<?xml version="1.0"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

<xsl:template name="content_poll">
  <xsl:variable name="content" select="/page/content/topic" />
  <h1 class="pollTitle"><xsl:value-of select="$content/title" disable-output-escaping="yes"/> 
    
  </h1>
  <div class="pollDescription">
    <xsl:value-of select="$content/text" disable-output-escaping="yes"/>
  </div>
  
  <xsl:if test="$content/categ">
    <xsl:call-template name="content_poll_categs">
      <xsl:with-param name="content" select="$content"/>
    </xsl:call-template>
  </xsl:if>
  <xsl:if test="$content/poll/@showdialog = 'no'">
    <xsl:call-template name="content_poll_only_results">
      <xsl:with-param name="content" select="$content"/>
    </xsl:call-template>
  </xsl:if>
  <xsl:if test="$content/poll/@showdialog = 'yes'">
    <xsl:call-template name="content_poll_form">
      <xsl:with-param name="content" select="$content"/>
    </xsl:call-template>
  </xsl:if>
  <xsl:if test="$content/error">
    <xsl:call-template name="content_poll_error">
      <xsl:with-param name="content" select="$content"/>
    </xsl:call-template>
  </xsl:if>  
</xsl:template>

<xsl:template name="content_poll_error">
  <xsl:param name="content" />
  <div class="pollError">
    <xsl:value-of select="$content/poll/@title" disable-output-escaping="yes"/>
    <xsl:value-of select="$content/error" disable-output-escaping="yes"/>
  </div>
</xsl:template>

<xsl:template name="content_poll_form">
  <xsl:param name="content" />
  <form class="pollForm" action="{$content/poll/@action}" method="post">
    <div class="question">
      <div class="date">
        (<xsl:call-template name="formatLongDate">
          <xsl:with-param name="isodate" select="$content/poll/@start_time"/>
        </xsl:call-template>
        - 
        <xsl:call-template name="formatLongDate">
          <xsl:with-param name="isodate" select="$content/poll/@end_time"/>
        </xsl:call-template>)
      </div>
      <xsl:value-of select="$content/poll/@title" disable-output-escaping="yes"/>
    </div>
    <input type="hidden" name="{$content/poll/@fieldname}" value="{$content/poll/@id}" />
    <xsl:for-each select="$content/poll/answer">
      <div>
        <input type="radio" name="{@fieldname}" value="{@id}" /> - 
          <xsl:value-of select="."/>
      </div>
    </xsl:for-each>
    <div class="button">
      <input class="button" type="submit" value="Abstimmen" />
    </div>
    <br clear="all" class="clear" />
  </form>
</xsl:template>

<xsl:template name="content_poll_only_results">
  <xsl:param name="content" />
  <div class="poll">
    <div class="votedResults">
      <div class="question">
        <div class="date">
          (<xsl:call-template name="formatShortDate">
            <xsl:with-param name="isodate" select="$content/poll/@start_time"/>
          </xsl:call-template>
          - 
          <xsl:call-template name="formatShortDate">
            <xsl:with-param name="isodate" select="$content/poll/@end_time"/>
          </xsl:call-template>)
        </div>
        <xsl:value-of select="$content/poll/@title" disable-output-escaping="yes"/>      
      </div>
      <xsl:variable name="voted" select="number($content/poll/voted)" />
      <xsl:variable name="maxResult">
        <xsl:call-template name="max">
          <xsl:with-param name="values" select="$content/poll/answer/@result"/>
        </xsl:call-template>
      </xsl:variable>
      <table border="0" class="answers">
        <xsl:for-each select="$content/poll/answer">
          <tr>
            <td class="oneAnswer"><xsl:value-of select="."/></td>
            <td class="answerResult"><xsl:value-of select="@result"/></td>
            <td class="answerProzent">
              (<xsl:call-template name="formatNumber">
                 <xsl:with-param name="float" select="100 * number(@result) div $voted"/>
                 <xsl:with-param name="pattern">##0.0</xsl:with-param>
               </xsl:call-template>%)
            </td>
          </tr>
          <tr>
            <td colspan="3">
              <xsl:if test="@result &gt;= 0">
              <div class="balken">
                <xsl:choose>
                  <xsl:when test="@result &gt; 0">
                    <div class="progress{floor(@result div $maxResult * 5)}" style="width: {round(@result div $maxResult * 100)}%">
                      <xsl:call-template name="space"/>
                    </div>
                  </xsl:when>
                  <xsl:otherwise>
                    <xsl:call-template name="space"/>
                  </xsl:otherwise>
                </xsl:choose>
              </div>
            </xsl:if>
            </td>
          </tr>
        </xsl:for-each>
      </table>
      <br clear="all" class="clear" />
    </div>
  </div>
</xsl:template>

<xsl:template name="content_poll_categs">
    <xsl:param name="content" />
    <xsl:if test="$content/categ/poll/@status = 'active'">
      <h2 class="pollActionTitle">
        Active Umfragen:  
      </h2>
      <div class="pollCategs">
        <xsl:for-each select="$content/categ/poll">
          <div>
            <xsl:if test="@status = 'active'">
              <a href="{@href}">
                <xsl:value-of select="@title"/>
              </a>
            </xsl:if>
          </div>
        </xsl:for-each>
      </div>
    </xsl:if>
    <xsl:if test="$content/categ/poll/@status = 'closed'">
      <h2 class="pollActionTitle">
        Inaktive Umfragen:  
      </h2>
      <div class="pollCategs">
        <xsl:for-each select="$content/categ/poll">
          <div>
            <xsl:if test="@status = 'closed'">
              <a href="{@href}">
                <xsl:value-of select="@title"/>
              </a>
            </xsl:if>
          </div>
        </xsl:for-each>
      </div>
    </xsl:if>
</xsl:template>


</xsl:stylesheet>