<?xml version="1.0"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns="http://www.w3.org/1999/xhtml">

<xsl:param name="PAGE_THEME_PATH" />
<xsl:param name="PAGE_WEB_PATH" />

<xsl:output method="xml" encoding="utf-8" standalone="no" indent="yes" omit-xml-declaration="yes" />
  
<xsl:template name="space">
  <xsl:text disable-output-escaping="yes">&amp;nbsp;</xsl:text>
</xsl:template>
<xsl:template name="copyright">
  <xsl:text disable-output-escaping="yes">&amp;copy;</xsl:text>
</xsl:template>

<xsl:template name="monthcalendar_month">
  <xsl:param name="month"/>
  <a href="{$month/@href}">
  <xsl:variable name="monthtitle">
    <xsl:call-template name="monthname">
      <xsl:with-param name="month" select="$month/@month"/>
    </xsl:call-template> 
  </xsl:variable>
   <xsl:choose>
     <xsl:when test="$month[@position='prior']">
       &#xAB;
     </xsl:when>
     <xsl:when test="$month[@position='next']">
       &#xBB;
     </xsl:when>
     <xsl:otherwise><xsl:value-of select="$monthtitle"/></xsl:otherwise>
   </xsl:choose>  
  </a>
</xsl:template>

<xsl:template name="monthcalendar_day">
  <xsl:param name="day"/>
  <xsl:param name="class">
    <xsl:choose>
      <xsl:when test="$day/@type='filled'">calseldata</xsl:when>
      <xsl:otherwise>caldata</xsl:otherwise>
    </xsl:choose>
  </xsl:param>
  <td>
    <xsl:attribute name="class">
      <xsl:choose>
        <xsl:when test="($day/@href) and ($day/@selected)">
          <xsl:value-of select="$class"/> select
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="$class"/>
        </xsl:otherwise>      
      </xsl:choose>
    </xsl:attribute>
    <xsl:if test="$day/@dayspan > 1">
      <xsl:attribute name="colspan"><xsl:value-of select="$day/@dayspan"/></xsl:attribute>
    </xsl:if>
    <xsl:choose>
      <xsl:when test="($day/@href) and ($day/@selected)">
        <a href="{$day/@href}" title="{$day/@hint}"><xsl:value-of select="$day"/></a>
      </xsl:when>
      <xsl:when test="$day/@href">
        <a href="{$day/@href}" title="{$day/@hint}"><xsl:value-of select="$day"/></a>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$day"/>
      </xsl:otherwise>
    </xsl:choose>
  </td>
</xsl:template>

<xsl:template name="monthname">
  <xsl:param name="month"/>
  <xsl:choose>
    <xsl:when test="$month = 1">Januar</xsl:when>
    <xsl:when test="$month = 2">Februar</xsl:when>
    <xsl:when test="$month = 3">M&#xE4;rz</xsl:when>
    <xsl:when test="$month = 4">April</xsl:when>
    <xsl:when test="$month = 5">Mai</xsl:when>
    <xsl:when test="$month = 6">Juni</xsl:when>
    <xsl:when test="$month = 7">Juli</xsl:when>
    <xsl:when test="$month = 8">August</xsl:when>
    <xsl:when test="$month = 9">September</xsl:when>
    <xsl:when test="$month = 10">Oktober</xsl:when>
    <xsl:when test="$month = 11">November</xsl:when>
    <xsl:otherwise>Dezember</xsl:otherwise>
  </xsl:choose>
</xsl:template>

<xsl:template name="wdayshort">
  <xsl:param name="wday"/>
  <xsl:choose>
    <xsl:when test="$wday = 1">Mo</xsl:when>
    <xsl:when test="$wday = 2">Di</xsl:when>
    <xsl:when test="$wday = 3">Mi</xsl:when>
    <xsl:when test="$wday = 4">Do</xsl:when>
    <xsl:when test="$wday = 5">Fr</xsl:when>
    <xsl:when test="$wday = 6">Sa</xsl:when>
    <xsl:otherwise>So</xsl:otherwise>
  </xsl:choose>
</xsl:template>

<xsl:template name="wdaytitle">
  <xsl:param name="wday"/>
  <xsl:choose>
    <xsl:when test="$wday = 1">Montag</xsl:when>
    <xsl:when test="$wday = 2">Dienstag</xsl:when>
    <xsl:when test="$wday = 3">Mittwoch</xsl:when>
    <xsl:when test="$wday = 4">Donnerstag</xsl:when>
    <xsl:when test="$wday = 5">Freitag</xsl:when>
    <xsl:when test="$wday = 6">Samstag</xsl:when>
    <xsl:otherwise>Sonntag</xsl:otherwise>
  </xsl:choose>
</xsl:template>

<xsl:template name="monthcalendar" match="monthcalendar">
<table border="0" class="calender">
  <xsl:if test="monthnav">
    <tr>
      <td class="calheader navi">
        <xsl:call-template name="monthcalendar_month">
          <xsl:with-param name="month" select="./monthnav/month[@position='prior']"/>
        </xsl:call-template>
      </td>
      <td class="calheader" colspan="6">
        <xsl:call-template name="monthcalendar_month">
          <xsl:with-param name="month" select="./monthnav/month[@position='actual']"/>
        </xsl:call-template>
      </td>
      <td class="calheader navi">
        <xsl:call-template name="monthcalendar_month">
          <xsl:with-param name="month" select="./monthnav/month[@position='next']"/>
        </xsl:call-template>
      </td>
    </tr>
  </xsl:if>
  <xsl:if test="weekdays">
    <tr>
      <td class="calheader">
      </td>
      <xsl:for-each select="weekdays/wday">
        <xsl:variable name="wdayshort">
          <xsl:call-template name="wdayshort">
            <xsl:with-param name="wday" select="@wday"/>
          </xsl:call-template> 
        </xsl:variable>
        <xsl:variable name="wdaytitle">
          <xsl:call-template name="wdaytitle">
            <xsl:with-param name="wday" select="@wday"/>
          </xsl:call-template> 
        </xsl:variable>
        <td class="calheader"><a href="{@href}" title="{$wdaytitle}"><xsl:value-of select="$wdayshort"/></a></td>
      </xsl:for-each> 
    </tr>
  </xsl:if>
  <xsl:for-each select="weeks/week">
  <tr>
    <td class="calheader"><a href="{@href}"><xsl:value-of select="@no"/></a></td>
    <xsl:for-each select="day">
      <xsl:call-template name="monthcalendar_day">
        <xsl:with-param name="day" select="."/>
      </xsl:call-template>
    </xsl:for-each>
  </tr>
  </xsl:for-each>
</table>
</xsl:template>

</xsl:stylesheet>