<?xml version="1.0"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns="http://www.w3.org/1999/xhtml">

<xsl:template name="content_casestudy">
  <xsl:param name="content" select="/page/content/topic"/>
  <xsl:if test="$content/image">    
    <a href="{$content/navigation/navlink[@dir='index']/@href}" class="back">Zur&#xFC;ck</a>
  </xsl:if>
  <h1>
    <xsl:choose>
      <xsl:when test="$content/title/text() != ''">
        <xsl:value-of select="$content/title" disable-output-escaping="yes"/></xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$PAGE_TITLE" disable-output-escaping="yes"/>
      </xsl:otherwise>
    </xsl:choose>
    <xsl:if test="$content/subtitle/text() != ''">
      <span class="contentSubtitle">
        <xsl:value-of select="$content/subtitle" disable-output-escaping="yes"/>
      </span>
    </xsl:if>   
  </h1>
  <div class="contentCaseStudy">
    <xsl:choose>
      <xsl:when test="$content/image">
        <xsl:if test="count($content/thumbnails//thumb[count(.//img) &gt; 0]) &gt; 1">
          <xsl:call-template name="casestudy_thumbtable">
          <xsl:with-param name="thumbs" select="$content/thumbnails//thumb[count(.//img) &gt; 0]"/>
          </xsl:call-template>
        </xsl:if>
        <xsl:if test="$content/imagetitle/text() != ''">
          <h3 class="imageTitle"><xsl:value-of select="$content/imagetitle/text()"/></h3>
        </xsl:if>
        <div class="image">
          <xsl:apply-templates select="$content/image/*"/>
        </div>
        <xsl:if test="$content/imagecomment/text() != ''">
          <div class="imageDescription">
        	  <xsl:value-of select="$content/imagecomment"/>
          </div>
          <br clear="all" class="clear"/>
        </xsl:if>
      </xsl:when>
      <xsl:otherwise>
     	  <table class="caseStudy">
     		  <tr>
            <td class="caseStudyText">
              <xsl:apply-templates select="$content/text"/>
            </td>
            <xsl:if test="$content/thumbnails/line/thumb/a/@href !=''">
              <td class="caseStudyThumbs">
                <xsl:for-each select="$content/thumbnails//thumb">
     			        <div><xsl:apply-templates/></div>
     			      </xsl:for-each>
     		      </td>
            </xsl:if>
          </tr>
        </table>
      </xsl:otherwise>
    </xsl:choose>
  </div>
</xsl:template>

<xsl:template name="casestudy_thumbtable">
  <xsl:param name="thumbs" />
    <table border="0" class="thumbNails">
    <xsl:variable name="count" select="count($thumbs)" />
    <xsl:variable name="lineCount">2</xsl:variable>
    <xsl:for-each select="$thumbs[(position() mod $lineCount) = 1]">
      <xsl:variable name="indexmin" select="position() * $lineCount - $lineCount" />
      <xsl:variable name="indexmax" select="position() * $lineCount" />
      <xsl:call-template name="casestudy_thumbline">
        <xsl:with-param name="thumbs" 
          select="$thumbs[((position() &gt; $indexmin) and (position() &lt;= $indexmax))]"/>
        <xsl:with-param name="thumbsCount" select="$count" />
        <xsl:with-param name="lineCount" select="$lineCount"/>
        <xsl:with-param name="lineNo" select="position()"/>
      </xsl:call-template>
    </xsl:for-each>	
    </table>
</xsl:template>

<xsl:template name="casestudy_thumbline">
  <xsl:param name="thumbs"/>
  <xsl:param name="thumbsCount"/>
  <xsl:param name="lineCount"/>
  <xsl:param name="lineNo"/>
  <xsl:variable name="count" select="count($thumbs)" />
  <tr>
    <xsl:for-each select="$thumbs">
      <xsl:call-template name="casestudy_thumb">
        <xsl:with-param name="thumb" select="."/>  
        <xsl:with-param name="pos" select="position()"/>    
      </xsl:call-template>
    </xsl:for-each>
    <xsl:variable name="colspan" select="$lineCount - $count"/>
    <xsl:if test="($colspan &gt; 0) and ($lineNo &gt; 1)">
      <td>
        <xsl:if test="$colspan &gt; 1">
          <xsl:attribute name="colspan"><xsl:value-of select="$colspan"/></xsl:attribute>
        </xsl:if>
        <xsl:call-template name="space"/>
      </td> 
    </xsl:if>
  </tr>
</xsl:template>

<xsl:template name="casestudy_thumb">
  <xsl:param name="thumb"/>
  <xsl:param name="pos"/>
  <xsl:if test="$thumb">
  <xsl:variable name="selected">
    <xsl:choose>
      <xsl:when test="$thumb/@selected">selectedThumb</xsl:when>
    </xsl:choose>
  </xsl:variable>
  <td class="{$selected}"><xsl:apply-templates select="$thumb//a|img"/></td>
  </xsl:if>
</xsl:template>

</xsl:stylesheet>
