<?xml version="1.0"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns="http://www.w3.org/1999/xhtml">
<xsl:template name="content_thumbs">
  <xsl:param name="content" select="/page/content/topic"/>
  <div class="contentTopic">
    <h1><xsl:value-of select="$PAGE_TITLE"/></h1>
  </div>
  <div class="thumbs">
    <xsl:call-template name="content_thumbs_navigation">
      <xsl:with-param name="links" select="$content/navigation/navlink"/>
    </xsl:call-template>
    <xsl:if test="$content/thumbnails/thumb">
      <xsl:call-template name="content_thumbs_thumbnails">
        <xsl:with-param name="thumbs" select="$content/thumbnails/thumb"/>
        <xsl:with-param name="options" select="$content/options"/>
      </xsl:call-template>
    </xsl:if>
  	<xsl:if test="$content/image">
      <xsl:if test="$content/imagetitle != ''">
        <div class="contentTopic">
          <h2><xsl:apply-templates select="$content/imagetitle"/></h2>
          <xsl:if test="$content/imagecomment/text()">
            <div class="contentText">
              <xsl:apply-templates select="$content/imagecomment"/>
            </div>
          </xsl:if>
        </div>
      </xsl:if>
      <div class="largeImage">
        <xsl:apply-templates select="$content/image//img|object"/>
      </div>
  	</xsl:if>
    <xsl:call-template name="content_thumbs_navigation">
      <xsl:with-param name="links" select="$content/navigation/navlink"/>
    </xsl:call-template>
  </div>
</xsl:template>

<xsl:template name="content_thumbs_navigation">
  <xsl:param name="links"/>
	<xsl:if test="count($links) &gt; 0">
    <div class="navigation">
  	  <xsl:if test="$links[@dir='prior']">
				<a href="{$links[@dir='prior']/@href}" class="linkLeft">Vorherige Seite</a>
		  </xsl:if>
  	  <xsl:if test="$links[@dir='next']">
				<a href="{$links[@dir='next']/@href}" class="linkRight">N&#xE4;chste Seite</a>
  		</xsl:if>
  	  <xsl:if test="$links[@dir='index']">
  		  <a href="{$links[@dir='index']/@href}" class="linkCenter">&#xDC;bersicht</a>
  		</xsl:if>
      <br clear="all" class="clear"/>
    </div>
	</xsl:if>
</xsl:template>

<xsl:template name="content_thumbs_thumbnails">
  <xsl:param name="thumbs"/>
  <xsl:param name="options"/>
  <xsl:variable name="colCount">
    <xsl:choose>
      <xsl:when test="$options/maxperline"><xsl:value-of select="$options/maxperline"/></xsl:when>
      <xsl:otherwise>3</xsl:otherwise>
    </xsl:choose>
  </xsl:variable>
  <table class="contentThumbnails">
    <xsl:for-each select="$thumbs[(position() mod $colCount) = 1]">
      <xsl:variable name="indexmin" select="position() * $colCount - $colCount" />
      <xsl:variable name="indexmax" select="position() * $colCount" />
      <xsl:call-template name="content_thumbs_line">
        <xsl:with-param name="thumbs" select="$thumbs[((position() &gt; $indexmin) and (position() &lt;= $indexmax))]"/>
        <xsl:with-param name="colCount" select="$colCount"/>
        <xsl:with-param name="rowIdx" select="position()"/>
        <xsl:with-param name="options" select="$options"/>
      </xsl:call-template>
    </xsl:for-each>	
  </table>
</xsl:template>

<xsl:template name="content_thumbs_line">
  <xsl:param name="thumbs"/>
  <xsl:param name="colCount"/>
  <xsl:param name="rowIdx"/>
  <xsl:param name="options"/>
  <xsl:variable name="thumbsCount" select="count($thumbs)"/>
  <xsl:variable name="colWidth"><xsl:value-of select="floor(100 div $colCount)"/></xsl:variable>
  <tr>
    <xsl:for-each select="$thumbs">
      <xsl:variable name="style">
        <xsl:choose>
          <xsl:when test="$rowIdx &gt; 1"></xsl:when>
          <xsl:when test="position() = last()">width: <xsl:value-of select="100 - ($colWidth * (position() - 1))"/>%;</xsl:when>
          <xsl:otherwise>width: <xsl:value-of select="$colWidth"/>%;</xsl:otherwise>
        </xsl:choose>
      </xsl:variable>
      <td>
        <xsl:if test="$style != ''">
          <xsl:attribute name="style"><xsl:value-of select="$style"/></xsl:attribute>
        </xsl:if>
        <xsl:call-template name="content_thumbs_thumbnail">
          <xsl:with-param name="thumb" select="."/>
          <xsl:with-param name="options" select="$options"/>
        </xsl:call-template>
      </td>
    </xsl:for-each>
    <xsl:if test="$thumbsCount &lt; $colCount">
      <xsl:variable name="colspan" select="$colCount - $thumbsCount"/>
      <td>
        <xsl:if test="$colspan &gt; 1">
          <xsl:attribute name="colspan"><xsl:value-of select="$colspan"/></xsl:attribute>
        </xsl:if>
        <xsl:call-template name="space"/>
      </td>
    </xsl:if>
  </tr>
</xsl:template>

<xsl:template name="content_thumbs_thumbnail">
  <xsl:param name="thumb"/>
  <xsl:param name="options"/>
  <div style="width: {$options/thumbwidth}px; height: {$options/thumbheight}px; line-height: {$options/thumbheight}px;">
    <a href="{$thumb//a/@href}"><xsl:copy-of select=".//img"/></a>  
  </div>
  <xsl:if test="$thumb//div[@class='papayaImageSubtitle']">
    <div class="imageSubTitle"><xsl:value-of select="$thumb//div[@class='papayaImageSubtitle']" disable-output-escaping="yes"/></div>
  </xsl:if>
</xsl:template>

</xsl:stylesheet>