<?xml version="1.0"?> 
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns="http://www.w3.org/1999/xhtml">

<xsl:import href="./base/base.xsl" />
<xsl:import href="./lang/language.xsl" />
<xsl:import href="./lang/de.xsl"/>
<xsl:import href="./lang/en.xsl"/>

<xsl:import href="./content/categ.xsl" />
<xsl:import href="./content/topic.xsl" />

<xsl:output method="xml" encoding="utf-8" standalone="yes" doctype-public="-//W3C//DTD XHTML 1.0 Strict//EN" doctype-system="http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd" indent="yes" omit-xml-declaration="yes" /> 

<!-- Params -->
<xsl:param name="PAGE_THEME_PATH" />
<xsl:param name="PAGE_WEB_PATH" />
<xsl:param name="PAGE_TITLE" />
<xsl:param name="PAGE_PRINT_URL" />
  
<xsl:template match="page">
<html>
  <head>
    <title><xsl:value-of select="meta/metatags/pagetitle"/> - <xsl:value-of select="$PAGE_TITLE" disable-output-escaping="yes" /></title>  
    <xsl:apply-templates select="meta/metatags"/>
    <link rel="stylesheet" type="text/css" href="{$PAGE_THEME_PATH}style.css" />
    <link rel="stylesheet" type="text/css" href="{$PAGE_THEME_PATH}style_box.css" />
    <link rel="shortcut icon" href="/favicon.ico" />
    <xsl:call-template name="cssfiles"/>
    <script type="text/javascript" src="{$PAGE_WEB_PATH}papaya-script/popup.js"><xsl:text> </xsl:text></script>
    <script type="text/javascript" src="{$PAGE_WEB_PATH}papaya-script/flashmode.js"><xsl:text> </xsl:text></script>
    <script type="text/vbscript" src="{$PAGE_WEB_PATH}papaya-script/flashmode.vbs"><xsl:text> </xsl:text></script>
    <script type="text/javascript" src="{$PAGE_WEB_PATH}papaya-script/flashfile.js"><xsl:text> </xsl:text></script>
    <meta name="robots" content="index,follow" />
  </head>
  <body>
    <div id="content">
      <xsl:if test="count(boxes/box[@group = 'right']) = 0">
        <xsl:attribute name="class">largeContent</xsl:attribute>
      </xsl:if>
      <xsl:call-template name="content_area"/>
      <div id="footer">
        powered by <a href="http://www.papaya-cms.com/">papaya CMS</a>
      </div>
    </div>
    <div id="left">
      <xsl:if test="boxes/box[@group = 'navigation']">
        <div id="navigation">
          <xsl:for-each select="boxes/box[@group = 'navigation']">
            <div class="box navigation">
              <xsl:if test="@name != ''">
                <div class="title"><xsl:value-of select="@name" disable-output-escaping="yes"/></div>
              </xsl:if>
              <xsl:value-of select="." disable-output-escaping="yes"/>
            </div>
          </xsl:for-each>
        </div>
      </xsl:if>
      <xsl:if test="boxes/box[@group = 'left']">
        <div id="leftBoxes">
          <xsl:for-each select="boxes/box[@group = 'left']">
            <div class="box">
              <xsl:if test="@title != ''">
                <div class="title"><xsl:value-of select="@title" disable-output-escaping="yes"/></div>
              </xsl:if>
              <xsl:value-of select="." disable-output-escaping="yes"/>
            </div>
          </xsl:for-each>
	      </div>
      </xsl:if>
    </div>
    <xsl:if test="count(boxes/box[@group = 'right']) &gt; 0">
      <div id="right">
        <xsl:for-each select="boxes/box[@group = 'right']">
          <div class="box">
            <xsl:if test="@title != ''">
              <div class="boxTitle"><xsl:value-of select="@title" disable-output-escaping="yes"/></div>
            </xsl:if>
            <div class="boxData"><xsl:value-of select="." disable-output-escaping="yes"/></div>
          </div>
        </xsl:for-each>
      </div>
    </xsl:if>
    <div id="hbar"><xsl:text> </xsl:text></div>
    <div id="header"><a href="./"><img src="{$PAGE_THEME_PATH}pics/logo.gif" alt="" id="img_logo" /></a></div>
  </body>
</html>
</xsl:template>

<xsl:template name="content_area">
  <xsl:variable name="module" select="content/topic/@module"/>
  <xsl:choose>
    <xsl:when test="$module = 'content_categ' or $module = 'content_categimg'">
      <xsl:call-template name="content_categ"/>
    </xsl:when>
    <xsl:when test="$module = 'content_topic' or $module = 'content_imgtopic'">
      <xsl:call-template name="content_topic"/>
    </xsl:when>
    <xsl:otherwise>
      <xsl:call-template name="content_default"/>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<xsl:template name="cssfiles">
</xsl:template>

</xsl:stylesheet>