<?php
/**
* Page module casestudy
*
* @package module_default
* @author Thomas Weinert <weinert@papaya-cms.com>
* @version $Id: content_casestudy.php,v 1.5 2006/02/23 10:48:26 weinert Exp $
*/

/**
* Basic class page module
*/
require_once(PAPAYA_INCLUDE_PATH.'system/base_content.php');
/**
* file manager
*/
require_once(PAPAYA_INCLUDE_PATH.'system/base_fileman.php');
/**
* Page module casestudy
*
* combines article and image gallery
*
* @package module_default
* @author Thomas Weinert <weinert@papaya-cms.com>
*/
class content_casestudy extends base_content {

  var $editFields = array(
    'nl2br'=>array ('Automatic linebreak', 'isNum', FALSE, 'combo', array(0=>'Yes', 1=>'No'),
      'Papaya will apply your linebreaks to the output page.'),
    'title'=>array ('Title', 'isSomeText', TRUE, 'input', 60),
    'subtitle'=>array ('Subtitle', 'isSomeText', FALSE, 'input', 300),
    'teaser'=>array ('Teaser', 'isSomeText', TRUE, 'simplerichtext', 10),
    'text'=>array ('Text', 'isSomeText', FALSE, 'richtext', 30),
    'Box',
    'fb_title'=>array ('Title', 'isSomeText', FALSE, 'input', 300),
    'fb_text'=>array ('Text', 'isSomeText', FALSE, 'simplerichtext', 10),
    'Images',
    'directory'=>array ('Folder', 'isNum', TRUE, 'function', 'mediadirectory', '', 0),
    'width'=>array ('Width', 'isNum', TRUE, 'input', 5, '', 640),
    'height'=>array ('Height', 'isNum', TRUE, 'input', 5, '', 480),
    'Thumbnail',
    'thumbwidth'=>array ('Width', 'isNum', TRUE, 'input', 5, '', 100),
    'thumbheight'=>array ('Height', 'isNum', TRUE, 'input', 5, '', 100),
  );

  /**
  * Get parsed data
  *
  * @access public
  * @return string
  */
  function getParsedData() {
    $result = "";
    $result .= sprintf('<title>%s</title>'.LF,
      $this->getXHTMLString($this->data['title']));
    $result .= sprintf('<subtitle>%s</subtitle>'.LF,
      $this->getXHTMLString($this->data['subtitle']));
    $result .= sprintf('<teaser>%s</teaser>'.LF,
      $this->getXHTMLString($this->data['teaser'], !((bool)$this->data['nl2br'])));
    $result .= sprintf("<text>%s</text>",
      $this->getXHTMLString($this->data['text'], !((bool)$this->data['nl2br'])));
    $result .= sprintf('<factbox title="%s">%s</factbox>'.LF,
      htmlspecialchars($this->getXHTMLString($this->data['fb_title'])),
      $this->getXHTMLString($this->data['fb_text'], !((bool)$this->data['nl2br'])));
    $result .= $this->getThumbs();
    return $result;
  }

  /**
  * Get thumbs
  *
  * @access public
  * @return string
  */
  function getThumbs() {
    $result = "";
    $selected = "";

    if (isset($this->topicObj->topicId)) {
      $this->baseLink = $this->getBaseLink($this->topicObj->topicId);
    }

    $fileManager = &new base_filemanager;
    $fileManager->dataDirectory = PAPAYA_PATH_MEDIAFILES;
    $fileManager->thumbnailDirectory = PAPAYA_PATH_THUMBFILES;
    $fileManager->folderId = (int)$this->data['directory'];

    $fileManager->dataLoadFileList($this->topicObj->getContentLanguageId(), TRUE, TRUE);

    $count = count($fileManager->files);
    $min = 0;
    $max = (6 < $count) ? 5 : $count-1;

    if (isset($this->params['mode']) && isset($this->params['img']) && ($this->params['mode'] == 'max') && ($this->params['img'] >= 0)) {
      //Groe Ansicht
      $result .= '<image>';
        $file = $fileManager->files[$this->params['img']];
        $result .= sprintf('<papaya:media src="%s" width="%d" height="%d" align="center"/>'.LF,
          $file['file_id'], (int)$this->data['width'], (int)$this->data['height']);
      $result .= '</image>';
      $result .= sprintf('<imagetitle>%s</imagetitle>', htmlspecialchars($file['file_title']));
      $result .= sprintf('<imagecomment>%s</imagecomment>', htmlspecialchars($file['file_comment']));
      $result .= '<navigation>'.LF;
      $result .= sprintf('<navlink dir="index" href="%s?%s[idx]=%d" />'.LF,
        $this->baseLink, $this->paramName, (int)$this->params['idx']);
      $result .= '</navigation>'.LF;
    }
    $result .= '<thumbnails>'.LF;
    $result .= '<line>'.LF;
    for ($i = $min; $i <= $max; $i++) {
      $file = $fileManager->files[$i];
      $selected = (@$this->params['img'] == $i) ? ' selected="selected"' : '';
      $href = sprintf('%s?%s[mode]=max&amp;%s[idx]=%d&amp;%s[img]=%d',
            $this->baseLink, $this->paramName,
          $this->paramName, @$this->params['idx'],
          $this->paramName, $i);

      $result .= sprintf('<thumb %s><papaya:media href="%s" src="%s" width="%d" height="%d" align="center" lspace="0" tspace="0" rspace="0" bspace="0" /></thumb>'.LF,
        $selected, $href, $file['file_id'], (int)$this->data['thumbwidth'], (int)$this->data['thumbheight']);
      $spacer = @$this->data['maxperline'] - (($i+1) % @$this->data['maxperline']);
      $lineEnd = (isset($this->data['maxperline']) && (($i+1) % $this->data['maxperline']) == 0);
      if ($lineEnd && ($i < ($max))) {
        $result .= '</line><line>'.LF;
      }
    }
    if (isset($spacer) && $spacer > 0 && ($spacer < $this->data['maxperline'])) {
      $result .= sprintf('<spacer span="%d"/>'.LF, $spacer);
    }
    $result .= '</line>'.LF;
    $result .= '</thumbnails>'.LF;
    return $result;
  }

  /**
  * Get parsed teaser
  *
  * @access public
  * @return string
  */
  function getParsedTeaser() {
    $result = "";
    $result .= sprintf('<title>%s</title>'.LF,
      $this->getXHTMLString($this->data['title']));
    $result .= sprintf('<subtitle>%s</subtitle>'.LF,
      $this->getXHTMLString($this->data['subtitle']));
    $result .= sprintf('<text>%s</text>'.LF,
      $this->getXHTMLString($this->data['teaser'], TRUE));
    return $result;
  }

  /**
  * Media directory
  *
  * @param string $name
  * @param string $field
  * @param integer $data
  * @access public
  * @return string
  */
  function mediadirectory($name, $field, $data) {
    $result = '';

    $fileManager = &new base_filemanager;
    $fileManager->dataDirectory = PAPAYA_PATH_MEDIAFILES;
    $fileManager->thumbnailDirectory = PAPAYA_PATH_THUMBFILES;

    $fileManager->dataLoadFolderList($this->topicObj->getContentLanguageId());
    $this->folders = $fileManager->folders;
    $this->folderLinks = $fileManager->folderLinks;

    $result .= sprintf('<select name="%s[%s]" class="dialogSelect dialogScale">'.LF,
      $this->paramName, $name);
    $result .= $this->getFolder(0,(int)$data, 0);
    $result .= '</select>'.LF;

    return $result;
  }

  /**
  * Get folder
  *
  * @param integer $id
  * @param integer $selId
  * @param integer $indent optional, default value 0
  * @access public
  * @return string
  */
  function getFolder($id, $selId, $indent=0) {
    $result = '';
    $folder = $this->folders[$id];
    $subFolders = @$this->folderLinks[$id]['CHILDS'];
    if (isset($folder) && is_array($folder)) {
      if ($selId == $id) {
        $selected = ' selected="selected"';
      } else {
        $selected = '';
      }
      $result .= sprintf('<option value="%d"%s>%s%s</option>'.LF,
        $folder['folder_id'], $selected, str_repeat('-',$indent*2), htmlspecialchars($folder['folder_title']));
      if (isset($subFolders) && is_array($subFolders)) {
        foreach($subFolders as $sub) {
          $result .= $this->getFolder($sub, $selId, $indent+1);
        }
      }
    }
    return $result;
  }
}

?>