<?php
/**
* Page module - category with image
*
* @package module_default
* @author Thomas Weinert <weinert@papaya-cms.com>
* @version $Id: content_categimg.php,v 1.12 2006/04/27 09:34:59 weinert Exp $
*/

/**
* Basic class page module
*/
require_once(PAPAYA_INCLUDE_PATH.'system/base_content.php');
/**
* Article list
*/
require_once(PAPAYA_INCLUDE_PATH.'system/base_topiclist.php');

/**
* Page module - category with image
*
* Display subordinate as page teaser as images
*
* @package module_default
* @author Thomas Weinert <weinert@papaya-cms.com>
*/
class content_categimg extends base_content {

  var $editFields = array(
    'max'=>array ('Count', 'isNum', TRUE, 'input', 5, '', '10000'),
    'columns'=>array ('Column count', 'isNum', TRUE, 'input', 1, '', '2'),
    'sort'=>array ('Sort', 'isNum', TRUE, 'combo', array(0=>'Aufsteigend', 1=>'Absteigend')),
    'Image',
    'image'=>array ('Image', 'isSomeText', FALSE, 'image', 400),
    'imgalign'=>array ('Image align', 'isAlpha', TRUE, 'combo', array('left'=>'left', 'right'=>'right')),
    'breakstyle'=>array ('Text float', 'isAlpha', TRUE, 'combo',
      array('none'=>'None','side'=>'Side', 'center'=>'Center')),
    'Texts',
    'nl2br'=>array ('Automatic linebreak', 'isNum', FALSE, 'combo', array(0=>'Yes', 1=>'No'),
      'Papaya will apply your linebreaks to the output page.'),
    'title'=>array ('Title', 'isSomeText', FALSE, 'input', 400),
    'subtitle'=>array ('Subtitle', 'isSomeText', FALSE, 'input', 400),
    'teaser'=>array ('Teaser', 'isSomeText', FALSE, 'simplerichtext', 10),
    'text'=>array ('Text', 'isSomeText', FALSE, 'richtext', 20),
    'Thumbnails',
    'thumbwidth'=>array ('Width', 'isNum', TRUE, 'input', 5, '', 100),
    'thumbheight'=>array ('Height', 'isNum', TRUE, 'input', 5, '', 100),
    'resizemode'=>array ('Resize mode', 'isAlpha', TRUE, 'combo',
      array('max'=>'Maximal', 'min'=>'Minimum', 'mincrop'=>'Minimum crop', 'abs'=>'Absolute'), '', 'max'),
  );

  /**
  * Get parsed data
  *
  * @access public
  * @return string
  */
  function getParsedData() {
    $result = sprintf('<title encoded="%s">%s</title>'.LF,
      rawurlencode(@$this->data['title']), $this->getXHTMLString(@$this->data['title']));
    $result .= sprintf('<subtitle>%s</subtitle>'.LF,
      $this->getXHTMLString(@$this->data['subtitle']));
    $result .= sprintf('<image align="%s" break="%s">%s</image>'.LF,
      htmlspecialchars(@$this->data['imgalign']), htmlspecialchars(@$this->data['breakstyle']),
      $this->getPapayaImageTag(@$this->data['image']));
    $result .= sprintf("<teaser>%s</teaser>", @$this->getXHTMLString($this->data['teaser'],
      !((bool)$this->data['nl2br'])));
    $result .= sprintf("<text>%s</text>", $this->getXHTMLString(@$this->data['text'],  !((bool)@$this->data['nl2br'])));
    $result .= sprintf('<columns>%d</columns>'.LF, (int)@$this->data['columns']);

    $topicList = &new base_topiclist;
    $topicList->databaseURI = $this->topicObj->databaseURI;
    $topicList->tableTopics = $this->topicObj->tableTopics;
    $topicList->tableTopicsTrans = $this->topicObj->tableTopicsTrans;
    $topicClass = get_class($this->topicObj);
    $topicList->loadList($this->topicObj->topicId, @(int)$this->topicObj->topic['TRANSLATION']['lng_id'],
      TRUE, ((bool)@$this->data['sort']));
    if (isset($this->data['max']) && $this->data['max'] < 1) {
      $this->data['max'] = 1000;
    }
    $subTopicString = papaya_strings::entityToXML($topicList->getList($topicClass, @(int)$this->data['max']));
    $result .= $subTopicString;
    if (trim($subTopicString) != '') {
      if ($subTopics = $this->createXMLTree($subTopicString)) {
        $parent = NULL;
        $template = &new simple_xmlnode($parent, 'subtopic');
        $templateImage = &new simple_xmlnode($parent, 'image');
        unset($elements);
        unset($thumbs);
        $subTopics->extract($elements, $template);
        if (isset($elements["nodes"]) && is_array($elements["nodes"])) {
          foreach($elements["nodes"] as $node) {
            if (is_object($node)) {
              unset($imageNodes);
              unset($imageNode);
              unset($papayaNode);
              $node->extract($imageNodes, $templateImage);
              if (isset($imageNodes["nodes"]) && is_array($imageNodes["nodes"])) {
                $imageNode = reset($imageNodes["nodes"]);
                $papayaNode = @$imageNode->children[0];
                if (is_object($papayaNode) && @in_array($papayaNode->name, array('papaya:media', 'ndim:media'))) {
                  $thumbs[$node->attributes['no']] = '<papaya:media src="'.$papayaNode->attributes['src'].'" width="'.@(int)$this->data['thumbwidth'].'" height="'.@(int)$this->data['thumbheight'].'" resize="'.@papaya_strings::escapeHTMLChars($this->data['resizemode']).'" />';
                }
              }
            }
          }
        }
        if (isset($thumbs) && is_array($thumbs)) {
          $result .= '<subtopicthumbs>';
          foreach($thumbs as $no=>$thumb) {
            $result .= '<thumb topic="'.(int)$no.'">'.$thumb.'</thumb>';
          }
          $result .= '</subtopicthumbs>';
        }
      }
    }
    return $result;
  }

  /**
  * Get parsed teaser
  *
  * @access public
  * @return string
  */
  function getParsedTeaser() {
    $result = sprintf('<title>%s</title>'.LF,
      $this->getXHTMLString(@$this->data['title']));
    $result .= sprintf('<subtitle>%s</subtitle>'.LF,
      $this->getXHTMLString(@$this->data['subtitle']));
    if (isset($this->data['teaser']) && trim($this->data['teaser'] != '')) {
      $result .= sprintf("<text>%s</text>", @$this->getXHTMLString($this->data['teaser'], !((bool)$this->data['nl2br'])));
    } elseif (isset($this->data['text']) && trim($this->data['text']) != '') {
      $teaser = str_replace("\r\n","\n",$this->data['text']);
      if (preg_match("/^(.+)([\n]{2})/sU", $teaser, $regs)) {
        $teaser = $regs[1];
      }
      $result .= sprintf('<text>%s</text>'.LF, $this->getXHTMLString($teaser, !((bool)$this->data['nl2br'])));
    }
    $result .= sprintf('<image align="%s" break="%s">%s</image>'.LF,
      htmlspecialchars(@$this->data['imgalign']), htmlspecialchars(@$this->data['breakstyle']),
      $this->getPapayaImageTag(@$this->data['image']));
    return $result;
  }
}

?>