<?php
/**
* Cronjob-module sends reminders of todos
*
* @package module_base
* @author David Rekowski <rekowski@papaya-cms.com>
* @version $Id: cronjob_todos_reminder.php,v 1.6 2006/04/25 12:45:58 weinert Exp $
*/

/**
* Basic class Cronjobs
*/
require_once(PAPAYA_INCLUDE_PATH.'system/base_cronjob.php');
require_once(PAPAYA_INCLUDE_PATH.'system/sys_base_db.php');

/**
* Cronjob-module sends reminders of todos
*
* @package module_base
* @author David Rekowski <rekowski@papaya-cms.com>
*/
class cronjob_todos_reminder extends base_cronjob {
  var $modified = FALSE;

  var $editFields = array(
    'subject' => array('Subject', 'isNoHTML', TRUE, 'input', 100, '', 'You\'ve got something to do.'),
    'todo'    => array('TodoList', 'isNoHTML', TRUE, 'textarea', 10,
'cronjobID: {%id%} Todo: {%todo%} Date: {%from%} - {%to%} Topic: {%topic%} Comment: {%comment%} Status: {%status%} Priority: {%priority%}',
"cronjobID:  {%id%}
Todo:       {%todo%}
Date:       {%from%} - {%to%}
Topic:      {%topic%}
Comment:    {%comment%}
Status:     {%status%}
Priority:   {%priority%}
------------------------------
"
),
    'body'    => array('Email Body', 'isNoHTML', TRUE, 'textarea', 10, "You have something to do with the following:\n\n{%content%}", "You have something to do with the following:\n\n{%content%}"),
  );
  var $tableTopics = PAPAYA_DB_TBL_TOPICS;
  var $tableTopicsPublic = PAPAYA_DB_TBL_TOPICS_PUBLIC;
  var $tableAuthUser = PAPAYA_DB_TBL_AUTHUSER;

  function __construct($param) {
    $this->layout = &$param->layout;
    $this->dialog = &$param->dialog;
    $this->paramName = 'tr';
    $this->sessionParamName = 'PAPAYA_SESS_'.$this->paramName;
    $this->initializeParams();
    $this->sessionParams = $this->getSessionValue($this->sessionParamName);
    $this->setSessionValue($this->sessionParamName, $this->sessionParams);

    $this->tableTodos  = PAPAYA_DB_TABLEPREFIX."_todos";
    $this->tableTopicsTrans = PAPAYA_DB_TABLEPREFIX."_topic_trans";
    $this->tableTopicsPublicTrans = PAPAYA_DB_TABLEPREFIX."_topic_public_trans";
    $this->db = &new base_db;
    return parent::__construct($paramName);
  }

  function cronjob_catalog_subscriptions($param) {
    return $this->__construct(&$param);
  }

  function getReminder() {
    $result = array();
    $sql = "SELECT todo.todo_id, todo.title, todo.priority, todo.date_from, todo.date_to, todo.comment, todo.status,
              user.email,
              tt.topic_title
            FROM %s AS todo
              LEFT OUTER JOIN %s AS tt ON (tt.topic_id = todo.topic_id)
              LEFT OUTER JOIN %s AS user ON (user.user_id = todo.user_id_to)
            ORDER BY date_to";
    $params = array($this->tableTodos, $this->tableTopicsPublicTrans, $this->tableAuthUser);
    if ($res = $this->db->databaseQueryFmt($sql, $params)) {
      while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $result[$row['email']][] = $row;
      }
    }
    return $result;
  }

  function loadModifiedTopics() {
    $sql = "SELECT topic_title, t.topic_id, t.topic_created, email
             FROM %s AS t
             LEFT JOIN %s AS tt ON ( t.topic_id = tt.topic_id )
             LEFT JOIN %s AS tp ON ( t.topic_id = tp.topic_id )
             LEFT JOIN papaya_auth_user AS u ON ( t.author_id = u.user_id )
            WHERE tp.topic_id IS NULL
            ORDER BY t.topic_id";

    $params = array($this->tableTopics, $this->tableTopicsTrans, $this->tableTopicsPublic);

    if ($res = $this->db->databaseQueryFmt($sql, $params, $max)) {
      while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->modifiedTopics[$row["email"]][] = $row;
      }
    }
  }

  function execute() {
    if ($reminder = $this->getReminder()) {
      $this->loadModifiedTopics();
      // send emails
      include_once(PAPAYA_INCLUDE_PATH.'system/sys_email.php');
      $emailObj = &new email;

      require_once(PAPAYA_INCLUDE_PATH.'system/base_statictables.php');
      $states = base_statictables::getTodoStates();
      $priorities = base_statictables::getTodoPriorities();

      $mailsCount = 0;

      foreach($reminder as $email => $todos) {
        $content = '';
        $bodyData = array();
        if (trim($email) != '') {
          foreach($todos as $todo) {
            $content = array(
              '{%id%}'       => $todo['todo_id'],
              '{%todo%}'     => $todo['title'],
              '{%from%}'     => date('d.m.Y H:i', (int)$todo['date_from']),
              '{%to%}'       => date('d.m.Y H:i', (int)$todo['date_to']),
              '{%topic%}'    => $todo['topic_title'],
              '{%comment%}'  => $todo['comment'],
              '{%status%}'   => $states[$todo['status']],
              '{%priority%}' => $priorities[$todo['priority']],
            );
            $bodyData['content'] .= strtr($this->data['todo'], $content).LF;
          }
          if (isset($this->modifiedTopics[$email]) && is_array($this->modifiedTopics[$email]) && count($this->modifiedTopics[$email]) > 0) {
            foreach($this->modifiedTopics[$email] as $email => $topic) {
              $bodyData['topics'] .= sprintf("topic %s (%d)\ncreated: %s".LF,
                $topic['topic_title'], $topic['topic_id'],
                date('d.m.Y H:i', $topic['topic_created']));
            }
          }
          if (isset($bodyData)) {
            $emailObj->setTemplate('body', $this->data['body'], $bodyData);
            if ($emailObj->send($email, $this->data['subject'])) {
              $mailsCount++;
            }
          }
        }
      }
      printf('%d emails sent.'.LF, $mailsCount);
      return 0;
    }
  }

  /**
  * Check execution parameters
  *
  * @access public
  * @return boolean Execution possible?
  */
  function checkExecParams() {
    if ($this->data['subject'] != '' && $this->data['todo'] != '' && $this->data['body'] != '') {
      return TRUE;
    }
    return FALSE;
  }

}

?>