<?php
/**
* Time calculation - particular week day monthly
*
* @package module_time
* @author Thomas Weinert <weinert@papaya-cms.com>
* @version $Id: time_weekdaypermonth.php,v 1.2 2006/04/26 10:28:59 weinert Exp $
*/

/**
* Basic class time calculation
*/
require_once(PAPAYA_INCLUDE_PATH.'system/base_crontime.php');

/**
* Time calculation - particular week day monthly
*
* @package module_time
* @author Thomas Weinert <weinert@papaya-cms.com>
*/
class time_weekdaypermonth extends base_crontime {

  var $editFields = array(
    'weekday'=>array ('Weekday', 'isNum', FALSE, 'combo',
      array(1=>'Montag',2=>'Dienstag',3=>'Mittwoch',4=>'Donnerstag',5=>'Freitag', 6=>'Samstag', 7=>'Sonntag'), '', 1),
    'repeatmode'=>array ('Repeat', 'isNum', FALSE, 'combo',
      array(
        0=>'every 1. [weekday]',
        1=>'every 2. [weekday]',
        2=>'every 3. [weekday]',
        3=>'every 4. [weekday]',
        4=>'every last [weekday]',
        5=>'every week',
        6=>'every 2 week',
        7=>'every 3 week',
        8=>'every 4 week'), '', 0),
    'repeatmonth'=>array ('Month', 'isNum', FALSE, 'combo',
      array(
        0=>'every month',
        1=>'every 2. month',
        2=>'every 3. month',
        3=>'every 6. month'), '', 0),
  );

  var $secOfDay = 86400;

  /**
  * Get next date time
  *
  * @param integer $from
  * @access public
  * @return mixed
  */
  function getNextDateTime($from) {
    if ($this->data['repeatmode'] >= 0 && $this->data['repeatmode'] <= 4) {
      $base = $this->parseDateToArray($from);
      $days = $this->getWeekDaysOfMonth($base, $this->data['weekday']);
      if ($this->data['repeatmode'] < 4) {
        if ($from < ($result = $days[$this->data['repeatmode']])) {
          return $result;
        } else {
          $base = $this->parseDateToArray(mktime(0,0,0,$base['month']+$this->getPlusMonths(),1,$base['year']));
          $days = $this->getWeekDaysOfMonth($base, $this->data['weekday']);
          return $days[$this->data['repeatmode']];
        }
      } else {
        if ($from < ($result = end($days))) {
          return $result;
        } else {
          $base = $this->parseDateToArray(mktime(0,0,0,$base['month']+$this->getPlusMonths(),1,$base['year']));
          $days = $this->getWeekDaysOfMonth($base, $this->data['weekday']);
          return end($days);
        }
      }
    } elseif ($this->data['repeatmode'] >= 5 && $this->data['repeatmode'] <= 8) {
      $day = $from;
      if (($this->data['weekday']) < 1) $this->data['weekday'] = 1;
      if (($this->data['weekday']) > 7) $this->data['weekday'] = 7;
      $max = (int)$this->data['repeatmode']-4;
      $found = 0;
      while($found < $max) {
        $day = $this->dayOnly($day+$this->secOfDay+3601);
        if ($this->getWDay($day) == $this->data['weekday']) {
          ++$found;
        }
      }
      return $day;
    }
  }

  /**
  * Get plus months
  *
  * @access public
  * @return integer
  */
  function getPlusMonths() {
    switch((int)$this->data['repeatmonth']) {
      case 3 : return 6;
      case 2 : return 3;
      case 1 : return 2;
      default : return 1;
    }
  }

  /**
  * Get week days of month
  *
  * @param array $base
  * @param integer $weekDay
  * @access public
  * @return integer
  */
  function getWeekDaysOfMonth($base, $weekDay) {
    list($firstDay,$lastDay) = $this->getBorderDays($base);
    unset($weekDays);
    for ($day = $firstDay;
          $this->dayOnly($day) <= $lastDay;
          $day += $this->secOfDay) {
      if ($this->getWDay($day) == $weekDay) {
        $weekDays[] = $this->dayOnly($day);
      }
    }
    return $weekDays;
  }

  /**
  * Get border days
  *
  * @param array $date
  * @access public
  * @return array
  */
  function getBorderDays($date) {
    $firstDay = mktime(0,0,0,$date['month'],1,$date['year']);
    $lastDay = mktime(0,0,0,$date['month'],date('t',$date['date']),$date['year']);
    return array($firstDay,$lastDay);
  }

  /**
  * Transform date to array
  *
  * @param integer $date
  * @access public
  * @return array
  */
  function parseDateToArray($date) {
    $result['date'] = $date;
    $result['day'] = date('j',$date);
    $result['month'] = date('n',$date);
    $result['year'] = date('Y',$date);
    $result['wday'] = $this->getWDay($date);
    return $result;
  }

  /**
  * Get week day
  *
  * @param integer $date
  * @access public
  * @return string
  */
  function getWDay($date) {
    $result = strftime('%u',$date);
    return $result;
  }


  /**
  * Get day only
  *
  * @param string $date
  * @access public
  * @return integer
  */
  function dayOnly($date) {
    $dayArr = explode('-',date('Y-m-d', $date));
    if (isset($dayArr) && is_array($dayArr)) {
      return mktime(0,0,0,$dayArr[1],$dayArr[2],$dayArr[0]);
    }
    return 0;
  }

}
?>