<?php
/**
* FAQ Module
*
* @package module_faq
* @author Bodo Grassmann <grassmann@papaya-cms.com>
* @version $Id: base_faq.php,v 1.13 2006/05/15 09:28:46 weinert Exp $
*/

/**
* Basic class for database access
*/
require_once(PAPAYA_INCLUDE_PATH.'system/sys_base_db.php');

/**
* FAQ Module
*
* @author Bodo Grassmann <grassmann@papaya-cms.com>
* @package module_faq
*/
class base_faq extends base_db {

  /**
  * Surfer table
  * @var String $tableSurfer
  */
  var $tableSurfer = "";

  /**
  * Faq table
  * @var String $tableFaqs
  */
  var $tableFaqs = '';
  /**
  * Faq groups table
  * @var String $tableFaqgroups
  */
  var $tableFaqgroups = '';
  /**
  * Entry table
  * @var String $tableEntries
  */
  var $tableEntries = '';
  /**
  * Comment table
  * @var String $tableComments
  */
  var $tableComments = '';

  var $inputFieldSize = 'x-large';

  /**
  * Faq parameter name
  * @var string
  */
  var $paramName = '';
  /**
  * Faq seassion parameter name
  * @var string
  */
  var $sessionParamName = '';
  /**
  * Parameters
  * @var array
  */
  var $params = NULL;

  /**
  * Faqs
  * @var array
  */
  var $faqs = NULL;
  /**
  * Faq groups
  * @var array
  */
  var $faqGroups = NULL;
  /**
  * Faq group
  * @var array
  */
  var $faqGroup = NULL;

  /**
  * Faq
  * @var array
  */
  var $faq = NULL;

  /**
  * Cache search results
  * @var array
  */
  var $cacheSearchResults = FALSE;

  /**
  * Object base dialog
  * @var object base_dialog
  */
  var $faqDialog = NULL;
  /**
  * Full text search parameter
  * @var array
  */
  var $fullTextSearch = PAPAYA_SEARCH_BOOLEAN;
  /**
  * Entries
  * @var array
  */
  var $entries = NULL;
  /**
  * Entry
  * @var array
  */
  var $entry = NULL;
  /**
  * Search dialog
  * @var object base_dialog
  */
  var $searchDialog = NULL;
  /**
  * Output dialog
  * @var object base_dialog
  */
  var $outputDialog = NULL;

  /**
  * Comments
  * @var array
  */
  var $comments = NULL;
  /**
  * Comment
  * @var array
  */
  var $comment = NULL;


  /**
  * Check surfer rights
  *
  *  MUST BE WRITTEN
  *
  * @access public
  * @return TRUE
  */
  function checkSurferPerm() {
    return TRUE;
  }

  /**
  * Constructor
  *
  * @param string $paramName optional, default value 'ff'
  * @access public
  */
  function __construct($paramName = 'ff') {
    $this->paramName = $paramName;
    $this->sessionParamName = 'PAPAYA_SESS_'.$paramName;
    $this->tableFaqs = PAPAYA_DB_TABLEPREFIX.'_faq';
    $this->tableFaqgroups = PAPAYA_DB_TABLEPREFIX.'_faqgroups';
    $this->tableEntries = PAPAYA_DB_TABLEPREFIX.'_faqentries';
    $this->tableComments = PAPAYA_DB_TABLEPREFIX.'_faqnotes';
    $this->tableSurfer = PAPAYA_DB_TABLEPREFIX.'_surfer';
  }

  /**
  * Initialize parameters
  *
  * @access public
  */
  function initialize() {
    $this->initializeParams();
    $this->sessionParams = $this->getSessionValue($this->sessionParamName);
    $this->initializeSessionParam('faq_id', array('cmd','faqgroup_id','entry_id','note_id') );
    $this->initializeSessionParam('faqgroup_id', array('cmd','entry_id','note_id') );
    $this->initializeSessionParam('entry_id', array('cmd','note_id') );
    $this->initializeSessionParam('note_id', array('cmd') );
  }

  /**
  * Execute - basic function for handling parameters
  *
  * @access public
  */
  function execute() {
    switch (@$this->params['cmd']) {
      case 'add_faq' :
        if ( $this->module->hasPerm(3) ) {
          if ($newId = $this->addFaq()) {
            $this->addMsg(MSG_INFO, $this->_gt('FAQ added.'));
            $this->params['faq_id'] = $newId;
            $this->initializeSessionParam('faq_id', array('cmd'));
          } else {
            $this->addMsg(MSG_ERROR, $this->_gt('Database error! Changes not saved.'));
          }
        }
        break;
      case 'del_faq' :
        if ( $this->module->hasPerm(3) ) {
          if (isset($this->params['confirm_delete']) && $this->params['confirm_delete']) {
            if ($this->faqExists($this->params['faq_id'])) {
              if ($this->deleteFaq($this->params['faq_id'])) {
                $this->addMsg(MSG_INFO, $this->_gt('Faq deleted.'));
                $this->params['faq_id'] = 0;
                $this->initializeSessionParam('faq_id', array('cmd'));
              } else {
                $this->addMsg(MSG_ERROR, $this->_gt('Database error! Changes not saved.'));
              }
            } else {
              $this->addMsg(MSG_WARNING, $this->_gt('Faq not found.'));
            }
          }
        }
        break;
      case 'edit_faq' :
        if ( $this->module->hasPerm(3) || $this->module->hasPerm(3) ) {
          $this->loadFaqs($this->params['faq_id']);
          $this->initializeFaqEditform();
          if ($this->faqDialog->modified()) {
            if ($this->faqDialog->checkDialogInput()) {
              if ($this->saveFaq()) {
                $this->addMsg(MSG_INFO, sprintf($this->_gt('%s modified.'), $this->_gt('Faq')));
              } else {
                $this->addMsg(MSG_ERROR, $this->_gt('Database error! Changes not saved.'));
              }
            }
          }
        }
        break;
      case 'add_faqgroup' :
        if ( $this->module->hasPerm(3) ) {
          if ($newId = $this->addFaqGroup((int)$this->params['faq_id'])) {
            $this->addMsg(MSG_INFO, $this->_gt('Faq Group added.'));
            $this->params['faqgroup_id'] = $newId;
            $this->initializeSessionParam('faqgroup_id', array('cmd'));
          } else {
            $this->addMsg(MSG_ERROR, $this->_gt('Database error! Changes not saved.'));
          }
        }
        break;
      case 'del_faqgroup' :
        if ($this->module->hasPerm(3)) {
          if (isset($this->params['confirm_delete']) && $this->params['confirm_delete']) {
            if ($this->faqGroupExists($this->params['faqgroup_id'])) {
              if ($this->deleteFaqGroup($this->params['faqgroup_id'])) {
                $this->addMsg(MSG_INFO, $this->_gt('Faq Group deleted.'));
                $this->params['faqgroup_id'] = 0;
                $this->initializeSessionParam('faqgroup_id', array('cmd'));
              } else {
                $this->addMsg(MSG_ERROR, $this->_gt('Database error! Changes not saved.'));
              }
            } else {
              $this->addMsg(MSG_WARNING, $this->_gt('Faq Group not found.'));
            }
          }
        }
        break;
      case 'edit_faqgroup' :
        if ( $this->module->hasPerm(3) || $this->module->hasPerm(3) ) {
          $this->loadFaqs();
          $this->loadFaqGroups($this->params['faq_id']);
          $this->initializeFaqGroupEditForm();
          if ($this->faqDialog->modified()) {
            if ($this->faqDialog->checkDialogInput()) {
              if ($this->saveFaqGroup()) {
                $this->addMsg(MSG_INFO, sprintf($this->_gt('%s modified.'), $this->_gt('FAQ group')));
              } else {
                $this->addMsg(MSG_ERROR, $this->_gt('Database error! Changes not saved.'));
              }
            }
          }
        }
        break;
      case 'add_entry' :
        if ( $this->module->hasPerm(2) || $this->module->hasPerm(3) ) {
          if ($newId = $this->addEntrie((int)$this->params['faqgroup_id'])) {
            $this->addMsg(MSG_INFO, $this->_gt('Entry added.'));
            $this->params['entry_id'] = $newId;
            $this->initializeSessionParam('entry_id', array('cmd'));
          } else {
            $this->addMsg(MSG_ERROR, $this->_gt('Database error! Changes not saved.'));
          }
        }
        break;
    case 'edit_entry' :
        if ( $this->module->hasPerm(2) || $this->module->hasPerm(3) ) {
          $this->loadFaqs();
          $this->loadFaqGroups($this->params['faq_id']);
          $this->loadEntries($this->params['faqgroup_id']);
          $this->initializeEntryEditForm();
          if ($this->faqDialog->modified()) {
            if ($this->faqDialog->checkDialogInput()) {
              if ($this->saveEntry()) {
                $this->addMsg(MSG_INFO, sprintf($this->_gt('%s modified.'), $this->_gt('Entry')));
              } else {
                $this->addMsg(MSG_ERROR, $this->_gt('Database error! Changes not saved.'));
              }
            }
          }
        }
        break;
    case 'del_entry' :
        if ( $this->module->hasPerm(2) || $this->module->hasPerm(3) ) {
          if (isset($this->params['confirm_delete']) && $this->params['confirm_delete']) {
            if ($this->entryExists($this->params['entry_id'])) {
              if ($this->deleteEntry($this->params['entry_id'])) {
                $this->addMsg(MSG_INFO, $this->_gt('Entry deleted.'));
                $this->params['entry_id'] = 0;
                $this->initializeSessionParam('entry_id', array('cmd'));
              } else {
                $this->addMsg(MSG_ERROR, $this->_gt('Database error! Changes not saved.'));
              }
            } else {
              $this->addMsg(MSG_WARNING, $this->_gt('Faq Group not found.'));
            }
          }
        }
        break;
    case 'add_comment' :
          if ($this->module->hasPerm(2) || $this->module->hasPerm(3)) {
            if ($newId = $this->addComment((int)$this->params['entry_id'])) {
              $this->addMsg(MSG_INFO, $this->_gt('Comment added.'));
              $this->params['note_id'] = $newId;
              $this->initializeSessionParam('note_id', array('cmd'));
            } else {
              $this->addMsg(MSG_ERROR, $this->_gt('Database error! Changes not saved.'));
            }
          }
          break;
    case 'del_comment' :
        if ($this->module->hasPerm(2) || $this->module->hasPerm(3)) {
          if (isset($this->params['confirm_delete']) && $this->params['confirm_delete']) {
            if ($this->commentExists($this->params['note_id'])) {
              if ($this->deleteComment($this->params['note_id'])) {
                $this->addMsg(MSG_INFO, $this->_gt('Comment deleted.'));
                $this->params['note_id'] = 0;
                $this->initializeSessionParam('note_id', array('cmd'));
              } else {
                $this->addMsg(MSG_ERROR, $this->_gt('Database error! Changes not saved.'));
              }
            } else {
              $this->addMsg(MSG_WARNING, $this->_gt('Faq Entry not found.'));
            }
          }
        }
        break;
    case 'edit_comment' :
        if ($this->module->hasPerm(2) || $this->module->hasPerm(3)) {
          $this->loadFaqs();
          $this->loadFaqGroups($this->params['faq_id']);
          $this->loadEntries($this->params['faqgroup_id']);
          $this->loadComments($this->params['entry_id']);
          $this->initializeCommentEditForm();
          if ($this->faqDialog->modified()) {
            if ($this->faqDialog->checkDialogInput()) {
              if ($this->saveComment()) {
                $this->addMsg(MSG_INFO, sprintf($this->_gt('%s modified.'), $this->_gt('Comment')));
              } else {
                $this->addMsg(MSG_ERROR, $this->_gt('Database error! Changes not saved.'));
              }
            }
          }
        }
        break;

      }
    //ablegen der neuen seassion daten
    $this->setSessionValue($this->sessionParamName, $this->sessionParams);
    $this->loadFaqs();
    $this->loadFaqGroups($this->params['faq_id']);
    $this->loadEntries($this->params['faqgroup_id']);
    $this->loadComments($this->params['entry_id']);
  }

  /**
  * Get XML output
  *
  * @access public
  * @return string
  */
  function getXML() {
    if (is_object($this->layout)) {
      $this->getXMLButtons();
      $this->getXMLTreeList();
      $this->getXMLEntryList();
      switch (@$this->params['cmd']) {
        case 'edit_faq' :
          unset($this->comments);
          unset($this->entries);
          unset($this->faqGroups);
          $this->getXMLFaqForm();
          break;
        case 'edit_faqgroup' :
          unset($this->comments);
          unset($this->entries);
          $this->getXMLFaqGroupForm();
          break;
        case 'edit_entry' :
          $this->getXMLEntryForm();
          break;
        case 'del_faq' :
          $this->getXMLDelFaqForm();
          break;
        case 'del_faqgroup' :
          $this->getXMLDelFaqGroupForm();
          break;
        case 'del_entry' :
          $this->getXMLDelEntryForm();
          break;
        case 'del_comment' :
          $this->getXMLDelCommentForm();
          break;
        default :
          // die eione  mit details
          if (isset($this->comments[$this->params['note_id']]) && is_array($this->comments[$this->params['note_id']])) {
          $this->getXMLCommentForm();
          }elseif (isset($this->entries[$this->params['entry_id']]) && is_array($this->entries[$this->params['entry_id']])) {
          $this->getXMLEntryForm();
          }elseif (isset($this->faqGroups[$this->params['faqgroup_id']]) && is_array($this->faqGroups[$this->params['faqgroup_id']])) {
          $this->getXMLFaqGroupForm();
          }elseif (isset($this->faqs[$this->params['faq_id']]) && is_array($this->faqs[$this->params['faq_id']])) {
          $this->getXMLFaqForm();
          }
      }
    $this->getXMLCommentList();
    }
  }

  /**
  * Get XML faq formular
  *
  * @access public
  */
  function getXMLFaqForm() {
    if ($this->module->hasPerm(3)) {
      if (isset($this->faqs[$this->params['faq_id']]) && is_array($this->faqs[$this->params['faq_id']])) {
        $this->initializeFaqEditForm();
        $this->faqDialog->inputFieldSize = $this->inputFieldSize;
        $this->faqDialog->baseLink = $this->baseLink;
        $this->faqDialog->dialogTitle = htmlspecialchars($this->_gt('Properties'));
        $this->faqDialog->dialogDoubleButtons = FALSE;
        $this->layout->add($this->faqDialog->getDialogXML());
      }
    }else{
      if (isset($this->faqs[$this->params['faq_id']]) && is_array($this->faqs[$this->params['faq_id']])) {
        $ouputXML='
        <dialog title="'.$this->_gt('Properties').'" width="100%">
          <listview width="100%">
            <items>
              <listitem title="'.$this->_gt('Title').'">
                <subitem>'.$this->faqs[$this->params['faq_id']]['faq_title'].'</subitem>
              </listitem>

              <listitem valign="top" title="'.$this->_gt('Description').'">
                <subitem>'.$this->faqs[$this->params['faq_id']]['faq_descr'].'</subitem>
              </listitem>
            </items>
          </listview>
        </dialog>';
        $this->layout->add($ouputXML);
      }
    }
  }

  /**
  * Get XML faq group formular
  *
  * @access public
  */
  function getXMLFaqGroupForm() {
    if ( $this->module->hasPerm(3) ) {
      if (isset($this->faqGroups[$this->params['faqgroup_id']]) && is_array($this->faqGroups[$this->params['faqgroup_id']])) {
        $this->initializeFaqGroupEditForm();
        $this->faqDialog->inputFieldSize = $this->inputFieldSize;
        $this->faqDialog->baseLink = $this->baseLink;
        $this->faqDialog->dialogTitle = htmlspecialchars($this->_gt('Properties'));
        $this->faqDialog->dialogDoubleButtons = FALSE;
        $this->layout->add($this->faqDialog->getDialogXML());
      }
    }else{
      if (isset($this->faqGroups[$this->params['faqgroup_id']]) && is_array($this->faqGroups[$this->params['faqgroup_id']])) {
        $ouputXML='
        <dialog title="'.$this->_gt('Properties').'" width="100%">
          <listview width="100%">
            <items>
              <listitem title="'.$this->_gt('Title').'">
                <subitem>'.$this->faqGroups[$this->params['faqgroup_id']]['faqgroup_title'].'</subitem>
              </listitem>

              <listitem valign="top" title="'.$this->_gt('Description').'">
                <subitem>'.$this->faqGroups[$this->params['faqgroup_id']]['faqgroup_descr'].'</subitem>
              </listitem>
            </items>
          </listview>
        </dialog>';
        $this->layout->add($ouputXML);
      }
    }
  }


  /**
  * Get XML entry formular
  *
  * @access public
  */
  function getXMLEntryForm() {
    if ( $this->module->hasPerm(2) || $this->module->hasPerm(3) ) {
      if (isset($this->entries[$this->params['entry_id']]) && is_array($this->entries[$this->params['entry_id']])) {
        $this->initializeEntryEditForm();
        $this->faqDialog->inputFieldSize = $this->inputFieldSize;
        $this->faqDialog->baseLink = $this->baseLink;
        $this->faqDialog->dialogTitle = htmlspecialchars($this->_gt('Properties'));
        $this->faqDialog->dialogDoubleButtons = FALSE;
        $this->layout->add($this->faqDialog->getDialogXML());
      }
    }else{
      if (isset($this->entries[$this->params['entry_id']]) && is_array($this->entries[$this->params['entry_id']])) {
        $ouputXML='
        <dialog title="'.$this->_gt('Properties').'" width="100%">
          <listview width="100%">
            <items>
              <listitem title="'.$this->_gt('Title').'">
                <subitem>'.$this->entries[$this->params['entry_id']]['entry_title'].'</subitem>
              </listitem>

              <listitem title="'.$this->_gt('Question').'">
                <subitem>'.$this->entries[$this->params['entry_id']]['entry_question'].'</subitem>
              </listitem>

              <listitem title="'.$this->_gt('Answer').'">
                <subitem>'.$this->entries[$this->params['entry_id']]['entry_answer'].'</subitem>
              </listitem>

            </items>
          </listview>
        </dialog>';
        $this->layout->add($ouputXML);
      }
    }
  }

  /**
  * Get XML comment formular
  *
  * @access public
  */
  function getXMLCommentForm() {
    if ( $this->module->hasPerm(2) || $this->module->hasPerm(3) ) {
      if (isset($this->comments[$this->params['note_id']]) && is_array($this->comments[$this->params['note_id']])) {
        $this->initializeCommentEditForm();
        $this->faqDialog->inputFieldSize = $this->inputFieldSize;
        $this->faqDialog->baseLink = $this->baseLink;
        $this->faqDialog->dialogTitle = htmlspecialchars($this->_gt('Properties'));
        $this->faqDialog->dialogDoubleButtons = FALSE;
        $this->layout->add($this->faqDialog->getDialogXML());
      }
    }else{
      if (isset($this->comments[$this->params['note_id']]) && is_array($this->comments[$this->params['note_id']])) {
        $ouputXML='
        <dialog title="'.$this->_gt('Properties').'" width="100%">
          <listview width="100%">
            <items>
              <listitem title="'.$this->_gt('Username').'">
                <subitem>'.$this->comments[$this->params['note_id']]['note_username'].'</subitem>
              </listitem>

              <listitem title="'.$this->_gt('Created').'">
                <subitem>'.date('Y-m-d H:i:s', $this->comments[$this->params['note_id']]['note_created']).'</subitem>
              </listitem>

              <listitem title="'.$this->_gt('Content').'">
                <subitem>'.$this->comments[$this->params['note_id']]['note_content'].'</subitem>
              </listitem>

            </items>
          </listview>
        </dialog>';
        $this->layout->add($ouputXML);
      }
    }
  }


  /**
  * Build link
  *
  * @param mixed $params optional, default value NULL
  * @param mixed $file optional, default value NULL
  * @param string $seperator optional, default value '&amp;'
  * @access public
  * @return string
  */
  function buildLink($params = NULL, $file = '') {
    $link = (trim($file) != '') ? $file : $this->baseLink;
    return $this->getLink($params, NULL, $link);
  }

  /**
  * Get XML delete faq group formular
  *
  * @access public
  */
  function getXMLDelFaqGroupForm() {
    if (isset($this->faqGroups[$this->params['faqgroup_id']]) && is_array($this->faqGroups[$this->params['faqgroup_id']])) {
      include_once(PAPAYA_INCLUDE_PATH.'system/base_msgdialog.php');
      $hidden = array(
        'cmd' => 'del_faqgroup',
        'faqgroup_id' => $this->params['faqgroup_id'],
        'confirm_delete' => 1,
      );
      $msg = sprintf($this->_gt('Delete faqgroup "%s" (%s)?'),
        htmlspecialchars($this->faqGroups[$this->params['faqgroup_id']]['faqgroup_title']),
        (int)$this->params['faqgroup_id']);
      $dialog = &new base_msgdialog($this, $this->paramName, $hidden, $msg, 'question');
      $dialog->baseLink = $this->baseLink;
      $dialog->msgs = &$this->msgs;
      $dialog->buttonTitle = 'Delete';
      $this->layout->add($dialog->getMsgDialog());
    }
  }

  /**
  * Get XML delete faq formular
  *
  * @access public
  */
  function getXMLDelFaqForm() {
    if (isset($this->faqs[$this->params['faq_id']]) && is_array($this->faqs[$this->params['faq_id']])) {
      include_once(PAPAYA_INCLUDE_PATH.'system/base_msgdialog.php');
      $hidden = array(
        'cmd' => 'del_faq',
        'faq_id' => $this->params['faq_id'],
        'confirm_delete' => 1,
      );
      $msg = sprintf($this->_gt('Delete faq "%s" (%s)?'),
        htmlspecialchars($this->faqs[$this->params['faq_id']]['faq_title']),
        (int)$this->params['faq_id']);
      $dialog = &new base_msgdialog($this, $this->paramName, $hidden, $msg, 'question');
      $dialog->baseLink = $this->baseLink;
      $dialog->msgs = &$this->msgs;
      $dialog->buttonTitle = 'Delete';
      $this->layout->add($dialog->getMsgDialog());
    }
  }

  /**
  * Get XML faq list
  *
  * @access public
  * @return string
  */
  function getXMLFaqList() {
    if (isset($this->faqs) && is_array($this->faqs)) {
      $result = sprintf('<listview title="%s" width="200">'.LF, $this->_gt('FAQ'));
      $result .= '<items>'.LF;
      foreach($this->faqs as $id => $faq) {
        if (isset($faq) && is_array($faq)) {
          $selected = ($this->params['faq_id'] == $id) ? ' selected="selected"' : '';
          $result .= sprintf('<listitem href="%s" title="%s" image="%s" %s/>'.LF,
            $this->buildLink(array('faq_id'=>(int)$id)),
            htmlspecialchars($faq['faq_title']),
            $this->images[(($this->params['faq_id'] == $id)  ? 57 : 56)], $selected);
        }
      }
      $result .= '</items>'.LF;
      $result .= '</listview>'.LF;
      $this->layout->addLeft($result);
    }
  }

  /**
  * Get XML tree list
  *
  * @access public
  */
  function getXMLTreeList() {
    if (isset($this->faqs) && is_array($this->faqs)) {
      $result = sprintf('<listview title="%s" width="200">'.LF, $this->_gt('Tree'));
      $result .= '<items>'.LF;
      foreach($this->faqs as $id => $faq) {
        if (isset($faq) && is_array($faq)) {
          if($this->faqs[$id]['groupcount'] > 0 ){
            $selected = ($this->params['faq_id'] == $id) ? 'node="open" selected="selected"' : 'node="close"';
          }else{
            $selected = ($this->params['faq_id'] == $id) ? 'node="empty" selected="selected"' : 'node="empty"';
          }
          $result .= sprintf('<listitem href="%s" title="%s" nhref="%s" %s />'.LF,
          $this->buildLink(array('faq_id' => (int)$id,'cmd' => 'edit_faq')),
          htmlspecialchars($faq['faq_title']),
          $this->buildLink(array('faq_id' => (int)$id)),
          $selected);
          if((isset($this->faqGroups) && is_array($this->faqGroups))&&($this->params['faq_id'] == $id)) {
            $result .= $this->getXMLFaqGroupTreeList($this->params['faq_id']);
          }

        }
      }
      $result .= '</items>'.LF;
      $result .= '</listview>'.LF;
      $this->layout->addLeft($result);
    }
  }

  /**
  * Get XML faq group tree list
  *
  * @param integer $i
  * @access public
  * @return string
  */
  function getXMLFaqGroupTreeList($i) {
    $result = '';
    if (isset($this->faqGroups) && is_array($this->faqGroups)) {
      foreach($this->faqGroups as $id => $faqGroup) {
        if (isset($faqGroup) && is_array($faqGroup)) {
          $selected = ($this->params['faqgroup_id'] == $id) ? 'selected="selected"' : '';
          if($faqGroup['faq_id'] == $i){
            $result .= sprintf('<listitem indent="1" href="%s" title="%s" image="%s" %s/>'.LF,
            $this->buildLink(array('faqgroup_id' => (int)$id,'cmd' => 'edit_faqgroup')),
            htmlspecialchars($faqGroup['faqgroup_title']),
            $this->images[(($this->params['faqgroup_id'] == $id)  ? 57 : 56)], $selected);
          }
        }
      }
      return $result;
    }
  }

  /**
  * Get XML faq group list
  *
  * @access public
  */
  function getXMLFaqGroupList() {
    if (isset($this->faqGroups) && is_array($this->faqGroups)) {
      $result = sprintf('<listview title="%s" width="200">'.LF, $this->_gt('FAQ Group'));
      $result .= '<items>'.LF;
      foreach($this->faqGroups as $id => $faqGroup) {
        if (isset($faqGroup) && is_array($faqGroup)) {
          $selected = ($this->params['faqgroup_id'] == $id) ? ' selected="selected"' : '';
          $result .= sprintf('<listitem href="%s" title="%s" image="%s" %s/>'.LF,
            $this->buildLink(array('faqgroup_id' => (int)$id)),
            htmlspecialchars($faqGroup['faqgroup_title']),
            $this->images[(($this->params['faqgroup_id'] == $id)  ? 57 : 56)], $selected);
        }
      }
      $result .= '</items>'.LF;
      $result .= '</listview>'.LF;
      $this->layout->addLeft($result);
    }
  }

  /**
  * Get XML Buttons
  *
  * @access public
  */
  function getXMLButtons() {
    include_once(PAPAYA_INCLUDE_PATH.'system/base_btnbuilder.php');
    $toolbar = &new base_btnbuilder;
    $toolbar->images = &$this->images;
    if ( $this->module->hasPerm(2) || $this->module->hasPerm(3) ) {
      if( $this->module->hasPerm(3) ) {
        $toolbar->addButton('New FAQ', $this->buildLink(array('cmd'=>'add_faq')), 70, '', FALSE);
      }
      if (isset($this->faqs[$this->params['faq_id']]) && is_array($this->faqs[$this->params['faq_id']])) {
        if( $this->module->hasPerm(3) ) {
          $toolbar->addButton('Delete FAQ',
            $this->buildLink(array('cmd' => 'del_faq', 'faq_id' => (int)$this->params['faq_id'])),
            10, '', FALSE);
          $toolbar->addSeperator();
          $toolbar->addButton('New group',
          $this->buildLink(array('cmd' => 'add_faqgroup', 'faq_id' => (int)$this->params['faq_id'])),
            70, '', FALSE);
        }
        if (isset($this->faqGroups[$this->params['faqgroup_id']]) && is_array($this->faqGroups[$this->params['faqgroup_id']])) {
          if( $this->module->hasPerm(3) ) {
            $toolbar->addButton('Delete group',
            $this->buildLink(array('cmd' => 'del_faqgroup', 'faqgroup_id' => (int)$this->params['faqgroup_id'])),
            10, '', FALSE);
            $toolbar->addSeperator();
          }
            $toolbar->addButton('New entry',
            $this->buildLink(array('cmd' => 'add_entry', 'faqgroup_id'=>(int)$this->params['faqgroup_id'])),
            70, '', FALSE);

          if (isset($this->entries[$this->params['entry_id']]) && is_array($this->entries[$this->params['entry_id']])) {
            $toolbar->addButton('Delete entry',
              $this->buildLink(array('cmd' => 'del_entry', 'entry_id' => (int)$this->params['entry_id'])),
              10, '', FALSE);
              $toolbar->addSeperator();
              $toolbar->addButton('New comment',
              $this->buildLink(array('cmd' => 'add_comment', 'entry_id' => (int)$this->params['entry_id'])),
              70, '', FALSE);

            if (isset($this->comments[$this->params['note_id']]) && is_array($this->comments[$this->params['note_id']])) {
              $toolbar->addButton('Delete comment',
                $this->buildLink(array('cmd' => 'del_comment', 'note_id' => (int)$this->params['note_id'])),
                10, '', FALSE);
                $toolbar->addSeperator();
            }
          }
        }
      }
    }
    if ($str = $toolbar->getXML()) {
      $this->layout->addMenu(sprintf('<menu ident="%s">%s</menu>'.LF, 'edit', $str));
    }
  }

  /**
  * Save faq changes
  *
  * @access public
  * @return boolean
  */
  function saveFaq() {
    $data = array(
      'faq_title' => $this->params['faq_title'],
      'faq_descr' => $this->params['faq_descr']
    );
    return (FALSE !== $this->databaseUpdateRecord($this->tableFaqs, $data, 'faq_id', (int)$this->params['faq_id']));
  }

  /**
  * Initialize faq edit formular
  *
  * @access public
  */
  function initializeFaqEditForm() {
    if (!is_object($this->faqDialog)) {
      include_once(PAPAYA_INCLUDE_PATH.'system/base_dialog.php');
      $data = $this->faqs[$this->params['faq_id']];
      $hidden = array('cmd' => 'edit_faq', 'save' => 1, 'faq_id' => $data['faq_id']);
      $fields = array(
        'faq_title' => array('Title', 'isNoHTML', TRUE, 'input', 200),
        'faq_descr' => array('Description', 'isSomeText', FALSE, 'textarea', 8)
      );
      $this->faqDialog = &new base_dialog($this, $this->paramName, $fields, $data, $hidden);
      $this->faqDialog->msgs = &$this->msgs;
      $this->faqDialog->loadParams();
    }
  }

  /**
  * Initialize faq group edit formular
  *
  * @access public
  */
  function initializeFaqGroupEditForm() {
    if (!is_object($this->faqDialog)) {
      include_once(PAPAYA_INCLUDE_PATH.'system/base_dialog.php');
      $data = $this->faqGroups[$this->params['faqgroup_id']];
      $hidden = array('cmd' => 'edit_faqgroup', 'save' => 1, 'faqgroup_id' => $data['faqgroup_id']);
      $fields = array(
        'faqgroup_title' => array('Title', 'isNoHTML', TRUE, 'input', 200),
        'faqgroup_descr' => array('Description', 'isSomeText', FALSE, 'textarea', 8)
      );
      $this->faqDialog = &new base_dialog($this, $this->paramName, $fields, $data, $hidden);
      $this->faqDialog->msgs = &$this->msgs;
      $this->faqDialog->loadParams();
    }
  }

  /**
  * Initialize entry edit
  *
  * @access public
  */
  function initializeEntryEditForm() {
    if (!is_object($this->faqDialog)) {
      include_once(PAPAYA_INCLUDE_PATH.'system/base_dialog.php');
      $data = $this->entries[$this->params['entry_id']];
      $hidden = array('cmd' => 'edit_entry', 'save' => 1, 'entry_id' => $data['entry_id']);
      $fields = array(
        'entry_title' => array('Title', 'isNoHTML', TRUE, 'input', 200),
        'entry_question' => array('Question', 'isSomeText', TRUE, 'textarea', 8),
        'entry_answer' => array('Answer', 'isSomeText', TRUE, 'textarea', 8)
      );
      $this->faqDialog = &new base_dialog($this, $this->paramName, $fields, $data, $hidden);
      $this->faqDialog->msgs = &$this->msgs;
      $this->faqDialog->loadParams();
    }
  }

  /**
  * Initialize
  *
  * @access public
  */
  function initializeCommentEditForm() {
    if (!is_object($this->faqDialog)) {
      include_once(PAPAYA_INCLUDE_PATH.'system/base_dialog.php');
      $data = $this->comments[$this->params['note_id']];
      $hidden = array('cmd' => 'edit_comment', 'save' => 1, 'note_id' => $data['note_id']);
      $fields = array(
        'note_username' => array('Username', 'isNoHTML', TRUE, 'input', 50),
        'note_content' => array('Content', 'isNoHTML', TRUE, 'textarea', 8)
      );
      $this->faqDialog = &new base_dialog($this, $this->paramName, $fields, $data, $hidden);
      $this->faqDialog->msgs = &$this->msgs;
      $this->faqDialog->loadParams();
    }
  }


  /**
  * Load faqs
  *
  * @param mixed $id optional, default value NULL
  * @access public
  * @return boolean
  */
  function loadFaqs($id = NULL) {
    unset($this->faqs);
    unset($this->faqTree);
    if (isset($id)) {
      $sql = "SELECT faq_id, faq_title, faq_descr
                FROM %s
               WHERE faq_id = '%d'
               ORDER BY faq_title ASC";
    } else {
      $sql = "SELECT faq_id, faq_title, faq_descr
                FROM %s
               ORDER BY faq_title ASC";
    }
    if ($res = $this->databaseQueryFmt($sql, array($this->tableFaqs, (int)$id))) {
      while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $row['groupcount'] = 0;
        $this->faqs[(int)$row['faq_id']] = $row;
      }
      return $this->checkFaqGroups();
    }
    return FALSE;
  }


  /**
  * Load faq
  *
  * @param integer $id
  * @access public
  * @return boolean
  */
  function loadFaq($id) {
    unset($this->faq);
    if (isset($id) && $id > 0) {
      $sql = "SELECT faq_id, faq_title, faq_descr
                FROM %s
               WHERE faq_id = '%d'
               ORDER BY faq_title ASC";
      if ($res = $this->databaseQueryFmt($sql, array($this->tableFaqs, (int)$id))) {
        if($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
          $this->faq = $row;
          return TRUE;
        }
      }
    }
    return FALSE;
  }

  /**
  * Faq exists
  *
  * @param integer $id
  * @access public
  * @return boolean
  */
  function faqExists($id) {
    $sql = "SELECT count(*) FROM %s WHERE faq_id = '%d'";
    $params = array($this->tableFaqs, $id);
    if ($res = $this->databaseQueryFmt($sql, $params)) {
      if ($row = $res->fetchRow()) {
        return ((bool)$row[0] > 0);
      }
    }
    return FALSE;
  }

  /**
  * Add faq
  *
  * @access public
  * @return mixed
  */
  function addFaq() {
    return $this->databaseInsertRecord($this->tableFaqs, 'faq_id',
    array('faq_title' => $this->_gt('New Faq')));
  }

  /**
  * Add faq group
  *
  * @param integer $parent
  * @access public
  * @return mixed
  */
  function addFaqGroup($parent) {
    if ($this->faqExists($parent)) {
      return $this->databaseInsertRecord($this->tableFaqgroups, 'faqgroup_id',
        array('faq_id' => $parent, 'faqgroup_title' => $this->_gt('New Faq Group')));
    }
    return FALSE;
  }

  /**
  * Add entrie
  *
  * @param integer $parent
  * @access public
  * @return mixed
  */
  function addEntrie($parent) {
    if ($this->faqGroupExists($parent)) {
      return $this->databaseInsertRecord($this->tableEntries, 'entry_id',
        array('faqgroup_id' => $parent, 'entry_title' => $this->_gt('New Entry')));
    }
    return FALSE;
  }


  /**
  * Add comment
  *
  * @param integer $parent
  * @access public
  * @return mixed
  */
  function addComment($parent) {
    if ($this->entryExists($parent)) {
      return $this->databaseInsertRecord($this->tableComments, 'note_id',
        array('entry_id' => $parent, 'note_username'=>$this->_gt('New Comment')));
    }
    return FALSE;
  }

  /**
  * Add comment output
  *
  * @param integer $parent
  * @param string $text
  * @param string $username
  * @param array $data
  * @access public
  * @return mixed
  */
  function addCommentOutput($parent,$text,$username,$data) {
    if ($this->entryExists($parent)) {
      if(
          $data['post_comments'] != -1
          &&
          !( is_object($GLOBALS['PAPAYA_SURFER'])
            && $GLOBALS['PAPAYA_SURFER']->isValid == TRUE
            && $this->checkSurferPerm($data['post_comments'])
          )

        ){
        return FALSE;
      } else {
        return $this->databaseInsertRecord($this->tableComments, 'note_id',
          array('entry_id' => $parent, 'note_content' => $text, 'note_username' => $username, 'note_created' => time()));
      }
    }
    return FALSE;
  }



  /**
  * Faq group exists
  *
  * @param integer $id
  * @access public
  * @return boolean
  */
  function faqGroupExists($id) {
    $sql = "SELECT count(*) FROM %s WHERE faqgroup_id = '%d'";
    $params = array($this->tableFaqgroups, $id);
    if ($res = $this->databaseQueryFmt($sql, $params)) {
      if ($row = $res->fetchRow()) {
        return ((bool)$row[0] > 0);
      }
    }
    return FALSE;
  }

  /**
  * Delete faq
  *
  * @param integer $id
  * @access public
  * @return mixed returns db_deleteRecord
  */
  function deleteFaq($id) {
    $this->loadFaqGroups($id);
    if(isset($this->faqGroups) && is_array($this->faqGroups)){
      foreach($this->faqGroups as $faqGroupId=>$faqGroup){
        $this->deleteFaqGroup($faqGroupId);
      }
    }
    unset($this->faqs);
    return $this->databaseDeleteRecord($this->tableFaqs, 'faq_id', $id);
  }

  /**
  * Delete faq group
  *
  * @param integer $i
  * @access public
  * @return boolean
  */
  function deleteFaqGroup($i) {
    $this->loadEntries($i);
    if(isset($this->entries) && is_array($this->entries)){
      foreach($this->entries as $enid => $entry){
        $this->databaseDeleteRecord($this->tableComments, 'entry_id', $enid);
      }
    }
    if (FALSE !== $this->databaseDeleteRecord($this->tableEntries, 'faqgroup_id', $i)) {
      if (FALSE !== $this->databaseDeleteRecord($this->tableFaqgroups, 'faqgroup_id', $i)) {
        unset($this->comments);
        unset($this->entries);
        unset($this->faqGroups);
        return TRUE;
      }
    }
    return FALSE;
  }

  /**
  * Check faq groups
  *
  * @access public
  * @return boolean
  */
  function checkFaqGroups() {
    $sql = "SELECT count(faqgroup_id), faq_id
              FROM %s
             GROUP BY faq_id";
    $params = array($this->tableFaqgroups);
    if ($res = $this->databaseQueryFmt($sql, $params)){
      while($row = $res->fetchRow()) {
        $this->faqs[(int)$row[1]]['groupcount'] = (int)$row[0];
      }
      return TRUE;
    }
    return FALSE;
  }

  /**
  * Load faq groups
  *
  * @param integer $id
  * @access public
  * @return boolean
  */
  function loadFaqGroups($id) {
    unset($this->faqGroups);
    if ($id) {
      $sql = "SELECT f.faqgroup_id, f.faq_id, f.faqgroup_title, f.faqgroup_descr, COUNT(e.entry_id) as entry_count
                FROM %s AS f
                LEFT OUTER JOIN %s AS e ON f.faqgroup_id = e.faqgroup_id
               WHERE f.faq_id = %d
               GROUP BY f.faqgroup_id, f.faq_id, f.faqgroup_title, f.faqgroup_descr
               ORDER BY f.faqgroup_title ASC, f.faqgroup_id ASC";
    } else {
      return FALSE;
    }
    $params = array($this->tableFaqgroups, $this->tableEntries, (int)$id);
    if ($res = $this->databaseQueryFmt($sql, $params)) {
      while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->faqGroups[(int)$row['faqgroup_id']] = $row;
      }
      return TRUE;
    }
    return FALSE;
  }

  /**
  * Load faq group
  *
  * @param integer $id
  * @access public
  * @return boolean
  */
  function loadFaqGroup($id) {
    unset($this->faqGroups);
    if ($id) {
      $sql = "SELECT faqgroup_id, faq_id, faqgroup_title, faqgroup_descr
                FROM %s
               WHERE faqgroup_id = %d
               ORDER BY faqgroup_title ASC, faqgroup_id ASC";
    } else {
      return FALSE;
    }
    $params = array($this->tableFaqgroups, (int)$id);
    if ($res = $this->databaseQueryFmt($sql, $params)) {
      while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->faqGroup = $row;
      }
      return TRUE;
    }
    return FALSE;
  }

  /**
  * Load Entries
  *
  * @param integer $id
  * @access public
  * @return boolean
  */
  function loadEntries($id) {
    unset($this->entries);
    if ($id) {
    $sql = "SELECT entry_id, entry_title, faqgroup_id, entry_question, entry_answer
              FROM %s
             WHERE faqgroup_id = %d
             ORDER BY entry_question, entry_id DESC";
    } else {
      return FALSE;
    }
    $params = array($this->tableEntries, $id);
    if ($res = $this->databaseQueryFmt($sql, $params)) {
      while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->entries[(int)$row['entry_id']] = $row;
      }
      return TRUE;
    }
    return FALSE;
  }

  /**
  * Load entries
  *
  * @param integer $id
  * @access public
  * @return boolean
  */
  function loadEntry($id) {
    unset($this->entries);
    if ($id) {
    $sql = "SELECT faqgroup_id
              FROM %s
             WHERE entry_id = %d
             ORDER BY entry_id DESC";
    } else {
      return FALSE;
    }
    $params = array($this->tableEntries, $id);
    if ($res = $this->databaseQueryFmt($sql, $params)) {
      while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->entry = $row;
      }
      return TRUE;
    }
    return FALSE;
  }

  /**
  * Delete entry
  *
  * @param integer $i
  * @access public
  * @return mixed return value of db_delete record
  */
  function deleteEntry($i) {
    $temp=$this->comments;
    $this->loadComments($i);
    if(isset($this->comments) && is_array($this->comments)){
      foreach($this->comments as $id => $comment){
        if($comment['entry_id'] = $i){
          $this->databaseDeleteRecord($this->tableComments, 'entry_id', $i);
        }
      }
    }
    unset ($this->comments);
    unset ($this->entries);
    return $this->databaseDeleteRecord($this->tableEntries, 'entry_id', $i);
  }

  /**
  * Get XML entry list
  *
  * @access public
  */
  function getXMLEntryList() {
    if (isset($this->entries) && is_array($this->entries)) {
      $result = sprintf('<listview title="%s" width="200">'.LF, $this->_gt('FAQ Entries'));
      $result .= '<items>'.LF;
      foreach($this->entries as $id => $entry) {
        if (isset($entry) && is_array($entry)) {
          $selected = ($this->params['entry_id'] == $id) ? ' selected="selected"' : '';
          $result .= sprintf('<listitem href="%s" title="%s" image="%s" %s/>'.LF,
            $this->buildLink(array('entry_id' => (int)$id,'cmd' => 'edit_entry')),
            papaya_strings::escapeHTMLChars($entry['entry_title']),
            $this->images[(($this->params['entry_id'] == $id)  ? 57 : 56)], $selected);
        }
      }
      $result .= '</items>'.LF;
      $result .= '</listview>'.LF;
      $this->layout->addLeft($result);
    }
  }

  /**
  * Get XML delete entry formular
  *
  * @access public
  */
  function getXMLDelEntryForm() {
    if (isset($this->entries[$this->params['entry_id']]) && is_array($this->entries[$this->params['entry_id']])) {
      include_once(PAPAYA_INCLUDE_PATH.'system/base_msgdialog.php');
      $hidden = array(
        'cmd' => 'del_entry',
        'entry_id' => $this->params['entry_id'],
        'confirm_delete' => 1,
      );
      $msg = sprintf($this->_gt('Delete Entry "%s" (%s)?'),
        htmlspecialchars($this->entries[$this->params['entry_id']]['entry_title']),
        (int)$this->params['entry_id']);
      $dialog = &new base_msgdialog($this, $this->paramName, $hidden, $msg, 'question');
      $dialog->baseLink = $this->baseLink;
      $dialog->msgs = &$this->msgs;
      $dialog->buttonTitle = 'Delete';
      $this->layout->add($dialog->getMsgDialog());
    }
  }

  /**
  * Get XML delete comment
  *
  * @access public
  */
  function getXMLDelCommentForm() {
    if (isset($this->comments[$this->params['note_id']]) && is_array($this->comments[$this->params['note_id']])) {
      include_once(PAPAYA_INCLUDE_PATH.'system/base_msgdialog.php');
      $hidden = array(
        'cmd' => 'del_comment',
        'note_id' => $this->params['note_id'],
        'confirm_delete' => 1,
      );
      $msg = sprintf($this->_gt('Delete Comment "%s" (%s)?'),
        htmlspecialchars($this->comments[$this->params['note_id']]['note_username']),
        (int)$this->params['note_id']);
      $dialog = &new base_msgdialog($this, $this->paramName, $hidden, $msg, 'note_content');
      $dialog->baseLink = $this->baseLink;
      $dialog->msgs = &$this->msgs;
      $dialog->buttonTitle = 'Delete';
      $this->layout->add($dialog->getMsgDialog());
    }
  }

  /**
  * Entry exists
  *
  * @param integer $id
  * @access public
  * @return boolean
  */
  function entryExists($id) {
    $sql = "SELECT count(*)
              FROM %s
             WHERE entry_id = '%d'";
    $params = array($this->tableEntries, $id);
    if ($res = $this->databaseQueryFmt($sql, $params)) {
      if ($row = $res->fetchRow()) {
        return ((bool)$row[0] > 0);
      }
    }
    return FALSE;
  }

  /**
  * Load Comments
  *
  * @param integer $id
  * @access public
  * @return boolean
  */
  function loadComments($id) {
    unset($this->comments);
    if ($id) {
    $sql = "SELECT note_id, entry_id, note_username, note_content, note_created
              FROM %s
             WHERE entry_id = %d
             ORDER BY note_id DESC";
    } else {
      return FALSE;
    }
    $params = array($this->tableComments, $id);
    if ($res = $this->databaseQueryFmt($sql, $params)) {
      while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->comments[(int)$row['note_id']] = $row;
      }
      return TRUE;
    }
    return FALSE;
  }

  /**
  * Load comments
  *
  * @param integer $id
  * @access public
  * @return boolean
  */
  function loadComment($id) {
    unset($this->comments);
    if ($id) {
    $sql = "SELECT entry_id
              FROM %s
             WHERE note_id = %d
             ORDER BY note_id DESC";
    } else {
      return FALSE;
    }
    $params = array($this->tableComments, $id);
    if ($res = $this->databaseQueryFmt($sql, $params)) {
      while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->comment = $row;
      }
      return TRUE;
    }
    return FALSE;
  }

  /**
  * Get XML comment list
  *
  * @access public
  */
  function getXMLCommentList() {
    if (isset($this->comments) && is_array($this->comments)) {
      $result = sprintf('<listview width="100%%" title="%s">'.LF, $this->_gt('Comments'));
      $result .= '<cols>'.LF;
      $result .= sprintf('<col>%s</col>', $this->_gt('Username'));
      $result .= sprintf('<col align="center">%s</col>'.LF, $this->_gt('Content'));
      $result .= sprintf('<col align="center">%s</col>'.LF, $this->_gt('Created'));
      $result .= '</cols>'.LF;
      $result .= '<items>'.LF;
      foreach($this->comments as $id => $comment) {
        $selected = ($comment['note_id'] == $this->params['note_id']) ? ' selected="selected"' : '';
        $result .= sprintf('<listitem href="%s" title="%s" image="%s" %s>'.LF,
            $this->buildLink(array('note_id'=>(int)$id)),
            htmlspecialchars($comment['note_username']),
            $this->images[(($this->params['note_id'] == $id)  ? 94 : 94)], $selected);
        $result .= sprintf('<subitem align="center">%s</subitem>'.LF,
          $comment['note_content']);
        $result .= sprintf('<subitem align="center">%s</subitem>'.LF,
          htmlspecialchars(date('Y-m-d H:i:s', $comment['note_created'])));
        $result .= '</listitem>'.LF;
      }
      $result .= '</items>'.LF;
      $result .= '</listview>'.LF;
      $this->layout->add($result);
    }
  }

  /**
  * Comment exist
  *
  * @param integer $id
  * @access public
  * @return boolean
  */
  function commentExists($id) {
    $sql = "SELECT count(*)
              FROM %s
             WHERE note_id = '%d'";
    $params = array($this->tableComments, $id);
    if ($res = $this->databaseQueryFmt($sql, $params)) {
      if ($row = $res->fetchRow()) {
        return ((bool)$row[0] > 0);
      }
    }
    return FALSE;
  }

  /**
  * Delete comment
  *
  * @param integer $id
  * @access public
  * @return mixed return value of db_deleteRecord
  */
  function deleteComment($id) {
    unset ($this->comments);
    return $this->databaseDeleteRecord($this->tableComments, 'note_id', $id);
  }

  /**
  * Save FAQ group
  *
  * @access public
  * @return boolean
  */
  function saveFaqGroup() {
    $data = array(
      'faqgroup_title' => $this->params['faqgroup_title'],
      'faqgroup_descr' => $this->params['faqgroup_descr']
    );
    return (FALSE !== $this->databaseUpdateRecord($this->tableFaqgroups, $data, 'faqgroup_id', (int)$this->params['faqgroup_id']));
  }

  /**
  * Save entry
  *
  * @access public
  * @return boolean
  */
  function saveEntry() {
    $data = array(
      'entry_title' => $this->params['entry_title'],
      'entry_question' => $this->params['entry_question'],
      'entry_answer' => $this->params['entry_answer']
    );
    return (FALSE !== $this->databaseUpdateRecord($this->tableEntries, $data, 'entry_id', (int)$this->params['entry_id']));
  }

  /**
  * SaveComment
  *
  * @access public
  * @return boolean
  */
  function saveComment() {
    $data = array(
      'note_username' => $this->params['note_username'],
      'note_content' => $this->params['note_content']
    );
    return (FALSE !== $this->databaseUpdateRecord($this->tableComments, $data, 'note_id', (int)$this->params['note_id']));
  }

  /**
  * Get output
  *
  * @param array $data
  * @access public
  * @return string
  */
  function getOutput($data) {
    $result = '';
    $faqId=$data['faq'];
    $this->initializeParams();
    $this->backLink = '';
    $this->params['faq_id']=$faqId;
    if(isset($this->params['entry_id']) && isset($this->params['cmd']) && $this->params['entry_id'] > 0 && $this->params['cmd'] == 'add_note') {
      $this->addCommentOutput((int)$this->params['entry_id'], $this->params['note_content'], @$this->params['note_username'], $data);
    }

    if(isset($this->params['entry_id']) && $this->params['entry_id'] > 0) {
      if($this->loadEntry($this->params['entry_id'])){
        $this->params['faqgroup_id']=$this->entry['faqgroup_id'];
      }
      $this->loadComments($this->params['entry_id']);
      $this->loadEntries($this->entry['faqgroup_id']);
      $this->loadFaqGroup($this->entry['faqgroup_id']);
    }elseif(isset($this->params['faqgroup_id']) && $this->params['faqgroup_id'] > 0) {
      $this->loadEntries($this->params['faqgroup_id']);
    }
    $this->loadFaq($faqId);
    $this->loadFaqGroups($faqId);
    $result .= $this->getXMLOutputTreeList($data);
    return $result;
  }

  /**
  * Get XML output tree list
  *
  * @param array $data
  * @access public
  * @return string or FALSE
  */
  function getXMLOutputTreeList($data) {
    $faqId = $data['faq'];
    $result = '';
    if (isset($this->faq) && is_array($this->faq)) {
      $result .= sprintf('<faq href="%s" title="%s" shref="%s">'.LF,
      $this->buildLink(array('faq_id' => (int)$faqId)),
      htmlspecialchars(@$this->faq['faq_title']),
      $this->buildLink(array('faq_id' => (int)$faqId,'search' => 1))
      );
      $result.= sprintf('<descr><![CDATA[%s]]></descr>'.LF, htmlspecialchars($this->faq['faq_descr']));
      if (@$this->params['search']) {
        $this->initializeSearchForm();
        if (trim(@$this->params['searchfor']) != '') {
          $searched = $this->searchFaq($faqId, $this->params['searchfor']);
          if ($searched) {
            if (isset($this->entries) && is_array($this->entries)) {
              $result .= $this->getOutputEntries( $data );
            } else {
              $result .= sprintf('<message no="2"><![CDATA[%s]]></message>'.LF, htmlspecialchars("msg_noresult")); // msg_noresult
            }
          }
        } else {
          $result .= sprintf('<message no="1"><![CDATA[%s]]></message>'.LF, htmlspecialchars("msg_invalid"));
        }
        $result .= $this->getSearchForm("caption_search");
        $this->backLink = $this->buildLink(array(
          'faq_id' => (int)$this->params['faq_id']));
      }
      if(isset($this->faqGroups) && is_array($this->faqGroups)) {
        $result .= $this->getXMLOutputFaqGroupTreeList($data);
      }
      $result .= '</faq>'.LF;
      return $result;
    }
    return FALSE;
  }

  /**
  * Get XML output FAQ group tree list
  *
  * @param array $data
  * @access public
  * @return string or FALSE
  */
  function getXMLOutputFaqGroupTreeList($data) {
    $i = $data['faq'];
    $result = '';
    if (isset($this->faqGroups) && is_array($this->faqGroups)) {
      $result .= '<groups>'.LF;
      foreach(@$this->faqGroups as $id => $faqGroup) {
        if (isset($faqGroup) && is_array($faqGroup)) {
          $selected = (@$this->params['faqgroup_id'] == $id) ? 'selected="selected"' : '';
          if($faqGroup['faq_id'] == $i){
            $result .= sprintf('<group href="%s" title="%s" entries="%d" %s>'.LF,
            $this->buildLink(array('faqgroup_id' => (int)$id)),
            papaya_strings::escapeHTMLChars(@$faqGroup['faqgroup_title']),
            (int)@$faqGroup['entry_count'],
            $selected);
            if(@$this->params['faqgroup_id'] == $id){
              if(''!=$faqGroup['faqgroup_descr']){
                $result.=sprintf('<descr><![CDATA[%s]]></descr>'.LF,
                papaya_strings::escapeHTMLChars($faqGroup['faqgroup_descr']));
              }
            }
            if( (isset($this->entries) && is_array($this->entries)) && (@$this->params['faqgroup_id'] == $id) ) {
              $result .= $this->getXMLOutputEntryList($id,$data);
            }
          $result .= '</group>'.LF;
          }
        }
      }
      $result .= '</groups>'.LF;
      return $result;
    }
    return FALSE;
  }

  /**
  * Get XML output entry list
  *
  * @param integer $i
  * @param array $data
  * @access public
  * @return string or FALSE
  */
  function getXMLOutputEntryList($i,$data) {
    $result = '';
    if (isset($this->entries) && is_array($this->entries)) {
      include_once(PAPAYA_INCLUDE_PATH.'system/papaya_parser.php');
      $parser = &new papaya_parser;
      $parser->tableTopics = $this->tableTopics;
      $parser->tableTopicsTrans = $this->tableTopicsTrans;
      
      $result .= '<entries>'.LF;
      foreach($this->entries as $id => $entry) {
        if (isset($entry) && is_array($entry)) {
          $selected = (@$this->params['entry_id'] == $id) ? 'selected="selected"' : '';
          if($entry['faqgroup_id'] == $i){
            $result .= sprintf('<entry  href="%s" title="%s" %s>'.LF,
            $this->buildLink(array('entry_id' => (int)$id)),
            papaya_strings::escapeHTMLChars($entry['entry_title']),
            $selected);
            if(@!$this->params['entry_id'] == $id) {
              $result .= sprintf('<question><![CDATA[%s]]></question>'.LF,
                $this->module->getXHTMLString(
                $this->stringCut($parser->parse($entry['entry_question'], $this->contentLanguageId), 300), TRUE));
              $result .= sprintf('<answer><![CDATA[%s]]></answer>'.LF,
                $this->module->getXHTMLString(
                $this->stringCut($parser->parse($entry['entry_answer'], $this->contentLanguageId), 300), TRUE));
            } else {
              if(''!=$entry['entry_question']){
                $result.=sprintf('<question><![CDATA[%s]]></question>'.LF,
                $this->module->getXHTMLString(trim($entry['entry_question']),TRUE));
              }
              if(''!=$entry['entry_answer']){
                $result.=sprintf('<answer><![CDATA[%s]]></answer>'.LF,
                $this->module->getXHTMLString($entry['entry_answer'],TRUE));
              }
            }
            if((isset($this->comments) && is_array($this->comments))&&($this->params['entry_id'] == $id)) {
              $result .= $this->getXMLOutputCommentList($this->params['entry_id']);
            }
            if(@$this->params['entry_id'] == $id) {
              if($data['post_comments'] != -1) {
                if($this->module->checkSurferPerm($data['post_comments'])){
                  $result .= $this->getOutputPermForm();
                  $result .= '</entry>'.LF;
                } else {
                  $result .= '</entry>'.LF;
                }
              } else {
                if( is_object($GLOBALS['PAPAYA_SURFER'])
                      && $GLOBALS['PAPAYA_SURFER']->isValid == TRUE
                    ){
                  $result .= $this->getOutputPermForm();
                  $result .= '</entry>'.LF;
                } else {
                  $result .= $this->getOutputAnonForm();
                  $result .= '</entry>'.LF;
                }
              }
            } else {
              $result .= '</entry>'.LF;
            }
          }
        }
      }
      $result .= '</entries>'.LF;
      return $result;
    }
    return FALSE;
  }

  /**
  * Get XML output comment list
  *
  * @param integer $i
  * @access public
  * @return string or FALSE
  */
  function getXMLOutputCommentList($i) {
    $result = '';
    if (isset($this->comments) && is_array($this->comments)) {
      $result .= '<notes>';
      foreach($this->comments as $id => $note) {
        if (isset($note) && is_array($note)) {
          $selected = (@$this->params['note_id'] == $id) ? 'selected="selected"' : '';
          if($note['entry_id'] == $i){
            $result .= sprintf('<note href="%s" user="%s" created="%s" %s><![CDATA[%s]]>'.LF,
            $this->buildLink(array('note_id' => (int)$id)),
            $note['note_username'],
            date('Y-m-d H:i:s', $note['note_created']),
            $selected,
            $this->module->getXHTMLString($note['note_content'], TRUE));
            $result.='</note>'.LF;
          }
        }
      }
      $result .='</notes>'.LF;
      return $result;
    }
    return FALSE;
  }

  /**
  * Initialize output anonym formular
  *
  * @access public
  */
  function initializeOutputAnonForm() {
    if (!is_object($this->outputDialog)) {
      include_once(PAPAYA_INCLUDE_PATH.'system/base_dialog.php');
      $hidden = array('cmd' => 'add_note', 'save' => 1,
        'entry_id' => (int)$this->params['entry_id']);
      $fields = array(
        'note_username' => array('Username', 'isSomeText', TRUE, 'input', 200),
        'note_content' => array('Content', 'isSomeText', TRUE, 'textarea', 8)
      );
      $this->outputDialog = &new base_dialog($this, $this->paramName, $fields, $data, $hidden);
      $this->outputDialog->msgs = &$this->msgs;
      $this->outputDialog->loadParams();
    }
  }

  /**
  * initialize output search formular
  *
  * @access public
  */
  function initializeOutputSearchForm() {
    if (!is_object($this->outputDialog)) {
      include_once(PAPAYA_INCLUDE_PATH.'system/base_dialog.php');
      $hidden = array(
        'entry_id' => (int)@$this->params['entry_id']);
      $this->outputDialog = &new base_dialog($this, $this->paramName, $fields, $data, $hidden);
      $this->outputDialog->msgs = &$this->msgs;
      $this->outputDialog->loadParams();
    }
  }

  /**
  * Initialize output permission formular
  *
  * @access public
  */
  function initializeOutputPermForm() {
    if (!is_object($this->outputDialog)) {
      include_once(PAPAYA_INCLUDE_PATH.'system/base_dialog.php');


      if (is_object($GLOBALS['PAPAYA_SURFER'])) {
        $data['note_username'] = $GLOBALS['PAPAYA_SURFER']->surferGivenname.' '.$GLOBALS['PAPAYA_SURFER']->surferSurname;
        $usernameInput = FALSE;
      } else {
        $usernameInput = TRUE;
      }
      $hidden = array('cmd' => 'add_note', 'save' => 1,
        'entry_id' => (int)$this->params['entry_id']);
      $fields = array(
        'note_username' => array('Username', 'isNoHTML', TRUE, (($usernameInput) ? 'input' : 'disabled_input'), 100),
        'note_content' => array('Message', 'isSomeText', TRUE, 'textarea', 8)
      );
      $this->outputDialog = &new base_dialog($this, $this->paramName, $fields, $data, $hidden);
      $this->outputDialog->msgs = &$this->msgs;
      $this->outputDialog->loadParams();
    }
  }

  /**
  * Get output anonym formular
  *
  * @access public
  * @return string
  */
  function getOutputAnonForm() {
    $this->initializeOutputAnonForm();
    $this->outputDialog->baseLink = $this->baseLink;
    $this->outputDialog->dialogTitle = htmlspecialchars(@$caption);
    $this->outputDialog->dialogDoubleButtons = FALSE;
    return '<newdlg>'.$this->outputDialog->getDialogXML().'</newdlg>';
  }

  /**
  * Get output permission formular
  *
  * @access public
  * @return string
  */
  function getOutputPermForm() {
    $this->initializeOutputPermForm();
    $this->outputDialog->baseLink = @$this->baseLink;
    $this->outputDialog->dialogTitle = @htmlspecialchars($caption);
    $this->outputDialog->dialogDoubleButtons = FALSE;
    return '<newdlg>'.$this->outputDialog->getDialogXML().'</newdlg>';
  }

  /**
  * Initialize search formular
  *
  * @access public
  */
  function initializeSearchForm() {
    if (!is_object($this->searchDialog)) {
      include_once(PAPAYA_INCLUDE_PATH.'system/base_dialog.php');
      $hidden = array(
        'note_id' => 0,
        'faqgropup_id' => (int)@$this->params['faqgroup_id'],
        'faq_id' => (int)$this->params['faq_id'],
        'entry_id' => @(int)$this->params['entry_id'],
        'search' => TRUE
      );
      $fields = array(
        'searchfor' => array('Search', 'isNoHTML', TRUE, 'input', 200),
      );

      $this->searchDialog = &new base_dialog($this, $this->paramName, $fields, $data, $hidden);
      $this->searchDialog->msgs = &$this->msgs;
      $this->searchDialog->loadParams();
    }
  }

  /**
  * Get search formular
  *
  * @param string $caption
  * @access public
  * @return string
  */
  function getSearchForm($caption) {
    $this->initializeOutputSearchForm();
    $this->searchDialog->baseLink = $this->baseLink;
    $this->searchDialog->dialogTitle = htmlspecialchars($caption);
    $this->searchDialog->dialogDoubleButtons = FALSE;
    return '<searchdlg>'.$this->searchDialog->getDialogXML().'</searchdlg>';
  }

  /**
  * Search entries
  *
  * @param integer $faqId
  * @param string $searchFor
  * @param array &$entryIds
  * @access public
  * @return boolean
  */
  function searchEntries($faqId, $searchFor, &$entryIds) {
    $entryIds = NULL;
    include_once(PAPAYA_INCLUDE_PATH.'system/base_searchstringparser.php');
    $parser = &new searchstringparser;
    $filter = $parser->getSQL($searchFor,
      array('e.entry_question', 'e.entry_answer', 'e.entry_title', 'c.note_content'),
      $this->fullTextSearch);

    if ($filter) {
      $sql = "SELECT e.entry_id
                FROM %s g, %s e
                LEFT OUTER JOIN %s c ON c.entry_id = e.entry_id
               WHERE g.faq_id = %d
                 AND e.faqgroup_id = g.faqgroup_id
                 AND ( ".str_replace('%', '%%', $filter)." )";
      $params = array($this->tableFaqgroups, $this->tableEntries, $this->tableComments, $faqId);
      if ($res = $this->databaseQueryFmt($sql, $params)) {
        while($row = $res->fetchRow()) {
          if ($row[0] > 0) {
            $entryIds[$row[0]] = 1;
          } else {
            $entyIds[$row[1]] = 1;
          }
        }
        if (isset($entryIds) && is_array($entryIds)) $entryIds = array_keys($entryIds);
        return TRUE;
      }
    }
    return FALSE;
  }

  /**
  * Search faq
  *
  * @param integer $faqId
  * @param string $searchFor
  * @access public
  * @return boolean
  */
  function searchFaq($faqId, $searchFor) {
    unset($this->entries);
    unset($entryIds);
    $searched = FALSE;
    if ($this->cacheSearchResults) {
      $searchResult = $this->getSessionValue($this->sessionParamName.'_searchresult');
      if (($searchResult['searchfor'] == $searchFor) && ($searchResult['time'] > time())) {
        $entryIds = $searchResult['entry_ids'];
        $searched = TRUE;
      } else {
        $searched = $this->searchFaqGroups($faqId, $searchFor, $entryIds);
        if ($searched) {
          $searchResult = array('searchfor' => $searchFor, 'time' => time()+1800, 'entry_ids' => $entryIds);
          $this->setSessionValue($this->sessionParamName.'_searchresult', $searchResult);
        }
      }
    } else {
      $searched = $this->searchEntries($faqId, $searchFor, $entryIds);
    }
    if (isset($entryIds) && is_array($entryIds)) {
      $filter = (count($entryIds) > 1)
        ? "IN ('".implode("','", $entryIds)."')"
        : " = '".(int)$entryIds[0]."'";
      $sql = "SELECT entry_id, entry_question, entry_title
                FROM %s
               WHERE entry_id $filter ";
      if ($res = $this->databaseQueryFmt($sql, array($this->tableEntries))) {
        while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
          $this->entries[$row['entry_id']] = $row;
        }
        return TRUE;
      }
    } else {
      return $searched;
    }
    return FALSE;
  }

  /**
  * Get output thread list
  *
  * @param array $data
  * @access public
  * @return string
  */
  function getOutputEntries($data) {
    $faqId=$data['faq'];
    $result = '';
    if (isset($this->entries) && is_array($this->entries)) {
      $result = sprintf('<results >'.LF);
      foreach($this->entries as $entry) {
        $selected = ($entry['entry_id'] == $this->params['entry_id']) ? ' selected="selected"' : '';
        $result .= sprintf('<result href="%s" title="%s" %s>'.LF,
          $this->buildLink(
            array('entry_id'=>(int)$entry['entry_id'])
          ),
          htmlspecialchars($entry['entry_title']),
          $selected,
          htmlspecialchars($entry['entry_title']));
        $result .= '<![CDATA['.htmlspecialchars($entry['entry_question']).']]>';
        $result .= '</result>'.LF;
      }
      $result .= '</results>'.LF;
    }
    return $result;
  }

  /**
  * string cut
  *
  * @param string $string
  * @access public
  * @return string
  */
  function stringCut($string, $maxLength = 300){
    $teaser = str_replace("\r\n","\n", strip_tags($string));
    if (preg_match("/^(.+)([\n]{2})/sU", $teaser, $regs)) {
      return $regs[1];
    }
    return substr($teaser, 0, $maxLength);
  }
}

?>
