<?php
/**
* Function class discussion
*
* @package module_forum
* @author Thomas Weinert <weinert@papaya-cms.com>
* @version $Id: admin_forum.php,v 1.7 2006/05/05 10:11:01 kelm Exp $
*/

/**
* forum base functionality
*/
require_once(dirname(__FILE__).'/base_forum.php');

/**
* Function class discussion
*
* @author Thomas Weinert <weinert@papaya-cms.com>
* @package module_forum
*/
class admin_forum extends base_forum {
  /**
  * Categories table
  * @var string $tableCategs
  */
  var $tableCategs = '';
  /**
  * Forum / boards table
  * @var string $tableBoards
  */
  var $tableBoards = '';
  /**
  * Entry table
  * @var string $tableEntries
  */
  var $tableEntries = '';
  /**
  * Surfer table
  * @var string $tableSurfers
  */
  var $tableSurfers = '';

  /**
  * Css input class
  * @var string
  */
  var $inputFieldSize = 'x-large';

  /**
  * Parameter prefix name
  * @var string
  */
  var $paramName = '';
  /**
  * Seassion parameter prefix name
  * @var string
  */
  var $sessionParamName = '';
  /**
  * Parameters
  * @var array
  */
  var $params = NULL;

  /**
  * Categories
  * @var array
  */
  var $categs = NULL;
  var $categ = NULL;

  /**
  * Category tree
  * @var array
  */
  var $categTree = NULL;
  /**
  * Boards
  * @var array
  */
  var $boards = NULL;
  /**
  * Board
  * @var array
  */
  var $board = NULL;

  /**
  * Entries
  * @var array
  */
  var $entries = NULL;
  /**
  * Entry
  * @var array
  */
  var $entry = NULL;
  /**
  * Entry tree
  * @var array
  */
  var $entryTree = NULL;

  /**
  * Threads
  * @var array
  */
  var $threads = NULL;
  /**
  * Thread id list
  * @var array
  */
  var $entryIdList = NULL;

  /**
  * Full text search ?
  * @var array
  */
  var $fullTextSearch = PAPAYA_SEARCH_BOOLEAN;
  /**
  * Cache search results
  * @var boolean
  */
  var $cacheSearchResults = FALSE;


  var $cuttedCategs = NULL;
  var $cuttedList = NULL;
  var $cuttedThreads = NULL;
  var $categsById = NULL;

  var $allowedTags = array('b', 'i', 'tt');

  /**
  * Initialize - Load parameters and seassion variable
  */
  function initialize() {
    $this->initializeParams();
    $this->sessionParams = $this->getSessionValue($this->sessionParamName);
    $this->initializeSessionParam('categ_id', array('forum_id', 'thread_id', 'entry_id'));
    $this->initializeSessionParam('forum_id', array('cmd', 'thread_id', 'entry_id', 'offset'));
    $this->initializeSessionParam('thread_id', array('cmd', 'entry_id'));
    $this->initializeSessionParam('entry_id', array('cmd'));

    if (!isset($this->params['offset'])) {
      $this->params['offset'] = 0;
    }
  }

  /**
  * Execute - basic function for handling parameters
  */
  function execute() {
    if (isset($this->sessionParams['categopen']) && is_array($this->sessionParams['categopen'])) {
      $this->categsOpen = $this->sessionParams['categopen'];
    } else {
      $this->categsOpen = array();
    }
    switch (@$this->params['cmd']) {
      case 'open' :
        $this->categsOpen[(int)$this->params['categ_id']] = TRUE;
        break;
      case 'close' :
        unset($this->categsOpen[(int)$this->params['categ_id']]);
        break;

      case 'repair' :
        $this->repairPaths();
        break;

      case 'add_categ' :
        if ($newId = $this->addCateg((int)$this->params['categ_id'])) {
          $this->addMsg(MSG_INFO, $this->_gt('Category added.'));
          $this->params['categ_id'] = $newId;
          $this->initializeSessionParam('categ_id', array('cmd', 'forum_id', 'thread_id', 'entry_id'));
        } else {
          $this->addMsg(MSG_ERROR, $this->_gt('Database error! Changes not saved.'));
        }
        break;
      case 'del_categ' :
        if (isset($this->params['confirm_delete']) && $this->params['confirm_delete']) {
          if ($this->categExists($this->params['categ_id'])) {
            if ($this->categisEmpty($this->params['categ_id'])) {
              if ($this->deleteCateg($this->params['categ_id'])) {
                $this->addMsg(MSG_INFO, $this->_gt('Category deleted.'));
                if ($this->categExists($this->params['categ_prev'])) {
                  $this->params['categ_id'] = $this->params['categ_prev'];
                } else {
                  $this->params['categ_id'] = 0;
                }
                $this->initializeSessionParam('categ_id', array('cmd', 'forum_id', 'thread_id', 'entry_id'));
              } else {
                $this->addMsg(MSG_ERROR, $this->_gt('Database error! Changes not saved.'));
              }
            } else {
              $this->addMsg(MSG_WARNING, $this->_gt('Category is not empty.'));
              $this->params['cmd'] = '';
            }
          } else {
            $this->addMsg(MSG_WARNING, $this->_gt('Category not found.'));
          }
        }
        break;
      case 'edit_categ' :
        $this->loadCategs();
        $this->loadCateg();
        $this->initializeCategEditform();
        if ($this->categDialog->modified()) {
          if ($this->categDialog->checkDialogInput()) {
            if ($this->saveCateg()) {
              $this->addMsg(MSG_INFO, sprintf($this->_gt('%s modified.'), $this->_gt('Category')));
            } else {
              $this->addMsg(MSG_ERROR, $this->_gt('Database error! Changes not saved.'));
            }
          }
        }
        break;
      case 'cut_categ' :
        if ($this->categExists($this->params['categ_id'])){
          if ($this->cutCateg((int)$this->params['categ_id'])) {
            $this->params['cmd'] = NULL;
            $this->params['categ_id'] = 0;
            $this->initializeSessionParam('categ_id', array('cmd', 'forum_id'));
            $this->addMsg(MSG_INFO, sprintf($this->_gt('%s cut out.'), $this->_gt('Categ')));
          } else {
            $this->addMsg(MSG_ERROR, $this->_gt('Database error!.'));
          }
        }
        break;

      case 'paste_categ' :
        if (isset($this->params['cut_categ_id']) &&
          $this->categExists($this->params['cut_categ_id']) &&
          isset($this->params['categ_id']) && $this->params['categ_id'] >= 0){

          if ($this->pasteCateg( $this->params['categ_id'],
            (int)$this->params['cut_categ_id'])) {

            $this->params['cmd'] = NULL;
            $this->params['categ_id'] = $this->params['cut_categ_id'];
            $this->initializeSessionParam('categ_id', array('cmd', 'forum_id'));
            $this->addMsg(MSG_INFO, sprintf($this->_gt('%s paste.'), $this->_gt('Categ')));
          } else {
            $this->addMsg(MSG_ERROR, $this->_gt('Database error!.'));
          }
        }
        break;

      case 'paste_forum' :
        if (isset($this->params['cut_forum_id']) &&
          $this->forumExists($this->params['cut_forum_id']) &&
          isset($this->params['categ_id']) && $this->categExists($this->params['categ_id'])){

          if ($this->pasteForum($this->params['cut_forum_id'], $this->params['categ_id'])) {
            $this->params['cmd'] = NULL;
            $this->params['forum_id'] = $this->params['cut_forum_id'];
            $this->initializeSessionParam('forum_id',array('offset','cmd'));
            $this->addMsg(MSG_INFO, sprintf($this->_gt('%s paste.'), $this->_gt('Forum')));
          } else {
            $this->addMsg(MSG_ERROR, $this->_gt('Database error!.'));
          }
        }
        break;

      case 'add_forum' :
        if ($newId = $this->addForum((int)$this->params['categ_id'])) {
          $this->addMsg(MSG_INFO, $this->_gt('Forum added.'));
          $this->params['forum_id'] = $newId;
          $this->initializeSessionParam('forum_id', array('cmd', 'thread_id', 'entry_id', 'offset'));
        } else {
          $this->addMsg(MSG_ERROR, $this->_gt('Database error! Changes not saved.'));
        }
        break;
      case 'del_forum' :
        if (isset($this->params['confirm_delete']) && $this->params['confirm_delete']) {
          if ($this->forumExists($this->params['forum_id'])) {
            if ($this->deleteForum($this->params['forum_id'])) {
              $this->addMsg(MSG_INFO, $this->_gt('Forum deleted.'));
              $this->params['forum_id'] = 0;
              $this->initializeSessionParam('forum_id', array('cmd', 'thread_id', 'entry_id'));
            } else {
              $this->addMsg(MSG_ERROR, $this->_gt('Database error! Changes not saved.'));
            }
          } else {
            $this->addMsg(MSG_WARNING, $this->_gt('Forum not found.'));
          }
        }
        break;
      case 'edit_forum' :
        $this->loadCategs();
        $this->loadBoards();
        $this->initializeForumEditform();
        if ($this->forumDialog->modified()) {
          if ($this->forumDialog->checkDialogInput()) {
            if ($this->saveForum()) {
              $this->addMsg(MSG_INFO, sprintf($this->_gt('%s modified.'), $this->_gt('Forum')));
            } else {
              $this->addMsg(MSG_ERROR, $this->_gt('Database error! Changes not saved.'));
            }
          }
        }
        break;

      case 'cut_forum' :
        if ($this->forumExists($this->params['forum_id'])){
          if ($this->cutForum((int)$this->params['forum_id'])) {
            $this->params['cmd'] = NULL;
            $this->params['forum_id'] = 0;
            $this->initializeSessionParam('forum_id', array('cmd', 'entry_id'));
            $this->addMsg(MSG_INFO, sprintf($this->_gt('%s cut out.'), $this->_gt('Forum')));
          } else {
            $this->addMsg(MSG_ERROR, $this->_gt('Database error!.'));
          }
        }
        break;
      case 'edit_entry' :
        $this->loadEntry($this->params['entry_id']);
        $this->initializeEntryEditform();
        if ($this->entryDialog->modified()) {
          if ($this->entryDialog->checkDialogInput()) {
            if ($this->saveEntry()) {
              $this->addMsg(MSG_INFO, sprintf($this->_gt('%s modified.'), $this->_gt('Entry')));
            } else {
              $this->addMsg(MSG_ERROR, $this->_gt('Database error! Changes not saved.'));
            }
          }
        }
        break;
      case 'del_entry' :
        if (isset($this->params['confirm_delete']) && $this->params['confirm_delete']) {
          if ($this->deleteEntry((int)$this->params['entry_id'],
              (bool)@$this->params['delete_all'])) {
            $this->addMsg(MSG_INFO, sprintf($this->_gt('%s deleted.'), $this->_gt('Entry')));
          } else {
            $this->addMsg(MSG_ERROR, $this->_gt('Database error!.'));
          }
        }
        break;
      case 'del_thread' :
        if (isset($this->params['confirm_delete']) && $this->params['confirm_delete']) {
          if ($this->deleteThread((int)$this->params['thread_id'])) {
            $this->addMsg(MSG_INFO, sprintf($this->_gt('%s deleted.'), $this->_gt('Thread')));
          } else {
            $this->addMsg(MSG_ERROR, $this->_gt('Database error!.'));
          }
        }
        break;
      case 'cut_thread' :
        if ($this->entryExists($this->params['thread_id'])){
          if ($this->cutThread((int)$this->params['thread_id'])) {
            $this->params['cmd'] = NULL;
            $this->params['thread_id'] = 0;
            $this->initializeSessionParam('thread_id', array('cmd', 'entry_id'));
            $this->addMsg(MSG_INFO, sprintf($this->_gt('%s cut out.'), $this->_gt('Thread')));
          } else {
            $this->addMsg(MSG_ERROR, $this->_gt('Database error!.'));
          }
        }
        break;
      case 'paste_thread' :
        if (isset($this->params['cut_thread_id']) &&
          $this->entryExists($this->params['cut_thread_id']) &&
          isset($this->params['forum_id']) &&
          $this->forumExists($this->params['forum_id']) ){
          if ($this->pasteThread()) {
            $this->params['cmd'] = NULL;
            $this->params['thread_id'] = $this->params['cut_thread_id'];
            $this->initializeSessionParam('thread_id', array('cmd', 'entry_id'));
            $this->addMsg(MSG_INFO, sprintf($this->_gt('%s paste.'), $this->_gt('Thread')));
          } else {
            $this->addMsg(MSG_ERROR, $this->_gt('Database error!.'));
          }
        }
        break;
    }
    $this->sessionParams['categopen'] = $this->categsOpen;
    $this->setSessionValue($this->sessionParamName, $this->sessionParams);
    $this->loadCategs();
    $this->loadBoards();
    $this->loadThreads(
      (isset($this->params['forum_id'])) ? (int)$this->params['forum_id'] : 0,
      (isset($this->params['offset'])) ? (int)$this->params['offset'] : 0, 20);
    $this->loadThread(@(int)$this->params['thread_id']);
    $this->loadEntry(@(int)$this->params['entry_id']);
    if ($this->cutOutExists()) {
      $this->loadCutedElement();
    }
  }

  /**
  * Return XML data
  * @return string
  */
  function getXML() {
    if (is_object($this->layout)) {
      $this->getXMLButtons();
      $this->getXMLCategTree();
      $this->getXMLForumList();
      switch (@$this->params['cmd']) {
        case 'del_categ' :
          $this->getXMLDelCategForm();
          break;
        case 'del_forum' :
          $this->getXMLDelForumForm();
          break;
        case 'del_thread' :
          $this->getXMLDelThreadForm();
          break;
        case 'del_entry' :
          $this->getXMLDelEntryForm();
          break;
        default :
          if (isset($this->entries[@(int)$this->params['entry_id']]) &&
              is_array($this->entries[@(int)$this->params['entry_id']])) {
            $this->getXMLEntryForm();
            $this->getXMLThread();
            $this->getXMLThreadList();
          } elseif (isset($this->entries[@(int)$this->params['thread_id']]) &&
              is_array($this->entries[@(int)$this->params['thread_id']])) {
            $this->getXMLThread();
            $this->getXMLThreadList();
          } elseif (isset($this->boards[@(int)$this->params['forum_id']]) &&
              is_array($this->boards[@(int)$this->params['forum_id']])) {
            $this->loadBoard($this->params['forum_id']);
            $this->getXMLForumForm();
            $this->getXMLThreadList();
          } elseif (isset($this->categs[@(int)$this->params['categ_id']]) &&
              is_array($this->categs[@(int)$this->params['categ_id']])) {
            $this->loadCateg();
            $this->getXMLCategForm();
          }
          if ( (isset($this->cuttedCategs) && is_array($this->cuttedCategs)) ||
            (isset($this->cuttedList) && is_array($this->cuttedList)) ||
            (isset($this->cuttedThreads) && is_array($this->cuttedThreads))) {

            $this->getXMLCutedList();
          }
      }
    }
  }

  /**
  * Get XML for buttons
  */
  function getXMLButtons() {
    include_once(PAPAYA_INCLUDE_PATH.'system/base_btnbuilder.php');
    $toolbar = &new base_btnbuilder;
    $toolbar->images = &$this->images;

    $toolbar->addButton('Repair path', $this->buildLink(array('cmd'=>'repair')),
      58, '', FALSE
    );

    $toolbar->addButton('New category',
      $this->buildLink(array('cmd'=>'add_categ', 'categ_id'=>(int)$this->params['categ_id'])),
      70, '', FALSE);
    if (isset($this->categs[$this->params['categ_id']]) && is_array($this->categs[$this->params['categ_id']])) {

      if (isset($this->params['categ_id']) && $this->params['categ_id'] > 0 &&
        (!isset($this->params['forum_id']) || $this->params['forum_id'] == 0) &&
        (!isset($this->params['thread_id']) || $this->params['thread_id'] == 0)) {
      $toolbar->addButton('Cut category',
        $this->buildLink(array('cmd'=>'cut_categ',
        'categ_id'=>(int)$this->params['categ_id'])),
        61, '', FALSE
      );
      $toolbar->addButton('Delete category',
        $this->buildLink(array('cmd'=>'del_categ', 'categ_id'=>(int)$this->params['categ_id'])),
        10, '', FALSE);
      }

      $toolbar->addSeperator();
      $toolbar->addButton('New forum',
        $this->buildLink(array('cmd'=>'add_forum', 'categ_id'=>(int)$this->params['categ_id'])),
        70, '', FALSE);
      if (isset($this->boards[@(int)$this->params['forum_id']]) &&
          is_array($this->boards[@(int)$this->params['forum_id']])) {

        if (isset($this->params['categ_id']) && $this->params['categ_id'] > 0 &&
          (isset($this->params['forum_id']) || $this->params['forum_id'] > 0) &&
          (!isset($this->params['thread_id']) || $this->params['thread_id'] == 0)){
          $toolbar->addButton('Cut Forum',
            $this->buildLink(array('cmd'=>'cut_forum',
            'forum_id'=>(int)$this->params['forum_id'])),
            61, '', FALSE
          );
          $toolbar->addButton('Delete forum',
            $this->buildLink(array('cmd'=>'del_forum', 'forum_id'=>(int)$this->params['forum_id'])),
            10, '', FALSE);
        }
        if (isset($this->entries[(int)@$this->params['thread_id']]) &&
          is_array($this->entries[(int)$this->params['thread_id']])) {

          $toolbar->addSeperator();
          $toolbar->addButton('Cut Thread',
            $this->buildLink(array('cmd'=>'cut_thread',
            'thread_id'=>(int)$this->params['thread_id'])),
            61, '', FALSE
          );

          $toolbar->addButton('Delete thread',
            $this->buildLink(array('cmd'=>'del_thread', 'thread_id'=>(int)$this->params['thread_id'])),
            10, '', FALSE);
          $toolbar->addSeperator();
        }
        $toolbar->addSeperator();
        if (isset($this->entry) && is_array($this->entry)) {
          if (isset($this->entryTree[$this->entry['entry_id']]) && is_array($this->entryTree[$this->entry['entry_id']]))
          $toolbar->addButton('Delete subthread',
            $this->buildLink(array('cmd'=>'del_entry', 'entry_id'=>(int)$this->params['entry_id'],
            'delete_all'=>1)),
            10, '', FALSE);
          $toolbar->addButton('Delete entry',
            $this->buildLink(array('cmd'=>'del_entry', 'entry_id'=>(int)$this->params['entry_id'])),
            10, '', FALSE);
          $toolbar->addSeperator();
        }
      }
    }

    if ($str = $toolbar->getXML()) {
      $this->layout->addMenu(sprintf('<menu ident="%s">%s</menu>'.LF, 'edit', $str));
    }
  }

  /**
  * Add category
  *
  * @param $parent
  * @access public
  * @return mixed
  */
  function addCateg($parent) {
    if ($this->categExists($parent) || ($parent == 0)) {
      return $this->databaseInsertRecord($this->tableCategs, 'forumcat_id',
        array('forumcat_prev'=>$parent, 'forumcat_title'=>$this->_gt('New category')));
    } else {
      return TRUE;
    }
  }

  /**
  * Delete Category
  *
  * @param integer $id category id
  * @access public
  * @return mixed mixed FALSE or number of affected_rows or database result object
  */
  function deleteCateg($id) {
    return $this->databaseDeleteRecord($this->tableCategs, 'forumcat_id', $id);
  }

  /**
  * Save category
  *
  * @access public
  * @return boolean
  */
  function saveCateg() {
    $data = array(
      'forumcat_title' => $this->params['forumcat_title'],
      'forumcat_desc' => $this->params['forumcat_desc']
    );
    return (FALSE !== $this->databaseUpdateRecord($this->tableCategs, $data, 'forumcat_id', (int)$this->params['categ_id']));
  }



  /**
  * Get XML category tree
  */
  function getXMLCategTree() {
    if (isset($this->categs) && is_array($this->categs)) {
      $result = sprintf('<listview title="%s" width="200">'.LF, $this->_gt('Categories'));
      $result .= '<items>'.LF;
      $selected = ($this->params['categ_id'] == 0) ? ' selected="selected"' : '';
      $result .= sprintf('<listitem href="%s" title="%s" image="%s" %s/>'.LF,
        $this->buildLink(array('categ_id'=>0,'categ_id'=>0)).'&ldb[forum_id]=0',
        $this->_gt('Base'),
        $this->images[55],
        $selected
      );
      $result .= $this->getXMLCategSubTree(0, 0);
      $result .= '</items>'.LF;
      $result .= '</listview>'.LF;
      $this->layout->addLeft($result);
    }
  }

  /**
  * Branch of Category tree
  * @param integer $parent Parent-ID
  * @param integer $indent shifting
  * @return string $result XML
  */
  function getXMLCategSubTree($parent, $indent) {
    $result = '';
    if (isset($this->categTree[$parent]) && is_array($this->categTree[$parent]) && (isset($this->categsOpen[$parent]) || ($parent == 0))) {
      foreach($this->categTree[$parent] as $id) {
        $result .= $this->getXMLCategEntry($id, $indent);
      }
    }
    return $result;
  }

  /**
  * Element of category tree
  * @param integer $id ID
  * @param integer $indent shifting
  * @return string $result XML
  */
  function getXMLCategEntry($id, $indent) {
    $result = '';
    if (isset($this->categs[$id]) && is_array($this->categs[$id])) {
      $empty = (bool)(!@is_array($this->categTree[$id]));
      $opened = (bool)(isset($this->categsOpen[$id]) && (!$empty));
      if ($empty) {
        $nodeHref = FALSE;
        $node = ' node="empty"';
      } elseif ($opened) {
        $nodeHref = $this->buildLink(array('cmd'=>'close', 'categ_id' => (int)$id));
        $node = sprintf(' node="open" nhref="%s"',$nodeHref);
      } else {
        $nodeHref = $this->buildLink(array('cmd'=>'open', 'categ_id' => (int)$id));
        $node = sprintf(' node="close" nhref="%s"',$nodeHref);
      }
      $selected = ($this->params['categ_id'] == $id) ? ' selected="selected"' : '';
      $result .= sprintf('<listitem href="%s" title="%s" indent="%d" %s %s/>'.LF,
        $this->buildLink(array('forum_id'=> 0, 'categ_id' => (int)$id)),
        htmlspecialchars($this->categs[$id]['forumcat_title']),
        $indent, $node, $selected);
      $result .= $this->getXMLCategSubTree($id, $indent+1);
    }
    return $result;
  }

  /**
  * Delete category formular
  *
  * @access public
  */
  function getXMLDelCategForm() {
    if (isset($this->categs[$this->params['categ_id']]) && is_array($this->categs[$this->params['categ_id']])) {
      include_once(PAPAYA_INCLUDE_PATH.'system/base_msgdialog.php');
      $hidden = array(
        'cmd' => 'del_categ',
        'categ_id' => $this->params['categ_id'],
        'categ_prev' => $this->categs[$this->params['categ_id']]['forumcat_prev'],
        'confirm_delete' =>1,
      );
      $msg = sprintf($this->_gt('Delete categ "%s" (%s)?'),
        htmlspecialchars($this->categs[$this->params['categ_id']]['forumcat_title']),
        (int)$this->params['categ_id']);
      $dialog = &new base_msgdialog($this, $this->paramName, $hidden, $msg, 'question');
      $dialog->baseLink = $this->baseLink;
      $dialog->msgs = &$this->msgs;
      $dialog->buttonTitle = 'Delete';
      $this->layout->add($dialog->getMsgDialog());
    }
  }

  /**
  * Initialize categ edit formular
  *
  * @access public
  */
  function initializeCategEditForm() {
    if (!(isset($this->categDialog) && is_object($this->categDialog))) {
      include_once(PAPAYA_INCLUDE_PATH.'system/base_dialog.php');
      $data = $this->categs[$this->params['categ_id']];
      $hidden = array('cmd'=>'edit_categ', 'save'=>1, 'categ_id'=>$this->params['categ_id']);
      $fields = array(
        'forumcat_title' => array('Title', 'isNoHTML', TRUE, 'input', 200),
        'forumcat_desc' => array('Description', 'isNoHTML', FALSE, 'textarea', 8)
      );
      $this->categDialog = &new base_dialog($this, $this->paramName, $fields, $data, $hidden);
      $this->categDialog->msgs = &$this->msgs;
      $this->categDialog->loadParams();
    }
  }

  /**
  * Get category formular XML
  *
  * @access public
  */
  function getXMLCategForm() {
    if (isset($this->categs[$this->params['categ_id']]) && is_array($this->categs[$this->params['categ_id']])) {
      $this->initializeCategEditForm();
      $this->categDialog->inputFieldSize = $this->inputFieldSize;
      $this->categDialog->baseLink = $this->baseLink;
      $this->categDialog->dialogTitle = htmlspecialchars($this->_gt('Properties'));
      $this->categDialog->dialogDoubleButtons = FALSE;
      $this->layout->add($this->categDialog->getDialogXML());
    }
  }

  /**
  * Delete forum
  *
  * @param integer $id
  * @access public
  * @return mixed FALSE or number of affected_rows or database result object
  */
  function deleteForum($id) {
    if ($this->databaseDeleteRecord($this->tableEntries, 'forum_id', $id) !== FALSE) {
      return ($this->databaseDeleteRecord($this->tableBoards, 'forum_id', $id) !== FALSE);
    }
    return FALSE;
  }

  /**
  * Save forum
  *
  * @access public
  * @return boolean
  */
  function saveForum() {
    $data = array(
      'forum_title' => $this->params['forum_title'],
      'forum_desc' => $this->params['forum_desc']
    );
    return (FALSE !== $this->databaseUpdateRecord($this->tableBoards, $data, 'forum_id', (int)$this->params['forum_id']));
  }

  /**
  * Get XML for forum list
  *
  * @access public
  */
  function getXMLForumList() {
    if (isset($this->boards) && is_array($this->boards)) {
      $result = sprintf('<listview title="%s" width="200">'.LF, $this->_gt('Forums'));
      $result .= '<items>'.LF;
      foreach($this->boards as $id=>$board) {
        if (isset($board) && is_array($board)) {
          $selected = (isset($this->params['forum_id']) && $this->params['forum_id'] == $id) ?
            ' selected="selected"' : '';
          $result .= sprintf('<listitem href="%s" title="%s" image="%s" %s/>'.LF,
            $this->buildLink(array('forum_id'=>(int)$id, 'thread_id' => 0)),
            htmlspecialchars($board['forum_title']),
            $this->images[((isset($this->params['forum_id']) && $this->params['forum_id'] == $id)  ? 57 : 56)], $selected);
        }
      }
      $result .= '</items>'.LF;
      $result .= '</listview>'.LF;
      $this->layout->addLeft($result);
    }
  }

  /**
  * Get XML for delete forum formular
  *
  * @access public
  */
  function getXMLDelForumForm() {
    if (isset($this->boards[$this->params['forum_id']]) && is_array($this->boards[$this->params['forum_id']])) {
      include_once(PAPAYA_INCLUDE_PATH.'system/base_msgdialog.php');
      $hidden = array(
        'cmd' => 'del_forum',
        'forum_id' => $this->params['forum_id'],
        'confirm_delete' =>1,
      );
      $msg = sprintf($this->_gt('Delete forum "%s" (%s)?'),
        htmlspecialchars($this->boards[$this->params['forum_id']]['forum_title']),
        (int)$this->params['forum_id']);
      $dialog = &new base_msgdialog($this, $this->paramName, $hidden, $msg, 'question');
      $dialog->baseLink = $this->baseLink;
      $dialog->msgs = &$this->msgs;
      $dialog->buttonTitle = 'Delete';
      $this->layout->add($dialog->getMsgDialog());
    }
  }

  /**
  * Initialize forum edit formular
  *
  * @access public
  */
  function initializeForumEditForm() {
    if (!(isset($this->forumDialog) && is_object($this->forumDialog))) {
      include_once(PAPAYA_INCLUDE_PATH.'system/base_dialog.php');
      $data = $this->board;
      $hidden = array('cmd'=>'edit_forum', 'save'=>1, 'forum_id'=>$data['forum_id']);
      $fields = array(
        'forum_title' => array('Title', 'isNoHTML', TRUE, 'input', 200),
        'forum_desc' => array('Description', 'isNoHTML', FALSE, 'textarea', 8)
      );
      $this->forumDialog = &new base_dialog($this, $this->paramName, $fields, $data, $hidden);
      $this->forumDialog->msgs = &$this->msgs;
      $this->forumDialog->loadParams();
    }
  }

  /**
  * Get XML for froum formular
  *
  * @access public
  */
  function getXMLForumForm() {
    if (isset($this->boards[$this->params['forum_id']]) && is_array($this->boards[$this->params['forum_id']])) {
      $this->initializeForumEditForm();
      $this->forumDialog->inputFieldSize = $this->inputFieldSize;
      $this->forumDialog->baseLink = $this->baseLink;
      $this->forumDialog->dialogTitle = htmlspecialchars($this->_gt('Properties'));
      $this->forumDialog->dialogDoubleButtons = FALSE;
      $this->layout->add($this->forumDialog->getDialogXML());
    }
  }

  /**
  * Delete entry
  *
  * @param integer $id
  * @param boolean $deleteSub optional, default value TRUE
  * @access public
  * @return boolean
  */
  function deleteEntry($id, $deleteSub = TRUE) {
    $this->loadEntry($id);
    $tid = ($this->entry['entry_tid'] > 0) ? $this->entry['entry_tid'] : $this->entry['entry_id'];
    $this->loadThread($tid);
    if (isset($this->entry) && is_array($this->entry)) {
      if ($deleteSub) {
        $ids = NULL;
        if (isset($this->entryTree[$id]) && is_array($this->entryTree[$id])) {
          //direkte subids ermitteln
          $ids = $this->entryTree[$id];
          if ($deleteSub) $ids = array_merge($ids, $this->getChildIds($ids));
        }
        //alle lschen
        if (isset($ids) && is_array($ids) && count($ids) >= 1) {
          $ids[] = $id;
          $filter = ' entry_id IN ("'.implode('","', $ids).'")';
          if (FALSE !== $this->databaseDeleteRecord($this->tableEntries, $filter)) {
            $this->updateThread($tid);
            return TRUE;
          }
        } else {
          if (FALSE !== $this->databaseDeleteRecord($this->tableEntries, 'entry_id', $id)) {
            $this->updateThread($tid);
            return TRUE;
          }
        }
      } else {
        //untergeordnete nach oben verschieben
        $pid = (int)$this->entry['entry_pid'];
        $data['entry_pid'] = $pid;
        if ($pid <= 0) $data['entry_tid'] = 0;
        $this->databaseUpdateRecord($this->tableEntries, $data, 'entry_pid', (int)$id);
        //eintrag lschen
        if (FALSE !== $this->databaseDeleteRecord($this->tableEntries, 'entry_id', $id)) {
          $this->updateThread($tid);
          return TRUE;
        }
      }
    }
    return FALSE;
  }

  /**
  * Delete thread
  *
  * @param integer $id
  * @access public
  * @return boolean
  */
  function deleteThread($id) {
    $filter = array(
      'entry_tid' => $id,
      'OR',
      'entry_id' => $id,
    );
    if (FALSE !== $this->databaseDeleteRecord($this->tableEntries, $filter)) {
      $this->params['cmd'] = '';
      $this->initializeSessionParam('cmd', array('thread_id'));
      return TRUE;
    } else {
      return FALSE;
    }
  }
  
  /**
  * Get XML for delete thread formular
  *
  * @access public
  */
  function getXMLDelThreadForm() {
    if (isset($this->entries[(int)$this->params['thread_id']]) &&
        is_array($this->entries[(int)$this->params['thread_id']])) {
      include_once(PAPAYA_INCLUDE_PATH.'system/base_msgdialog.php');
      $hidden = array(
        'cmd' => 'del_thread',
        'thread_id' => $this->params['thread_id'],
        'confirm_delete' =>1,
      );
      $entry = $this->entries[(int)$this->params['thread_id']];
      $msg = sprintf($this->_gt('Delete thread "%s" (%s) from "%s"?'),
        htmlspecialchars($entry['entry_subject']),
        (int)$entry['entry_id'],
        htmlspecialchars($entry['entry_username']));
      $dialog = &new base_msgdialog($this, $this->paramName, $hidden, $msg, 'question');
      $dialog->baseLink = $this->baseLink;
      $dialog->msgs = &$this->msgs;
      $dialog->buttonTitle = 'Delete';
      $this->layout->add($dialog->getMsgDialog());
    }
  }

  /**
  * Get XML for delte entry formular
  *
  * @access public
  */
  function getXMLDelEntryForm() {
    if (isset($this->entry) && is_array($this->entry)) {
      include_once(PAPAYA_INCLUDE_PATH.'system/base_msgdialog.php');
      $hidden = array(
        'cmd' => 'del_entry',
        'entry_id' => $this->entry['entry_id'],
        'delete_all' => (bool)@$this->params['delete_all'],
        'confirm_delete' =>1,
      );
      $msg = sprintf($this->_gt('Delete entry "%s" (%s) from "%s"?'),
        htmlspecialchars($this->entry['entry_subject']),
        (int)$this->entry['entry_id'],
        htmlspecialchars($this->entry['entry_username']));
      $dialog = &new base_msgdialog($this, $this->paramName, $hidden, $msg, 'question');
      $dialog->baseLink = $this->baseLink;
      $dialog->msgs = &$this->msgs;
      $dialog->buttonTitle = 'Delete';
      $this->layout->add($dialog->getMsgDialog());
    }
  }

  /**
  * Initialize entry edit formular
  *
  * @access public
  */
  function initializeEntryEditForm() {
    if (!(isset($this->entryDialog) && is_object($this->entryDialog))) {
      include_once(PAPAYA_INCLUDE_PATH.'system/base_dialog.php');
      $data = $this->entry;
      $hidden = array('cmd'=>'edit_entry', 'save'=>1, 'entry_id'=>$data['entry_id']);
      $fields = array(
        'entry_subject' => array('Subject', 'isNoHTML', TRUE, 'input', 200),
        'entry_text' => array('Message', 'isSomeText', TRUE, 'textarea', 8),
        'entry_username' => array('Username', 'isNoHTML', TRUE, 'input', 100),
        'entry_useremail' => array('EMail', 'isEMail', TRUE, 'input', 150),
        'entry_userregistered' => array('Registered User', 'isNum', FALSE, 'yesno', 1, '', 0, 'left'),
        'entry_sendanswers' => array('Send answers', 'isNum', FALSE, 'yesno', 1, '', 0, 'left'),
      );

      $this->entryDialog = &new base_dialog($this, $this->paramName, $fields, $data, $hidden);
      $this->entryDialog->textYes = $this->_gt('Yes');
      $this->entryDialog->textNo = $this->_gt('No');
      $this->entryDialog->msgs = &$this->msgs;
      $this->entryDialog->loadParams();
    }
  }

  /**
  * Get XML for entry formular
  *
  * @access public
  */
  function getXMLEntryForm() {
    if (isset($this->entry) && is_array($this->entry)) {
      $this->initializeEntryEditForm();
      $this->entryDialog->inputFieldSize = $this->inputFieldSize;
      $this->entryDialog->baseLink = $this->baseLink;
      $this->entryDialog->dialogTitle = htmlspecialchars($this->_gt('Properties'));
      $this->entryDialog->dialogDoubleButtons = FALSE;
      $this->layout->add($this->entryDialog->getDialogXML());
    }
  }

  /**
  * Get XML for thread list
  *
  * @access public
  */
  function getXMLThreadList() {
    if (isset($this->threads) && is_array($this->threads)) {
      $result = sprintf('<listview width="100%%" title="%s">', $this->_gt('Threads'));
      $result .= '<cols>';
      $result .= sprintf('<col>%s</col>', $this->_gt('Subject'));
      $result .= sprintf('<col align="center">%s</col>', $this->_gt('Created'));
      $result .= sprintf('<col align="center">%s</col>', $this->_gt('Answers'));
      $result .= sprintf('<col align="center">%s</col>', $this->_gt('Modified'));
      $result .= '</cols>';
      $result .= '<items>';
      foreach($this->threads as $thread) {
        $selected = (isset($this->params['thread_id']) && $thread['entry_id'] == $this->params['thread_id']) ? ' selected="selected"' : '';
        $result .= sprintf('<listitem href="%s" title="%s" image="%s" %s>',
          $this->buildLink(array('thread_id'=>(int)$thread['entry_id'],
            'offset'=>(int)@$this->params['offset'])),
          htmlspecialchars($thread['entry_subject']),
          $this->images[94], $selected);
        $result .= sprintf('<subitem align="center">%s</subitem>',
          htmlspecialchars(date('Y-m-d H:i:s', $thread['entry_created'])));
        $result .= sprintf('<subitem align="center">%s</subitem>',
          (int)$thread['entry_thread_count']);
        $result .= sprintf('<subitem align="center">%s</subitem>',
          htmlspecialchars(date('Y-m-d H:i:s', $thread['entry_thread_modified'])));
        $result .= '</listitem>';
      }
      $result .= '</items>';
      $result .= '</listview>';
      $this->layout->add($result);
    }
  }

  /**
  * Get XML for thread output
  *
  * @access public
  */
  function getXMLThread() {
    $entry = $this->entries[(int)$this->params['thread_id']];
    if (isset($entry) && is_array($entry)) {
      $result = sprintf('<listview width="100%%" title="%s">', $this->_gt('Thread'));
      $result .= '<cols>';
      $result .= sprintf('<col>%s</col>', $this->_gt('Subject'));
      $result .= sprintf('<col align="center">%s</col>', $this->_gt('Username'));
      $result .= sprintf('<col align="center">%s</col>', $this->_gt('Created'));
      $result .= sprintf('<col align="center">%s</col>', $this->_gt('Modified'));
      $result .= '</cols>';
      $result .= '<items>';
      $result .= $this->getXMLThreadElement($entry['entry_id'], 0);
      $result .= '</items>';
      $result .= '</listview>';
      $this->layout->add($result);
    }
  }

  /**
  * Get XML for thread element output
  *
  * @param integer $id
  * @param integer $indent
  * @access public
  * @return string
  */
  function getXMLThreadElement($id, $indent) {
    $entry = $this->entries[(int)$id];
    $result = '';
    if (isset($entry) && is_array($entry)) {
      $selected = ($entry['entry_id'] == $this->params['entry_id']) ? ' selected="selected"' : '';
      $threadId = ($entry['entry_tid'] > 0) ? $entry['entry_tid'] : $entry['entry_id'];
      $result .= sprintf('<listitem href="%s" title="%s" image="%s" indent="%d" %s>',
        $this->buildLink(array('thread_id'=>(int)$threadId, 'entry_id'=>(int)$entry['entry_id'],
          'offset'=>(int)@$this->params['offset'])),
        htmlspecialchars($entry['entry_subject']),
        $this->images[94], $indent, $selected);
      $result .= sprintf('<subitem align="center">%s</subitem>',
        htmlspecialchars($entry['entry_username']));
      $result .= sprintf('<subitem align="center">%s</subitem>',
        htmlspecialchars(date('Y-m-d H:i:s', $entry['entry_created'])));
      $result .= sprintf('<subitem align="center">%s</subitem>',
        htmlspecialchars(date('Y-m-d H:i:s', $entry['entry_modified'])));
      $result .= '</listitem>';
      $result .= $this->getXMLSubThread($entry['entry_id'], $indent+1);
      return $result;
    }
    return '';
  }

  /**
  * Get XML sub thread
  *
  * @param integer $pid
  * @param integer $indent
  * @access public
  * @return string
  */
  function getXMLSubThread($pid, $indent) {
    $result = '';
    if (isset($this->entryTree[$pid]) && is_array($this->entryTree[$pid])) {
      foreach($this->entryTree[$pid] as $id) {
        $result .= $this->getXMLThreadElement($id, $indent);
      }
    }
    return $result;
  }
  
  /**
  * Cut Categ to the clipboard
  *
  * @access public
  * @return mixed FALSE or number of affected_rows or database result object
  */
  function cutCateg () {
    $data = array('forumcat_prev' => -1);
    return (FALSE !== $this->databaseUpdateRecord($this->tableCategs,
      $data, 'forumcat_id', (int)$this->params['categ_id'])
    );
  }

  /**
  * Cut Forum to the clipboard
  *
  * @access public
  * @return mixed FALSE or number of affected_rows or database result object
  */
  function cutForum () {
    $data = array('forumcat_id' => -1);
    return (FALSE !== $this->databaseUpdateRecord($this->tableBoards,
      $data, 'forum_id', (int)$this->params['forum_id'])
    );
  }

  /**
  * Cut Thread to the clipboard
  *
  * @access public
  * @return mixed FALSE or number of affected_rows or database result object
  */
  function cutThread () {
    $data = array('forum_id'=>'-1');
    $cond = array('entry_id' => $this->params['thread_id'],
      'or', 'entry_tid' => $this->params['thread_id']
    );
    return (FALSE !== $this->databaseUpdateRecord($this->tableEntries, $data, $cond));
  }

  /**
  * Check if exists cuted Elements
  *
  * @access public
  * @return boolean
  */
  function cutOutExists () {
    $exists = FALSE;
    $sql = "SELECT count(*) FROM %s WHERE forumcat_prev = '%d'";
    $params = array($this->tableCategs, -1);
    if ($res = $this->databaseQueryFmt($sql, $params)) {
      if ($row = $res->fetchRow()) {
        if ((bool)$row[0] > 0 ) {
          $exists = TRUE;
        }
      }
    }

    $sql = "SELECT count(*) FROM %s WHERE forumcat_id = '%d'";
    $params = array($this->tableBoards, -1);
    if ($res = $this->databaseQueryFmt($sql, $params)) {
      if ($row = $res->fetchRow()) {
        if ((bool)$row[0] > 0 ) {
          $exists = TRUE;
        }
      }
    }

    $sql = "SELECT count(*) FROM %s WHERE forum_id = '%d'";
    $params = array($this->tableEntries, -1);
    if ($res = $this->databaseQueryFmt($sql, $params)) {
      if ($row = $res->fetchRow()) {
        if ((bool)$row[0] > 0 ) {
          $exists = TRUE;
        }
      }
    }
    return $exists;
  }

  /**
  * Get XML for Clipboard
  *
  * @access public
  */
  function getXMLCutedList() {
      $result = sprintf('<listview title="%s" width="200">'.LF, $this->_gt('Clipboard'));
      $result .= '<items>'.LF;
      $result .= $this->getXMLCutedElements();
      $result .= '</items>'.LF;
      $result .= '</listview>'.LF;
      $this->layout->addLeft($result);
  }

  /**
  * Get XML of elements for clipboard
  *
  * @access public
  * @return string $result XML
  */
  function getXMLCutedElements() {
    $result = '';
    if (isset($this->cuttedCategs) && is_array($this->cuttedCategs)){
      foreach($this->cuttedCategs as $cuttedCateg) {
        $result .= sprintf('<listitem title="%s" image="%s">'.LF,
          $cuttedCateg['forumcat_title'], $this->images[56]
        );
        $result .= sprintf('<subitem align="right">'.LF);
        $result .= sprintf('<a href="%s">'.LF,
          $this->buildLink(array('cmd'=>'paste_categ',
            'cut_categ_id'=>(int)$cuttedCateg['forumcat_id'],
            'categ_id'=>$this->params['categ_id']
          ))
        );
        $result .= sprintf('<glyph src="paste.gif"/>'.LF);
        $result .= '</a>'.LF;
        $result .= '</subitem>'.LF;
        $result .= '</listitem>'.LF;

      }
    }
    if (isset($this->cuttedList) && is_array($this->cuttedList)){
      foreach($this->cuttedList as $cuttedForum) {

        $result .= sprintf('<listitem title="%s" image="%s">'.LF,
          $cuttedForum['forum_title'],
          $this->images[56]
        );
        $result .= sprintf('<subitem align="%s">'.LF, "right");
        $result .= sprintf('<a href="%s">'.LF,
          $this->buildLink(array('cmd'=>'paste_forum',
            'cut_forum_id'=>(int)$cuttedForum['forum_id'])
          )
        );
        $result .= sprintf('<glyph src="%s"/>'.LF, 'paste.gif');
        $result .= '</a>'.LF;
        $result .= '</subitem>'.LF;
        $result .= '</listitem>'.LF;

      }
    }

    if (isset($this->cuttedThreads) && is_array($this->cuttedThreads)){
      foreach($this->cuttedThreads as $cuttedThread) {

        $result .= sprintf('<listitem title="%s" image="%s">'.LF,
          $cuttedThread['entry_subject'],
          $this->images[94]
        );
        $result .= sprintf('<subitem align="%s">'.LF, "right");
        $result .= sprintf('<a href="%s">'.LF,
          $this->buildLink(array('cmd'=>'paste_thread',
            'cut_thread_id'=>(int)$cuttedThread['entry_id'])
          )
        );
        $result .= sprintf('<glyph src="%s"/>'.LF, 'paste.gif');
        $result .= '</a>'.LF;
        $result .= '</subitem>'.LF;
        $result .= '</listitem>'.LF;

      }
    }
    return $result;
  }

  /**
  * Load Elements from clipboard
  *
  * @access public
  */
  function loadCutedElement() {
    $sql = 'SELECT forumcat_id,
                    forumcat_prev,
                    forumcat_title
              FROM %s
              WHERE forumcat_prev = %d';
    $params = array($this->tableCategs, -1);

    if ($res = $this->databaseQueryFmt($sql, $params)) {
      while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->cuttedCategs[(int)$row['forumcat_id']] = $row;
      }
    }

    $sql = 'SELECT forum_id,
                    forumcat_id,
                    forum_title
              FROM %s
              WHERE forumcat_id = %d';
    $params = array($this->tableBoards, -1);
    if ($res = $this->databaseQueryFmt($sql, $params)) {
      while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->cuttedList[(int)$row['forum_id']] = $row;
      }
    }

    $sql = 'SELECT entry_id,
                    entry_subject
              FROM %s
              WHERE forum_id = %d
              AND entry_pid = %d';
    $params = array($this->tableEntries, -1, 0);
    if ($res = $this->databaseQueryFmt($sql, $params)) {
      while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->cuttedThreads[(int)$row['entry_id']] = $row;
      }
    }
  }

  /**
  * Paste Categ from clipboard into Categstree
  *
  * @param integer $parent
  * @param integer $categ
  * @access public
  * @return mixed FALSE or number of affected_rows or database result object
  */
  function pasteCateg($parent, $categ) {
    $this->loadCateg($this->params['categ_id']);
/*     if (isset($this->categ) && is_array($this->categ)){
      $path = $this->categ['linkcateg_path'].$this->categ['linkcateg_id'].';';
    } else {
      $path = ';0;';
    } */
    $data = array(
      'forumcat_prev' => $parent
      //'linkcateg_path'=>$path
    );
    unset($this->categ);
    return (FALSE !== $this->databaseUpdateRecord($this->tableCategs, $data,
      ' forumcat_id', (int)$this->params['cut_categ_id'])
    );
  }

  /**
  * Paste Forum from clipboard into Categ
  *
  * @param integer $linkId
  * @param integer $categId
  * @access public
  * @return mixed FALSE or number of affected_rows or database result object
  */
  function pasteForum ($forumId, $categId) {
    $data = array('forumcat_id' => $categId);
    return (FALSE !== $this->databaseUpdateRecord($this->tableBoards,
      $data, 'forum_id', (int)$this->params['cut_forum_id'])
    );
  }

  /**
  * Paste Thread from clipboard into catgorie
  *
  * @access public
  * @return
  */
  function pasteThread () {
    $data = array('forum_id' => $this->params['forum_id']);
    $cond = array('entry_id' => $this->params['cut_thread_id'],
      'or', 'entry_tid' => $this->params['cut_thread_id']
    );
    return (FALSE !== $this->databaseUpdateRecord($this->tableEntries, $data, $cond));
  }

  /**
  * Calculate preview path
  *
  * @param $folderId
  * @access public
  * @return string
  */
  function calcPrevPath($categId) {
    if ($categId > 0 && isset($this->categs[$categId])) {
      $categ = $this->categs[$categId];
      if (!isset($categ['newpath'])) {
        if ($categ['forumcat_prev'] > 0) {
          if (isset($this->categs[$categ['forumcat_prev']])) {
            $newPath = $this->calcPrevPath($categ['forumcat_prev']).$categ['forumcat_prev'].';';
          } else {
            $newPath = ";0;";
          }
        } else {
          $newPath = ";0;";
        }
        $this->categs[$categId]['newpath'] = $newPath;
        return $newPath;
      } else {
        return $categ['newpath'];
      }
    }
    return '';
  }

  /**
  * Repair paths
  *
  * @access public
  */
  function repairPaths() {
    $this->loadCategs(NULL, TRUE);
    $count = 0;
    if (isset($this->categs) && is_array($this->categs)) {
      foreach($this->categs as $categId=>$categ) {
        if ($categId > 0) {
          $oldPath = $categ['forumcat_path'];
          $path = $this->calcPrevPath($categId);
          if ($oldPath != $path) {
            $data = array('forumcat_path'=>$path);

            $this->databaseUpdateRecord($this->tableCategs,
              $data, 'forumcat_id', $categId
            );
            $count++;
          }
        }
      }
      if ($count > 0) {
        $this->addMsg(MSG_INFO, sprintf($this->_gt('%s paths changed.'), $count));
      }
    }
  }
  
  function getForumCombo($paramName, $fieldName, $data) {
    $this->loadCategs();
    $this->loadBoards(TRUE);
    $result = sprintf('<select name="%s[%s]" class="dialogSelect dialogScale">'.LF, $paramName, $fieldName);
    if (isset($this->categs) && is_array($this->categs)) {
      $result .= $this->getForumComboSubTree(0,0, $data);
    }
    $result .= '</select>'.LF;
    return $result;
  }
  
  /**
  * Get forum combo sub tree
  *
  * @param integer $parent
  * @param integer $indent
  * @param array $data
  * @access public
  * @return string
  */
  function getForumComboSubTree($parent, $indent, $data) {
    $result = '';
    if (isset($this->categTree[$parent]) && is_array($this->categTree[$parent])) {
      foreach($this->categTree[$parent] as $id) {
        $result .= $this->getForumComboCateg($id, $indent, $data);
      }
    }
    return $result;
  }

  /**
  * Get forum combo category
  *
  * @param integer $id
  * @param integer $indent
  * @param array $data
  * @access public
  * @return string
  */
  function getForumComboCateg($id, $indent, $data) {
    $result = '';
    if (isset($this->categs[$id]) && is_array($this->categs[$id])) {
      $categ = $this->categs[$id];
      if ($indent > 0) {
        $indentString = "'".str_repeat('-', $indent).'->';
      } else {
        $indentString = '';
      }
      $selected = (($data['id'] == $id) && ($data['mode'] == 'categ'))
        ? ' selected="selected"' : '';
      $result .= sprintf('<option value="categ;%d" %s>%s %s</option>'.LF,
        $id, $selected ,
        $indentString, papaya_strings::escapeHTMLChars($this->categs[$id]['forumcat_title']));
      if (isset($categ['boards']) && is_array($categ['boards'])) {
        foreach ($categ['boards'] as $boardId) {
          $board = $this->boards[$boardId];
          $indentString = "'".str_repeat('-', $indent*4).'->';
          if (isset($board) && is_array($board)) {
            $selected = (($board['forum_id'] == $data['id']) && ($data['mode'] == 'forum'))
              ? ' selected="selected"' : '';
            $result .= sprintf('<option value="forum;%d" %s>%s %s</option>', $board['forum_id'], $selected,
              $indentString, papaya_strings::escapeHTMLChars($board['forum_title']));
          }
        }
      }
      $result .= $this->getForumComboSubTree($id, $indent+1, $data);
    }
    return $result;
  }
}
?>
