<?php
/**
* Page module - configurable feedback formular
*
* Configured by xml-file
*
* @package module_feedback
* @author Thomas Weinert <weinert@papaya-cms.com>
* @version $Id: content_form.php,v 1.4 2006/01/23 09:38:50 weinert Exp $
*/

/**
* Basic class page module
*/
require_once(PAPAYA_INCLUDE_PATH.'system/base_content.php');

define('PAPAYA_XMLFILES_PATH', PAPAYA_PATH_DATA.'xml/');

/**
* Seitenmodul konfigurierbares Feedbackformular
*
* Configured by xml-file
*
* @package module_feedback
* @author Thomas Weinert <weinert@papaya-cms.com>
*/
class content_form extends base_content {

  /**
  * cacheable ?
  * @var boolean
  */
  var $cacheable = FALSE;

  /**
  * Content edit fields
  * @var array
  */
  var $editFields = array(
    'xmlfile'=>  array('XML-Formular', 'isFile', TRUE, 'filecombo', array(PAPAYA_XMLFILES_PATH, '/^\w+\.xml$/i'), ''),
    'mailto' => array('Empfnger', 'isEMail', TRUE, 'input', 60, '', 'webmaster@localhost'),
    'subject' => array('Betreff', 'isNoHTML', TRUE, 'input', 60, '', 'Feeback von der Webseite'),
    'mailtext' => array('E-Mail Text', 'isSomeText', FALSE, 'textarea', 6, '',
      'Formular von der Webseite'),
    'Page',
    'text' => array('Fester Text', 'isSomeText', FALSE, 'textarea', 6, '',
      ''),
    'msg_hello' => array ('Text Aufforderung', 'isSomeText', FALSE, 'textarea', 6, '', ''),
    'msg_error' => array ('Text Eingabefehler', 'isSomeText', FALSE, 'textarea', 6, '',
      'Bitte prfen Sie die folgenden Felder.'),
    'msg_send' => array ('Text Besttigung', 'isSomeText', FALSE, 'textarea', 6, '',
      'Vielen Dank. Die Nachricht wurde versendet.'),
    'msg_notsend' => array ('Text Sendefehler', 'isSomeText', FALSE, 'textarea', 6, '',
      'Beim Senden ist ein Fehler aufgetreten. Bitte versuchen Sie es noch einmal.')
  );

  var $inputName = 'xmldialog';

  /**
  * On load function
  *
  * @access public
  */
  function onLoad() {
    $this->initializeLookups(basename(@$this->data['xmlfile']));
  }

  /**
  * Get parsed data
  *
  * @access public
  * @return string
  */
  function getParsedData() {
    $xmlData .= sprintf('<text>%s</text>'.LF, $this->getXHTMLString(@$this->data['text'], TRUE));
    if ($this->params['send']) {
      $error = $this->checkBoxInput();
      if (isset($error) && is_array($error)) {
        foreach($error as $key => $isError) {
          if (!$isError) unset($error[$key]);
        }
      }
      if (isset($error) && is_array($error) && count($error) > 0) {
        $xmlData .= $this->getBoxForm();
        $xmlData .= sprintf('<message>%s <ul>'.LF, $this->getXHTMLString(@$this->data['msg_error'], TRUE));
        if (isset($error) && is_array($error)) {
          foreach($error as $key => $isError) {
            if ($isError) {
              $xmlData .= '<li class="topictext">'.$this->getXHTMLString($this->inputFields[$key][0]).'</li>'.LF;
            }
          }
        }
        $xmlData .= '</ul></message>'.LF;
      } else {
        if ($this->sendEmail()) {
          $xmlData .= sprintf('<message>%s</message>'.LF, $this->getXHTMLString(@$this->data['msg_send'], TRUE));
        } else {
          $xmlData .= sprintf('<message>%s</message>'.LF, $this->getXHTMLString(@$this->data['msg_notsend'], TRUE));
        }
      }
    } else {
      $xmlData .= $this->getBoxForm();
      $xmlData .= sprintf('<message>%s</message>'.LF, $this->getXHTMLString(@$this->data['msg_hello'], TRUE));
    }
    $xmlData .= sprintf('<dialogname>%s</dialogname>'.LF, $this->inputName);
    return $xmlData;
  }

  /**
  * Get parameter name
  *
  * @param string $paramName
  * @access public
  * @return string
  */
  function getParamName($paramName) {
    return $paramName.'_cof';
  }

  /**
  * Send email
  *
  * @access public
  * @return integer
  */
  function sendEmail() {
    $msg = @$this->data['mailtext']."\n\n";
    foreach ($this->inputFields as $key => $val) {
      if (trim($val[0]) != '') {
        if (isset($this->params[$key]) && is_array($this->params[$key])) {
          $data = implode(', ',$this->params[$key]);
        } elseif (substr_count($this->params[$key], "\n") > 0) {
          $data = "\n\n".$this->params[$key].LF;
        } else {
          $data = $this->params[$key];
        }
        $msg .= str_pad($val[0], 30, ' ', STR_PAD_RIGHT).': '.$data.LF;
      }
    }
    $result = mail(@$this->data['mailto'], @$this->data['subject'], $msg);
    return $result;
  }


  /**
  * initialize Lookups
  * @param string $fileName filename
  */
  function initializeLookups($fileName) {
    $this->loadInputFields(basename($fileName));
    if (isset($this->inputFields) && is_array($this->inputFields)) {
      $this->editFields[] = 'LookUps';
        foreach($this->inputFields as $key=>$val) {
        if (preg_match('/^lookup_/', $val[3])) {
          $name = 'lookup_'.$val[4];
          $element = array ($val[0], 'isSomeText', FALSE, 'textarea', 6);
          $this->editFields[$name] = $element;
        }
      }
    }
  }

  /**
  * Initialize box formular
  */
  function initializeBoxForm() {
    if (!@is_object($this->dialog)) {
      include_once(PAPAYA_INCLUDE_PATH.'system/base_dialog.php');
      $hidden = array('send' => 1);
      $data = NULL;
      $this->boxForm = &new base_dialog($this, $this->paramName, $this->inputFields, $data, $hidden);
      $this->boxForm->loadParams();
      $this->boxForm->inputFieldSize = $this->inputFieldSize;
      $this->boxForm->baseLink = $this->baseLink;
    }
  }

  /**
  * Return box formular
  * @return string XML des Dialoges
  */
  function getBoxForm() {
    $this->initializeBoxForm();
    return $this->boxForm->getDialogXML();
  }

  /**
  * Check box input
  * @return array error
  */
  function checkBoxInput() {
    $this->initializeBoxForm();
    if ($result = $this->boxForm->checkDialogInput()) {
      $this->boxData = $this->boxForm->data;
      return FALSE;
    } else {
      return $this->boxForm->inputErrors;
    }
  }

  /**
  * Load input fields
  * @param string $fileName filename
  */
  function loadInputFields($fileName) {
    unset($this->inputFields);
    if (file_exists(PAPAYA_XMLFILES_PATH.$fileName) && ($fileName != '')) {
      if ($xmlTree = $this->createXMLTree(file_get_contents(PAPAYA_XMLFILES_PATH.$fileName))) {
        $this->readInputFields($xmlTree);
      }
    }
  }

  /**
  * Read input fields
  * @param reference &$xmlTree XML-tree
  */
  function readInputFields(&$xmlTree) {
    $parent = NULL;
    $template = &new simple_xmlnode($parent, 'dialog');
    $xmlTree->extract($elements, $template);
    if (isset($elements["nodes"]) && is_array($elements["nodes"])) {
      foreach($elements["nodes"] as $node) {
        if (is_object($node)) {
          $this->inputName = $node->attributes["name"];
        }
      }
    }
    $template = &new simple_xmlnode($parent, 'element');
    $xmlTree->extract($elements, $template);
    if (isset($elements["nodes"]) && is_array($elements["nodes"])) {
      foreach($elements["nodes"] as $node) {
        if (is_object($node)) {
          $xId = FALSE;
          if (isset($node->attributes["name"])) {
            $xId = $node->attributes["name"];
            $xData = array(@$node->attributes["caption"], @$node->attributes["check"], (int)@$node->attributes["needed"],
              @$node->attributes["type"], @$node->attributes["typeparam"], @$node->attributes["hint"],
              @$node->attributes["default"], @$node->attributes["align"]);
            $this->inputFields[$xId] = $xData;
          }
        }
      }
    }
  }
}

?>