<?php
set_time_limit(60);

/**
* PDF output filter
*
* @package papaya_module_pdf
* @author Thomas Weinert <weinert@papaya-cms.com>
* @version $Id: filter_pdf.php,v 1.1 2006/04/27 15:37:16 weinert Exp $
*/

/**
* Basic class output filters
*/
require_once (PAPAYA_INCLUDE_PATH.'system/base_outputfilter.php');

define('PAPAYA_PATH_PDFFILES', PAPAYA_PATH_TEMPLATES.'pdf/');

/**
* PDF output filter
*
* @package papaya_module_pdf
* @author Thomas Weinert <weinert@papaya-cms.com>
*/
class filter_pdf extends base_outputfilter {

	var $editFields = array(
		'xslfile' => array('XSLT Template', 'isFile', TRUE, 'filecombo',
      array(PAPAYA_PATH_PDFFILES, '/^\w+\.xsl$/i'), ''),
		'fullpage' => array('Full page', 'isNum', TRUE, 'yesno', '', '', 1));

	var $errorMessage = '';

	var $pdf = NULL;

  var $outputCharset = 'UTF-8';

  var $badStrings = array('&amp;nbsp;', '&amp;ndash;', '&amp;bdquo;', '&amp;ldquo;', '&amp;hellip;', '&amp;lsquo;', '&amp;rsquo;');
  var $badStringRepl = array(' ', '-', ',,', '"', '...', ",", "'");

	function parsePage(&$topic, &$layout) {
		$xslFile = PAPAYA_PATH_PDFFILES.$this->data['xslfile'];
    
		$layout->setXSL($xslFile);
		$output = str_replace($this->badStrings, $this->badStringRepl, $layout->xhtml());
        
    //dump the transformed xml
		if (isset($_GET['dump'])) {
			header('Content-type: text/xml');
      switch ($_GET['dump']) {
        case 'pdf' : echo $output; break;
        case 'xml' : echo $layout->xmlData; break;
      }
			exit();
		}
		$xmlTree = simple_xmltree::create();
		$xmlTree->loadFromString($output);
    
    include_once(dirname(__FILE__).'/papaya_pdf_options.php');
    include_once(dirname(__FILE__).'/papaya_pdf.php');
    
		$this->pdfConf = &new papaya_pdf_options();
    $this->pdf = &new papaya_pdf($this->pdfConf);
        
		if (isset($xmlTree->root) && is_object($xmlTree->root)) {
      if (isset($xmlTree->root->attributes['charset'])) {
        $this->outputCharset = $xmlTree->root->attributes['charset'];
      }
      
      $nodeCover = NULL;
      $nodeContent = NULL;
      $nodeFinal = NULL;
      $nodeHeader = NULL;
      $nodeFooter = NULL;
      
      if (isset($xmlTree->root->children) && count($xmlTree->root->children) > 0) {
        foreach(array_keys($xmlTree->root->children) as $idx) {
          if (isset($xmlTree->root->children[$idx]) && is_object($xmlTree->root->children[$idx])) {
            switch($xmlTree->root->children[$idx]->name) {
            case 'layout' : 
              $this->pdfConf->initialize($this, $xmlTree->root->children[$idx]);      
              $this->pdfConf->pdfInitialize($this->pdf);             
            case 'cover' :
              $nodeCover = &$xmlTree->root->children[$idx];
              break;
            case 'content' :
              $nodeContent = &$xmlTree->root->children[$idx];
              break;
            case 'final' :
              $nodeFinal = &$xmlTree->root->children[$idx];
              break;
            case 'header' :
              $nodeHeader = &$xmlTree->root->children[$idx];
              break;
            case 'footer' :
              $nodeFooter = &$xmlTree->root->children[$idx];
              break;
            }
          }
        }
      }
      $this->pdfConf->setFooter($nodeFooter);
      $this->pdfConf->setHeader($nodeHeader);
      
      $this->outputPageNamed($nodeCover, 'cover');
      $this->outputPage($nodeContent); 
      $this->outputPageNamed($nodeFinal, 'final');
		}
    $docTitle = $this->getWebLink($topic->topicId, $topic->getContentLanguageId(), 
      NULL, NULL, NULL, $topic->topic['TRANSLATION']['topic_title']);
    $result = $this->pdf->Output($docTitle, 'S');
    return $result;
	}

  function checkConfiguration($page = TRUE) {
		$xslFile = PAPAYA_PATH_PDFFILES.$this->data['xslfile'];
		if (!(is_file($xslFile) && is_readable($xslFile))) {
			$this->errorMessage = 'XSLT Template not readable.';
			return FALSE;
		}
		return TRUE;
	}

  function filterString($str) {
    $result = str_replace(array("\r\n", "\n\r", "\r", "\n"), " ", $str);
    $result = papaya_strings::ensureUTF8(trim($result));
    if (strtoupper($this->outputCharset) != 'UTF-8') {
      return papaya_strings::utf8ToCharset($result, $this->outputCharset);
    } else {
      return $result;
    }
  }
  
  function outputPageNamed(&$node, $pageName) {
    if (isset($node)) { 
      $this->pdfConf->pdfAddPage($this->pdf, $pageName, TRUE);
      if ($node->childCount() > 0) {
        foreach(array_keys($node->children) as $idx) {
          $str = '';
          if (isset($node->children[$idx]) && is_object($node->children[$idx]) &&
              $node->children[$idx]->name == 'element') {
            $elementNode = &$node->children[$idx];
            $for = @$elementNode->attributes['id'];
            if ($for != '' && $this->pdfConf->pdfInitElement($this->pdf, $pageName, $for)) {
              $this->pdf->openElement($elementNode->attributes);
              $this->outputDataChildren($elementNode);
              $this->pdf->closeElement($elementNode->attributes);
            }
          }
        }
      }
    }  
  }
  
  function outputPage(&$node) {
    $this->pdf->breakBeforeNextSection = TRUE;
    if (isset($node) && isset($node->children) && is_array($node->children) && count($node->children) > 0) {
      foreach(array_keys($node->children) as $idx) {
        $str = '';
        if (isset($node->children[$idx]) && is_object($node->children[$idx]) &&
            $node->children[$idx]->name == 'section') {
          $sectionNode = &$node->children[$idx];
          $this->pdf->openSection($sectionNode->attributes);
          $this->outputDataChildren($node->children[$idx]);
          $this->pdf->closeSection($sectionNode->attributes);
        }
      }    
    }  
  }

  function outputDataChildren(&$node) {
    if (isset($node->children) && is_array($node->children)) {
      foreach (array_keys($node->children) as $idx) {
        $childNode = &$node->children[$idx];
        $str = '';      
        if (is_a($childNode, 'simple_xmltextnode')) {
          $childNode->printText($str);
          $str = $this->filterString($str, TRUE);
          $this->pdf->writeText($str);
        } elseif (is_a($childNode, 'simple_xmlnode')) {
          if ($childNode->name == 'img') {
            if ($fileName = $this->getImageForPDF($childNode->attributes)) {
              $this->pdf->writeImage($fileName);
            }
          } else {
            $this->pdf->openTag($childNode->name, $childNode->attributes);
            // go into childnodes
            if ($childNode->childCount() > 0) {
              $this->outputDataChildren($childNode);
            }    
            $this->pdf->closeTag($childNode->name, $childNode->attributes);
          }
        }
      }
    }
  }

  function setDataFooter(&$node) {
    if (isset($node) && isset($node->children) && is_array($node->children) && count($node->children) > 0) {
      $str = '';
      $node->printText($str);
      $str = $this->filterString($str, TRUE);
      $this->pdf->footerText = $str;
    }
  }

  function getImageForPDF($imgAttr) {
    $imgURL = @$imgAttr['src'];
    $pImage = '~/?(sid([a-z]*([\da-f]{32}))/)?(([a-z\d_-]+/)*)([a-z\d_-]+)\.(image)(\.(jpg))(\?.*)?~i';
    $pMedia = '~^/?(sid([a-z]*([\da-f]{32}))/)?(([a-z\d_-]+/)*)([a-z\d_-]+\.(media|thumb)(\.(preview))?(\.(([a-z\d_]+)(\.([a-z]+)))))~i';
    if (preg_match($pImage, $imgURL, $regs)) {
      if (!(isset($this->imgGenerator) && is_object($this->imgGenerator))) {
        include_once(PAPAYA_INCLUDE_PATH.'system/base_imagegenerator.php');
        $this->imgGenerator = &new base_imagegenerator();
      }
      $params = $this->decodeQueryString($regs[10]);
      if ($this->imgGenerator->loadByIdent(@$regs[6])) {
        if ($fileName = $this->imgGenerator->generateImage(FALSE, @$params['img'])) {
          return $fileName;
        }
      }
    } elseif (preg_match($pMedia, $imgURL, $regs)) {
      $fileName = $regs[11];      
      switch ($regs[7]) {
      case 'thumb' :
        if (file_exists(PAPAYA_PATH_THUMBFILES.$fileName)) {
          $path = PAPAYA_PATH_THUMBFILES;
          break;
        }
      default :
        $path = PAPAYA_PATH_MEDIAFILES;
        break;
      }

      if (file_exists($path.$fileName) && is_file($path.$fileName) && is_readable($path.$fileName)) {
        list($imgWidth, $imgHeight, $format) = getimagesize($path.$fileName);
        if($format == 2 || $format == 3) {
          return $path.$fileName;
        } elseif ($format == 1) {
          if (preg_match('#^(.*?)(_([a-z]+))?_(\d+x\d+)(\.\w+)$#', $fileName, $regs)) {
            $fileName = $regs[1].$regs[5];
          } elseif (preg_match('#^(\w{32}\.\w+)$#', $fileName, $regs)) {
            $fileName = $regs[1];
          }
          $path = PAPAYA_PATH_MEDIAFILES;
          if (file_exists($path.$fileName) && is_file($path.$fileName) && is_readable($path.$fileName)) {
            include_once(PAPAYA_INCLUDE_PATH.'system/base_thumbnail.php');
            $thumbnail = &new base_thumbnail;
            $thumbnail->baseDirectory = $path;
            $thumbnail->thumbnailDirectory = PAPAYA_PATH_THUMBFILES;
            $thumbnail->width = $imgWidth;
            $thumbnail->height = $imgHeight;
            $thumbnail->resizeFlag = 'abs';
            if ($thumbFileName = $thumbnail->getThumb($fileName, '#FFFFFF')) {
              return $thumbnail->thumbnailDirectory.$thumbFileName;
            }
          }
        }
      }
    }
    return NULL;
  }
  
  function decodeQueryString($queryString) {
    $queryParams = array();
    if (preg_match_all('~(^|[?&])(([^=&?]+)=([^&]+))|([^&]+)~', $queryString, $matches, PREG_SET_ORDER)) {
      foreach ($matches as $match) {
        if (isset($match[3]) && $match[3] != '') {
          $null = NULL;
          if (preg_match_all('~([^\[\]]+)(\[|\]|$)~', $match[3], $nameParts)) {
            $currentParam = &$queryParams;
            $max = count($nameParts[1])-1;
            foreach ($nameParts[1] as $idx=>$namePart) {
              if ($max == $idx) {
                $currentParam[$namePart] = &$match[4];
              } else {
                $currentParam = &$currentParam[$namePart];
              }
            }
          }
        } elseif (isset($match[5])) {
          $queryParams[$match[5]] = TRUE;
        }
      }
    }
    return $queryParams;
  }
}
?>
