<?php
/**
* Implement FPDF/FPDI for papaya CMS
*
* @package papaya_module_pdf
* @author Thomas Weinert <weinert@papaya-cms.com>
* @author Sascha Kersten <kersten@papaya-cms.com>
* @version $Id: papaya_pdf.php,v 1.1 2006/04/27 15:37:16 weinert Exp $
*/

/**
* Define font path for fpdf default fonts
*/
define('FPDF_FONTPATH', '');

require_once(dirname(__FILE__).'/external/fpdf/ufpdf.php');

/**
* Implement FPDF/FPDI for papaya CMS
*
* @package papaya_module_pdf
* @author Thomas Weinert <weinert@papaya-cms.com>
* @author Sascha Kersten <kersten@papaya-cms.com>
*/
class papaya_pdf extends UFPDF {

  /**
  * configuration oebject
  * @var object papaya_pdf_options
  */
  var $confObj = NULL;

  /**
  * current page template name
  * @var string
  */
  var $currentPageTemplate = NULL;
  /**
  * current column number
  * @var integer
  */
  var $currentColumn = 1;

  /**
  * break before next section
  * @var boolean
  */
  var $breakBeforeNextSection = FALSE;

  /**
  * default styles
  * @var array
  */
  var $defaultStyle = array(
    'bold' => 0, 'italic' => 0, 'underline' => 0, 'list' => 0,
    'align' => 'left', 'fgcolor' => '#000', 'bgcolor' => ''
  );
  /**
  * current style
  * @var array
  */
  var $currentStyle = NULL;
  /**
  * element buffer
  * @var array
  */
  var $elementBuffer = array();
  /**
  * element mode (text | table | outputtable)
  * @var string
  */
  var $elementMode = 'text';
  /**
  * table buffer
  * @var object papaya_pdf_table
  */
  var $tableBuffer = NULL;
  /**
  * table cell buffer
  * @var object papaya_pdf_table_cell
  */
  var $currentTableCell = NULL;
  
  /**
  * indent step size
  * @var float
  */
  var $indentStep = 5;
  /**
  * bullet
  * @var string
  */
  var $listBullet = '-'; //"";
  /**
  * bullet font
  * @var string
  */
  var $listBulletFont = "Optima";
  /**
  * bullet font size
  * @var string
  */
  var $listBulletSize = 10;
  
  /**
  * status "in paragraph"
  * @var boolean
  */
  var $inParagraph = TRUE;
  /**
  * status "in column"
  * @var boolean
  */
  var $inColumn = TRUE;
  /**
  * status "in image"
  * @var boolean
  */
  var $inImage = TRUE;
  /**
  * status "in text"
  * @var boolean
  */
  var $inText = FALSE;

  /**
  * points to mm
  * @var float
  */
  var $dtpPoint = 0.352778;
  /**
  * image dpi
  * @var float
  */
  var $mediaDpi = 120;
  /**
  * image dpmm
  * @var float
  */
  var $mediaK = 0;
  
  /**
  * line space
  * @var float
  */
  var $lineSpacePt = 3.5;

  /**
  * constructor 
  *
  * @param object papaya_pdf_options &$conf
  * @access public
  */
	function __construct(&$conf) {
		parent::fpdi();
    $this->confObj = &$conf;
    $this->currentStyle = $this->defaultStyle;
    $this->mediaK = $this->mediaDpi / 25.4;
	}

  /**
  * PHP 4 constructor redirect
  *
  * @param object papaya_pdf_options &$conf
  * @access public
  * @return
  */
	function papaya_pdf(&$conf) {
		$this->__construct($conf);
	}

  /**
  * work with absolute font paths
  *
  * @access public
  * @return string empty font path ''
  */
  function _getfontpath() {
  	return '';
  }

  /**
  * css hex color to rgb array
  *
  * @param string $colorStr
  * @access public
  * @return array
  */
  function getRGB($colorStr) {
    $colorStr = ltrim($colorStr, "#");
    if ($colorStr <= 3) {
      while (strlen($colorStr) < 3) {
        $colorStr .= '0';
      }
      $r = hexdec(str_repeat(substr($colorStr, 0, 1), 2));
      $g = hexdec(str_repeat(substr($colorStr, 1, 1), 2));
      $b = hexdec(str_repeat(substr($colorStr, 2, 1), 2));    
    } else {
      while (strlen($colorStr) < 6) {
        $colorStr .= '0';
      }
      $r = hexdec(substr($colorStr, 0, 2));
      $g = hexdec(substr($colorStr, 2, 2));
      $b = hexdec(substr($colorStr, 4, 2));      
    }
    return array($r, $g, $b);
  }
  
  /**
  * set foreground color
  *
  * @param mixed $color
  * @access public
  */
  function setFGColor($color) {
    if (empty($color)) {
      $this->currentFGColor = array(0, 0, 0);    
    } elseif (!is_array($color)) {
      $this->currentFGColor = $this->getRGB($color);
    } else {
      $this->currentFGColor = $color;    
    }
    $this->SetTextColor($this->currentFGColor[0], $this->currentFGColor[0], $this->currentFGColor[0]);
  }
  
  /**
  * set background color
  *
  * @param mixed $color
  * @access public
  */
  function setBGColor($color) {
    if (empty($color)) {
      $this->currentBGColor = array(255, 255, 255);
    } elseif (!is_array($color)) {
      $this->currentBGColor = $this->getRGB($color);
    } else {
      $this->currentBGColor = $color;    
    }
    $this->SetFillColor($this->currentBGColor[0], $this->currentBGColor[0], $this->currentBGColor[0]);
  }

  /**
  * embed font style file
  *
  * @param string $family font family
  * @param string $style font style, default value ''
  * @param string $file font filename, default value ''
  * @access public
  */
  function addFont($family, $style  ='', $file = '') {
	  $family = strtolower($family);
    if (!(file_exists($file) && is_file($file) && is_readable($file))) {
      $file = dirname(__FILE__).'/font/'.$file;
    }
    if (file_exists($file) && is_file($file) && is_readable($file)) {
      $style = strtoupper($style);
	    if ($style == 'IB') {
    		$style = 'BI';
      }
	    $fontKey = $family.$style;
	    if (isset($this->fonts[$fontKey])) {
		    $this->Error('Font already added: '.$family.' '.$style);
      }

      $path = dirname($file);
	    include($file);
      $file = $path.'/'.$file;

	    if (!isset($name)) {
    		$this->Error('Could not include font definition file');
      }
	    $i = count($this->fonts) + 1;
    	$this->fonts[$fontKey] = array (
        'i'    => $i,
        'type' => $type,
        'name' => $name,
        'desc' => $desc,
        'up'   => $up,
        'ut'   => $ut,
        'cw'   => $cw,
        //'enc'  => $enc,
        'file' => $file,
        'ctg' => $path.'/'.$ctg
      );
	    if ($diff) {
		    //Search existing encodings
		    $d = 0;
	    	$nb = count($this->diffs);
		    for($i = 1; $i <= $nb; $i++) {
			    if($this->diffs[$i]==$diff) {
				   $d=$i;
			    	break;
    			}
    		}
    		if ($d == 0) {
    			$d = $nb + 1;
		    	$this->diffs[$d] = $diff;
        }
		    $this->fonts[$fontKey]['diff']=$d;
      }
  	  if ($type=='TrueTypeUnicode') {
  		  $this->FontFiles[$file] = array('length1' => $originalsize);
    	} else {
  	  	$this->FontFiles[$file] = array('length1' => $size1, 'length2' => $size2);
  	  }
    } else {
      $this->Error('Font file not found: '.$file);
    }
  }
  
  /**
  * begin a new page or select an existing one
  *
  * @param string $orientation
  * @access public
  */
  function _beginpage($orientation) {
  	$this->page++;
    if (!isset($this->pages[$this->page])) {
    	$this->pages[$this->page] = '';
    }
  	$this->state = 2;
  	$this->x = $this->lMargin;
  	$this->y = $this->tMargin;
  	$this->FontFamily = '';
  	//Page orientation
  	if (!$orientation) {
  		$orientation = $this->DefOrientation;
  	} else {
  		$orientation = strtoupper($orientation{0});
  		if($orientation != $this->DefOrientation) {
  			$this->OrientationChanges[$this->page] = true;
      }
  	}
  	if ($orientation != $this->CurOrientation) {
  		//Change orientation
  		if($orientation == 'P') {
  			$this->wPt = $this->fwPt;
  			$this->hPt = $this->fhPt;
  			$this->w = $this->fw;
  			$this->h = $this->fh;
  		}	else {
  			$this->wPt = $this->fhPt;
  			$this->hPt = $this->fwPt;
  			$this->w = $this->fh;
  			$this->h = $this->fw;
  		}
  		$this->PageBreakTrigger = $this->h - $this->bMargin;
  		$this->CurOrientation = $orientation;
  	}
  }

 /**
 * add a page
 *
 * @param string $orientation optional, default value 'P'
 * @param string $pageName page configuration name, default value NULL
 * @param boolean $resetStyle optional, default value FALSE
 * @access public
 */
	function addPage($orientation = 'P', $pageName = NULL, $resetStyle = FALSE) {
		parent::addPage($orientation);
    if (isset($pageName)) {
      $this->confObj->pdfInitPage($this, $pageName, $resetStyle);
    } else {
      $this->confObj->pdfInitPage($this, $this->currentPageTemplate, $resetStyle);
    }
    $this->inColumn = FALSE;  
    $this->inParagraph = FALSE;
    $this->inText = FALSE;
	}

  /**
  * set page style
  *
  * @param array $pageStyle
  * @access public
  */
  function setPageStyle($pageStyle) {
    $this->resetTextStatus();
    $this->setFont($pageStyle['font']['family'], '', $pageStyle['font']['size']);
    $this->setAttributeStyles($pageStyle, TRUE);
  }

  /**
  * get current status - buffer, styles, ...
  *
  * @access public
  * @return array
  */
  function getTextStatus() {
    return array(
      'fontFamily' => $this->FontFamily,
      'fontStyle' => $this->FontStyle,
      'fontSizePt' => $this->FontSizePt,
      'style' => $this->currentStyle,
      'elementBuffer' => $this->elementBuffer,
      'elementMode' => $this->elementMode,
    );
  }

  /**
  * set current status - buffer, styles, ...
  *
  * @param array $status
  * @access public
  */
  function setTextStatus($status) {
    $this->setFont($status['fontFamily'], $status['fontStyle'], $status['fontSizePt']);
    $this->elementBuffer = $status['elementBuffer'];
    $this->currentStyle = $status['style'];
    $this->elementMode = $status['elementMode'];
  }

  /**
  * reset current status to defaults - buffer, styles, ...
  *
  * @access public
  */
  function resetTextStatus() {
    $this->elementBuffer = array();
    $this->currentStyle = $this->defaultStyle;
    $this->elementMode = 'text';
  }

  /**
  * page or column break
  *
  * @access public
  * @return boolean new page
  */
  function acceptPageBreak() {
    if ($this->confObj->pdfInitColumn($this, $this->currentPageTemplate, $this->currentColumn + 1)) {
      $this->inColumn = FALSE;  
      $this->inParagraph = FALSE;
      $this->inText = FALSE;
      return FALSE;
    } elseif ($this->AutoPageBreak) {
      //Automatic page break
      //get current "indent"
  		$offset = $this->x - $this->lMargin;

      //close page
	  	$ws = $this->ws;
	    $k = $this->k;
      if ($ws > 0) {
			  $this->ws = 0;
			  $this->_out('0 Tw');
		  }

      $this->AddPage($this->CurOrientation, $this->currentPageTemplate, FALSE);
      //set current "indent"
		  $this->x += $offset;

      //open page
		  if ($ws > 0) {
			  $this->ws = $ws;
			  $this->_out(sprintf('%.3f Tw', $ws * $k));
		  }
      $this->inColumn = FALSE;
      $this->inParagraph = FALSE;
      $this->inText = FALSE;
      return TRUE;
    }
    return FALSE;
  }
  
  /**
  * check for page break trigger
  *
  * @param $nextElementHeight
  * @access public
  * @return boolean
  */
  function checkPageBreakTrigger($nextElementHeight) {
    $y = $this->GetY() + $nextElementHeight;
    if ($this->elementMode == 'tableoutput') {
      $y += $this->tableBuffer->padding;
    }
    return ((!$this->InFooter) && $y >= $this->PageBreakTrigger);
  }
  
  /**
  * select page an column
  *
  * @param integer $page
  * @param integer $column
  * @access public
  * @return
  */
  function gotoPageColumn($page, $column) {
    if ($page != $this->page || $column != $this->currentColumn) {
      $this->page = $page;
      $this->currentColumn = $column;
      $this->confObj->pdfInitColumn($this, $this->currentPageTemplate, $this->currentColumn);
    }
  }

  /**
  * calculate line height for a given count of lines using current font height
  *
  * @param integer $lines line count
  * @access public
  * @return float line height in mm
  */
  function getLineHeight($lines = 1) {
    return (($this->FontSizePt + $this->lineSpacePt) * $lines * $this->dtpPoint);
  }

  /**
  * calculate text height
  *
  * @access public
  * @return float line height in mm
  */
  function getTextHeight() {
    return ($this->FontSizePt * $this->dtpPoint);
  }

  /**
  * get line width
  *
  * @access public
  * @return float line width
  */
  function getLineWidth() {
    if (isset($this->currentTableCell) && $this->elementMode == 'tableoutput') {
      return ($this->currentTableCell->getWidth() - ($this->tableBuffer->padding * 2));    
    } else {
      return $this->getColumnWidth();
    }
  }
  
  /**
  * get column width
  *
  * @access public
  * @return float
  */
  function getColumnWidth() {
    return ($this->w - $this->lMargin - $this->rMargin);
  }
  
  /**
  * return line left start
  *
  * @access public
  * @return float
  */
  function getLineLeft() {
    if (isset($this->currentTableCell) && $this->elementMode == 'tableoutput') {
      return $this->getColumnLeft() + $this->currentTableCell->getLeft() + $this->tableBuffer->padding;    
    } else {
      return $this->getColumnLeft();
    }   
  }
  
  /**
  * get column left
  *
  * @access public
  * @return float
  */
  function getColumnLeft() {
    return $this->lMargin;
  }

  /**
  * calculate image height for an image and add image infos
  *
  * @param string $file image file name
  * @param float $width image width
  * @access public
  * @return array width, height
  */
  function calcImageSize($file, $maxWidth) {
	  //add an image to the list and return height
	  if(!isset($this->images[$file])) {
		  $mqr=get_magic_quotes_runtime();
		  set_magic_quotes_runtime(0);

		  //First use of image, get info
      list(,,$imgType) = @getimagesize($file);
		  if (!($imgType == 2 || $imgType == 3)) {
			  $this->Error('Image is not JPEG or PNG: '.$file);
      } elseif ($imgType == 2) {
        $type = 'jpg';
			  $info = $this->_parsejpg($file);
      } elseif ($imgType == 3) {
        $type = 'png';
			  $info = $this->_parsepng($file);
      }
  		set_magic_quotes_runtime($mqr);
  		$info['i'] = count($this->images) + 1;
  		$this->images[$file] = $info;
  	}	else {
  		$info=$this->images[$file];
    }
    $imgWidth  = $info['w'] / $this->mediaK;
    $imgHeight = $info['h'] / $this->mediaK;
    if ($maxWidth < $imgWidth) {
      return array(
        'width'  => $maxWidth,
        'height' => $imgHeight / ($imgWidth / $maxWidth)
      );
    } else {
      return array(
        'width'  => $imgWidth,
        'height' => $imgHeight
      );
    }
  }

  /**
  * Output some text
  *
  * @param string $str
  * @access public
  */
  function writeText($str) {
    if ($this->elementMode == 'table') {
      $this->addTableContentText($str);
    } else {
      $this->beginParagraph();
      //add text to buffer
      array_push($this->elementBuffer, array($str, $this->getCurrentElementStyle()));
    }
  }
  
  /**
  * write element buffer
  *
  * @access public
  */
  function writeBuffer() {
    if (count($this->elementBuffer) <= 0) {
      //nothing to do
      return 0;
    }    
    $paraStyle = $this->elementBuffer[0][1];
    $this->activateCurrentStyle($paraStyle);
    $para = $this->getParagraphSize($paraStyle);
    
    if ($this->checkPageBreakTrigger($para['lineheight'])) {
      //column or page break
      $this->acceptPageBreak();
      $para = $this->getParagraphSize($paraStyle);
    }
    
    $this->writeListBullet($paraStyle, $para);
    
    $lineBuffer = array();
    $lineLength = 0;
    $lineStyles = array();
    foreach(array_keys($this->elementBuffer) as $elementIdx) {
      $style = &$this->elementBuffer[$elementIdx][1];
      $this->activateCurrentStyle($style);
      $words = explode(' ', $this->elementBuffer[$elementIdx][0]);
      foreach($words as $wordIdx=>$word) {
        $wordLength = $this->GetStringWidth($word);
        if (count($lineBuffer) > 0) {
          $wordSpace = $this->GetStringWidth(' ');
        } else {
          $wordSpace = 0;
        }
        if ($lineLength + $wordLength + $wordSpace > $para['textwidth']) {
          //output line
          if ($this->writeBufferLine($para, $lineLength, $lineBuffer, $lineStyles, $paraStyle['align'])) {
            //enough space for another line on this page?
            if ($this->checkPageBreakTrigger($para['lineheight'] * 2)) {
              //column or page break
              $this->acceptPageBreak();
              $this->beginParagraph();
              $para = $this->getParagraphSize($paraStyle);
            } else {
              //enough space - go to next line
              $this->Ln($para['lineheight']);
            }
            $wordSpace = 0;
          }
          //add word to next line
          $lineStyles[count($lineBuffer)] = &$style;
        } else {
          //first word in element - remember style
          if ($wordIdx == 0) {
            $lineStyles[count($lineBuffer)] = &$style;
          }
        }
        //add content to line variables
        $lineBuffer[count($lineBuffer)] = $word;
        $lineLength += $wordLength + $wordSpace;
      }
    }
    //check line buffer - this is the last line of this paragraph
    if ($this->writeBufferLine($para, $lineLength, $lineBuffer, $lineStyles, $paraStyle['align'], TRUE)) {      
      //enough space for another line on this page?
      if ($this->checkPageBreakTrigger($para['lineheight'])) {
        //column or page break
        $this->acceptPageBreak();
      }
    }
    //clear element buffer
    $this->elementBuffer = array();
  }
  
  /**
  * write the current line buffer to pdf
  *
  * @param array $para paragraph measures
  * @param float $lineLength
  * @param array &$lineBuffer
  * @param array &$lineStyles
  * @param string $align
  * @param boolean $lastLine optional, default value FALSE
  * @access public
  * @return boolean linefeed
  */
  function writeBufferLine($para, &$lineLength, &$lineBuffer, &$lineStyles, $align, $lastLine = FALSE) {
    if (count($lineBuffer) > 0) {   
      $this->inText = TRUE;
      $this->inColumn = TRUE; 
      $this->inParagraph = TRUE; 
      
      if ($lastLine && $align == 'justify') {
         $align = 'left';
      }      
      // paint background
      if (isset($lineStyles[0])) {
        //style changes
        $this->activateCurrentStyle($lineStyles[0]);
      }
      if (!empty($para['bgcolor'])) {
        $this->Rect($para['left'], $this->GetY(), $para['width'], $para['lineheight'], 'F');
      }
      
      switch($align) {
      case 'justify' : 
        $x = $para['left'] + $para['indent'];
        $y = $this->GetY() + $para['textheight'];
        //calc space size
        $spaceIncWidth = 0;
        if (count($lineBuffer) > 1) {
          $spaceIncWidth = ($para['textwidth'] - $lineLength) / (count($lineBuffer) - 1);
          if ($spaceIncWidth < 0) {
            $spaceIncWidth = 0;
          }
        }
        foreach($lineBuffer as $wordIdx=>$word) {
          if (isset($lineStyles[$wordIdx])) {
            //style changes
            $this->activateCurrentStyle($lineStyles[$wordIdx]);
          }
          $this->Text($x, $y, $word);
          //end of word, add a space
          $x += $this->GetStringWidth($word) + $this->GetStringWidth(' ') + $spaceIncWidth;
        }              
        break;
      case 'center' :
        //get left start
        $x = $para['left'] + (($para['width'] - $lineLength - $para['indent']) / 2) + $para['indent'];
        $y = $this->GetY() + $para['textheight'];
        foreach($lineBuffer as $wordIdx=>$word) {
          if (isset($lineStyles[$wordIdx])) {
            //style changes
            $this->activateCurrentStyle($lineStyles[$wordIdx]);
          }
          $this->Text($x, $y, $word);
          //end of word, add a space
          $x += $this->GetStringWidth($word) + $this->GetStringWidth(' ');
        }
        break;
      case 'right' :
        //get left start
        $x = $para['left'] + $para['textwidth'] - $lineLength;
        $y = $this->GetY() + $para['textheight'];
        foreach($lineBuffer as $wordIdx=>$lineWord) {
          if (isset($lineStyles[$wordIdx])) {
            //style changes
            $this->activateCurrentStyle($lineStyles[$wordIdx]);
          }
          $this->Text($x, $y, $word);
          //end of word, add a space
          $x += $this->GetStringWidth($word) + $this->GetStringWidth(' ');
        }
        break;
      case 'left' :
      default :
        //get left start
        $x = $para['left'] + $para['indent'];
        $y = $this->GetY() + $para['textheight'];
        foreach($lineBuffer as $wordIdx=>$word) {
          if (isset($lineStyles[$wordIdx])) {
            //style changes
            $this->activateCurrentStyle($lineStyles[$wordIdx]);
          }
          $this->Text($x, $y, $word);
          //end of word, add a space
          $x += $this->GetStringWidth($word) + $this->GetStringWidth(' ');
        }
        break;
      }
      $result = TRUE;
    } else {
      $result = FALSE;
    }
    //clear line buffer variables
    $lineBuffer = array();
    $lineLength = 0;
    $lineStyles = array();
    return $result;
  }
  
  /**
  * write list bullet to pdf document
  *
  * @param array &$paraStyle
  * @param array $para
  * @access public
  */
  function writeListBullet(&$paraStyle, $para) {
    if ($this->getListIndent($paraStyle) > 0) {
      $font = array(
        'family' => $this->FontFamily,
        'style' => $this->FontStyle,
        'sizePt' => $this->FontSizePt
      );
      $this->setFont($this->listBulletFont, '', $this->listBulletSize);      
      if ($paraStyle['align'] == 'right') {
        $x = $para['left'] + $para['width'] - $para['indent'];
        $bulletIndent = ($para['indent'] - $this->GetStringWidth($this->listBullet)) / 2;
        $this->Text($x + $bulletIndent, $this->GetY() + $para['textheight'], $this->listBullet);
      } else {
        $x = $para['left'];
        $bulletIndent = ($para['indent'] - $this->GetStringWidth($this->listBullet)) / 2;
        $this->Text($x + $bulletIndent, $this->GetY() + $para['textheight'], $this->listBullet);      
      }
      $this->setFont($font['family'], $font['style'], $font['sizePt']);      
    }
  }
  
  /**
  * get paragraph size
  *
  * @param array &$paraStyle
  * @access public
  * @return array
  */
  function getParagraphSize(&$paraStyle) {
    $result = array(
      'width' => $this->getLineWidth(),
      'lineheight' => $this->getLineHeight(),
      'textheight' => $this->getTextHeight(),
      'indent' => $this->getListIndent($paraStyle),
      'left' => $this->getLineLeft(),
      'bgcolor' => @$paraStyle['bgcolor']
    );  
    $result['textwidth'] = $result['width'] - $result['indent'];
    return $result;
  }
  
  /**
  * begin a new paragraph
  *
  * @access public
  */
  function beginParagraph() {
    if (!$this->inParagraph) {
      if ($this->inColumn && (!$this->inImage)) {
        $this->Ln($this->getLineHeight());
      }
      $this->inParagraph = TRUE;
      $this->inColumn = TRUE;
      $this->inImage = FALSE;
      $this->inText = FALSE;
    }
  }
  
  /**
  * end current paragraph
  *
  * @access public
  * @return
  */
  function endParagraph() {
    if (isset($this->inParagraph) && $this->inParagraph) {
      if (!$this->inImage) {
        $this->Ln($this->getLineHeight());
      }
      $this->inParagraph = FALSE;
      $this->inImage = FALSE;
      $this->inText = FALSE;
    }
  }

  /**
  * Output an image
  *
  * @param string $fileName
  * @access public
  */
  function writeImage($fileName) {
    if ($this->elementMode == 'table') {
      $imageSize = $this->calcImageSize($fileName, $this->getLineWidth());
      $this->addTableContentImage($fileName, $imageSize['width'], $imageSize['height']);      
    } else {
      $this->writeBuffer();
      $imageSize = $this->calcImageSize($fileName, $this->getLineWidth());
      if ($this->inParagraph && $this->inColumn) {
        $this->endParagraph();
      }
      if ($this->checkPageBreakTrigger($imageSize['height'])) {
        $this->AcceptPageBreak();
      }
      $this->activateCurrentStyle($this->getCurrentElementStyle());
      
      //the width could change at a column or page break
      $lineWidth = $this->getLineWidth();
      $imageSize = $this->calcImageSize($fileName, $lineWidth);
      $lineHeight = $this->getLineHeight();
      $imageBottom = $this->GetY() + (ceil($imageSize['height'] / $lineHeight) * $lineHeight);
      
      $this->Rect($this->getLineLeft(), $this->GetY(), $this->getLineWidth(), $imageBottom - $this->GetY(), 'F');
      $this->Image($fileName, $this->getLineLeft(), $this->GetY(), $imageSize['width'], $imageSize['height']);
      
      $this->SetY($imageBottom);
      
      if ($this->checkPageBreakTrigger($imageSize['height'])) {
        $this->AcceptPageBreak();
      }
      $this->Rect($this->getLineLeft(), $this->GetY(), $this->getLineWidth(), $lineHeight, 'F');
      
      $this->inImage = TRUE;
      $this->inColumn = TRUE;
      $this->inParagraph = TRUE;
    }
  }
  
  /**
  * write a table
  *
  * @access public
  */
  function writeTable() {
    if (isset($this->tableBuffer)) {
      $this->endParagraph();
      $this->beginParagraph();
      $rowCount = $this->tableBuffer->rowCount();
      if ($rowCount > 0) {
        $row = &$this->tableBuffer->getRowByIndex(0);
        $row->setLeftTop($this->page, $this->currentColumn, $this->GetX(), $this->GetY());
      }
      for ($i = 0; $i < $rowCount; $i++) {
        $this->writeTableRow($i);
      } 
      $this->inParagraph = TRUE;
      $this->endParagraph();
      $this->beginParagraph();
    }
  }
  
  /**
  * write a table row
  *
  * @param integer $rowIdx
  * @access public
  */
  function writeTableRow($rowIdx) {
    if (isset($this->tableBuffer)) {
      if ($row = &$this->tableBuffer->getRowByIndex($rowIdx)) {
        $leftTop = $row->getLeftTop();
        if ($leftTop['y'] + ceil($row->minHeight) > $this->PageBreakTrigger && !$this->InFooter) {
          $this->AcceptPageBreak();
          $row->setLeftTop($this->page, $this->currentColumn, $this->GetX(), $this->GetY());
          $leftTop = $row->getLeftTop();
        }
        
        $this->gotoPageColumn($leftTop['page'], $leftTop['col']);
        $this->writeTableBorderH($leftTop['x'], $leftTop['y'], $this->tableBuffer->width);
        
        $cellCount = $row->cellCount();
        for ($i = 0; $i < $cellCount; $i++) {
          $this->writeTableCell($row->getCellByIndex($i), $leftTop);
          $row->setLeftBottom($this->page, $this->currentColumn, $this->GetX(), $this->GetY());
        }
        
        $leftBottom = $row->getLeftBottom();
        if ($leftTop['page'] == $leftBottom['page'] && $leftTop['col'] == $leftBottom['col']) {
          $y = $leftTop['y'];
          $rowHeight = $leftBottom['y'] - $leftTop['y'];
          for ($i = 0; $i < $cellCount; $i++) {
            $cell = &$row->getCellByIndex($i);
            $left = $cell->getLeft();
            if ($i == 0) {
              $this->writeTableBorderV($this->lMargin + $left, $y, $rowHeight);
            }
            $this->writeTableBorderV($this->lMargin + $left + $cell->getWidth(), $y, $rowHeight); 
          }
          $this->writeTableBorderH($leftBottom['x'], $leftBottom['y'], $this->tableBuffer->width); 
        } else {
          $pageColCount = $this->confObj->pdfColumnCount($this, $this->currentPageTemplate);
          for ($page = $leftBottom['page']; $page >= $leftTop['page']; $page--) {
            for ($pageCol = $pageColCount; $pageCol >= 1; $pageCol--) {
              if (($page == $leftBottom['page'] && $pageCol > $leftBottom['col']) ||
                  ($page == $leftTop['page'] && $pageCol < $leftTop['col'])) {
                continue;
              }
              $this->gotoPageColumn($page, $pageCol);   
              $bottom = $this->h - $this->bMargin;
              if ($page == $leftTop['page'] && $pageCol == $leftTop['col']) {
                //table start column
                $this->writeTableBorderH($this->lMargin, $bottom, $this->tableBuffer->width);
                $y = $leftTop['y'];
                $rowHeight = $bottom - $leftTop['y'];                
              } elseif ($page == $leftBottom['page'] && $pageCol == $leftBottom['col']) {
                //table end column
                $this->writeTableBorderH($this->lMargin, $this->tMargin, $this->tableBuffer->width);               
                $y = $this->tMargin;
                $rowHeight = $leftBottom['y'] - $this->tMargin;
              } else {
                //all columns between
                $this->writeTableBorderH($this->lMargin, $this->tMargin, $this->tableBuffer->width);
                $this->writeTableBorderH($this->lMargin, $bottom, $this->tableBuffer->width);               
                $y = $this->tMargin;
                $rowHeight = $bottom - $this->tMargin;
              }
              for ($i = 0; $i < $cellCount; $i++) {
                $cell = &$row->getCellByIndex($i);
                $left = $cell->getLeft();
                if ($i == 0) {
                  $this->writeTableBorderV($this->lMargin + $left, $y, $rowHeight);
                }
                $this->writeTableBorderV($this->lMargin + $left + $cell->getWidth(), $y, $rowHeight); 
              }
            }
          }
        }
      }
    }  
  }
  
  /**
  * write a table cell
  *
  * @param integer &$cell
  * @param array $rowLeftTop
  * @access public
  */
  function writeTableCell(&$cell, $rowLeftTop) {
    if (isset($cell)) {
      if ($this->page != $rowLeftTop['page'] || $this->currentColumn != $rowLeftTop['col']) {
        $this->gotoPageColumn($rowLeftTop['page'], $rowLeftTop['col']);
      }
      $this->setXY($rowLeftTop['x'] + $cell->getLeft(), $rowLeftTop['y'] + $this->tableBuffer->padding);
      $this->currentTableCell = &$cell;
      $this->elementMode = 'tableoutput';
      $this->inColumn = FALSE;
      $this->inParagraph = FALSE;
      $this->inText = FALSE;
      $this->setAttributeStyles($cell->getAttributes(), TRUE);
      $cell->outputData($this);
      $this->setAttributeStyles($cell->getAttributes(), FALSE);
      $this->writeBuffer();
      $this->endParagraph();
      $this->elementMode = 'text';
      $null = NULL;
      $this->currentTableCell = &$null;
    }
  }
  
  /**
  * paint a horizontal table border  
  *
  * @param float $x
  * @param float $y
  * @param float $width
  * @access public
  */
  function writeTableBorderH($x, $y, $width) {
    $this->Line($x, $y, $x + $width, $y);
  }
  
  /**
  * paint a vertical table border  
  *
  * @param float $x
  * @param float $y
  * @param float $width
  * @access public
  */
  function writeTableBorderV($x, $y, $height) {
    $this->Line($x, $y, $x, $y + $height);
  }
  
  /**
  * line break
  *
  * @param $height
  * @access public
  * @return
  */
  function Ln($height) {
    parent::Ln($height);
  }

  /**
  * open an element
  *
  * @param array $attr
  * @access public
  */
  function openElement($attr) {
    if (isset($attr['align']) && in_array($attr['align'], array('right', 'justify', 'left'))) {
      $this->currentAlign = $attr['align'];
    }
    $this->inColumn = FALSE;
    $this->inParagraph = FALSE;
    $this->inText = FALSE;
  }

  /**
  * close/end the current element
  *
  * @param array $attr
  * @access public
  */
  function closeElement($attr) {
    $this->writeBuffer();
    $this->inParagraph = FALSE;
  }

  /**
  * open a section 
  *
  * @param array $attr
  * @access public
  */
  function openSection($attr) {
    if (@$attr['break-before'] == 'yes' || $this->breakBeforeNextSection) {
      if (isset($attr['page']) && $attr['page'] != '') {
        $this->confObj->pdfAddPage($this, $attr['page'], TRUE);
      } else {
        $this->confObj->pdfAddPage($this, $this->currentPageTemplate, TRUE);
      }
      $this->breakBeforeNextSection = FALSE;
    }
    $this->setAttributeStyles($attr, TRUE);
    $this->beginParagraph();
  }

  /**
  * close current section
  *
  * @param array $attr
  * @access public
  */
  function closeSection($attr) {
    $this->writeBuffer();
    $this->endParagraph();
    
    $this->setAttributeStyles($attr, FALSE);
    
    if (@$attr['break-after'] == 'yes') {
      $this->breakBeforeNextSection = TRUE;
    }
  }

  /**
  * open a tag
  *
  * @param string $tag
  * @param array $attr
  * @access public
  */
  function openTag($tag, $attr) {
    switch ($tag) {
    case 'table' : 
      $this->openTable($attr);
      return;
    case 'tr' : 
      $this->openTableRow($attr);
      return;
    case 'th' : 
      $this->openTableCell($attr, TRUE);
      return;
    case 'td' : 
      $this->openTableCell($attr);
      return;
    }
    if ($this->elementMode == 'table') {
      $this->openTableContentTag($tag, $attr);
    } else {
      switch ($tag) {
      case 'h1' :
      case 'h2' :
      case 'h3' :
      case 'h4' :
      case 'h5' :
        $this->writeBuffer();
        $this->setStyle('b', TRUE);
        $this->setAttributeStyles($attr, TRUE);
        $this->beginParagraph();
        break;
      case 'p' :
        $this->writeBuffer();
        $this->setAttributeStyles($attr, TRUE);
        if ($this->inText) {
          $this->endParagraph();
        }
        $this->beginParagraph();
        break;
      case 'ul' :
        $this->writeBuffer();
        if ($this->inText) {
          $this->endParagraph();
        }
        $this->setStyle('list', TRUE);
        $this->setAttributeStyles($attr, TRUE);
        $this->beginParagraph();
        break;
      case 'li' :
        //nothing special at list item start
        break;
      case 'b' :
        $this->setStyle('b', TRUE);
        break;
      case 'i' :
        $this->setStyle('i', TRUE);
        break;
      case 'u' :
        $this->setStyle('u', TRUE);
        break;
      case 'image' :
        $this->writeBuffer();
        if ($this->elementMode == 'tableoutput' && isset($attr['src']) && 
            file_exists($attr['src']) && is_file($attr['src']) && is_readable($attr['src'])) {
          $this->writeImage($attr['src']);
        }
        break;
      case 'br' :
        $this->writeBuffer();
        if ($this->inColumn) {
          $this->Ln($this->getLineHeight());
          if ($this->checkPagebreakTrigger($this->getLineHeight())) {
            $this->acceptPageBreak();
          }
          $this->Rect($this->getLineLeft(), $this->GetY(), $this->getLineWidth(), $this->getLineHeight(), 'F');
        }
        break;
      }
    }
  }

  /**
  * close current tag
  *
  * @param string $tag
  * @param array $attr
  * @access public
  */
  function closeTag($tag, $attr) {
    switch ($tag) {
    case 'table' : 
      $this->closeTable();
      return;
    case 'tr' : 
      $this->closeTableRow();
      return;
    case 'th' : 
      $this->closeTableCell(TRUE);
      return;
    case 'td' : 
      $this->closeTableCell();
      return;
    }
    if ($this->elementMode == 'table') {
      $this->closeTableContentTag($tag, $attr);
    } else {
      switch($tag) {
      case 'h1' :
      case 'h2' :
      case 'h3' :
      case 'h4' :
      case 'h5' :
        $this->writeBuffer();
        $this->endParagraph();
        $this->setStyle('b', FALSE);
        $this->setAttributeStyles($attr, FALSE);
        break;
      case 'p' :
        $this->writeBuffer();
        $this->endParagraph();
        $this->setAttributeStyles($attr, FALSE);
        break;
      case 'ul' :
        $this->writeBuffer();
        //$this->endParagraph();
        $this->setStyle('list', FALSE);
        $this->setAttributeStyles($attr, FALSE);
        break;
      case 'li' :
        $this->writeBuffer();
        $this->Ln($this->getLineHeight());
        break;
      case 'b' :
        $this->setStyle('b', FALSE);
        break;
      case 'i' :
        $this->setStyle('i', FALSE);
        break;
      case 'u' :
        $this->setStyle('u', FALSE);
        break;
      }
    }
  }
  
  /**
  * open a table (create table buffer)
  *
  * @param array $attr
  * @access public
  * @return
  */
  function openTable($attr) {
    $this->writeBuffer();
    include_once(dirname(__FILE__).'/papaya_pdf_table.php');
    if (!isset($this->tableBuffer)) {
      $this->tableBuffer = &new papaya_pdf_table($this, $attr);
    }
  }
  
  /**
  * open a table row (write to table buffer)
  *
  * @param array $attr
  * @access public
  * @return
  */
  function openTableRow($attr) {
    $this->writeBuffer();  
    if (isset($this->tableBuffer)) {
      $this->tableBuffer->addRow($attr);
    }
  }
  
  /**
  * open a table cell (write to table buffer)
  *
  * @param array $attr
  * @access public
  */
  function openTableCell($attr) {
    $this->writeBuffer();  
    if (isset($this->tableBuffer)) {
      $this->tableBuffer->addCell($attr);  
      $this->elementMode = 'table';  
    }
  }
  
  /**
  * add table cell text content  (write to table buffer)
  *
  * @param string $str text
  * @access public
  * @return
  */
  function addTableContentText($str) {
    if (isset($this->tableBuffer)) {
      $this->tableBuffer->addCellContent($str);  
    }  
  }
  
  /**
  * add table cell image content (write to table buffer)
  *
  * @param string $fileName
  * @param float $width
  * @param float $height
  * @access public
  */
  function addTableContentImage($fileName, $width, $height) {
    if (isset($this->tableBuffer)) {
      $this->tableBuffer->addCellContentImage($fileName, $width, $height);  
    }  
  }
  
  /**
  * open a tag inside a table cell (write to table buffer)
  *
  * @param string $tag
  * @param array $attr
  * @access public
  * @return
  */
  function openTableContentTag($tag, $attr) {
    if (isset($this->tableBuffer)) {
      $this->tableBuffer->addCellContentTag($tag, $attr); 
    }
  }
  
  /**
  * close a tag inside a table cell (write to table buffer)
  *
  * @param $tag
  * @param $attr
  * @access public
  * @return
  */
  function closeTableContentTag($tag, $attr) {
    if (isset($this->tableBuffer)) {
      $this->tableBuffer->endCellContentTag($tag, $attr);  
    }  
  }
  
  /**
  * close current table cell (write to table buffer)
  *
  * @access public
  */
  function closeTableCell() {
    if (isset($this->tableBuffer)) {
      $this->elementBuffer = array();
      $this->tableBuffer->endCell();
      $this->elementMode = 'text';
    }
  }
  
  /**
  * close current table row (write to table buffer)
  *
  * @access public
  * @return
  */
  function closeTableRow() {
    $this->writeBuffer();
    if (isset($this->tableBuffer)) {
      //close cell if exists
      $this->closeTableCell();
      //close row
      $this->tableBuffer->endRow();
    }  
  }
  
  /**
  * close table, calculate widths and trigger output, reset table buffer
  *
  * @access public
  */
  function closeTable() {
    $this->writeBuffer();
    if (isset($this->tableBuffer)) {
      //close row if exists
      $this->closeTableRow();
      $this->tableBuffer->calcColumnWidths($this->getColumnWidth());
      $this->writeTable();
      unset($this->tableBuffer);
    }
  }

  /**
  * set integer text style
  *
  * @param string $tag
  * @param boolean $enable
  * @access public
  */
  function setStyle($tag, $enable) {
    switch ($tag) {
    case 'b' :
      $this->currentStyle['bold'] += ($enable ? 1 : -1);
      break;
    case 'i' :
      $this->currentStyle['italic'] += ($enable ? 1 : -1);
      break;
    case 'u' :
      $this->currentStyle['underline'] += ($enable ? 1 : -1);
      break;
    case 'list' :
      $this->currentStyle['list'] += ($enable ? 1 : -1);
      break;
    }
  }
  
  /**
  * set string text style
  *
  * @param $attr
  * @param $enable
  * @access public
  * @return
  */
  function setAttributeStyles($attr, $enable) {
    $attrNames = array('align', 'bgcolor', 'fgcolor');
    foreach ($attrNames as $attrName) {
      if (isset($attr[$attrName])) {
        if ($enable) {
          $this->addAttributeStyle($attrName, $attr[$attrName]);
        } else {
          $this->removeAttributeStyle($attrName, $attr[$attrName]);        
        }
      }
    }
  }
  
  /**
  * add string text style
  *
  * @param string $attr
  * @param string $value
  * @access public
  */
  function addAttributeStyle($attr, $value) {
    if (isset($value) && $value != '') {
      switch ($attr) {
      case 'align' :
        if (in_array($value, array('left', 'right', 'center', 'justify'))) {
          if (is_array($this->currentStyle['align'])) {
            array_unshift($this->currentStyle['align'], (string)$value);
          } else {
            $this->currentStyle['align'] = array((string)$value);
          }
        }
        break;
      case 'bgcolor' :
        if (is_array($this->currentStyle['bgcolor'])) {
          array_unshift($this->currentStyle['bgcolor'], (string)$value);
        } else {
          $this->currentStyle['bgcolor'] = array((string)$value);
        }
        break;
      case 'fgcolor' :
        if (is_array($this->currentStyle['fgcolor'])) {
          array_unshift($this->currentStyle['fgcolor'], (string)$value);
        } else {
          $this->currentStyle['fgcolor'] = array((string)$value);
        }
        break;  
      }
    }
  }
  
  /**
  * remove string text style
  *
  * @param string $attr
  * @param string $value
  * @access public
  */
  function removeAttributeStyle($attr, $value) {
    if (isset($value) && $value != '') {
      switch ($attr) {
      case 'align' :
        if (in_array($value, array('left', 'right', 'center', 'justify'))) {
          array_shift($this->currentStyle['align']);
          if (count($this->currentStyle['align']) <= 0) {
            $this->currentStyle['align'] = $this->defaultStyle['align'];
          }
        }
        break;
      case 'bgcolor' :
        array_shift($this->currentStyle['bgcolor']);
        if (count($this->currentStyle['bgcolor']) <= 0) {
          $this->currentStyle['bgcolor'] = $this->defaultStyle['bgcolor'];
        }
        break;
      case 'fgcolor' :
        array_shift($this->currentStyle['fgcolor']);  
        if (count($this->currentStyle['bgcolor']) <= 0) {
          $this->currentStyle['bgcolor'] = $this->defaultStyle['bgcolor'];
        }    
        break;
      }
    }
  }
  
  /**
  * get current text style
  *
  * @access public
  * @return array
  */
  function getCurrentElementStyle() {
    $result = $this->defaultStyle;
    if (isset($this->currentStyle)) {
      foreach ($this->defaultStyle as $styleItem => $styleDefault) {
        $result[$styleItem] = $styleDefault;
        if (isset($this->currentStyle[$styleItem])) {
          if (is_array($this->currentStyle[$styleItem]) && ($this->currentStyle[$styleItem]) > 0) {
            $result[$styleItem] = reset($this->currentStyle[$styleItem]);
          } elseif (is_string($this->currentStyle[$styleItem]) || is_int($this->currentStyle[$styleItem])) {
            $result[$styleItem] = $this->currentStyle[$styleItem];        
          }
        }
      }
    } else {
      return $this->defaultStyle;
    }
    return $result;
  }

  /**
  * activate current style
  *
  * @param array curStyle
  * @access public
  */
  function activateCurrentStyle($curStyle) {
    $style = '';
    $style .= ($curStyle['bold'] > 0) ? 'B' : '';
    $style .= ($curStyle['italic'] > 0) ? 'I' : '';
    $style .= ($curStyle['underline'] > 0) ? 'U' : '';
    $this->SetFont('',$style);
    $this->setFGColor($curStyle['fgcolor']);
    $this->setBGColor($curStyle['bgcolor']);
  }

  /**
  * get current list indent
  *
  * @param array $curStyle
  * @access public
  */
  function getListIndent($curStyle) {
    if (isset($curStyle) && ($curStyle['list'] >= 0)) {
      return $this->indentStep * $curStyle['list'];
    }
    return 0;
  }

}
?>