<?php
/**
* Options for PDF output
* 
* @package papaya_module_pdf 
* @author Thomas Weinert <weinert@papaya-cms.com>
* @version $Id: papaya_pdf_options.php,v 1.2 2006/05/12 16:55:21 weinert Exp $
*/

/**
* Options for PDF output
*
* @package papaya_module_pdf 
* @author Thomas Weinert <weinert@papaya-cms.com>
*/
class papaya_pdf_options {

  /**
  * owner object (pdf filter)
  * @var object filter_pdf $owner
  */
  var $owner = NULL;

  /**
  * font definitions
  * @var array $fonts 
  */
  var $fonts = array();
  
  /**
  * page definitions
  * @var array $pages
  */
  var $pages = array();

  /**
  * page templates 
  */
  var $templates = array();
  
  /**
  * header data
  * @var object simple_xmlnode $headerNode
  */
  var $headerNode = NULL;
  
  /**
  * footer data
  * @var object simple_xmlnode $footerNode
  */
  var $footerNode = NULL;
  
  /**
  * default margins
  * @var array
  */
  var $defaultMargin = array('top' => 0, 'right' => 0, 'bottom' => 0, 'left' => 0);
  /**
  * default font family and size
  * @var 
  */
  var $defaultFont = array('family' => 'Times', 'size' => 10);
  /**
  * default content alignment
  * @var string
  */
  var $defaultAlign = 'left';
  
  /**
  * initialize configuration data
  *
  * @param object simple_xmlnode &$data
  * @access public
  */
  function initialize(&$owner, &$data) {
    $this->owner = &$owner;
    if (is_object($data) && $data->childCount()) {
      foreach(array_keys($data->children) as $idx) {
        if (isset($data->children[$idx]->name) && is_a($data->children[$idx], 'simple_xmlnode')) {
          switch($data->children[$idx]->name) {
          case 'templates' :
            $this->addTemplates($data->children[$idx]);
            break;
          case 'fonts' :
            $this->addFonts($data->children[$idx]);
            break;
          case 'pages' :
            $this->addPages($data->children[$idx]);          
            break;
          }
        }
      }
    }
  }
  
  /**
  * add page background templates
  *
  * @param object simple_xmlnode &$data
  * @access public
  */
  function addTemplates(&$data) {
    if (is_object($data) && $data->childCount()) {
      foreach(array_keys($data->children) as $idx) {
        if (isset($data->children[$idx]->name) && is_a($data->children[$idx], 'simple_xmlnode') &&
            $data->children[$idx]->name == 'template') {
          $this->addTemplate($data->children[$idx]);
        }
      }
    }  
  }
  
  /**
  * add a page background template
  *
  * @param object simple_xmlnode &$data
  * @access public
  */
  function addTemplate(&$data) {
    if (isset($data->attributes['name']) && trim($data->attributes['name']) != '' &&
        isset($data->attributes['file']) && file_exists(PAPAYA_PATH_PDFFILES.$data->attributes['file']) &&
        is_file(PAPAYA_PATH_PDFFILES.$data->attributes['file']) &&
        is_readable(PAPAYA_PATH_PDFFILES.$data->attributes['file'])) {
      $this->templates[$data->attributes['name']] = array(
        'file' => PAPAYA_PATH_PDFFILES.$data->attributes['file'],
        'page' => @(int)$data->attributes['page'],
        'index' => -1
      );
    }
  }
  
  /**
  * add fonts
  *
  * @param object simple_xmlnode &$data
  * @access public
  */
  function addFonts(&$data) {
    if (is_object($data) && $data->childCount()) {
      foreach(array_keys($data->children) as $idx) {
        if (isset($data->children[$idx]->name) && is_a($data->children[$idx], 'simple_xmlnode') &&
            $data->children[$idx]->name == 'font') {
          $this->addFont($data->children[$idx]);
        }
      }
    }  
  }
  
  /**
  * add font
  *
  * @param object simple_xmlnode &$data
  * @access public
  */
  function addFont(&$data) {
    $font = array();
    if (isset($data->attributes['name']) && trim($data->attributes['name']) != '' &&
        $this->addFontType($font, $data)) {
      $this->addFontType($font, $data, 'bold');
      $this->addFontType($font, $data, 'italic');
      $this->addFontType($font, $data, 'bolditalic');
      $this->fonts[$data->attributes['name']] = $font;      
    }
  }
  
  /**
  * add font style (default | bold | italic | bold italic)
  *
  * @param array &$font
  * @param object simple_xmlnode &$data
  * @param string $name style, default value 'default'
  * @access public
  * @return
  */
  function addFontType(&$font, &$data, $name = 'default') {
    if (isset($data->attributes[$name]) && file_exists(PAPAYA_PATH_PDFFILES.$data->attributes[$name]) &&
        is_file(PAPAYA_PATH_PDFFILES.$data->attributes[$name]) &&
        is_readable(PAPAYA_PATH_PDFFILES.$data->attributes[$name])) {
      $font[$name] = PAPAYA_PATH_PDFFILES.$data->attributes[$name];
      return TRUE;
    } elseif (isset($font['default']) && $name != 'default') {
      $font[$name] = $font['default'];    
    } else {
      $font[$name] = '';    
    }
    return FALSE;
  }
  
  /**
  * add page configurations
  *
  * @param object simple_xmlnode &$data
  * @access public
  */
  function addPages(&$data) {
    if (is_object($data) && $data->childCount()) {
      $this->defaultFont = $this->parseFont(@$data->attributes['font']);
      $this->defaultMargin = $this->parseMargin(@$data->attributes['margin']);
      $this->defaultAlign = $this->parseAlign(@$data->attributes['align']);
      foreach(array_keys($data->children) as $idx) {
        if (isset($data->children[$idx]->name) && is_a($data->children[$idx], 'simple_xmlnode') &&
            $data->children[$idx]->name == 'page') {
          $this->addPage($data->children[$idx]);
        }
      }
    }  
  }
  
  /**
  * add page configuration
  *
  * @param object simple_xmlnode &$data
  * @access public
  */
  function addPage(&$data) {
    if (isset($data->attributes['name']) && trim($data->attributes['name']) != '') {
      $pageName = $data->attributes['name'];
      $page = array(
        'template' => NULL,
        'orientation' => 'vertical',
        'header' => FALSE,
        'footer' => FALSE,
        'mode' => 'columns'
      );
      if (isset($data->attributes['template']) && $data->attributes['template'] != '') {
        $page['template'] = $data->attributes['template'];
      }
      if (isset($data->attributes['orientation']) && $data->attributes['orientation'] == 'horizontal') {
        $page['orientation'] = 'horizontal';
      }
      $page['margin'] = $this->parseMargin(@$data->attributes['margin']);
      $page['font'] = $this->parseFont(@$data->attributes['font']);
      $page['align'] = $this->parseAlign(@$data->attributes['align']);
      
      switch (@$data->attributes['mode']) {
      case 'absolute' :
      case 'elements' :
        $page['mode'] = 'elements';
        foreach(array_keys($data->children) as $idx) {
          if (isset($data->children[$idx]->name) && is_a($data->children[$idx], 'simple_xmlnode')) {
            switch ($data->children[$idx]->name) {
            case 'element' :
              $this->addPageElement($page, $data->children[$idx]);
              break;
            case 'header' :
              $this->addPageHeader($page, $data->children[$idx]);
              break;
            case 'footer' :
              $this->addPageFooter($page, $data->children[$idx]);
              break;
            }
          }
        }
        break;
      case 'columns' :
      case 'text' :
      default :
        $page['columns'] = array();
        foreach(array_keys($data->children) as $idx) {
          if (isset($data->children[$idx]->name) && is_a($data->children[$idx], 'simple_xmlnode')) {
            switch ($data->children[$idx]->name) {
            case 'column' :
              $this->addPageColumn($page, $data->children[$idx]);
              break;
            case 'header' :
              $this->addPageHeader($page, $data->children[$idx]);
              break;
            case 'footer' :
              $this->addPageFooter($page, $data->children[$idx]);
              break;
            }
          }
        }      
        if (count($page['columns']) == 0) {
          $column = array(
            'margin' => $page['margin']
          );
          $page['columns'][1] = $column;
        }
        break;
      }
      $this->pages[$pageName] = $page;
    }
  }  
  
  /**
  * add page element configuration
  *
  * @param array &$page
  * @param object simple_xmlnode &$data
  * @access public
  */
  function addPageElement(&$page, &$data) {
    if (isset($page) && isset($data->attributes['for']) && $data->attributes['for'] != '') {
      $elementFor = $data->attributes['for'];
      $element = array(
        'margin' => $this->parseMargin(@$data->attributes['margin'], $page['margin']),
        'font' => $this->parseFont(@$data->attributes['font'], $page['font']),
        'align' => $this->parseAlign(@$data->attributes['align'], $page['align'])
      );
      $page['elements'][$elementFor] = $element;
    }
  }
  
  /**
  * add page column configuration
  *
  * @param array &$page
  * @param object simple_xmlnode &$data
  * @access public
  */
  function addPageColumn(&$page, &$data) {
    if (isset($page)) {
      $columnIdx = count($page['columns']) + 1;
      $column = array(
        'margin' => $this->parseMargin(@$data->attributes['margin'], $page['margin'])
      );
      $page['columns'][$columnIdx] = $column;
    }  
  }
  
  /**
  * add page header configuration
  *
  * @param array &$page
  * @param object simple_xmlnode &$data
  * @access public
  */
  function addPageHeader(&$page, &$data) { 
    if (isset($page) && isset($data->attributes['margin'])) {
      $page['header'] =  array(
        'margin' => $this->parseMargin(@$data->attributes['margin'], $page['margin']),
        'font' => $this->parseFont(@$data->attributes['font'], $this->defaultFont),
        'align' => $this->parseAlign(@$data->attributes['align'], $this->defaultAlign)
      );
    } 
  }
  
  /**
  * add page footer configuration
  *
  * @param array &$page
  * @param object simple_xmlnode &$data
  * @access public
  */
  function addPageFooter(&$page, &$data) {
    if (isset($page) && isset($data->attributes['margin'])) {
      $page['footer'] =  array(
        'margin' => $this->parseMargin(@$data->attributes['margin'], $page['margin']),
        'font' => $this->parseFont(@$data->attributes['font'], $this->defaultFont),
        'align' => $this->parseAlign(@$data->attributes['align'], $this->defaultAlign)
      );
    } 
  }
  
  /**
  * parse font attribute
  *
  * @param string $str
  * @param array $default defaults, default value NULL
  * @access public
  * @return array
  */
  function parseFont($str, $default = NULL) {
    if (!isset($default)) {
      $default = $this->defaultFont;
    }
    if (preg_match('~^(.+)\s+(\d+)$~', $str, $matches)) {
      $result = array(
        'family' => $matches[1],
        'size' => (int)$matches[2],
      );
    } elseif (preg_match('~^(\d+)$~', $str, $matches)) {
      $result = array(
        'family' => $default['family'],
        'size' => (int)$matches[2],
      );    
    } elseif ($str != '') {
      $result = array(
        'family' => $str,
        'size' => $default['family'],
      );        
    } else {
      $result = $default;
    }
    return $result;
  }
  
  /**
  * parse margin attribute (works like CSS)
  *
  * @param string $str
  * @param array $default defaults, default value NULL
  * @access public
  * @return array
  */
  function parseMargin($str, $default = NULL) {
    if (preg_match('~^(\d+(\.\d+)?)\s+(\d+(\.\d+)?)\s+(\d+(\.\d+)?)\s+(\d+(\.\d+)?)$~', $str, $matches)) {
      $result = array(
        'top' => $matches[1],
        'right' => $matches[3],
        'bottom' => $matches[5],
        'left' => $matches[7],
      );
    } elseif (preg_match('~^(\d+(\.\d+)?)\s+(\d+(\.\d+)?)\s+(\d+(\.\d+)?)$~', $str, $matches)) {
      $result = array(
        'top' => $matches[1],
        'right' => $matches[3],
        'bottom' => $matches[3],
        'left' => $matches[5],
      );
    } elseif (preg_match('~^(\d+(\.\d+)?)\s+(\d+(\.\d+)?)$~', $str, $matches)) {
      $result = array(
        'top' => $matches[1],
        'right' => $matches[3],
        'bottom' => $matches[3],
        'left' => $matches[1],
      );
    } elseif (preg_match('~^(\d+(\.\d+)?)$~', $str, $matches)) {
      $result = array(
        'top' => $matches[1],
        'right' => $matches[1],
        'bottom' => $matches[1],
        'left' => $matches[1],
      );
    } elseif (isset($default)) {
      return $default;
    } else {
      return $this->defaultMargin;
    }
    return $result;
  }
  
  /**
  * parse alignment attribute
  *
  * @param string $str
  * @param array $default defaults, default value NULL
  * @access public
  * @return
  */
  function parseAlign($str, $default = NULL) {
    switch ($str) {
    case 'justify' : 
    case 'right' :
    case 'left' :
    case 'center' :
      return $str; 
    default :
      if (isset($default)) {
        return $default;
      } else {
        return $this->defaultAlign;
      }
    }
  }
  
  /**
  * remember header data
  *
  * @param object simple_xmlnode &$node
  * @access public
  */
  function setHeader(&$node) {
    if (isset($node) && is_object($node)) {
      $this->headerNode = &$node;
    } else {
      $this->headerNode = NULL;
    }
  }
  
  /**
  * remember footer data
  *
  * @param object simple_xmlnode &$node
  * @access public
  */
  function setFooter(&$node) {
    if (isset($node) && is_object($node)) {
      $this->footerNode = &$node;
    } else {
      $this->footerNode = NULL;
    }  
  }
  
  /**
  * initialize pdf document
  *
  * @param object papaya_pdf &$pdf pdf document
  * @access public
  */
  function pdfInitialize(&$pdf) {
    $lastFile = NULL;
    foreach ($this->templates as $name=>$data) {
      if ((!isset($lastFile)) || $lastFile != $data['file']) {
		    $pdf->setSourceFile($data['file']);
        $lastFile = $data['file'];
      }
		  $index = $pdf->importPage($data['page']);
      $this->templates[$name]['index'] = $index;    
    }
    foreach ($this->fonts as $name=>$data) {
		  $pdf->addFont($name, '', $data['default']);
		  $pdf->addFont($name, 'B', $data['bold']);
		  $pdf->addFont($name, 'I', $data['italic']);
		  $pdf->addFont($name, 'BI', $data['bolditalic']);
    }
  }
  
  /**
  * get a page configuration
  *
  * @param string $pageName page configuration name
  * @access public
  * @return array
  */
  function &getPage($pageName) {
    if (isset($pageName) && isset($this->pages[$pageName])) {
      $page = &$this->pages[$pageName];
    } elseif (isset($this->pages['default'])) {
      $page = &$this->pages['default'];   
    } else {
      $page = NULL;
    }
    return $page;  
  }
  
  /**
  * add a new page to pdf document
  *
  * @param object papaya_pdf &$pdf pdf document
  * @param string $pageName page configuration name
  * @param boolean $resetStyle optional, default value FALSE
  * @access public
  */
  function pdfAddPage(&$pdf, $pageName, $resetStyle = FALSE) {
    $page = &$this->getPage($pageName);
    if (isset($page)) {
      $orientation = ($page['orientation'] == 'horizontal') ? 'L' : 'P';
      $pdf->AddPage($orientation, $pageName, $resetStyle);
      $pdf->currentPageTemplate = $pageName;
    } else {
      $pdf->AddPage();
    }
  }
  
  /**
  * initialize a new pdf page
  *
  * @param object papaya_pdf &$pdf pdf document
  * @param string $pageName page configuration name
  * @param boolean $resetStyle optional, default value FALSE
  * @access public
  */
  function pdfInitPage(&$pdf, $pageName, $resetStyle = FALSE) {
    $page = &$this->getPage($pageName);
    if (isset($page)) {
      if (isset($page['template']) && isset($this->templates[$page['template']]) &&
          isset($this->templates[$page['template']]['index']) &&
          $this->templates[$page['template']]['index'] > 0) {
        $pdf->useTemplate($this->templates[$page['template']]['index']);
      }
      
      $currentStatus = $pdf->getTextStatus();
      $pdf->resetTextStatus();      
      
      $this->pdfOutputDocumentElement(&$pdf, $pageName, 'header', $this->headerNode);
      $this->pdfOutputDocumentElement(&$pdf, $pageName, 'footer', $this->footerNode);
      
      $pdf->setTextStatus($currentStatus);
      
      switch($page['mode']) {
      case 'elements' : 
        $pdf->setPageStyle($page);
        $pdf->setAutoPageBreak(FALSE);
        $pdf->columnCount = 1;
        $pdf->currentColumn = 1;
        break;
      default :
        if ($this->pdfInitColumn($pdf, $pageName)) {
          if ($resetStyle) {
            $pdf->setPageStyle($page);
          }
        }
      }
    }
  }
  
  /**
  * initialize a column in the pdf document
  *
  * @param object papaya_pdf &$pdf pdf document
  * @param string $pageName page configuration name
  * @param integer $index optional, default value 1
  * @access public
  * @return boolean
  */
  function pdfInitColumn(&$pdf, $pageName, $index = 1) {
    $page = &$this->getPage($pageName);
    if (isset($page)) {
      if (isset($page['columns'][$index])) {
        $column = &$page['columns'][$index];
        $pdf->currentColumn = $index;
        $pdf->setXY($column['margin']['left'], $column['margin']['top']);
        $pdf->setMargins($column['margin']['left'], $column['margin']['top'], $column['margin']['right']);
        $pdf->setAutoPageBreak(TRUE, $column['margin']['bottom']);
        $pdf->CurOrientation = ($page['orientation'] == 'horizontal') ? 'L' : 'P';
        return TRUE;
      }
    }  
    return FALSE;
  }
  
  /**
  * get column count
  *
  * @param object papaya_pdf &$pdf pdf document
  * @param string $pageName page configuration name
  * @access public
  * @return integer
  */
  function pdfColumnCount(&$pdf, $pageName) {
    $page = &$this->getPage($pageName);
    if (isset($page)) {
      if (isset($page['columns'])) {
        return count($page['columns']);
      }
    }  
    return 1;  
  }
  
  /**
  * initialize a elment in the pdf document
  *
  * @param object papaya_pdf &$pdf pdf document
  * @param string $pageName page configuration name
  * @param string $elementName element configuration name
  * @access public
  * @return boolean
  */
  function pdfInitElement(&$pdf, $pageName, $elementName) {
    $page = &$this->getPage($pageName);
    if (isset($page)) {
      if (isset($page['elements'][$elementName])) {
        $element = &$page['elements'][$elementName];
        $pdf->setXY($element['margin']['left'], $element['margin']['top']);
        $pdf->setMargins($element['margin']['left'], $element['margin']['top'], $element['margin']['right']);
        $pdf->setFont($element['font']['family'], '', $element['font']['size']);
        $pdf->currentStyle['align'] = $element['align'];
        $pdf->setAutoPageBreak(FALSE);
      }
      return TRUE;
    }
    return FALSE;
  }
  
  /**
  * write a element to the pdf document
  *
  * @param object papaya_pdf &$pdf pdf document
  * @param string $pageName page configuration name
  * @param string $elementName element configuration name
  * @param object simple_xmlnode &$elementData
  * @access public
  */
  function pdfOutputDocumentElement(&$pdf, $pageName, $elementName, &$elementData) {
    if (isset($elementData) && is_object($elementData)) {
      $page = &$this->getPage($pageName);
      if (isset($page) && isset($page[$elementName]) && is_array($page[$elementName])) {
        $element = &$page[$elementName];
        $pdf->setXY($element['margin']['left'], $element['margin']['top']);
        $pdf->setMargins($element['margin']['left'], $element['margin']['top'], $element['margin']['right']);
        $pdf->setFont($element['font']['family'], '', $element['font']['size']);
        $pdf->currentStyle['align'] = $element['align'];
        $pdf->setAutoPageBreak(FALSE);
        $pdf->openElement($elementData->attributes);
        $this->owner->outputDataChildren($elementData);
        $pdf->closeElement($elementData->attributes);
      }
    }
  }
}
?>