<?php 
/**
* class for a pdf table
* 
* @module papaya_pdf
* @author Thomas Weinert <weinert@papaya-cms.com>
* @version $Id: papaya_pdf_table.php,v 1.1 2006/04/27 15:37:16 weinert Exp $
*/

require_once(dirname(__FILE__).'/papaya_pdf_table_row.php');
require_once(dirname(__FILE__).'/papaya_pdf_table_cell.php');
require_once(dirname(__FILE__).'/papaya_pdf_table_col.php');

/**
* pdf table class
*/
class papaya_pdf_table {
  /**
  * array of papaya_pdf_table_row
  * @var array $rows
  */
  var $_rows = array();
  /**
  * array of columns
  * @var array
  */
  var $_cols = array();
  /**
  * current table row
  * @var object papaya_pdf_table_row
  */
  var $_currentRow = NULL;
  /**
  * current table row
  * @var object papaya_pdf_table_cell
  */
  var $_currentCell = NULL;
  
  /**
  * pdf document
  * @var object papaya_pdf
  */
  var $_pdfDocument = NULL;
  
  /**
  * table width
  * @var float;
  */
  var $width;
  
  /**
  * cell padding
  * @var float
  */
  var $padding = 1;
  
  /**
  * attrbutes
  * @var array
  */
  var $attributes = array();
  
  /**
  * constructor - remember pdf document object
  *
  * @param &$pdf
  * @access public
  */
  function __construct(&$pdf, $attr = NULL) {
    $this->_pdfDocument = &$pdf;
    if (isset($attr)) {
      $this->attributes = $attr;
    }
  }
  
  /**
  * PHP 4 constructor redirect
  *
  * @param &$pdf
  * @access public
  * @return
  */
  function papaya_pdf_table(&$pdf, $attr) {
    $this->__construct($pdf, $attr);
  }
  
  /**
  * add a new table row
  *
  * @access public
  */
  function addRow($attr) {
    $rowIdx = count($this->_rows);
    $this->_rows[$rowIdx] = &new papaya_pdf_table_row($this);
    $this->_currentRow = &$this->_rows[$rowIdx];
    $this->_currentRow->index = $rowIdx;
  }
  
  /**
  * end current row
  *
  * @access public
  */
  function endRow() {
    $null = NULL;
    $this->_currentRow = &$null;
  }

  /**
  * add a table cell
  *
  * @param boolean $headerCell new cell is header cell (optional, default value FALSE)
  * @access public
  */
  function addCell($attr, $headerCell = FALSE) {
    if (count($this->_rows) <= 0) {
      $this->addRow();
    }
    $rowIdx = count($this->_rows) - 1;
    if ($headerCell) {
      $this->_currentCell = &new papaya_pdf_table_headercell($this->_rows[$rowIdx], $attr);
    } else {
      $this->_currentCell = &new papaya_pdf_table_cell($this->_rows[$rowIdx], $attr);
    }
    if (!isset($this->_cols[$this->_currentCell->columnIndex])) {
      $this->_cols[$this->_currentCell->columnIndex] = &new papaya_pdf_table_column($this);
      $this->_cols[$this->_currentCell->columnIndex]->columnIndex = $this->_currentCell->columnIndex;
    }
  }
  
  /**
  * set text data for current cell
  *
  * @param string $str
  * @access public
  * @return boolean
  */
  function addCellContent($str) {
    if (isset($this->_currentCell)) {
      $this->_currentCell->addContent($str);
      return TRUE;
    }
    return FALSE;
  }
  
  /**
  * add an image to current cell content
  *
  * @param string $fileName
  * @param float $width
  * @param float $height
  * @access public
  * @return boolean
  */
  function addCellContentImage($fileName, $width, $height) {
    if (isset($this->_currentCell)) {
      $this->_currentCell->addContentImage($fileName, $width, $height);
      return TRUE;
    }
    return FALSE;
  }
  
  
  /**
  * set tag data for current cell
  *
  * @param string $tag tag name
  * @param array $attr tag attributes
  * @access public
  * @return boolean
  */
  function addCellContentTag($tag, $attr) {
    if (isset($this->_currentCell)) {
      $this->_currentCell->addContentTag($tag, $attr);
      return TRUE;
    }
    return FALSE;
  }
  
  /**
  * set tag data for current cell
  *
  * @param string $tag tag name
  * @param array $attr tag attributes
  * @access public
  * @return boolean
  */
  function endCellContentTag($tag, $attr) {
    if (isset($this->_currentCell)) {
      $this->_currentCell->endContentTag($tag, $attr);
      return TRUE;
    }
    return FALSE;
  }
  
  /**
  * end current cell
  *
  * @access public
  */
  function endCell() {
    if (isset($this->_currentCell)) {
      $this->_currentCell->updateColumnWidth();
    }
    $null = NULL;
    $this->_currentCell = &$null;
  }
  
  /**
  * 
  *
  * @param integer $colIdx
  * @param float $min needed min width
  * @param float $max needed max width
  * @access public
  */
  function updateColumnWidth($colIdx, $min, $max = 0) {
    if (isset($this->_cols[$colIdx])) {
      $this->_cols[$colIdx]->updateWidth($min, $max);
    }
  }
  
  /**
  * caclulate width for all columns
  *
  * @param $maxWidth
  * @access public
  * @return
  */
  function calcColumnWidths($maxWidth) {
    $tableMinWidth = 0;
    $tableWidthDiff = 0;
    foreach(array_keys($this->_cols) as $colIdx) {
      $tableMinWidth += $this->_cols[$colIdx]->getMinWidth();
      $tableWidthDiff += $this->_cols[$colIdx]->getWidthDiff();
    }
    if ($maxWidth > $tableMinWidth) {
      $widthOverhead = $maxWidth - $tableMinWidth;
      $this->width = 0;
      foreach(array_keys($this->_cols) as $colIdx) {
        $this->_cols[$colIdx]->calcWidth($widthOverhead, $tableWidthDiff);
        $this->width += $this->_cols[$colIdx]->width;
      }
    } else {
      $this->width = $tableMinWidth;
    }
  }
  
  function getMinRowHeight($rowIdx) {
    if (isset($this->_rows[(int)$rowIdx])) {
      return $this->_rows[(int)$rowIdx]->minHeight;
    } else {
      return $this->getLineHeight();
    }
  }
  
  /**
  * get min and max width for a string
  *
  * @param string $str
  * @access public
  * @return array
  */
  function getTextWidth($str) {
    $max = $this->_pdfDocument->GetStringWidth($str);
    $min = 0;
    $words = explode(' ', $str);
    if (is_array($words)) {
      foreach($words as $word) {
        $i = $this->_pdfDocument->GetStringWidth($word);
        if ($i > $min) {
          $min = $i;
        }
      }
    }
    return array($min, $max);
  }
  
  function getAttributes() {
    if (isset($this->attributes)) {
      return $this->attributes;
    } else {
      return array();
    }
  }
  
  function getLineHeight() {
    return $this->_pdfDocument->getLineHeight();
  }
  
  function rowCount() {
    return count($this->_rows);
  }
  
  function &getRowByIndex($rowIdx) {
    if (isset($this->_rows[$rowIdx])) {
      return $this->_rows[$rowIdx];
    }
    $null = NULL;
    return $null;
  }
  
  function &getColByIndex($colIdx) {
    if (isset($this->_cols[$colIdx])) {
      return $this->_cols[$colIdx];
    }
    $null = NULL;
    return $null;
  }
}
?>