<?php 
/**
* class for a pdf table cell
* 
* @module papaya_pdf
* @author Thomas Weinert <weinert@papaya-cms.com>
* @version $Id: papaya_pdf_table_cell.php,v 1.1 2006/04/27 15:37:16 weinert Exp $
*/

/**
* table data cell
*/
class papaya_pdf_table_cell {
  /**
  * parent row
  * @var object papaya_pdf_table_row
  */
  var $_row = NULL;
  /**
  * cell column index 
  * @var integer
  */
  var $columnIndex = 0;
  
  /**
  * cell content
  * @var string $_contentLine
  */
  var $_contentLine = '';
  
  /**
  * xml tree for content elements
  * @var object xmltree_common
  */
  var $_xmlTree = NULL;
  /**
  * current xml node for a content element
  * @var object simple_xmlnode
  */
  var $_currentNode = NULL;
  
  /**
  * minum height for this cell
  * @var float
  */
  var $_minHeight = NULL;
  
  /**
  * block tags - trigger a linebreak
  * @var array
  */
  var $blockElementTags = array('p', 'br', 'h1', 'h2', 'h3', 'h4', 'h5', 'li', 'ul');
  
  /**
  * attrbutes
  * @var array
  */
  var $attr = array();
  
  /**
  * table data cell contrcutor
  *
  * @param object papaya_pdf_table_row &$row
  * @access public
  */
  function __construct(&$row, $attr = NULL) {
    $this->_row = &$row;
    $this->_xmlTree = &new simple_xmltree;
    $null = NULL;
    $this->_xmlTree->root = &new simple_xmlnode($null, 'cell');
    $this->_currentNode = &$this->_xmlTree->root;
    $this->columnIndex = $this->_row->addCell($this);
    if (isset($attr)) {
      $this->attributes = $attr;
    }
  }
  
  /**
  * PHP 4 constructor redirect
  *
  * @param object papaya_pdf_table_row &$row
  * @access public
  */
  function papaya_pdf_table_cell(&$row, $attr = NULL) {
    $this->__construct($row, $attr);
  }
  
  /**
  * set content
  *
  * @param string $str
  * @access public
  */
  function addContent($str) {
    if (isset($this->_currentNode)) {
      new simple_xmltextnode($this->_currentNode, $str);
      $this->_contentLine .= $str;
      $this->updateCellHeight($this->_row->table->getLineHeight());
    }
  }
  
  /**
  * add a content image - this is a special tag
  *
  * @param string $fileName
  * @param float $width
  * @param float $height
  * @access public
  */
  function addContentImage($fileName, $width, $height) {
    if (isset($this->_currentNode)) {
      $this->updateColumnWidth();
      $this->_currentNode = &new simple_xmlnode($this->_currentNode, 'image', array('src' => $fileName));
      $this->_row->table->updateColumnWidth($this->columnIndex, $width);
      $this->updateCellHeight($this->_row->table->getLineHeight());
    }
  }
  
  /**
  * add a new content tag
  *
  * @param string $tag
  * @param array $attr
  * @access public
  */
  function addContentTag($tag, $attr) {
    if (isset($this->_currentNode)) {
      $this->_currentNode = &new simple_xmlnode($this->_currentNode, $tag, $attr);      
    } else {
      $this->_currentNode = &new simple_xmlnode($this->xmlTree->_root, $tag, $attr);          
    }
    if (in_array($tag, $this->blockElementTags)) {
      $this->updateColumnWidth();
    }
  }
  
  /**
  * end a content tag
  *
  * @param string $tag
  * @param array $attr
  * @access public
  */
  function endContentTag($tag, $attr) {
    if (isset($this->_currentNode) && isset($this->_currentNode->parent)) {
      $this->_currentNode = &$this->_currentNode->parent;      
    } else {
      $null = NULL;
      $this->_currentNode = $null;
    }
    if (in_array($tag, $this->blockElementTags)) {
      $this->updateColumnWidth();
    }  
  }
  
  /**
  * update column width  depending on text width
  *
  * @access public
  */
  function updateColumnWidth() {
    if (!empty($this->_contentLine)) {
      list($minWidth, $maxWidth) = $this->_row->table->getTextWidth($this->_contentLine);
      $this->_row->table->updateColumnWidth($this->columnIndex, $minWidth, $maxWidth);
      $this->_contentLine = '';
    }
  }
  
  /**
  * set the minimum height need for this cell content
  *
  * @param $minHeight
  * @access public
  */
  function updateCellHeight($minHeight) {
    if (!isset($this->_minHeight)) {
      $this->_minHeight = $minHeight;
      $this->_row->updateMinHeight($minHeight);
    }
  } 
  
  function getWidth() {
    $col = &$this->_row->table->getColByIndex($this->columnIndex);
    return $col->width;
  }
  
  function getLeft() {
    $col = &$this->_row->table->getColByIndex($this->columnIndex);
    return $col->getLeft();
  }
  
  function getAttributes() {
    $parentAttr = $this->_row->getAttributes();
    if (isset($this->attributes)) {
      return array_merge($parentAttr, $this->attributes);
    } else {
      return $parentAttr;
    }
  }
  
  function outputData(&$pdf) {
    if (isset($this->_xmlTree) && isset($this->_xmlTree->root)) {
      $this->outputDataNode($pdf, $this->_xmlTree->root);
    }
  }
  
  function outputDataNode(&$pdf, &$node) {
    if (isset($node->children)) {
      foreach($node->children as $subNode) {
        if (isset($subNode) && is_a($subNode, 'simple_xmlnode')) {
          $pdf->openTag($subNode->name, $subNode->attributes);
          $this->outputDataNode($pdf, $subNode);
          $pdf->closeTag($subNode->name, $subNode->attributes);
        } elseif (isset($subNode) && is_a($subNode, 'simple_xmltextnode')) {
          $str = '';
          $subNode->printText($str);
          $pdf->writeText($str);
        }
      }
    }  
  }
}

/**
* table header cell (different default formatting)
*/
class papaya_pdf_table_headercell extends papaya_pdf_table_cell {
 
}
?>