<?php 
/**
* class for a pdf table column
* 
* @module papaya_pdf
* @author Thomas Weinert <weinert@papaya-cms.com>
* @version $Id: papaya_pdf_table_col.php,v 1.1 2006/04/27 15:37:16 weinert Exp $
*/

/**
* table column
*/
class papaya_pdf_table_column {
  /**
  * table (parent object)
  * @var object papaya_pdf_table
  */
  var $table = NULL;
  /**
  * column index
  * @var integer
  */
  var $columnIndex = NULL;
  
  /**
  * needed minimum width 
  * @var float
  */
  var $_minWidth = 5;
  /**
  * needed maximum width
  * @var float
  */
  var $_maxWidth = 0;
  
  /**
  * calculated width
  * @var float
  */
  var $width = 0;
  
  /**
  * table row
  *
  * @param object papaya_pdf_table &$table
  * @access public
  * @return
  */
  function __construct(&$table) {
    $this->table = &$table;
  }
  
  /**
  * PHP 4 constructor redirect
  *
  * @param object papaya_pdf_table &$table
  * @access public
  * @return
  */
  function papaya_pdf_table_column(&$table) {
    $this->__construct($table);
  }
  
  /**
  * update needed min and max for width (increase only)
  *
  * @param float $min minimum width
  * @param float $max maximum width
  * @access public
  */
  function updateWidth($min, $max = 0) {
    $paddingInc = $this->table->padding * 2;
    if ($min + $paddingInc > $this->_minWidth) {
      $this->_minWidth = $min + $paddingInc;
    }
    if ($max + $paddingInc > $this->_maxWidth) {
      $this->_maxWidth = $max + $paddingInc;
    }
    if ($this->_minWidth > $this->_maxWidth){
      $this->_maxWidth = $this->_minWidth;
    }
    $this->width = $this->_minWidth;
  }
  
  /**
  * get minimum width
  *
  * @access public
  * @return float
  */
  function getMinWidth() {
    return $this->_minWidth;
  }
  
  /**
  * get differece between max and min width
  *
  * @access public
  * @return
  */
  function getWidthDiff() {
    return ($this->_maxWidth - $this->_minWidth);
  
  }
  
  function getLeft() {
    $result = 0;
    for ($i = 0; $i < $this->columnIndex; $i++) {
      $result += $this->table->_cols[$i]->width;
    }
    return $result;
  }
  
  /**
  * calculate width 
  *
  * @param float $widthOverhead
  * @param float $allColumnDiff
  * @access public
  */
  function calcWidth($widthOverhead, $allColumnDiff) {
    $diff = $this->_maxWidth - $this->_minWidth;
    $this->width = $this->_minWidth + ($widthOverhead * $diff / $allColumnDiff);
  }
}
?>