<?php
/**
* Page module - download list.
*
* @package module_mediadb
* @author Thomas Weinert <weinert@papaya-cms.com>
* @version $Id: content_download.php,v 1.7 2006/05/18 11:23:48 rekowski Exp $
*/

/**
* Basic class content
*/
require_once(PAPAYA_INCLUDE_PATH.'system/base_content.php');
/**
* File manager
*/
require_once(PAPAYA_INCLUDE_PATH.'system/base_fileman.php');
/**
* Page module - download list.
*
* @package module_mediadb
* @author Thomas Weinert <weinert@papaya-cms.com>
*/
class content_download extends base_content {

  /**
  * Content edit fields
  * @var array
  */
  var $editFields = array(
    'title'     => array( 'Title', 'isNoHTML', FALSE, 'input', 200),
    'directory' => array('Folder', 'isNum', TRUE, 'function', 'mediadirectory', '', 0),
    'text'      => array('Text', 'isSomeText', FALSE, 'simplerichtext', 10),
    'sort'      => array('Sort', 'isNum', TRUE, 'combo', array(0=>'Alphabetic', 1=>'Created ascending', 2=>'Created descending'))
  );

  /**
  * Get parsed teaser
  *
  * @access public
  * @return string
  */
  function getParsedTeaser() {
    $result .= sprintf('<title>%s</title>'.LF,
                $this->getXHTMLString($this->topicObj->topicTitle));
    $teaser = str_replace("\r\n","\n",$this->data['text']);
    if (preg_match("/^(.+)([\n]{2})/sU", $teaser, $regs)) {
      $teaser = $regs[1];
    }
    $result .= sprintf('<text>%s</text>'.LF,
                $this->getXHTMLString(strip_tags($teaser), TRUE));
    $result .= sprintf('<title>%s</title>'.LF,
                $this->getXHTMLString(strip_tags(@$this->data['title'])));
    return $result;
  }

  /**
  * Get parsed data
  *
  * @access public
  * @return string
  */
  function getParsedData() {
    $result="";
    $result .= sprintf("<text>%s</text>".LF, $this->getXHTMLString($this->data['text'], TRUE));
    $result .= sprintf('<title>%s</title>'.LF,
                $this->getXHTMLString(strip_tags(@$this->data['title'])));

    $this->initializeParams();

    $fileManager = &new base_filemanager;
    $fileManager->initializeParams();
    $fileManager->baseLink = $this->baseLink;

    $fileManager->dataDirectory = PAPAYA_PATH_MEDIAFILES;
    $fileManager->thumbnailDirectory = PAPAYA_PATH_THUMBFILES;
    if (isset($this->params['file_id']) && $fileManager->dataLoadFile(@$this->params['file_id'] ,$this->topicObj->getContentLanguageId())) {
      $val = $fileManager->selectedFile;
      $fileTitle = ($val['file_name'] != $val['file_id']) ? $val['file_name'] : '';
      $href = $fileManager->getWebMediaLink($val['file_id'], 'download', $fileTitle);
      $result .= sprintf('<file download="%s" file_id="%s" file_name="%s" file_size="%s kB" file_type="%s" file_date="%s" file_title="%s">%s</file>',
        $href, htmlspecialchars($val['file_id']), htmlspecialchars($fileTitle),
        round($val['file_size'] / 1024),
        htmlspecialchars($fileManager->getMimeType($val['file_type'], $val['file_id'])),
        date('Y-m-d H:i:s', $val['file_date']),
        htmlspecialchars($val['file_title']),
        htmlspecialchars($val['file_comment']));
    } else {
      $fileManager->folderId = (int)$this->data['directory'];
      $fileManager->dataLoadFolder($fileManager->folderId, $this->topicObj->getContentLanguageId());
      $fileManager->dataLoadFileList($this->topicObj->getContentLanguageId(), TRUE, FALSE);


      if (isset($fileManager->files)) {
        $files = $fileManager->files;
        $count = count($files);
        uasort($files, array(&$this, 'compareMediaRecords'));

        $result .= '<files count="'.(int)$count.'">';
        foreach($files as $id=>$val) {
          $href = $this->baseLink.'?'.$this->paramName.'[file_id]='.urlencode($val['file_id']);
          $fileTitle = ($val['file_name'] != $val['file_id']) ? $val['file_name'] : '';
          $result .= sprintf('<file href="%s" file_id="%s" file_name="%s" file_size="%s kB" file_type="%s" file_date="%s" file_title="%s" download="%s">%s</file>',
          $href, htmlspecialchars($val['file_id']), htmlspecialchars($val['file_name']),
          round($val['file_size'] / 1024),
          htmlspecialchars($fileManager->getMimeType($val['file_type'], $val['file_id'])),
          date('Y-m-d H:i:s', $val['file_date']),
          htmlspecialchars($val['file_title']),
          $fileManager->getWebMediaLink($val['file_id'], 'download', $fileTitle),
          htmlspecialchars($val['file_comment']));
        }
        $result .= "</files>";
      }
    }

    return $result;
  }

  function compareMediaRecords($a, $b) {
    $mode = @(int)$this->data['sort'];
    if ($a['file_sort'] != $b['file_sort']) {
      return strcasecmp($a['file_sort'], $b['file_sort']);
    }
    $result = 0;
    switch ($mode) {
      case 2 :
        if ($a['file_date'] != $b['file_date']) {
          $result = ($a['file_date'] > $b['file_date']) ? 1 : -1;
        }
        break;
      case 1 :
        if ($a['file_date'] != $b['file_date']) {
          $result = ($a['file_date'] > $b['file_date']) ? -1 : 1;
        }
        break;
    }
    if ($result == 0) {
      $strA = ($a['file_title'] != '') ? $a['file_title'] : $a['file_name'];
      $strB = ($b['file_title'] != '') ? $b['file_title'] : $b['file_name'];
      return strnatcasecmp($strA, $strB);
    } else {
      return $result;
    }
  }

  /**
  * Media directory
  *
  * @param string $name
  * @param array $field
  * @param integer $data
  * @access public
  * @return string
  */
  function mediadirectory($name, $field, $data) {
    $result = '';

    include_once(PAPAYA_INCLUDE_PATH.'system/papaya_fileman.php');
    $fileManager = &new papaya_filemanager;
    $fileManager->initialize();

    $fileManager->dataDirectory = PAPAYA_PATH_MEDIAFILES;
    $fileManager->thumbnailDirectory = PAPAYA_PATH_THUMBFILES;
    $fileManager->dataLoadFolderList($this->topicObj->getContentLanguageId());
    $this->folders = $fileManager->folders;
    $this->folderLinks = $fileManager->folderLinks;

    $result .= sprintf('<select name="%s[%s]" class="dialogSelect dialogScale">'.LF,
      $this->paramName, $name);
    $result .= $this->getFolder(0,(int)$data, 0);
    $result .= '</select>'.LF;

    return $result;
  }

  /**
  * Get folder
  *
  * @param integer $id
  * @param integer $selId
  * @param integer $indent optional, default value 0
  * @access public
  * @return string
  */
  function getFolder($id, $selId, $indent=0) {
    $result = '';
    $folder = $this->folders[$id];
    if (isset($this->folderLinks[$id]['CHILDS'])) {
      $subFolders = $this->folderLinks[$id]['CHILDS'];
    } else {
      $subFolders = NULL;
    }
    if (isset($folder) && is_array($folder)) {
      if ($selId == $id) {
        $selected = ' selected="selected"';
      } else {
        $selected = '';
      }
      $result .= sprintf('<option value="%d"%s>%s%s</option>'.LF,
        $folder['folder_id'], $selected, str_repeat('-',$indent*2), htmlspecialchars($folder['folder_title']));
      if (isset($subFolders) && is_array($subFolders)) {
        foreach($subFolders as $sub) {
          $result .= $this->getFolder($sub, $selId, $indent+1);
        }
      }
    }
    return $result;
  }

}

?>