<?php 
/**
* User autentification secure functions
*
* @package papaya
* @author Thomas Weinert <weinert@papaya-cms.com>
* @version $Id: base_auth_secure.php,v 1.3 2006/05/10 13:50:20 weinert Exp $
*/

/**
* Base class data access
*/
require_once(PAPAYA_INCLUDE_PATH.'system/sys_base_db.php');


/**
* check function library
*/
require_once(PAPAYA_INCLUDE_PATH.'system/sys_checkit.php');

/**
* 
* 
*/
class base_auth_secure extends base_db {
  
  var $tableAuthTry = PAPAYA_DB_TBL_AUTHTRY;
  var $tableAuthIp = PAPAYA_DB_TBL_AUTHIP;
  
  var $authTryGroup = 'default';
  
  function getPasswordHash($password) {
    if (defined('PAPAYA_PASSWORD_PREFIX')) {
      $password = PAPAYA_PASSWORD_PREFIX.$password;
    }  
    if (defined('PAPAYA_PASSWORD_SUFFIX')) {
      $password = $password.PAPAYA_PASSWORD_SUFFIX;
    }  
    if (defined('PAPAYA_PASSWORD_METHOD')) {
      $method = PAPAYA_PASSWORD_METHOD;
    } else {
      $method = 'md5';
    }
    switch ($method) {
      case 'sha1' :
        return substr(sha1($password), 0, 32);      
      case 'md5' :
      default :
        return md5($password);
      break;
    }
  }
  
  function checkLoginTry($userName, $group = NULL) {
    if (isset($group)) {
      $this->authTryGroup = $group;
    }
  
    $ip = @(string)$_SERVER['REMOTE_ADDR'];
    if (defined('PAPAYA_LOGIN_CHECKTIME') && PAPAYA_LOGIN_CHECKTIME > 0) {
      $time = time() - PAPAYA_LOGIN_CHECKTIME;
    } else {
      $time = time() - 600;
    }
    $countIp = 0;
    $countUser = 0;
    
    $sql = "SELECT COUNT(*) 
              FROM %s 
             WHERE authtry_time > %d 
               AND authtry_ip = '%s'";               
    if ($res = $this->databaseQueryFmt($sql, 
        array($this->tableAuthTry, $time, $ip))) {
      $countIp = $res->fetchField();    }
               
    $sql = "SELECT COUNT(*) 
              FROM %s 
             WHERE authtry_time > %d 
               AND authtry_username = '%s'";                         
    if ($res = $this->databaseQueryFmt($sql, 
        array($this->tableAuthTry, $time, $userName))) {
      $countUser = $res->fetchField();
    }
    
    $allowLoginTry = TRUE;
    if (defined('PAPAYA_LOGIN_RESTRICTION') && PAPAYA_LOGIN_RESTRICTION == 3) {
      //restricted to whitelist
      if ($this->getIpStatus($ip) != 1) {
         $allowLoginTry = FALSE;
      }
    } elseif (defined('PAPAYA_LOGIN_RESTRICTION') && 
      (PAPAYA_LOGIN_RESTRICTION == 1 || PAPAYA_LOGIN_RESTRICTION == 2)) {
      //blacklist restriction
      if ($this->getIpStatus($ip) == 2) {
         $allowLoginTry = FALSE;
      }
    } elseif (defined('PAPAYA_LOGIN_BLOCKCOUNT') && 
        ($countIp >= PAPAYA_LOGIN_BLOCKCOUNT || 
        $countUser >= PAPAYA_LOGIN_BLOCKCOUNT)) {
      if (defined('PAPAYA_LOGIN_RESTRICTION') && PAPAYA_LOGIN_RESTRICTION == 2) {
        //do not block ips in whitelist
        if ($this->getIpStatus($ip) != 1) {
          $allowLoginTry = FALSE;
        }
      } else {
        $allowLoginTry = FALSE;
      }
    } elseif (defined('PAPAYA_LOGIN_NOTIFYCOUNT') &&
        ($countIp >= PAPAYA_LOGIN_NOTIFYCOUNT || 
         $countUser >= PAPAYA_LOGIN_NOTIFYCOUNT)) {
      if (defined('PAPAYA_LOGIN_NOTIFYFACTOR') &&  defined('PAPAYA_LOGIN_NOTIFYFACTOR') >= 1) {
        $factor = (int)PAPAYA_LOGIN_NOTIFYFACTOR;
      } else {
        $factor = 10;      
      }
      if ($countIp % $factor == 0 || $countUser % $factor == 0) {
        $this->sendNotify($_SERVER['REMOTE_ADDR'], $countIp, $userName, $countUser);
      }
    } 
    
    if ($allowLoginTry) {
      $this->addLoginTry($userName, $ip);
      return TRUE;
    } else {
      return FALSE;
    }
  }
  
  function addLoginTry($userName, $ip) {
    $data = array(
      'authtry_username' => (string)$userName,
      'authtry_ip' => (string)$ip,
      'authtry_time' => time(),
      'authtry_group' => (string)$this->authTryGroup
    );
    $this->databaseInsertRecord($this->tableAuthTry, 'authtry_id', $data);
  }
  
  function sendNotify($Ip, $countIp, $userName, $countUser) {
    if (defined('PAPAYA_LOGIN_NOTIFYEMAIL') && 
        checkit::isEmail(PAPAYA_LOGIN_NOTIFYEMAIL, TRUE)) {
      include_once(PAPAYA_INCLUDE_PATH.'system/sys_email.php');
      $emailObj = &new email;
        
      $bodyTemplate = 
        $this->_gt('IP').': {%IP%} ({%IP_HOST%})'.LF.
        $this->_gt('Requests').': {%IP_COUNT%} '.LF.LF.
        $this->_gt('Username').': {%USERNAME%} ({%GROUP%})'.LF.
        $this->_gt('Requests').': {%USER_COUNT%} '.LF.LF.
        $this->_gt('User agent'.': {%USERAGENT%}');
      $subjectTemplate = 
        $this->_gt('WARNING! Mass login request on').' {%HOST%}';
        
      $data = array(
        'IP' => $Ip,
        'IP_HOST' => gethostbyaddr($Ip),
        'IP_COUNT' => $countIp,
        'USERNAME' => $userName,
        'USERAGENT' => (string)@$_SERVER['HTTP_USER_AGENT'],
        'USER_COUNT' => $countUser,
        'HOST' => $_SERVER['HTTP_HOST'],
        'GROUP' => $this->authTryGroup
      );
      
      $emailObj->setTemplate('body', $bodyTemplate, $data);
      $emailObj->setTemplate('subject', $subjectTemplate, $data);
      $emailObj->send(PAPAYA_LOGIN_NOTIFYEMAIL);
      $this->logMsg(MSG_WARNING, 1, $this->_gt('Brute force notification'),
        $emailObj->getBody());
    }
  }
  
  function getIPStatus($ip) {
    $sql = "SELECT auth_ip_status
              FROM %s
             WHERE auth_ip = '%s'";
    $params = array($this->tableAuthIp, $ip);
    if ($res = $this->databaseQueryFmt($sql, $params)) {
      if ($row = $res->fetchRow()) {
        return (int)$row[0];
      }
    }    
    return 0;
  }
}

?>