<?php
/**
* Design formulars
*
* @package papaya
* @author Thomas Weinert <weinert@papaya-cms.com>
* @version $Id: base_formdesigner.php,v 1.8 2006/04/26 10:29:00 weinert Exp $
*/


/**
* Base object
*/
require_once(PAPAYA_INCLUDE_PATH.'system/sys_base_object.php');

/**
* XML-Handling
*/
require_once(PAPAYA_INCLUDE_PATH.'system/base_formdsg_xml.php');

/**
* Design formulars
*
* @package papaya
* @author Thomas Weinert <weinert@papaya-cms.com>
*/
class base_formdesigner extends base_formdesigner_xml {
  var $field;

  var $inputFieldSize = 'medium';

  var $_modified = FALSE;

  var $lngId = NULL;
  
  /**
  * Initialize parameters
  *
  * @param $paramName
  * @param $fieldXML
  * @access public
  */
  function initialize($paramName, $fieldXML, $lngId) {
    $this->paramName = $paramName;
    $this->initializeParams();
    $this->xmlToFields($fieldXML);    
    $this->lngId = $lngId;    
  }

  /**
  * Basic function for handling user request
  *
  * @access public
  */
  function execute() {
    if (isset($this->params['fielddsg_field']) && isset($this->fields[$this->params['fielddsg_field']])) {
      $this->field = &$this->fields[$this->params['fielddsg_field']];
    }
    switch (@$this->params['fielddsg_cmd']) {
      case 'field_add' :
        $this->initFieldEditDialog(TRUE);       
        if ($this->fieldEditDialog->modified()) {      
          if ($this->fieldEditDialog->checkDialogInput()) {            
            if (isset($this->params['fielddsg_group']) && trim($this->params['fielddsg_group']) != '') {
              $groupTitle = $this->params['fielddsg_group'];
            } elseif (isset($this->params['fielddsg_groupsel']) && trim($this->params['fielddsg_groupsel']) != '') {
              $groupTitle = $this->params['fielddsg_groupsel'];
            } else {
              $groupTitle = NULL;
            }           
                       
            if ((!isset($this->fields[$this->params['fielddsg_name']])) && isset($groupTitle)) {
              $this->_modified = TRUE;
              $field = array(
                'name'          => $this->params['fielddsg_name'],
                'title'         => $this->params['fielddsg_title'],
                'type'          => $this->params['fielddsg_type'],
                'validatefunc'  => $this->params['fielddsg_valfunc'],
                'validateregex' => $this->params['fielddsg_valregex'],
                'needed'        => $this->params['fielddsg_needed'],
                'group'         => $groupTitle
              );
              if (isset($this->params['fielddsg_css']) && trim($this->params['fielddsg_css']) != '') {
                $field['css'] = $this->params['fielddsg_css'];
              } elseif (isset($this->params['fielddsg_csssel']) && trim($this->params['fielddsg_csssel']) != '') {
                $field['css'] = $this->params['fielddsg_csssel'];
              } else {
                $field['css'] = '';
              }
              if (!isset($this->groups[$groupTitle])) {
                $this->groups[$groupTitle]['title'] = $groupTitle;
              }
              $this->groups[$groupTitle]['fields'][] = $field['name'];
              $this->fields[$field['name']] = $field;
              $this->field = &$this->fields[$field['name']];
              $this->params['fielddsg_field'] = $field['name'];
              $this->params['fielddsg_cmd'] = 'field_edit';
              unset($this->fieldEditDialog);
            } else {
              $this->addMsg(MSG_ERROR, 'Field name already in use.');
            }
          }
        }
        break;
      case 'field_edit' :
        if (isset($this->field)) {
          $this->initFieldEditDialog();
          if ($this->fieldEditDialog->modified()) {
            if ($this->fieldEditDialog->checkDialogInput()) {
              $this->_modified = TRUE;
              if ($this->params['fielddsg_name'] != $this->field['name']) {
                if (!isset($this->fields[$this->params['fielddsg_name']])) {
                  $oldName = $this->field['name'];
                  $this->field['name'] = $this->params['fielddsg_name'];
                  $this->fields[$this->params['fielddsg_name']] = $this->fields[$oldName];
                  $this->field = &$this->fields[$this->params['fielddsg_name']];
                  unset($this->fields[$oldName]);
                  $group = &$this->groups[$this->field['group']];
                  $i = array_search($oldName, $group['fields']);
                  if ($i === 0 || $i > 0) {
                    $group['fields'][$i] = $this->field['name'];
                  }
                  $this->fieldEditDialog->hidden['fielddsg_field'] = $this->field['name'];
                } else {
                  $this->addMsg(MSG_ERROR, 'Field name already in use.');
                }
              }
              // Gruppe ndern
              if (isset($this->params['fielddsg_group']) && trim($this->params['fielddsg_group']) != '') {
                $groupTitle = $this->params['fielddsg_group'];
              } elseif (isset($this->params['fielddsg_groupsel']) && trim($this->params['fielddsg_groupsel']) != '') {
                $groupTitle = $this->params['fielddsg_groupsel'];
              } else {
                $groupTitle = NULL;
              }
              if ($groupTitle != $this->field['group']) {
                if (isset($this->groups[$groupTitle])) {
                  //gibts schon
                  $oldGroup = $this->field['group'];
                  $this->groups[$groupTitle]['fields'][] = $this->field['name'];
                  if (count($this->groups[$oldGroup]['fields']) <= 1) {
                    unset($this->groups[$oldGroup]);
                  } else {
                    $i = array_search($this->field['name'], $this->groups[$oldGroup]['fields']);
                    if ($i === 0 || $i > 0) {
                      unset($this->groups[$oldGroup]['fields'][$i]);
                    }
                  }
                } else {
                  foreach($this->groups as $key=>$val) {
                    if ($key == $this->field['group']) {
                      if (count($val['fields']) > 1) {
                        $i = array_search($this->field['name'], $val['fields']);
                        if ($i === 0 || $i > 0) {
                          unset($val['fields'][$i]);
                        }
                        $newGroups[$key] = $val;
                      }
                      $newGroups[$groupTitle]['title'] = $groupTitle;
                      $newGroups[$groupTitle]['fields'][] = $this->field['name'];
                    } else {
                      $newGroups[$key] = $val;
                    }
                  }
                  $this->groups = $newGroups;
                }
                $this->field['group'] = $groupTitle;
              }

              if (isset($this->params['fielddsg_css']) && trim($this->params['fielddsg_css']) != '') {
                $this->field['css'] = $this->params['fielddsg_css'];
              } elseif (isset($this->params['fielddsg_csssel']) && trim($this->params['fielddsg_csssel']) != '') {
                $this->field['css'] = $this->params['fielddsg_csssel'];
              } else {
                $this->field['css'] = '';
              }

              $this->field['title'] = $this->params['fielddsg_title'];
              $this->field['type'] = $this->params['fielddsg_type'];
              $this->field['validatefunc'] = $this->params['fielddsg_valfunc'];
              $this->field['validateregex'] = $this->params['fielddsg_valregex'];
              $this->field['needed'] = $this->params['fielddsg_needed'];
            }
          }
        }
        break;
      case 'param_edit' :
        if (isset($this->field)) {
          $this->initFieldTypeDialog();
          if (isset($this->fieldTypeDialog) && is_object($this->fieldTypeDialog)) {
            if ($this->fieldTypeDialog->modified()) {
              if ($this->fieldTypeDialog->checkDialogInput()) {
                $this->_modified = TRUE;
                $params = $this->types[$this->field['type']];
                if (is_array($params)) {
                  foreach($params as $attributeName=>$paramData) {
                    $this->field['typeparams'][$attributeName] = $this->params['fielddsg_tp_'.$attributeName];
                  }
                }
              }
            }
          }
        }
        break;
      case 'field_del' :
        if (isset($this->field)) {
          if (isset($this->params['fielddsg_confirm']) && $this->params['fielddsg_confirm']) {
            $this->_modified = TRUE;
            $group = &$this->groups[$this->field['group']];
            $i = array_search($this->field['name'], $group['fields']);
            if ($i === 0 || $i > 0) {
                if (count($group['fields']) < 2) {
                  unset($this->groups[$group['title']]);
                } else {
                  unset($group['fields'][$i]);
                }
            }
            unset($this->fields[$this->field['name']]);
            unset($this->field);
            unset($this->params['fielddsg_cmd']);
          }
        }
        break;
      case 'move_up' :
      case 'move_down' :
        if (isset($this->field) && isset($this->groups[$this->field['group']])) {
          //Feld bewegen
          $groupTitle = $this->field['group'];
          $i = array_search($this->field['name'], $this->groups[$groupTitle]['fields']);
          if ($i === 0 || $i > 0) {
            if ($this->params['fielddsg_cmd'] == 'move_down' &&
                count($this->groups[$groupTitle]['fields']) > $i+1) {
              $f1 = $this->groups[$groupTitle]['fields'][$i+1];
              $f2 = $this->groups[$groupTitle]['fields'][$i];
              $this->groups[$groupTitle]['fields'][$i+1] = $f2;
              $this->groups[$groupTitle]['fields'][$i] = $f1;
              $this->_modified = TRUE;
            } elseif ($this->params['fielddsg_cmd'] == 'move_up' && $i > 0) {
              $f1 = $this->groups[$groupTitle]['fields'][$i-1];
              $f2 = $this->groups[$groupTitle]['fields'][$i];
              $this->groups[$groupTitle]['fields'][$i-1] = $f2;
              $this->groups[$groupTitle]['fields'][$i] = $f1;
              $this->_modified = TRUE;
            }
          }
        } elseif (isset($this->params['fielddsg_group']) && isset($this->groups[$this->params['fielddsg_group']])) {
          //Gruppe bewegen
          $groupTitles = array_keys($this->groups);
          $i = array_search($this->params['fielddsg_group'], $groupTitles);
          if ($i === 0 || $i > 0) {
            if ($this->params['fielddsg_cmd'] == 'move_down' && count($groupTitles) > $i+1) {
              $g1 = $groupTitles[$i+1];
              $g2 = $groupTitles[$i];
              $groupTitles[$i+1] = $g2;
              $groupTitles[$i] = $g1;
              $this->_modified = TRUE;
            } elseif ($this->params['fielddsg_cmd'] == 'move_up' && $i > 0) {
              $g1 = $groupTitles[$i-1];
              $g2 = $groupTitles[$i];
              $groupTitles[$i-1] = $g2;
              $groupTitles[$i] = $g1;
              $this->_modified = TRUE;
            }
            if ($this->_modified) {
              foreach($groupTitles as $groupTitle) {
                $groups[$groupTitle] = $this->groups[$groupTitle];
              }
              $this->groups = $groups;
            }
          }
        }
        break;
    }
  }

  /**
  * Get dialog XML
  *
  * @access public
  * @return string XML or ''
  */
  function getDialogXML() {
    $result = '';    
    switch (@$this->params['fielddsg_cmd']) {
      case 'field_del' :
        if (isset($this->field)) {
          $result .= $this->getDelFieldForm();
        }
      case 'field_add' :
        $result .= $this->getFieldEditDialog(TRUE);
        break;
      case 'field_edit' :
      case 'param_edit' :
        if (isset($this->field)) {
          $result .= $this->getFieldEditDialog();
          $result .= $this->getFieldTypeDialog();
        }
        break;
    }
    return $result;
  }

  /**
  * Check if input modifies
  *
  * @access public
  * @return boolean
  */
  function modified() {
    return (boolean)$this->_modified;
  }

  /**
  * Get list XML
  *
  * @access public
  * @return string XML
  */
  function getListXML() {    
    $result = $this->getFieldsListView();
    return $result;
  }

  /**
  * Initialize field edit dialog
  *
  * @param boolean $newField optional, default value FALSE
  * @access public
  */
  function initFieldEditDialog($newField = FALSE) {
    if (!(isset($this->fieldEditDialog) && is_object($this->fieldEditDialog))) {
      include_once(PAPAYA_INCLUDE_PATH.'system/base_dialog.php');
      $hidden['save'] = 1;
      if ($newField) {
        $hidden['fielddsg_cmd'] = 'field_add';
      } else {
        $hidden['fielddsg_cmd'] = 'field_edit';
        $hidden['fielddsg_field'] = isset($this->params['fielddsg_field']) ? $this->params['fielddsg_field'] : '';
        $data = array(
          'fielddsg_name' => $this->field['name'],
          'fielddsg_title' => $this->field['title'],
          'fielddsg_type' => $this->field['type'],
          'fielddsg_valfunc' => $this->field['validatefunc'],
          'fielddsg_valregex' => $this->field['validateregex'],
          'fielddsg_needed' => $this->field['needed'],
          'fielddsg_csssel' => $this->field['css'],
          'fielddsg_groupsel' => $this->field['group']
        );
      }
      $yesnoArray = array(1=> $this->_gt('Yes'), 0=> $this->_gt('No'));
      $types = array();
      if (isset($this->types) && is_array($this->types)) {
        foreach(array_keys($this->types) as $typeName) {
          $types[$typeName] = $typeName;
        }
      }
      $fields = array(
        'fielddsg_name' => array('Name', 'isAlphaNum', TRUE, 'input', 20, '', ''),
        'fielddsg_title' => array('Title', 'isSomeText', TRUE, 'input', 40, '', ''),
        'fielddsg_type' => array('Type', 'isAlpha', TRUE, 'combo', $types, '', ''),
        'Validation',
        'fielddsg_valfunc' => array('Function', 'isAlphaChar', TRUE, 'function', 'getCheckFunctionsCombo', '', ''),
        'fielddsg_valregex' => array('PCRE', 'isNoHTML', FALSE, 'input', 40, '', ''),
        'fielddsg_needed' => array('Needed', 'isNum', TRUE, 'yesno', 1, '', '', 'center'),
        'CSS',
        'fielddsg_css' => array('Input', 'isAlphaNum', FALSE, 'input', 80, '', ''),
        'fielddsg_csssel' => array('Select', 'isAlphaNum', FALSE, 'function', 'getCSSCombo', '', ''),
        'Group',
        'fielddsg_group' => array('Input', 'isAlphaNum', TRUE, 'input', 100, '', ''),
      );
      if (isset($this->groups) && is_array($this->groups) && count($this->groups) > 0) {
        $fields['fielddsg_group'][2] = FALSE;
        $fields['fielddsg_groupsel'] = array('Select', 'isAlphaNum', FALSE, 'function', 'getGroupsCombo', '', '');
      }
      $this->fieldEditDialog = &new base_dialog($this, $this->paramName, $fields, $data, $hidden);
      $this->fieldEditDialog->dialogTitle = $this->_gt('Field');
      $this->fieldEditDialog->baseLink = $this->baseLink;
      $this->fieldEditDialog->inputFieldSize = $this->inputFieldSize;
      $this->fieldEditDialog->msgs = &$this->msgs;
      $this->fieldEditDialog->loadParams();
    }
  }

  /**
  * Get combo check-functions
  *
  * @param string $name
  * @param array $field
  * @param mixed $data
  * @access public
  * @return string XML or ''
  */
  function getCheckFunctionsCombo($name, $field, $data) {
    $result ="";
    $result .= sprintf('<select name="%s[%s]" class="dialogSelect dialogScale">'.LF,
      $this->paramName, $name);
    $methods = get_class_methods('checkit');
    if (is_array($methods) && count($methods) > 0) {
      $selected = ($data == -1) ? ' selected="selected"' : '';
      $result .= sprintf('<option value="%d" %s>[%s]</option>'.LF,
        -1, $selected, papaya_strings::escapeHTMLChars('PCRE'));
      foreach ($methods as $method) {
        if (substr($method, 0, 2) == 'is') {
          $selected = (strtolower($data) == strtolower($method)) ? ' selected="selected"' : '';
          $result .= sprintf('<option value="%s" %s>%s</option>'.LF,
            papaya_strings::escapeHTMLChars($method), $selected, papaya_strings::escapeHTMLChars($method));
        }
      }
    }
    $result .= '</select>'.LF;
    return $result;
  }

  /**
  * Get groups combo
  *
  * @param string $name
  * @param array $field
  * @param mixed $data
  * @access public
  * @return string XML or ''
  */
  function getGroupsCombo($name, $field, $data) {
    $result ="";
    if (is_array($this->groups)) {
      $result .= sprintf('<select name="%s[%s]" class="dialogSelect dialogScale">'.LF,
        $this->paramName, $name);

      foreach ($this->groups as $title=>$group) {
        $selected = ($data == $title) ? ' selected="selected"' : '';
        $result .= sprintf('<option value="%s" %s>%s</option>'.LF,
          papaya_strings::escapeHTMLChars($title), $selected, papaya_strings::escapeHTMLChars($title));
      }
      $result .= '</select>'.LF;
    }
    return $result;
  }

  /**
  * Get CSS
  *
  * @param string $name
  * @param array $field
  * @param mixed $data
  * @access public
  * @return string XML or ''
  */
  function getCSSCombo($name, $field, $data) {
    $result ="";
    if (isset($this->fields) && is_array($this->fields)) {
      $cssClasses = array();
      foreach ($this->fields as $field) {
        if (isset($field['css']) && trim($field['css']) != '') {
          $cssClasses[] = $field['css'];
        }
      }
      array_unique($cssClasses);

      $result .= sprintf('<select name="%s[%s]" class="dialogSelect dialogScale">'.LF,
        $this->paramName, $name);

      foreach ($cssClasses as $css) {
        $selected = ($data == $css) ? ' selected="selected"' : '';
        $result .= sprintf('<option value="%s" %s>%s</option>'.LF,
          papaya_strings::escapeHTMLChars($css), $selected, papaya_strings::escapeHTMLChars($css));
      }
      $result .= '</select>'.LF;
    }
    return $result;
  }

  /**
  * Get field edit dialog
  *
  * @access public
  * @return string XML or ''
  */
  function getFieldEditDialog() {
    $this->initFieldEditDialog();
    return $this->fieldEditDialog->getDialogXML();
  }

  /**
  * Initialize field type dialog
  *
  * @access public
  */
  function initFieldTypeDialog() {
    if (isset($this->field) &&
        isset($this->types[$this->field['type']]) && is_array($this->types[$this->field['type']])) {
      if (!(isset($this->fieldTypeDialog) && is_object($this->fieldTypeDialog))) {
        include_once(PAPAYA_INCLUDE_PATH.'system/base_dialog.php');
        $hidden['save'] = 1;
        $hidden['fielddsg_cmd'] = 'param_edit';
        $hidden['fielddsg_field'] = $this->field['name'];
        $fields = array();
        if (is_array($this->types[$this->field['type']])) {
          foreach($this->types[$this->field['type']] as $attributeName=>$field) {
            if (isset($this->field['typeparams'][$attributeName])) {
              $data['fielddsg_tp_'.$attributeName] = $this->field['typeparams'][$attributeName];
            }
            $fields['fielddsg_tp_'.$attributeName] = $field;
          }
        }
        $this->fieldTypeDialog = &new base_dialog($this, $this->paramName, $fields, $data, $hidden);
        $this->fieldTypeDialog->dialogTitle = $this->_gt('Parameters');
        $this->fieldTypeDialog->baseLink = $this->baseLink;
        $this->fieldTypeDialog->inputFieldSize = $this->inputFieldSize;
        $this->fieldTypeDialog->msgs = &$this->msgs;
        $this->fieldTypeDialog->loadParams();
      }
    }
  }

  /**
  * Get field type dialog
  *
  * @access public
  * @return string XML or ''
  */
  function getFieldTypeDialog() {
    $this->initFieldTypeDialog();
    if (isset($this->fieldTypeDialog) && is_object($this->fieldTypeDialog)) {
      return $this->fieldTypeDialog->getDialogXML();
    } else {
      return '';
    }
  }

  /**
  * Get delete field formular
  *
  * @access public
  * @return string XML or ''
  */
  function getDelFieldForm() {
    include_once(PAPAYA_INCLUDE_PATH.'system/base_msgdialog.php');
    $hidden = array(
      'fielddsg_cmd' => 'field_del',
      'fielddsg_field' => $this->field['name'],
      'fielddsg_confirm'=>1,
    );
    $msg = sprintf($this->_gt('Delete field "%s (%s)"?'),
      papaya_strings::escapeHTMLChars($this->field['name']), papaya_strings::escapeHTMLChars($this->field['title']));
    $dialog = &new base_msgdialog($this, $this->paramName, $hidden, $msg, 'question');
    $dialog->msgs = &$this->msgs;
    $dialog->baseLink = $this->baseLink;
    $dialog->buttonTitle = 'Delete';
    return $dialog->getMsgDialog();
  }

  /**
  * Get fields list view
  *
  * @access public
  * @return string XML or ''
  */
  function getFieldsListView() {
    if (isset($this->groups) && is_array($this->groups) && isset($this->fields) && is_array($this->fields)) {
      $result = sprintf('<listview title="%s">', papaya_strings::escapeHTMLChars($this->_gt('Fields')));
      $result .= '<cols>';
      $result .= sprintf('<col>%s</col>', papaya_strings::escapeHTMLChars($this->_gt('Name')));
      $result .= sprintf('<col>%s</col>', papaya_strings::escapeHTMLChars($this->_gt('Type')));
      $result .= '<col/>';
      $result .= '</cols>';
      $result .= '<items>';

      $groupMax = count($this->groups) - 1;
      $groupIdx = 0;
      foreach($this->groups as $group) {
        if (isset($group['fields']) && is_array($group['fields'])) {
          $result .= sprintf('<listitem title="%s">', papaya_strings::escapeHTMLChars($group['title']));
          $result .= '<subitem align="center">';
          if ($groupMax > 0) {
            if ($groupIdx > 0) {
              $result .= sprintf('<a href="%s"><glyph src="%s"/></a>',
                $this->baseLink.'?'.$this->paramName.'[fielddsg_cmd]=move_up&amp;'.
                  $this->paramName.'[fielddsg_group]='.urlencode($group['title']), $this->images[6]);
            } else {
              $result .= sprintf('<glyph src="%s"/>', $this->images[0]);
            }
            if ($groupIdx < $groupMax) {
              $result .= sprintf('<a href="%s"><glyph src="%s"/></a>',
                $this->baseLink.'?'.$this->paramName.'[fielddsg_cmd]=move_down&amp;'.
                $this->paramName.'[fielddsg_group]='.urlencode($group['title']), $this->images[7]);
            } else {
              $result .= sprintf('<glyph src="%s"/>', $this->images[0]);
            }
            $groupIdx++;
          }
          $result .= '</subitem>';
          $result .= '<subitem/>';
          $result .= '</listitem>';

          $fieldMax = count($group['fields']) - 1;
          $fieldIdx = 0;
          foreach($group['fields'] as $fieldName) {
            if (isset($this->fields[$fieldName])) {
              $field = $this->fields[$fieldName];
              $selected = (@$this->params['fielddsg_field'] == $field['name']) ? ' selected="selected"' : '';
              $result .= sprintf('<listitem indent="1" title="%s" href="%s"%s>', papaya_strings::escapeHTMLChars($field['name']),
                $this->baseLink.'?'.$this->paramName.'[fielddsg_cmd]=field_edit&amp;'.
                  $this->paramName.'[fielddsg_field]='.urlencode($field['name']),
                $selected);
              $result .= sprintf('<subitem>%s</subitem>', papaya_strings::escapeHTMLChars($field['type']));
              $result .= '<subitem align="center">';
              if ($fieldMax > 0) {
                if ($fieldIdx > 0) {
                  $result .= sprintf('<a href="%s"><glyph src="%s"/></a>',
                    $this->baseLink.'?'.$this->paramName.'[fielddsg_cmd]=move_up&amp;'.
                    $this->paramName.'[fielddsg_field]='.urlencode($field['name']), $this->images[6]);
                } else {
                  $result .= sprintf('<glyph src="%s"/>', $this->images[0]);
                }
                if ($fieldIdx < $fieldMax) {
                  $result .= sprintf('<a href="%s"><glyph src="%s"/></a>',
                    $this->baseLink.'?'.$this->paramName.'[fielddsg_cmd]=move_down&amp;'.
                    $this->paramName.'[fielddsg_field]='.urlencode($field['name']), $this->images[7]);
                } else {
                  $result .= sprintf('<glyph src="%s"/>', $this->images[0]);
                }
                $fieldIdx++;
              }
              $result .= '</subitem>';
              $result .= '</listitem>';
            }
          }
        }
        $groupIdx++;
      }
      $result .= '</items>';
      $result .= '</listview>';
      return $result;
    }
  }

  /**
  * Get XML buttons
  *
  * @access public
  * @return string '' or XML
  */
  function getButtonsXML() {
    include_once(PAPAYA_INCLUDE_PATH.'system/base_btnbuilder.php');
    $toolbar = &new base_btnbuilder;
    $toolbar->images = &$this->images;
    $toolbar->addButton('New field', $this->baseLink.'?'.$this->paramName.'[fielddsg_cmd]=field_add', 70);
    if (isset($this->field) && is_array($this->field)) {
      $toolbar->addSeperator();
      $toolbar->addButton('Delete field', $this->baseLink.'?'.$this->paramName.'[fielddsg_cmd]=field_del&amp;'.
        $this->paramName.'[fielddsg_field]='.urlencode($this->field['name']), 10);
    }
    return $toolbar->getXML();
  }

  /**
  * XML to fields
  *
  * @param string $xml
  * @access public
  */
  function xmlToFields($xml) {
    parent::xmlToFields($xml);
  }

  /**
  * fields to XML
  *
  * @access public
  * @return string XML
  */
  function fieldsToXML() {
    return parent::fieldsToXML();
  }

  /**
  * Create a XML Tree
  * @param string $xmlData XML-string
  * @return object simple_xmlnode $xmlTree XML-tree
  */
  function createXMLTree($xmlData) {
    if (trim($xmlData) != '') {
      $xmlTree = simple_xmltree::create();
      if (is_object($xmlTree)) {
        if (0 === $xmlTree->loadFromString($xmlData)) {
          if (is_object($xmlTree->root)) {
            return $xmlTree->root;
          }
        } elseif (defined('PAPAYA_DBG_XML_USERINPUT') && PAPAYA_DBG_XML_USERINPUT === '1') {
          $this->lastXMLError = @$xmlTree->lastError;
          unset($xmlTree);
        }
      }
    }
    return FALSE;
  }

  /**
  * Get media directory combo
  *
  * @param string $name
  * @param array $field
  * @param mixed $data
  * @access public
  * @return string '' or XML
  */
  function getMediaDirectoryCombo($name, $field, $data) {
    $result = '';

    include_once(PAPAYA_INCLUDE_PATH.'system/papaya_fileman.php');
    $fileManager = &new papaya_filemanager;
    $fileManager->initialize();

    $fileManager->dataDirectory = PAPAYA_PATH_MEDIAFILES;
    $fileManager->thumbnailDirectory = PAPAYA_PATH_THUMBFILES;

    $fileManager->dataLoadFolderList($this->lngId);
    $this->folders = $fileManager->folders;
    if ( isset($fileManager->folderLinks))
      $this->folderLinks = $fileManager->folderLinks;

    $result .= sprintf('<select name="%s[%s]" class="dialogSelect dialogScale">'.LF,
      $this->paramName, $name);
    $result .= $this->getFolder(0,(int)$data, 0);
    $result .= '</select>'.LF;

    return $result;
  }

  /**
  * Get folder
  *
  * @param integer $id
  * @param integer $selId
  * @param integer $indent optional, default value 0
  * @access public
  * @return string '' or XML
  */
  function getFolder($id, $selId, $indent=0) {
    $result = '';
    $folder = $this->folders[$id];
    $subFolders = @$this->folderLinks[$id]['CHILDS'];
    if (isset($folder) && is_array($folder)) {
      if ($selId == $id) {
        $selected = ' selected="selected"';
      } else {
        $selected = '';
      }
      $result .= sprintf('<option value="%d"%s>%s%s</option>'.LF,
        $folder['folder_id'], $selected, str_repeat('-',$indent*2), papaya_strings::escapeHTMLChars($folder['folder_title']));
      if (isset($subFolders) && is_array($subFolders)) {
        foreach($subFolders as $sub) {
          $result .= $this->getFolder($sub, $selId, $indent+1);
        }
      }
    }
    return $result;
  }
}

?>