<?php
/**
* formular designer
*
* @package papaya
* @author Thomas Weinert <weinert@papaya-cms.com>
* @version $Id: base_formdsg_xml.php,v 1.3 2006/04/26 10:29:00 weinert Exp $
*/


/**
* Basic object
*/
require_once(PAPAYA_INCLUDE_PATH.'system/sys_base_object.php');

/**
* XML-Handling
*/
require_once(PAPAYA_INCLUDE_PATH.'system/sys_simple_xmltree.php');

/**
* formular designer
*
* @package papaya
* @author Thomas Weinert <weinert@papaya-cms.com>
*/
class base_formdesigner_xml extends base_object {
  var $groups;
  var $fields;

  var $types = array(
    'input' => array(
      'maxlength' => array('Max length', 'isNum', TRUE, 'input', 3, '', 100)
    ),
    'checkbox' => array(
      'value_on' => array('On', 'isNoHTML', TRUE, 'input', 100, '', 'X'),
      'value_off' => array('Off', 'isNoHTML', TRUE, 'input', 100, '', 'O'),
      'default' => array('Default', 'isNum', TRUE, 'combo', array(1=>'Checked',0=>'Not checked'), '', 0),
      'caption' => array('Caption', 'isNoHTML', FALSE, 'input', 100, '', '')
    ),
    'checkboxes' => array(
      'items' => array('Items', 'isNoHTML', TRUE, 'textarea', 6, '', 'value=caption')
    ),
    'combobox' => array(
      'items' => array('Items', 'isNoHTML', TRUE, 'textarea', 6, '', 'value=caption')
    ),
    'file' => array(
      'folder' => array('Folder', 'isNum', TRUE, 'function', 'getMediaDirectoryCombo'),
      'delcaption' => array('Caption: Delete button', 'isAlphaNumChar', TRUE, 'input', 30, '', 'Delete')
    ),
    'image' => array(
      'folder' => array('Folder', 'isNum', TRUE, 'function', 'getMediaDirectoryCombo'),
      'delcaption' => array('Caption: Delete button', 'isAlphaNumChar', TRUE, 'input', 30, '', 'Delete'),
      'width' => array('Width', 'isNum', TRUE, 'input', 4, '', 240),
      'height' => array('Height', 'isNum', TRUE, 'input', 4, '', 180)
    ),
    'radio' => array(
      'items' => array('Items', 'isNoHTML', TRUE, 'textarea', 6, '', 'value=caption')
    ),
    'textarea' => array(
      'rows' => array('Rows', 'isNum', TRUE, 'input', 3, '', 5)
    ),
    'url' => array(
      'maxlength' => array('Max length', 'isNum', TRUE, 'input', 3, '', 200),
      'linktext' => array('Link text', 'isAlphaNumChar', FALSE, 'input', 40, '', ''),
      'target' => array('Link target', 'isAlphaNumChar', FALSE, 'combo',
        array('_blank'=>'_blank','_parent'=>'_parent','_self'=>'_self','_top'=>'_top'),
        '', '_self')
    ),

  );

  /**
  * XML to fields
  *
  * @param string $xml
  * @access public
  */
  function xmlToFields($xml) {
    $xmlTree = $this->createXMLTree($xml);
    if (is_object($xmlTree) && is_array($xmlTree->children)) {
      foreach($xmlTree->children as $node) {
        if (is_object($node)) {
          switch ($node->name) {
            case 'group' :
              unset($group);
              if (isset($node->children) &&  is_array($node->children) &&
                  count($node->children) > 0) {
                $group = array(
                  'title' => isset($node->attributes['title']) ? $node->attributes['title'] : _gt('Group')
                );
                foreach($node->children as $fieldNode) {
                  unset($field);
                  if (is_object($fieldNode) && ($fieldNode->name == 'field')) {
                    if (isset($fieldNode->attributes['name'])) {
                      $field = array(
                        'group' => $group['title'],
                        'name' => @$fieldNode->attributes['name'],
                        'title' => @$fieldNode->attributes['title'],
                        'css' => @$fieldNode->attributes['css'],
                        'validatefunc' => @$fieldNode->attributes['validatefunc'],
                        'validateregex'=> @$fieldNode->attributes['validateregex'],
                        'needed' => @$fieldNode->attributes['needed'],
                        'type' => 'input',
                        'typeparams' => array()
                      );
                      if (isset($fieldNode->children) &&  is_array($fieldNode->children)) {
                        foreach($fieldNode->children as $typeNode) {
                          if (isset($this->types[$typeNode->name])) {
                            $field['type'] = $typeNode->name;
                            $fieldType = $this->types[$field['type']];
                            if (isset($typeNode->children) && is_array($typeNode->children)) {
                              foreach($typeNode->children as $paramNode) {
                                if (isset($fieldType[$paramNode->name])) {
                                  $s = '';
                                  $paramNode->printText($s);
                                  $field['typeparams'][$paramNode->name] = simple_xmltree::XMLToStr($s);
                                }
                              }
                            }
                            break;
                          }
                        }
                      }
                      $this->fields[$field['name']] = $field;
                      $group['fields'][] = $field['name'];
                    }
                  }
                }
                $this->groups[$group['title']] = $group;
              }
              break;
          }
        }
      }
    }
  }

  /**
  * Fields to XML
  *
  * @access public
  * @return string XML
  */
  function fieldsToXML() {
    $result = '<fields>'.LF;
    if (isset($this->groups) && is_array($this->groups) && isset($this->fields) && is_array($this->fields)) {
      foreach($this->groups as $group) {
        if (isset($group['fields']) && is_array($group['fields'])) {
          $result .= sprintf('<group title="%s">'.LF, papaya_strings::escapeHTMLChars($group['title']));
          foreach($group['fields'] as $fieldName) {
            if (isset($this->fields[$fieldName])) {
              $field = $this->fields[$fieldName];
              $result .= sprintf('<field name="%s" title="%s" css="%s" validatefunc="%s" validateregex="%s" needed="%d">'.LF,
                @papaya_strings::escapeHTMLChars($field['name']),
                @papaya_strings::escapeHTMLChars($field['title']),
                @papaya_strings::escapeHTMLChars($field['css']),
                @papaya_strings::escapeHTMLChars($field['validatefunc']),
                @papaya_strings::escapeHTMLChars($field['validateregex']),
                @(int)($field['needed'])
              );
              if (isset($field['type'])) {
                $result .= '<'.$field['type'].'>'.LF;
                if (isset($field['typeparams']) && is_array($field['typeparams'])) {
                  foreach($field['typeparams'] as $attributeName=>$attributeValue) {
                      $result .= '<'.$attributeName.'><![CDATA['.simple_xmltree::strToXML($attributeValue).']]></'.$attributeName.'>'.LF;
                  }
                }
                $result .= '</'.$field['type'].'>'.LF;
              }
              $result .= '</field>'.LF;
            }
          }
          $result .= '</group>'.LF;
        }
      }
    }
    $result .= '</fields>'.LF;
    return $result;
  }

  /**
  * Create a XML tree
  * @param string $xmlData XML-string
  * @return object simple_xmlnode $xmlTree XML-tree
  */
  function createXMLTree($xmlData) {
    if (trim($xmlData) != '') {
      $xmlTree = simple_xmltree::create();
      if (is_object($xmlTree)) {
        if (0 === $xmlTree->loadFromString($xmlData)) {
          if (is_object($xmlTree->root)) {
            return $xmlTree->root;
          }
        } elseif (defined('PAPAYA_DBG_XML_USERINPUT') && PAPAYA_DBG_XML_USERINPUT === '1') {
          $this->lastXMLError = @$xmlTree->lastError;
          unset($xmlTree);
        }
      }
    }
    return FALSE;
  }
}

?>