<?php
/**
* basic object to manage dynamic images
*
* @package papaya
* @author Thomas Weinert <weinert@papaya-cms.com>
* @version $Id: base_imagegenerator.php,v 1.6 2006/03/27 09:46:08 weinert Exp $
*/

/**
* Basic objekt with database access
*/
require_once(PAPAYA_INCLUDE_PATH.'system/sys_base_db.php');
/**
* basic object to manage dynamic images
*
*
* @package papaya
* @author Thomas Weinert <weinert@papaya-cms.com>
*/
class base_imagegenerator extends base_db {

  var $paramName = 'img';
  
  var $tableImageConfs = PAPAYA_DB_TBL_IMAGES;
  var $tableModules = PAPAYA_DB_TBL_MODULES;
  var $tableModuleGroups = PAPAYA_DB_TBL_MODULEGROUPS;
  
  var $lastError = '';
  
  /**
  * selected image configuration data
  * @var array $imageConfs
  */
  var $imageConf = NULL;
  
  function loadByIdent($imageIdent) {
    unset($this->imageConf);
    $sql ="SELECT i.image_id, i.image_ident, i.image_title, i.image_data,
                  m.module_guid, m.module_path, m.module_file, m.module_class, m.module_title
             FROM %s AS i
             LEFT OUTER JOIN %s AS m ON (m.module_guid = i.module_guid)
            WHERE i.image_ident = '%s'";
    if ($res = $this->databaseQueryFmt($sql, array($this->tableImageConfs, $this->tableModules, $imageIdent))) {
      if ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->imageConf = $row;
        return TRUE;
      }
    }
    return FALSE;
  }
  
  function generateImage($outputImage = TRUE, $params = NULL) {
    if (isset($this->imageConf) && isset($this->imageConf['module_guid'])) {
      $parent = NULL;
      $moduleObj = &$this->createPluginObject($this->imageConf['module_path'].$this->imageConf['module_file'],
        $this->imageConf['module_class'], $this->imageConf['module_guid'], $parent);
      if (isset($moduleObj) && is_object($moduleObj)) {
        $moduleObj->images = &$this->images;
        $moduleObj->paramName = $this->paramName;
        $moduleObj->setData($this->imageConf['image_data']);
        if (isset($params)) {
          $this->params = $params;
        } else {
          $this->initializeParams();
        }
        if ($moduleObj->setAttributes($this->params)) {
          $cacheId = $this->getCacheId($moduleObj->getCacheId());
          $cacheFileName = PAPAYA_PATH_CACHE.$cacheId;
          if (file_exists($cacheFileName)) {
            if ($outputImage) {
              $this->sendContentHeader();
              readfile($cacheFileName);
            }
            return $cacheFileName;
          } else {
            if ($image = $moduleObj->generateImage(&$this)) {
              if ($this->saveImage($image, $cacheFileName)) {
                if ($outputImage) {
                  $this->sendContentHeader();
                  $this->saveImage($image, NULL);
                }
                return $cacheFileName;
              }
            } else {
              $this->lastError = $moduleObj->lastError;
            }
          }
        } else {
          $this->lastError = 'Invalid image attributes';  
        }
      } else {
        $this->lastError = 'Invalid image module';      
      }
    } else {
      $this->lastError = 'Image data not found';
    }
    return FALSE;
  }
  
  function sendContentHeader() {
    switch (@PAPAYA_THUMBS_FILETYPE) {
    case 1 : 
      header('Content-type: image/gif'); break;
    case 2 : 
      header('Content-type: image/jpeg'); break;
    case 3 : 
      header('Content-type: image/png'); break;
    }
    header('Expires: '.gmdate("D, d M Y H:i:s", time() + @(int)PAPAYA_CACHE_TIME)." GMT");
  }
  
  function getCacheId($moduleCacheId) {
    $ext = '.jpg';
    switch (@PAPAYA_THUMBS_FILETYPE) {
    case 1 : $ext = '.gif'; break;
    case 2 : $ext = '.jpg'; break;
    case 3 : $ext = '.png'; break;
    }
    return 'image_'.$this->escapeForFilename($this->imageConf['image_ident']).'_'.$moduleCacheId.$ext;
  }
  
  /**
  * loads an image
  *
  * @param string $fileName Dateiname der Bilddatei
  * @return resource
  */
  function &loadImage($fileName) {
    list(,,$fileType) = @getimagesize($fileName);
    $result = FALSE;
    switch ($fileType) {
      case 1 :
        $result = @imagecreatefromGIF($fileName);
        break;
      case 2 :
        $result = @imagecreatefromJPEG($fileName);
        break;
      case 3 :
        $result = @imagecreatefromPNG($fileName);
        break;
    }
    return $result;
  }
  
  function saveImage($im, $fileName) {
    switch (@PAPAYA_THUMBS_FILETYPE) {
      case 1 :
        $saved = imageGIF($im, $fileName);
        break;
      case 2 :
        $saved = imageJPEG($im, $fileName, 60);
        break;
      case 3 :
        $saved = imagePNG($im, $fileName);
        break;
      default :
        $this->lastError('Image output format not supported.');
        return FALSE;
    }
    return TRUE;
  }
  
  /**
  * delete cache for an ident
  *
  * @param string $ident
  * @access public
  * @return integer
  */
  function deleteCache($ident) {
    $identStr = 'image_'.$this->escapeForFilename($ident).'_';
    $counter = 0;
    if ($dh = opendir(PAPAYA_PATH_CACHE)) {
      while($file = readdir($dh)) {
        if (is_file(PAPAYA_PATH_CACHE.$file) && strpos($file, $identStr) === 0) {
          unlink(PAPAYA_PATH_CACHE.$file);
          $counter++;
        }
      }
    }
    return $counter;
  }
  
  /**
  * load a media file
  *
  * @param $dataStr
  * @access public
  * @return resource Image resource
  */
  function &getMediaFileImage($dataStr) {
    $result = NULL;
    if (trim($dataStr) != '' && file_exists(PAPAYA_PATH_MEDIAFILES.$dataStr)) {
      $result = &$this->loadImage(PAPAYA_PATH_MEDIAFILES.$dataStr);
    }
    return $result;
  }
  
  function colorToRGB($colorStr) {
    if (preg_match('~#[a-fA-F\d]{6}~', $colorStr)) {
      $result['red'] = hexdec(substr($colorStr, 1, 2));
      $result['green'] = hexdec(substr($colorStr, 3, 2));
      $result['blue'] = hexdec(substr($colorStr, 5, 2));
    } elseif (preg_match('~#[a-fA-F\d]{3}~', $colorStr)) {
      $result['red'] = hexdec(str_repeat(substr($colorStr, 1, 1), 2));
      $result['green'] = hexdec(str_repeat(substr($colorStr, 2, 1), 2));
      $result['blue'] = hexdec(str_repeat(substr($colorStr, 3, 2), 2));    
    } else {
      $result['red'] = 0;
      $result['green'] = 0;
      $result['blue'] = 0;        
    }
    return $result;
  }
}
?>
