<?php
/**
* message service
*
* @package papaya
* @author Thomas Weinert <weinert@papaya-cms.com>
* @version $Id: base_messages.php,v 1.5 2006/01/12 15:30:56 weinert Exp $
*/


/**
* basic class database access
*/
require_once(PAPAYA_INCLUDE_PATH.'system/sys_base_db.php');

/**
* message service
*
* @author Thomas Weinert <weinert@papaya-cms.com>
* @package papaya
*/
class base_messages extends base_db {
  var $tableAuthUser = PAPAYA_DB_TBL_AUTHUSER;
  var $tableMessages = PAPAYA_DB_TBL_MESSAGES;

  var $paramName = 'msg';
  var $sessionParamName = 'msg';
  var $params = NULL;
  var $sessionParams = NULL;
  var $baseLink = '';

  var $layout = NULL;
  var $images = NULL;
  var $authUser = NULL;

  var $messageList = NULL;
  var $messageTree = NULL;
  var $users = NULL;

  var $inputFieldSize = 'xx-large';

  var $newMessageFields = array(
    'to'      =>array('RCPT', 'isSomeText', TRUE, 'textarea', 3),
    'cc'      =>array('CC', 'isSomeText', FALSE, 'textarea', 3),
    'bcc'     =>array('BCC', 'isSomeText', FALSE, 'textarea', 3),
    'priority'=>array('Priority', 'isNum', TRUE, 'combo', array(0=>'normal',1=>'high',2=>'urgent')),
    'subject' =>array('Subject', 'isSomeText', TRUE, 'input', 200),
    'message' =>array('Text', 'isSomeText', TRUE, 'textarea', 30),
    //'mailme'  =>array('E-Mail an Absender', 'isNum', TRUE, 'yesno', '', '', 0, 'center')
  );


  /**
  * Initialize parameters
  *
  * @param string $paramName optional, default value 'msg'
  * @access public
  */
  function initialize($paramName = 'msg') {
    $this->paramName = $paramName;
    $this->sessionParamName = 'PAPAYA_SESS_messages_'.$paramName;
    $this->initializeParams();
    $this->sessionParams = $this->getSessionValue($this->sessionParamName);
    $this->initializeSessionParam('folder_id');
    $this->setSessionValue($this->sessionParamName, $this->sessionParams);
  }

  /**
  * Basic class for handling parameters
  *
  * @access public
  */
  function execute() {
    $this->params['msg_id'] = (isset($this->params['msg_id'])) ? (int)$this->params['msg_id'] : 0;
    $this->loadMessage($this->params['msg_id']);
    if (isset($this->message) && is_array($this->message) && ((int)$this->message['msg_folder_id'] >= (-1)) && ($this->message['msg_new'])) {
      if ($this->updateReaded($this->params['msg_id'])) {
        $this->loadMessage($this->params['msg_id']);
      }
    }
    switch (@$this->params['cmd']) {
      case 'reply' :
        if (isset($this->message) && is_array($this->message)) {
          $this->loadUsers();
          $this->params['reply_to'] = $this->params['msg_id'];
          $this->params['to'] = sprintf('%s <%s>',
            $this->getUserEmailName($this->users[$this->message['msg_from']]), $this->message['msg_from']);
          $this->initializeNewMessageForm();
          $this->dialog->params['reply_to'] = $this->params['msg_id'];
          $this->dialog->params['subject'] = (strpos($this->message['msg_subject'], 'Re:') !== 0) ?
            'Re: '.$this->message['msg_subject'] : $this->message['msg_subject'];
          $this->dialog->params['to'] = sprintf('%s <%s>',
            $this->getUserEmailName($this->users[$this->message['msg_from']]), $this->message['msg_from']);
          $this->dialog->params['message'] =
            sprintf('%s schreib am %s:', $this->getUserEmailName($this->users[$this->message['msg_from']]),
              date('Y-m-d H:i:s'))."\n\n".
            $this->rewrapMessage($this->message['msg_text'], 1, 75, TRUE);
        }
        $this->params['cmd'] = 'new';
        break;
      case 'forward' :
        if (isset($this->message) && is_array($this->message)) {
          $this->loadUsers();
          $this->params['reply_to'] = $this->params['msg_id'];
          $this->initializeNewMessageForm();
          $this->dialog->params['reply_to'] = $this->params['msg_id'];
          $this->dialog->params['subject'] = (strpos($this->message['msg_subject'], 'Fw:') !== 0) ?
            'Fw: '.$this->message['msg_subject'] : $this->message['msg_subject'];
          $this->dialog->params['message'] =
            sprintf('%s schrieb am %s:', $this->getUserEmailName($this->users[$this->message['msg_from']]),
              date('Y-m-d H:i:s'))."\n\n".
            $this->rewrapMessage($this->message['msg_text'], 0, 75, TRUE);
        }
        $this->params['cmd'] = 'new';
        break;
      case 'new' :
        $this->initializeNewMessageForm();
        if ($this->dialog->modified()) {
          if ($this->dialog->checkDialogInput()) {
            $this->sendMessage();
          }
        }
        break;
      case 'del' :
        if ($this->deleteMessage()) {
            $this->loadMessage($this->params['msg_id']);
        }
        break;
    }

    $this->loadMessages();
    $this->loadUsers();
  }

  /**
  * Initialize session parameters
  *
  * @param $name
  * @param mixed $resetParams optional, default value NULL
  * @access public
  */
  function initializeSessionParam($name, $resetParams = NULL) {
    if (isset($this->params[$name])) {
      if (isset($resetParams) && is_array($resetParams) && ($this->params[$name] != $this->sessionParams[$name])) {
        foreach($resetParams as $paramName) {
          unset($this->params[$paramName]);
          unset($this->sessionParams[$paramName]);
        }
      }
      $this->sessionParams[$name] = $this->params[$name];
    } else {
      $this->params[$name] = $this->sessionParams[$name];
    }
  }

  /**
  * Load messages
  *
  * @access public
  */
  function loadMessages() {
    unset($this->messageList);
    $sql = "SELECT msg_id, msg_owner_id, msg_prev_id, msg_folder_id,
              msg_to, msg_from, msg_datetime, msg_subject,
              msg_priority, msg_type, msg_new, msg_rel_topic_id, msg_rel_box_id
            FROM %s
            WHERE (msg_owner_id = '%s') AND (msg_folder_id = '%s')
            ORDER BY msg_datetime DESC";
    $params = array($this->tableMessages,$this->authUser->userId, (int)$this->params['folder_id']);
    if ($res = $this->databaseQueryFmt($sql,$params)) {
      while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->messageList[$row['msg_id']] = $row;
      }
    }
  }

  /**
  * Load message thread
  *
  * @param integer $id
  * @access public
  */
  function loadMessageThread($id) {
    unset($this->messageThread);
    $sql = "SELECT msg_id, msg_owner_id, msg_prev_id, msg_folder_id,
              msg_to, msg_from, msg_datetime, msg_subject,
              msg_priority, msg_type, msg_new, msg_rel_topic_id, msg_rel_box_id
            FROM %s
            WHERE (msg_owner_id = '%s')
              AND ((msg_thread_id = '%s') OR (msg_id = '%s'))
              AND (msg_folder_id >= -1)
            ORDER BY msg_datetime DESC";
    $params = array($this->tableMessages,$this->authUser->userId, (int)$id,  (int)$id);
    if ($res = $this->databaseQueryFmt($sql,$params)) {
      while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->messageThread[$row['msg_id']] = $row;
      }
    }
  }

  /**
  * Load message
  *
  * @param integer $id
  * @access public
  */
  function loadMessage($id) {
    unset($this->message);
    $sql = "SELECT msg_id, msg_owner_id, msg_prev_id, msg_folder_id,
              msg_to, msg_from, msg_cc, msg_bcc, msg_datetime,
              msg_subject, msg_text,
              msg_priority, msg_type, msg_new, msg_rel_topic_id, msg_rel_box_id
            FROM %s
            WHERE (msg_id = '%d') AND (msg_owner_id = '%s')";
    $params = array($this->tableMessages, (int)$id, $this->authUser->userId);
    if ($res = $this->databaseQueryFmt($sql,$params)) {
      while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->message = $row;
      }
    }
  }

  /**
  * Load user
  *
  * @access public
  */
  function loadUsers() {
    unset($this->users);
    $sql = "SELECT user_id, surname, givenname, email
            FROM %s
            ORDER BY surname, givenname";
    if ($res = $this->databaseQueryFmt($sql,array($this->tableAuthUser))) {
      while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->users[$row['user_id']] = $row;
      }
    }
  }

  /**
  * Load thread id
  *
  * @param integer $id
  * @access public
  * @return integer
  */
  function loadThreadId($id) {
    $sql = "SELECT msg_prev_id, msg_thread_id
            FROM %s
            WHERE msg_id = '%d'";
    if ($res = $this->databaseQueryFmt($sql,array($this->tableMessages,(int)$id))) {
      while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        return (int)(($row['msg_prev_id'] > 0) ? $row['msg_thread_id'] : $id);
      }
    }
    return 0;
  }

  /**
  * Decode addresses
  *
  * @param string $str
  * @access public
  * @return mixed array or NULL
  */
  function decodeAddresses($str) {
    $result = NULL;
    if (preg_match_all('#\s*([^<]+?[^<>]+)\s*<([^>]+)>(\s*,\s*)?#', $str, $regs, PREG_SET_ORDER)) {
      foreach($regs as $reg) {
        if (preg_match('/^[A-Fa-f\d]{32}$/', $reg[2])) {
          $result[] = array('name'=>$reg[1],'user_id'=>$reg[2]);
        } elseif (checkit::isEMail($reg[2],TRUE)) {
          $result[] = array('name'=>$reg[1],'email'=>$reg[2]);
        } else {
          $result[] = array('name'=>$reg[1]);
        }
      }
    }
    return $result;
  }

  /**
  * Update readed
  *
  * @param integer $id
  * @access public
  * @return boolaen
  */
  function updateReaded($id) {
    return (FALSE !== $this->databaseUpdateRecord($this->tableMessages, array('msg_new'=>0), 'msg_id', (int)$id));
  }

  /**
  * Get XML
  *
  * @access public
  */
  function getXML() {
    if (is_object($this->layout)) {
      $this->layout->setParam('COLUMNWIDTH_LEFT', 100);
      $this->layout->setParam('COLUMNWIDTH_CENTER', 400);
      $this->layout->setParam('COLUMNWIDTH_RIGHT', '100%');
      $this->getFolderPanel();
      switch (@$this->params['cmd']) {
        case 'new' :
          $this->layout->setParam('COLUMNWIDTH_CENTER', '100%');
          $this->layout->setParam('COLUMNWIDTH_RIGHT', 0);
          $this->getAddressJSForm();
          $this->getNewMessageForm();
          break;
        default :
          $this->getMessageView();
          $this->getMessageThread();
          $this->getMessageList();
      }
      $this->getButtons();
    }
  }

  /**
  * Get folder panel
  *
  * @access public
  */
  function getFolderPanel() {
    $result = '<outlookbar align="left">';
    $href = $this->baseLink.'?'.$this->paramName.'[folder_id]=';
    $result .= sprintf('<icon src="%s" subtitle="%s" href="%s%s"/>', 'msgboxin.gif', $this->getFolderTitle(0),
      $href, 0);
    $result .= sprintf('<icon src="%s" subtitle="%s" href="%s%s"/>', 'msgboxout.gif', $this->getFolderTitle(-1),
      $href, '-1');
    $result .= sprintf('<icon src="%s" subtitle="%s" href="%s%s"/>', 'trash.gif', $this->getFolderTitle(-2),
      $href, '-2');
    $result .= '</outlookbar>';
    $this->layout->addLeft($result);
  }

  /**
  * get message list
  *
  * @access public
  */
  function getMessageList() {
    if (isset($this->messageList) && is_array($this->messageList)) {
      $result = sprintf('<listview title="%s" width="400">'.LF,
        papaya_strings::escapeHTMLChars($this->getFolderTitle((int)$this->params['folder_id'])));
      $result .= '<cols>';
      $result .= '<col>Betreff/Absender</col>';
      $result .= '<col/>';
      $result .= '<col align="center">Zeit</col>';
      $result .= '</cols>';
      $result .= '<items>'.LF;
      $href = $this->baseLink.'?'.$this->paramName.'[msg_id]=';
      foreach($this->messageList as $message) {
        $result .= sprintf('<listitem title="%s" subtitle="%s" image="%s" href="%s" %s>',
          papaya_strings::escapeHTMLChars($message['msg_subject']),
          papaya_strings::escapeHTMLChars(@$this->getUserEmailName($this->users[$message['msg_from']])),
          $this->images[$this->getMsgGlyphIndex($message['msg_type'], $message['msg_new'],
            ($message['msg_id'] == $this->params['msg_id']))],
          $href.(int)$message['msg_id'],
          (($message['msg_id'] == $this->params['msg_id']) ? ' selected="selected"' : ''));
        $result .= sprintf('<subitem align="center"><glyph src="%s"/></subitem>',
          $this->images[$this->getPriorityGlyphIndex($message['msg_priority'])]);
        $result .= '<subitem align="center">'.date('Y-m-d H:i:s', $message['msg_datetime']).'</subitem>';
        $result .= '</listitem>'.LF;
      }
      $result .= '</items>'.LF;
      $result .= '</listview>'.LF;
      $this->layout->add($result);
    }
  }

  /**
  * Get message thread
  *
  * @access public
  */
  function getMessageThread() {
    if (isset($this->message) && is_array($this->message)) {
      $this->loadMessageThread($this->loadThreadId($this->message['msg_id']));
    }
    if (isset($this->messageThread) && is_array($this->messageThread)) {
      $rootIds = NULL;
      foreach($this->messageThread as $msg) {
        if (isset($this->messageThread[$msg['msg_prev_id']])) {
          $this->messageThread[$msg['msg_prev_id']]['CHILDS'][] = $msg['msg_id'];
        } else {
          $rootIds[] = $msg['msg_id'];
        }
      }
      $result = sprintf('<listview title="%s" width="400">'.LF,
        papaya_strings::escapeHTMLChars('Thread'));
      $result .= '<items>'.LF;
      $result .= $this->getMessageSubThread($rootIds, 0);
      $result .= '</items>'.LF;
      $result .= '</listview>'.LF;
      $this->layout->add($result);
    }
  }

  /**
  * Get message sub thread
  *
  * @param integer $ids
  * @param integer $indent
  * @access public
  * @return string '' or XML
  */
  function getMessageSubThread($ids, $indent) {
    $result = '';
    if (isset($ids) && is_array($ids) && isset($this->messageThread) && is_array($this->messageThread)) {
      $hrefPattern = $this->baseLink.'?'.$this->paramName.'[msg_id]=%s&amp;'.$this->paramName.'[folder_id]=%s';
      foreach($ids as $id) {
        $message = $this->messageThread[$id];
        if (isset($message) && is_array($message)) {
          $result .= sprintf('<listitem title="%s" subtitle="%s" image="%s" href="%s" indent="%s" %s>',
            papaya_strings::escapeHTMLChars($message['msg_subject']),
            papaya_strings::escapeHTMLChars($this->getUserEmailName($this->users[$this->message['msg_from']])),
            $this->images[$this->getMsgGlyphIndex($message['msg_type'], $message['msg_new'],
              ($message['msg_id'] == $this->params['msg_id']))],
            sprintf($hrefPattern, (int)$message['msg_id'], (int)$message['msg_folder_id']),
            $indent,
            (($message['msg_id'] == $this->params['msg_id']) ? ' selected="selected"' : ''));
          $result .= sprintf('<subitem align="center"><glyph src="%s" hint="%s"/></subitem>',
            $this->getFolderGlyph($message['msg_folder_id']), $this->getFolderTitle($message['msg_folder_id']));
          $result .= '</listitem>'.LF;
          if (isset($message['CHILDS']) && is_array($message['CHILDS'])) {
            $result .= $this->getMessageSubThread($message['CHILDS'], $indent+1);
          }
        }
      }
    }
    return $result;
  }

  /**
  * Get message view
  *
  * @access public
  */
  function getMessageView() {
    if (isset($this->message) && is_array($this->message)) {
      $result = '<sheet width="100%" align="center">';
      $result .= '<header>';
      $result .= '<lines>';
      $result .= sprintf('<line class="%s">%s</line>', 'headertitle', papaya_strings::escapeHTMLChars($this->message['msg_subject']));
      $href = $this->baseLink.'?'.$this->paramName.'[cmd]=new&amp;'.$this->paramName.'[to]=';
      $result .= sprintf('<line class="%s">Von: <a href="%s" class="email">%s</a></line>', 'headersubtitle',
          $href.$this->message['msg_from'] , papaya_strings::escapeHTMLChars($this->getUserEmailName($this->users[$this->message['msg_from']])));
      $result .= sprintf('<line class="%s">An: %s</line>', 'headersubtitle',
          $this->getAddressesStr($this->decodeAddresses($this->message['msg_to']), TRUE));
      if (trim($this->message['msg_cc'])) {
        $result .= sprintf('<line class="%s">CC: %s</line>', 'headersubtitle',
            $this->getAddressesStr($this->decodeAddresses($this->message['msg_cc']), TRUE));
      }
      if (trim($this->message['msg_bcc'])) {
        $result .= sprintf('<line class="%s">BCC: %s</line>', 'headersubtitle',
            $this->getAddressesStr($this->decodeAddresses($this->message['msg_bcc']), TRUE));
      }
      $result .= '</lines>';
      $result .= '<infos>';
      $result .= sprintf('<line>%s</line>', date('Y-m-d H:i:s', $this->message['msg_datetime']));
      switch($this->message['msg_priority']) {
        case 2 :
          $result .= sprintf('<line><span style="color: #FF0000">%s</span></line>', $this->_gt('Urgent'));
          break;
        case 1 :
          $result .= sprintf('<line>%s</line>', $this->_gt('High'));
          break;
      }
      if (($this->message['msg_rel_topic_id'] > 0) || ($this->message['msg_rel_box_id'] > 0)) {
        $result .= '<line>';
        if ($this->message['msg_rel_topic_id'] > 0) {
          $result .= sprintf('<a href="topic.php?p_id=%d"><glyph src="%s" hint="%s"/></a>',
            $this->message['msg_rel_topic_id'], $this->images[45], $this->_gt('Goto page'));
        }
        if ($this->message['msg_rel_box_id'] > 0) {
          $result .= sprintf('<a href="boxes.php?p_bid=%d&amp;p_cmd=chg_show&amp;p_mode=1"><glyph src="%s" hint="%s"/></a>',
            $this->message['msg_rel_box_id'], $this->images[5], $this->_gt('Goto box'));
        }
        $result .= '</line>';
      }
      $result .= '</infos>';
      $result .= '</header>';
      $result .= '<text>';
      $result .= $this->formatMessage($this->message['msg_text']);
      $result .= '</text>';
      $result .= '</sheet>';
      $this->layout->addRight($result);
    }
  }

  /**
  * Get message glyph index
  *
  * @param integer $type
  * @param boolean $new
  * @param boolean $opened
  * @access public
  * @return integer
  */
  function getMsgGlyphIndex($type, $new, $opened) {
    if ($opened) {
      return 81;
    } elseif ($new) {
      switch($type) {

        default : return 82;
      }
    } else {
      switch($type) {

        default : return 80;
      }
    }
  }

  /**
  * Get priority glyph index
  *
  * @param integer $priority
  * @access public
  * @return integer
  */
  function getPriorityGlyphIndex($priority) {
    switch($priority) {
      case 2 : return 85;
      case 1 : return 84;
      default : return 0;
    }
  }

  /**
  * Get folder title
  *
  * @param integer $index
  * @access public
  * @return string
  */
  function getFolderTitle($index) {
    switch ($index) {
      case 0 : return $this->_gt('Incoming mails');
      case -1 : return $this->_gt('Outgoing mails');
      case -2 : return $this->_gt('Trash');
      default : return '';
    }
  }

  /**
  * Get folder glyph
  *
  * @param integer $index
  * @access public
  * @return mixed
  */
  function getFolderGlyph($index) {
    switch ($index) {
      case 0 : return $this->images[88];
      case -1 : return $this->images[89];
      case -2 : return $this->images[10];
      default : return '';
    }
  }

  /**
  * Get addresses string
  *
  * @param array $addr
  * @param boolean $linked optional, default value FALSE
  * @access public
  * @return string
  */
  function getAddressesStr($addr, $linked = FALSE) {
    if (isset($addr) && is_array($addr)) {
      $href = $this->baseLink.'?'.$this->paramName.'[cmd]=new&amp;'.$this->paramName.'[to]=';
      $result = '';
      foreach($addr as $item) {
        if ($linked) {
          if (isset($item['user_id'])) {
            $result .= sprintf('<a href="%s" class="email">%s</a>, ',
              $href.urlencode(sprintf('%s <%s>', $item['name'], $item['user_id'])) , papaya_strings::escapeHTMLChars($item['name']));
          } elseif (isset($item['email'])) {
            $result .= sprintf('<a href="mailto:%s" class="email">%s</a>, ',
              $item['email'] , papaya_strings::escapeHTMLChars($item['name']));
          } else {
            $result .= papaya_strings::escapeHTMLChars($item['name']).', ';
          }
        } else {
          $result .= papaya_strings::escapeHTMLChars($item['name']).', ';
        }
      }
      return substr($result, 0, -2);
    }
    return '';
  }

  /**
  * Get address combo
  *
  * @param array $list
  * @param string $name
  * @access public
  * @return string html
  */
  function getAddressCombo($list, $name) {
    $result = '<table border="0" cellpadding="0" cellspacing="0"><tr>';
    $result .= sprintf('<td><select name="%s[%s]" class="%s" id="addrcombo_%s">'.LF,
      $this->paramName, $name, 'shortinput', $name);
    if (isset($list) && is_array($list)) {
      foreach($list as $user) {
        $yourself = ($user['user_id'] == $this->authUser->userId) ? TRUE : FALSE;
        $value = sprintf('%s <%s>', $this->getUserEmailName($user), $user['user_id']);
        $result .= sprintf('<option value="%s" class="%s" %s>%s</option>'.LF,
          papaya_strings::escapeHTMLChars($value),
          'listitemlow', ($yourself) ? ' style="color: #FF0000;"' : '',
          papaya_strings::escapeHTMLChars($this->getUserEmailName($user)));
      }
    }
    $result .= '</select></td>';
    $result .= sprintf('<td><glyph src="%s"/></td>', $this->images[0]);
    $result .= sprintf('<td><button type="button" onclick="addAddress(\'%s\');" class="formbutton"><glyph src="%s"/><span style="font-size: 9px">%s</span></button></td>',
      $name, $this->images[90], $this->_gt('Add'));
    $result .= sprintf('<td><button type="button" onclick="delAddress(\'%s\');" class="formbutton"><glyph src="%s"/><span style="font-size: 9px">%s</span></button></td>',
      $name, $this->images[91], $this->_gt('Delete'));
    $result .= '</tr></table>';
    return $result;
  }

  /**
  * Get address line
  *
  * @param array $addrList
  * @param array $fieldName
  * @param string $caption
  * @access public
  * @return string XML
  */
  function getAddressLine($addrList, $fieldName, $caption) {
    $result = sprintf('<line caption="%s">'.LF, papaya_strings::escapeHTMLChars($this->_gt($caption)));
    $result .= $this->getAddressCombo($addrList, $fieldName);
    $result .= '</line>'.LF;
    return $result;
  }

  /**
  * address string to Javascript
  *
  * @param string $str
  * @param array $field
  * @access public
  * @return string
  */
  function addrStrToJs($str, $field) {
    $addrs = $this->decodeAddresses($str);
    $result = '';
    if (isset($addrs) && is_array($addrs)) {
      $result .= sprintf('addr["%s"] = new Array;'.LF, $field);
      foreach($addrs as $key=>$val) {
        $address = (isset($val['user_id'])) ? $val['user_id'] : $val['email'];
        $result .= sprintf('addr["%s"][%s] = "%s <%s>";'.LF, $field, $key, $val['name'], $address);
      }
    }
    return $result;
  }

  /**
  * Get address javascript formular
  *
  * @access public
  */
  function getAddressJSForm() {
    $result = sprintf('<dialog action="#" title="%s" width="400">', $this->_gt('Address book'));
    $alertMessage = $this->_gt('Address already added.');
    $addr = $this->addrStrToJs(@$this->params['to'], 'to');
    $addr .= $this->addrStrToJs(@$this->params['cc'], 'cc');
    $addr .= $this->addrStrToJs(@$this->params['bcc'], 'bcc');
    $result .= <<<JAVASCRIPT
<script type="text/javascript" language="JavaScript" src="./script/mail.js"></script>
<script type="text/javascript" language="JavaScript"><![CDATA[
var addr = new Array;
var msg_wasadded = '$alertMessage';
$addr
]]></script>
JAVASCRIPT;
    $result .= '<lines>';
    $result .= $this->getAddressLine($this->users, 'to', 'RCPT:');
    $result .= $this->getAddressLine($this->users, 'cc', 'CC:');
    $result .= $this->getAddressLine($this->users, 'bcc', 'BCC:');
    $result .= '</lines>';
    $result .= '</dialog>';
    $this->layout->add($result);
  }

  /**
  * Initialize new message formular
  *
  * @access public
  */
  function initializeNewMessageForm() {
    if (!(isset($this->dialog) && is_object($this->dialog))) {
      include_once(PAPAYA_INCLUDE_PATH.'system//base_dialog.php');
      $hidden = array(
        'cmd'=>'new',
        'save'=>1,
        'reply_to'=>(int)@$this->params['reply_to']);
      $data = NULL;
      $this->dialog = &new base_dialog($this, $this->paramName, $this->newMessageFields, $data, $hidden);
      $this->dialog->msgs = &$this->msgs;
      $this->dialog->dialogTitle = $this->_gt('New message');
      $this->dialog->dialogId = 'newmsg';
      $this->dialog->inputFieldSize = $this->inputFieldSize;
      $this->dialog->dialogDoubleButtons = TRUE;
      $this->dialog->loadParams();
    }
  }

  /**
  * Get new message formular
  *
  * @access public
  */
  function getNewMessageForm() {
    $this->initializeNewMessageForm();
    $this->layout->add($this->dialog->getDialogXML());
  }

  /**
  * Send message
  *
  * @access public
  */
  function sendMessage() {
    $addr['to'] = $this->decodeAddresses(@$this->params['to']);
    $addr['cc'] = $this->decodeAddresses(@$this->params['cc']);
    $addr['bcc'] = $this->decodeAddresses(@$this->params['bcc']);
    if ($this->checkAndCompleteAddresses($addr)) {
      $email = array(
        'to'      => $this->getEMailString(@$addr['to']),
        'cc'      => $this->getEMailString(@$addr['cc']),
        'bcc'     => $this->getEMailString(@$addr['bcc']),
        'subject' => $this->params['subject'],
        'message' => $this->rewrapMessage($this->params['message']),
      );
      $addHeaders = '';
      if (is_string($email['cc']) && $email['cc'] != '') $addHeaders .= sprintf("CC: %s\r\n", $email['cc']);
      if (is_string($email['bcc']) && $email['bcc'] != '') $addHeaders .= sprintf("BCC: %s\r\n", $email['bcc']);
      if (checkit::isEmail($this->authUser->user['email'], TRUE)) {
        $addHeaders .= 'From: '.$this->getUserEmailAddress($this->authUser->user);
      }
      if (mail($email['to'], $email['subject'], $email['message'], $addHeaders)) {
        $this->addMsg(MSG_INFO, $this->_gt('Emails sent.'));
      } else {
        $this->addMsg(MSG_WARNING, $this->_gt('Could not sent all emails.'));
      }
      $record = array(
        'msg_to'      => $this->params['to'],
        'msg_cc'      => $this->params['cc'],
        'msg_from'    => $this->authUser->userId,
        'msg_datetime'=> time(),
        'msg_subject' => $this->params['subject'],
        'msg_text'    => $this->params['message'],
        'msg_priority'=> $this->params['priority'],
        'msg_prev_id' => ($this->params['reply_to']),
        'msg_thread_id' => ($this->loadThreadId($this->params['reply_to']))
      );
      $this->saveMessage(@$addr['to'], $record);
      $this->saveMessage(@$addr['cc'], $record);
      $this->saveMessage(@$addr['bcc'], $record);

      //Postausgang
      $record['msg_owner_id'] = $this->authUser->userId;
      $record['msg_bcc'] =  $this->params['bcc'];
      $record['msg_folder_id'] =  (-1);
      if (!$this->databaseInsertRecord($this->tableMessages, 'msg_id', $record)) {
        $this->addMsg(MSG_ERROR,
          $this->_gt('Database error').' - '.$this->_gt('Message not saved.'));
      }
    } else {
      $this->addMsg(MSG_ERROR, $this->_gt('No valid recepient specified'));
    }
  }

  /**
  * Save message
  *
  * @param array $addrList
  * @param integer $data
  * @access public
  */
  function saveMessage($addrList, $data) {
    if (isset($addrList) && is_array($addrList)) {
      foreach($addrList as $addr) {
        if (isset($addr['user_id'])) {
          $data['msg_owner_id'] = $addr['user_id'];
          if (!$this->databaseInsertRecord($this->tableMessages, 'msg_id', $data)) {
            $this->addMsg(MSG_ERROR, $this->_gt('Database error'));
          }
        }
      }
    }
  }

  /**
  * Delete Message
  *
  * @access public
  * @return boolean
  */
  function deleteMessage() {
    if (isset($this->message) && is_array($this->message)) {
      if ($this->message['msg_folder_id'] != (-2)) {
        if ($this->databaseUpdateRecord($this->tableMessages, array('msg_folder_id'=>'-2'),
              'msg_id', $this->message['msg_id'])) {
          unset($this->params['msg_id']);
          return TRUE;
        } else {
          $this->addMsg(MSG_ERROR, $this->_gt('Database error').' - '.
            $this->_gtf('Could not move message "%s (%d)" to the trash folder.',
                papaya_strings::escapeHTMLChars($this->message['msg_subject']), $this->message['msg_id']));
        }
      } else {
        if ($this->databaseDeleteRecord($this->tableMessages, 'msg_id', $this->message['msg_id'])) {
          unset($this->params['msg_id']);
          return TRUE;
        } else {
          $this->addMsg(MSG_ERROR, $this->_gt('Database error').' - '.
            $this->_gtf('Could not delete message "%s (%d)".',
                papaya_strings::escapeHTMLChars($this->message['msg_subject']), $this->message['msg_id']));
        }
      }
    } else {
        $this->addMsg(MSG_ERROR, $this->_gt('No message selected.'));
    }
    return FALSE;
  }

  /**
  * Get email string
  *
  * @param array $addrList
  * @access public
  * @return string
  */
  function getEMailString($addrList) {
    $result = '';
    if (isset($addrList) && is_array($addrList)) {
      foreach($addrList as $addr) {
        $result .= sprintf('%s <%s>, ', $addr['name'], $addr['email']);
      }
      $result = substr($result, 0, -2);
    }
    return $result;
  }

  /**
  * Check and complete addresses
  *
  * @param array &$addressLists
  * @access public
  * @return boolean
  */
  function checkAndCompleteAddresses(&$addressLists) {
    $this->loadUsers();
    $result = FALSE;
    $resultAddresses = NULL;
    foreach($addressLists as $field=>$list) {
      if (isset($list) && is_array($list)) {
        foreach($list as $key=>$val) {
          if (isset($val['email'])) {
            $resultAddresses[$field][] = $val;
            if ($field == 'to') $result = TRUE;
          } elseif (isset($val['user_id']) && (isset($this->users[$val['user_id']])) &&
              checkit::isEmail($this->users[$val['user_id']]['email'], TRUE)) {
            $val['email'] = $this->users[$val['user_id']]['email'];
            $resultAddresses[$field][] = $val;
            if ($field == 'to') $result = TRUE;
          } else {
            $this->addMsg(MSG_WARNING, $this->_gtf('Invalid Recipient "%s".', $val['name']));
          }
        }
      }
    }
    $addressLists = $resultAddresses;
    return $result;
  }

  /**
  * Get buttons
  *
  * @access public
  */
  function getButtons() {
    include_once(PAPAYA_INCLUDE_PATH.'system/base_btnbuilder.php');
    $toolbar = &new base_btnbuilder;
    $toolbar->images = &$this->images;

    $toolbar->addButton('New message',
        $this->baseLink.'?'.$this->paramName.'[cmd]=new', 41, '', @($this->params['cmd'] == 'new'));
    $toolbar->addSeperator();
    if (isset($this->message) && is_array($this->message)) {
      $userFrom = $this->users[$this->message['msg_from']];
      $toolbar->addButton('Answer',
          $this->baseLink.'?'.$this->paramName.'[cmd]=reply'.'&amp;'.
          $this->paramName.'[msg_id]='.$this->message['msg_id'],
        86, '');
      $toolbar->addButton('Forward',
          $this->baseLink.'?'.$this->paramName.'[cmd]=forward'.'&amp;'.
          $this->paramName.'[msg_id]='.$this->message['msg_id'],
        87, '');
      $toolbar->addButton('Delete',
          $this->baseLink.'?'.$this->paramName.'[cmd]=del'.'&amp;'.$this->paramName.'[msg_id]='.$this->message['msg_id'],
        10, '');
    }

    if ($str = $toolbar->getXML()) {
      $this->layout->addMenu(sprintf('<menu>%s</menu>'.LF, $str));
    }
  }

  /**
  * Parse quotes
  *
  * @param string $str
  * @param boolean $addOnly optional, default value FALSE
  * @access public
  * @return mixed array or NULL
  */
  function parseQuotes($str, $addOnly = FALSE) {
    $result = NULL;
    if (preg_match_all('#^(>*)(([ \t]*)(.*))$#m', $str, $lines, PREG_SET_ORDER)) {
      if (isset($lines) && is_array($lines)) {
        $counter = -1;
        $lastIndent = array('index'=>-1, 'indent'=>-1);
        foreach($lines as $line) {
          if (trim($line[0]) == '') {
            $result[++$counter]['data'] = '';
            $result[$counter]['indent'] = -1;
          } else {
            $append = FALSE;
            $indent = strlen($line[1]);
            if (($lastIndent['indent'] >= 0) && ($indent >= 0)) {
              $spaceIndent = ($lastIndent['indent'] < $indent) ? $lastIndent['indent'] : $indent;
              for ($i=$counter;$i > $lastIndent['index']; $i--) {
                $result[$i]['indent'] = $spaceIndent;
              }
            }
            if ($counter >= 0) {
              if ($result[$counter]['indent'] == $indent) {
                if ((!$addOnly) && (!preg_match('#^(-|(\d{1,3}\.))#', $line[4]))) {
                  $append = TRUE;
                }
              }
            }
            if ($append) {
              $result[$counter]['data'] .= " ".trim($line[2]);
            } else {
              $result[++$counter]['data'] = trim($line[2]);
              $result[$counter]['indent'] = $indent;
              $lastIndent = array('index'=>$counter, 'indent'=>$indent);
            }
          }
        }
      }
    }
    return $result;
  }

  /**
  * Rewrap message
  *
  * @param string $str
  * @param integer $addIndent optional, default value 0
  * @param integer $lineLength optional, default value 75
  * @param boolean $addOnly optional, default value TRUE
  * @access public
  * @return string
  */
  function rewrapMessage($str, $addIndent = 0, $lineLength = 75, $addOnly = TRUE) {
    $result = '';
    $parsed = $this->parseQuotes($str, $addOnly);
    if (isset($parsed) && is_array($parsed)) {
      foreach($parsed as $para) {
        $indent = $para['indent']+$addIndent;
        $indentStr = ($indent > 0) ? (str_repeat('>', $indent).' ') : '';
        $result .= $indentStr.wordwrap($para['data'], $lineLength-$indent, "\n".$indentStr, 1).LF;
      }
      return substr($result, 0, -1);
    }
    return '';
  }

  /**
  * Per formatted
  *
  * @param string $str
  * @access public
  * @return string
  */
  function preformatted($str) {
    $result = (trim($str) != '') ? $str : "\n\n";
    $result = preg_replace('#(\r\n)|(\n\r)|[\r\n]#', '<br />', papaya_strings::escapeHTMLChars($result));
    $result = preg_replace('#(^|\s)\*([^\*/_<>]+)\*(\s|$)#', '\1<b>*\2*</b>\3', $result);
    $result = preg_replace('#(^|\s)/([^\*/_<>]+)/(\s|$)#', '\1<i>/\2/</i>\3', $result);
    $result = preg_replace('#(^|\s)_([^\*/_<>]+)_(\s|$)#', '\1<u>_\2_</u>\3', $result);
    return $result;
  }

  /**
  * format message
  *
  * @param string $str
  * @access public
  * @return string
  */
  function formatMessage($str) {
    $parsed = $this->parseQuotes($str, TRUE);
    if (isset($parsed) && is_array($parsed)) {
      $result = '<div style="margin: 0px 4px 2px 0px;">';
      $indent = 0;
      $style = sprintf('margin: 2px; border-left: 2px solid %s; color: %s;', PAPAYA_BGCOLOR, PAPAYA_BGCOLOR);
      $startStr = sprintf('<div style="%s padding-left: 10px;">'.LF, $style);
      $endStr = '</div>'.LF;
      $ignoreSpace = FALSE;
      foreach($parsed as $para) {
        if ($para['indent'] >= 0) {
          if ($para['indent'] < $indent) {
            $result .= str_repeat($endStr, $indent-$para['indent']);
            $ignoreSpace = TRUE;
          } elseif ($para['indent'] > $indent) {
            $result .= str_repeat($startStr, $para['indent'] - $indent);
          }
          $indent = $para['indent'];
        }
        if ($ignoreSpace && (trim($str) != '')) {
          $ignoreSpace = FALSE;
          $result .= '<br />';
        } else {
          $result .= $this->preformatted(' '.$para['data']);
        }
      }
      $result .= str_repeat($endStr, $indent);
      $result .= '</div>';
      return $result;
    }
    return '';
  }

  function getUserEmailAddress(&$userData) {
    if (isset($userData)) {
      return $this->getUserEmailName($userData).' <'.$userData['email'].'>';
    }
    return '';
  }

  function getUserEmailName(&$userData) {
    if (isset($userData)) {
      return strtr($userData['givenname'].' '.$userData['surname'], ',;', '');
    }
    return '';

  }
}

?>
