<?php
/**
* Select view class
*
* @package papaya
* @author Thomas Weinert <weinert@papaya-cms.com>
* @version $Id: base_selectview.php,v 1.7 2006/04/20 18:24:00 weinert Exp $
*/


/**
* Basic class database access
*/
require_once(PAPAYA_INCLUDE_PATH.'system/sys_base_db.php');
/**
* Select view class
*
* Show a listview with views
*
* @package papaya
* @author Thomas Weinert <weinert@papaya-cms.com>
*/
class base_selectview extends base_db {

  var $tableViews = PAPAYA_DB_TBL_VIEWS;
  var $tableModules = PAPAYA_DB_TBL_MODULES;
  var $tableModuleGroups = PAPAYA_DB_TBL_MODULEGROUPS;

  var $views;
  var $moduleGroups;
  var $moduleGroupViews;

  var $moduleType = '';
  var $currentView = NULL;

  var $actionLink = '';

  function load($viewId, $moduleType) {
    $this->moduleType = $moduleType;
    $this->loadModuleGroups();
    $this->loadViews();
    $this->loadView($viewId);
  }

  /**
  * Load view details
  *
  * @param $viewId
  * @access public
  * @return boolean
  */
  function loadView($viewId) {
    unset($this->currentView);
    $sql = "SELECT v.view_id, v.view_title,
                    m.module_guid, m.module_title, m.module_file,
                    m.modulegroup_id
              FROM %s AS v, %s AS m
              WHERE m.module_guid = v.module_guid
                AND m.module_active = 1
                AND v.view_id = '%d'";
    if ($res = $this->databaseQueryFmt($sql, array($this->tableViews, $this->tableModules, $viewId))) {
      if($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->currentView = $row;
        return TRUE;
      }
    }
    return FALSE;
  }

  /**
  * Load view and module information
  *
  * @access public
  */
  function loadViews() {
    unset($this->views);
    $sql = "SELECT v.view_id, v.view_title,
                    m.module_guid, m.module_title, m.module_description,
                    m.modulegroup_id
              FROM %s AS v, %s AS m
              WHERE m.module_guid = v.module_guid
                AND m.module_type = '%s'
                AND m.module_active = 1
              ORDER BY v.view_title, m.module_title";
    if ($res = $this->databaseQueryFmt($sql, array($this->tableViews, $this->tableModules, $this->moduleType))) {
      while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->views[$row['view_id']] = $row;
        $this->moduleGroupViews[$row['modulegroup_id']][$row['view_id']] = &$this->views[$row['view_id']];
      }
    }
  }

  /**
  * Load all module groups
  *
  * @access public
  */
  function loadModuleGroups() {
    unset($this->moduleGroups);
    $sql = "SELECT modulegroup_id, modulegroup_title FROM %s ORDER BY modulegroup_title";
    if ($res = $this->databaseQueryFmt($sql, $this->tableModuleGroups)) {
      while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->moduleGroups[$row['modulegroup_id']] = $row;
      }
    }
  }

  function getXMLViewList() {
    if(isset($this->views) && is_array($this->views) && isset($this->moduleGroups) && is_array($this->moduleGroups)) {
      $result = sprintf('<listview title="%s">'.LF, $this->_gt('Modules/Views'));
      $result .= '<items>'.LF;
      foreach($this->moduleGroups as $groupId=>$group) {
        if (isset($this->moduleGroupViews[$groupId]) && is_array($this->moduleGroupViews[$groupId])) {
          $result .= sprintf('<listitem title="%s" image="%s">'.LF, papaya_strings::escapeHTMLChars($group['modulegroup_title']),
            papaya_strings::escapeHTMLChars($this->images[57]));
          $result .= '<subitem/><subitem/>'.LF;
          $result .= '</listitem>'.LF;
          foreach($this->moduleGroupViews[$groupId] as $viewId=>$view) {
            $selected = (isset($this->currentView['view_id']) && $viewId == $this->currentView['view_id'])
              ? ' selected="selected"' : '';
            $href = $this->actionLink.(int)$viewId;
            $result .= sprintf('<listitem title="%s" href="%s" indent="1" hint="%s" image="%s"%s>'.LF,
              papaya_strings::escapeHTMLChars($view['view_title']), $href,
              papaya_strings::escapeHTMLChars($view['module_description']), papaya_strings::escapeHTMLChars($this->images[92]), $selected);
            $result .= sprintf('<subitem>%s</subitem>'.LF, papaya_strings::escapeHTMLChars($view['module_title']));
            $result .= sprintf('<subitem><a href="%s"><glyph src="%s" alt="%s"/></a></subitem>'.LF,
              $href, papaya_strings::escapeHTMLChars($this->images[11]),
              papaya_strings::escapeHTMLChars($this->_gt('Select')));
            $result .= '</listitem>'.LF;
          }
        }
      }
      $result .= '</items>'.LF;
      $result .= '</listview>'.LF;
      return $result;
    } else {
      $this->addMsg(MSG_ERROR, $this->_gt('No views definied.'));
      return '';
    }
  }

  /**
  * generate list of selected views (similar to active boxes)
  *
  * @todo determine whether this is the correct place for this function or whether it should go to actbox_page_tatistic or elsewhere
  * @see actbox_page_statistic::getModulesList()
  * @param array $displayViews list of view_ids to select
  * @param boolean $displaySelected if true, shows selected views, else it shows all views NOT selected
  * @param string $result XML string of listview
  */
  function getXMLMultipleViewsList($displayViews, $displaySelected) {
    if(isset($this->views) && is_array($this->views) && isset($this->moduleGroups) && is_array($this->moduleGroups)) {
      $result = sprintf('<listview title="%s">'.LF, $this->_gt('Modules/Views'));
      $result .= '<items>'.LF;
      foreach($this->moduleGroups as $groupId=>$group) {
        $resultGroup = '';
        if (isset($this->moduleGroupViews[$groupId]) && is_array($this->moduleGroupViews[$groupId])) {
          $resultGroup .= sprintf('<listitem title="%s" image="%s">'.LF, papaya_strings::escapeHTMLChars($group['modulegroup_title']),
            papaya_strings::escapeHTMLChars($this->images[57]));
          $resultGroup .= '<subitem/><subitem/>'.LF;
          $resultGroup .= '</listitem>'.LF;
          foreach($this->moduleGroupViews[$groupId] as $viewId=>$view) {
            if (($displaySelected && is_array($displayViews) && in_array($viewId, $displayViews)) ||
                (!$displaySelected && (!is_array($displayViews) || !in_array($viewId, $displayViews)))) {
              $displayGroup = TRUE;
              $href = $this->actionLink.(int)$viewId;
              $resultGroup .= sprintf('<listitem title="%s" href="%s" indent="1" hint="%s" image="%s">'.LF,
                papaya_strings::escapeHTMLChars($view['view_title']), $href,
                papaya_strings::escapeHTMLChars($view['module_description']), papaya_strings::escapeHTMLChars($this->images[92]));
              $resultGroup .= sprintf('<subitem>%s</subitem>'.LF, papaya_strings::escapeHTMLChars($view['module_title']));
              if ($displaySelected) {
                $resultGroup .= sprintf('<subitem><a href="%s"><glyph src="%s" alt="%s"/></a></subitem>'.LF,
                  $href, papaya_strings::escapeHTMLChars($this->images[10]),
                  papaya_strings::escapeHTMLChars($this->_gt('Delete')));
              } else {
                $resultGroup .= sprintf('<subitem><a href="%s"><glyph src="%s" alt="%s"/></a></subitem>'.LF,
                  $href, papaya_strings::escapeHTMLChars($this->images[11]),
                  papaya_strings::escapeHTMLChars($this->_gt('Select')));
              }
                $resultGroup .= '</listitem>'.LF;
            }
          }
        }
        if (@$displayGroup) {
          $result .= $resultGroup;
          $displayGroup = FALSE;
        }
      }
      $result .= '</items>'.LF;
      $result .= '</listview>'.LF;
      return $result;
    } else {
      $this->addMsg(MSG_ERROR, $this->_gt('No views definied.'));
      return '';
    }
  }

}
?>
