<?php
/**
* Basic statistic object with logging
*
* @package papaya
* @author David Rekowski <rekowski@papaya-cms.com>
* @version $Id: base_statistic.php,v 1.85 2006/05/30 09:16:07 rekowski Exp $
*/


/**
* Basic class object
*/
require_once(PAPAYA_INCLUDE_PATH.'system/sys_base_db.php');

/**
* Basic statistic object
* @package papaya
* @author David Rekowski <rekowski@papaya-cms.com>
*/
class base_statistic extends base_db {
  /**
  * @var array $requestData data from parseRequestURI
  */
  var $requestData = array();

  var $options = NULL;

  function __construct() {
    parent::__construct();
    $this->tableStatisticOptions       = PAPAYA_DB_TABLEPREFIX.'_statistic_options';
    $this->tableStatisticRequests      = PAPAYA_DB_TABLEPREFIX.'_statistic_requests';
    $this->tableStatisticPages         = PAPAYA_DB_TABLEPREFIX.'_statistic_pages';
    $this->tableStatisticExitPages     = PAPAYA_DB_TABLEPREFIX.'_statistic_exitpages';
    $this->tableStatisticMediaRequests = PAPAYA_DB_TABLEPREFIX.'_statistic_media_requests';
  }

  function logExitPage($requestId, $url) {
    if (isset($requestId) && $url != '' && $this->loggable()) {
      $params = array('statistic_request_id' => $requestId, 'statistic_exitpage_url' => $url);
      $this->databaseInsertRecord($this->tableStatisticExitPages, NULL, $params);
    }
  }

  /**
  * logs page request
  * @return integer $requestId papaya_statistic_requests.statistic_request_id for use of reference with moduleactions
  */
  function logRequest($requestData, $lngId = 0, $cachedPage = FALSE) {
    $requestId = NULL;
    $this->requestData = $requestData;
    if ($this->loggable()) {
      $protocol = (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] == 'on') ? 'https' : 'http';

      if (defined("PAPAYA_STATISTIC_PRESERVE_IP") && PAPAYA_STATISTIC_PRESERVE_IP) {
        $remoteAddress = $_SERVER['REMOTE_ADDR'];
      } else {
        $remoteAddress = md5($_SERVER['REMOTE_ADDR']);
      }
      if (defined('PAPAYA_STATISTIC_PRESERVE_USERID') && PAPAYA_STATISTIC_PRESERVE_USERID &&
          isset($_SESSION['papaya_surfer_handle']) && $_SESSION['papaya_surfer_handle'] != 'NULL') {
        $userId = $this->getUserIdByHandle($_SESSION['papaya_surfer_handle']);
      } else {
        $userId = '';
      }
      $sid = (isset($GLOBALS['PAPAYA_SESSION']) && isset($GLOBALS['PAPAYA_SESSION']->sessionId)) ? $GLOBALS['PAPAYA_SESSION']->sessionId : '';

      $dataRequest['statistic_ip']        = $remoteAddress;
      $dataRequest['statistic_country']   = $this->getCountryByIP($_SERVER['REMOTE_ADDR']);
      $dataRequest['statistic_sid']       = $sid;
      $dataRequest['statistic_useragent'] = @(string)$_SERVER['HTTP_USER_AGENT'];
      $dataRequest['statistic_time']      = time();
      $dataRequest['statistic_uri']       = $protocol.'://'.$_SERVER['HTTP_HOST'].$_SERVER['REQUEST_URI'];
      $dataRequest['statistic_referer']   = @(string)$_SERVER['HTTP_REFERER'];
      $dataRequest['user_id']             = $userId;

      switch ($this->requestData['output']) {
      case 'image':
        break;
      case 'media':
        $dataRequest['statistic_media_mode'] = $this->requestData['ext'];
        $dataRequest['statistic_media_id']   = $this->requestData['media_id'];
        $this->databaseInsertRecord($this->tableStatisticMediaRequests, NULL, $dataRequest);
        return 0;
        break;
      case 'page':
      default:
        if ($this->requestData['page_id'] == NULL) {
          $dataPage['topic_id'] = PAPAYA_PAGEID_DEFAULT;
        } else {
          $dataPage['topic_id'] = $this->requestData['page_id'];
        }
        $dataPage['lng_id']             = $lngId;
        $dataPage['categ_id']           = $this->requestData['categ_id'];
        $dataPage['statistic_mode']     = $this->requestData['ext'];
        $dataPage['statistic_filename'] = $this->requestData['filename'];
        $sql = "SELECT statistic_page_id
                FROM %s
                WHERE topic_id = '%d' AND
                      lng_id = '%d' AND
                      categ_id = '%d' AND
                      statistic_mode = '%s' AND
                      statistic_filename = '%s'";
        $params = array($this->tableStatisticPages,
          $dataPage['topic_id'], $dataPage['lng_id'], $dataPage['categ_id'],
          $dataPage['statistic_mode'], $dataPage['statistic_filename']);
        if ($res = $this->databaseQueryFmt($sql, $params)) {
          if ($row = $res->fetchRow()) {
            $dataRequest['statistic_page_id'] = $row[0];
          } else {
            $dataRequest['statistic_page_id'] = $this->databaseInsertRecord($this->tableStatisticPages, 'statistic_page_id', $dataPage);
          }
        }
        if ($cachedPage) {
          $this->databaseInsertRecord($this->tableStatisticRequests, NULL, $dataRequest);
          return 0;
        } else {
          $requestId = $this->databaseInsertRecord($this->tableStatisticRequests, 'statistic_request_id', $dataRequest);
        }
        break;
      }
    }
    return $requestId;
  }

  function getCountryByIP($ip) {
    if (isset($ip) && $ip != '') {
      if (include_once(PAPAYA_INCLUDE_PATH.'external/geoip/geoip.inc')) {
        $gi = geoip_open(PAPAYA_INCLUDE_PATH.'external/geoip/geoip.dat', GEOIP_STANDARD);
        $country = geoip_country_code_by_addr($gi, $ip);
        geoip_close($gi);
        return $country;
      }
    }
    return FALSE;
  }

  /**
  * checks whether request is valid for logging (e.g. not favicon, no preview, not in list of ignored ips/hosts)
  * @return boolean TRUE if loggable, FALSE if should be ignored
  */
  function loggable() {
    if (defined('PAPAYA_ADMIN_PAGE') && PAPAYA_ADMIN_PAGE) {
      return FALSE;
    } elseif ($this->requestData['preview'] !== FALSE) {
      return FALSE;
    } elseif ($this->requestData['filename'] == 'favicon') {
      return FALSE;
    } else {
      if (!(isset($this->options) && is_array($this->options) && count($this->options) > 0)) {
        $this->loadConfiguration();
      }
      if (isset($this->ignoredIps) && is_array($this->ignoredIps) && isset($this->ignoredIps[$_SERVER['REMOTE_ADDR']])) {
        return FALSE;
      } elseif ($this->requestData['output'] == 'media' && isset($this->ignoredExtensions) && is_array($this->ignoredExtensions) &&
        isset($this->ignoredExtensions[substr($this->requestData['filename'], strrpos($this->requestData['filename'], '.')+1 )])) {
        return FALSE;
      }
    }
    return TRUE;
  }

  function loadConfiguration() {
    unset($this->options);
    unset($this->ignoredIps);
    unset($this->ignoredExtensions);
    $params = array($this->tableStatisticOptions);
    $sql = "SELECT statistic_option_id, statistic_option_name, statistic_option_value, statistic_option_title
            FROM %s
            ORDER BY statistic_option_name
            ";
    if ($res = $this->databaseQueryFmt($sql, $params)) {
      while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        switch ($row['statistic_option_name']) {
          case 'ignore_ip':
            $this->ignoredIps[$row['statistic_option_value']] = 1;
          break;
          case 'ignore_media_extension':
            $this->ignoredExtensions[$row['statistic_option_value']] = 1;
          break;
          default:
          break;
        }
        $this->options[] = $row;
      }
    }
  }

  function getUserIdByHandle($handle) {
    /* SQL Modifications
      ALTER TABLE `papaya_statistic_requests` CHANGE `user_id` `user_id` BIGINT DEFAULT NULL
      ALTER TABLE `papaya_statistic_media_requests` CHANGE `user_id` `user_id` BIGINT DEFAULT NULL
    */
    $id = '';
    if (include_once(PAPAYA_INCLUDE_PATH.'/modules/_base/community/base_surfers.php')) {
      if ($surfer = &new surfer_admin()) {
        $id = $surfer->getIdByHandle($handle);
      }
    }
    return $id;
  }

}
?>