<?php
/**
* Basic object for topic navigation
*
* @author Thomas Weinert <weinert@papaya-cms.com>
* @package papaya
* @version $Id: base_topic_navigation.php,v 1.4 2006/01/23 12:03:38 weinert Exp $
*/

/**
* Basic class database access
*/
require_once(PAPAYA_INCLUDE_PATH.'system/sys_base_db.php');
/**
* Basic object for topic navigation
* @author Thomas Weinert <weinert@papaya-cms.com>
* @package papaya
*/
class base_topic_navigation extends base_db {

  var $tableTopics = PAPAYA_DB_TBL_TOPICS;
  var $tableTopicsTrans = PAPAYA_DB_TBL_TOPICS_TRANS;

  /**
  * base topic navigation
  *
  * @param object base_topic &$topic
  * @param boolean $fields optional, default value FALSE
  * @access public
  */
  function __construct(&$topic, $fields=FALSE) {
    $this->topicObj = &$topic;
    $this->tableTopics = $this->topicObj->tableTopics;
    $this->tableTopicsTrans = $this->topicObj->tableTopicsTrans;
    $this->data = array(
      'root' => (int)$this->topicObj->topicPrev,
      'sort' => $fields['sort']
    );
  }

  function base_topic_navigation(&$topic, $fields = FALSE) {
    $this->__construct($topic, $fields);
  }

  /**
  * Initialize surfer
  *
  * @access public
  */
  function initializeSurfer() {
    if ($this->useSurfer) {
      include_once(PAPAYA_INCLUDE_PATH.'system/base_surfer.php');
      if (isset($GLOBALS['PAPAYA_SURFER'])) {
        $this->surfer = &$GLOBALS['PAPAYA_SURFER'];
      } else {
        $this->surfer = &new base_surfer(TRUE);
        if ($this->surfer->isValid) {
          $this->surfer->loadTopicIdList(TRUE);
        }
        $GLOBALS['PAPAYA_SURFER'] = &$this->surfer;
      }
    }
  }

  /**
  * Load navigation
  *
  * @param integer $parentTopicId active topic-ID
  * @return boolean
  */
  function load($lngId, $desc=FALSE) {
    unset($this->prevId);
    unset($this->nextId);
    unset($this->firstId);
    unset($this->lastId);
    $order = ($desc) ? 'DESC' : 'ASC';
    $sql = "SELECT t.topic_id, tt.topic_title, t.topic_created,
                   t.topic_weight, t.is_navigation
              FROM %s AS t
              LEFT OUTER JOIN %s AS tt ON (t.topic_id = tt.topic_id AND tt.lng_id = '%d')
             WHERE t.prev = '%d'
             ORDER BY t.topic_weight $order, t.topic_created $order";
    $params = array($this->tableTopics, $this->tableTopicsTrans, $lngId, $this->data['root']);
    if ($res = $this->databaseQueryFmt($sql,$params)) {
      while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        if ($this->checkFilter($row['topic_id'], $row)) {
          $this->topics[$row['topic_id']] = $row;
          if ($row['topic_id'] == $this->topicObj->topicId) {
            $this->prevId = $dummy['topic_id'];
          }
          if ($dummy['topic_id'] ==  $this->topicObj->topicId) {
            $this->nextId = $row['topic_id'];
          }
          $dummy = end($this->topics);
        }
      }
      $dummy = reset($this->topics);
      if (isset($this->prevId) && ($this->prevId > 0) && ($dummy['topic_id'] != $this->prevId)) {
        $this->firstId = $dummy['topic_id'];
      }
      $dummy = end($this->topics);
      if (isset($this->nextId) && ($this->nextId > 0) && ($dummy['topic_id'] != $this->nextId)) {
        $this->lastId = $dummy['topic_id'];
      }
      return TRUE;
    }
    return FALSE;
  }

  /**
  * Check filter
  *
  * @param integer $id
  * @param array &$row
  * @access public
  * @return boolean
  */
  function checkFilter($id, &$row) {
    if (isset($row) && is_array($row)) {
      if ($this->useSurfer && is_object($this->surfer)) {
        if (!$this->surfer->canView($id)) {
          return FALSE;
        }
      }
      return TRUE;
    }
    return FALSE;
  }


  /**
  * Get XML
  *
  * @access public
  * @return string
  */
  function getXML() {
    //Basis laden
    if ($this->load($this->topicObj->getContentLanguageId(), $this->data['sort'])) {
      //eventuell Surfer laden
      $this->initializeSurfer();
      return $this->getSitemap();

    }
    return '';
  }

  /**
  * Get Sitemap
  *
  * @access public
  * @return string
  */
  function getSitemap() {
    $result = sprintf('<sitemap title="%s">'.LF,
      papaya_strings::escapeHTMLChars($this->data['title']));
    if (isset($this->topics) && is_array($this->topics)) {
      if (isset($this->firstId) && ($this->firstId > 0)) {
        $result .= $this->getItem($this->firstId, 'first');
      }
      if (isset($this->prevId) && ($this->prevId > 0)) {
        $result .= $this->getItem($this->prevId, 'prev');
      }
      if (isset($this->nextId) && ($this->nextId > 0)) {
        $result .= $this->getItem($this->nextId, 'next');
      }
      if (isset($this->lastId) && ($this->lastId > 0)) {
        $result .= $this->getItem($this->lastId, 'last');
      }
    }
    $result .= '</sitemap>'.LF;
    return $result;
  }

  /**
  * Get item
  *
  * @param integer $id
  * @param $pos
  * @access public
  * @return string
  */
  function getItem($id, $pos) {
    $result .= '';
    $row = $this->topics[$id];
    if (isset($row) && is_array($row)) {
      if ($id == $this->$this->topicObj->topicId) {
        $focus = ' focus="focus"';
      } else {
        $focus = '';
      }
      $result .= sprintf('<mapitem id="%d" href="%s" title="%s" position="%s" visible="1" %s>'.LF,
        $id, $this->getLink(array('page_id'=>$id)),
        papaya_strings::escapeHTMLChars($row['topic_title']), $pos, $focus);
      $result .= '</mapitem>'.LF;
    }
    return $result;

  }
}

?>