<?php
/**
* Create topic list
*
* @author Thomas Weinert <weinert@papaya-cms.com>
* @package papaya
* @version $Id: base_topiclist.php,v 1.5 2006/04/20 18:24:00 weinert Exp $
*/


/**
* Basic class database access
*/
require_once(PAPAYA_INCLUDE_PATH.'system/sys_base_db.php');

/**
* Create topic list
*
* @author Thomas Weinert <weinert@papaya-cms.com>
* @package papaya
*/
class base_topiclist extends base_db {
  var $tableTopics = "";
  var $tableTopicsTrans = "";
  var $topics;
  var $parentTopicId = 0;
  var $lngId = 0;

  /**
  * Constructor
  *
  * @access public
  */
  function __construct() {
    $this->baseLink = $this->getBaseLink();
  }

  /**
  * Load list from database
  *
  * @param $parentTopicId
  * @param boolean $publishedOnly optional, default value TRUE
  * @param boolean $desc optional, default value FALSE
  * @param integer $max optional, default value 1000
  * @access public
  */
  function loadList($parentTopicId, $lngId, $publishedOnly=TRUE, $desc=FALSE, $max = 1000) {
    $this->lngId = $lngId;
    $iPrevTopicId = (integer) $parentTopicId;
    $this->parentTopicId = $iPrevTopicId;
    if ($desc) {
      $order = "DESC";
    } else {
      $order = "ASC";
    }
    $sql = "SELECT t.topic_id, t.prev, t.topic_created, t.topic_weight,
                   tt.topic_title 
              FROM %s AS t, %s AS tt
              WHERE t.prev = '%s' AND tt.topic_id = t.topic_id AND tt.lng_id = %d
              ORDER BY t.topic_weight $order, topic_created $order";
    $params = array($this->tableTopics,$this->tableTopicsTrans, $iPrevTopicId, $lngId);
    if ($res = $this->databaseQueryFmt($sql, $params, $max)) {
      while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->topics[$row["topic_id"]] = $row;
      }
    }
  }

  /**
  * Get List
  *
  * @param object base_topic $topicClass
  * @param integer $max optional, default value 1000
  * @access public
  * @return XML
  */
  function getList($topicClass, $max=1000) {
    $result = '';
    if (isset($this->topics) && is_array($this->topics)) {
      $counter = 0;
      $result .= '<subtopics>';
      foreach($this->topics as $topicId=>$topicData) {
        if (isset($GLOBALS['PAPAYA_PAGE']) && is_object($GLOBALS['PAPAYA_PAGE']))  {
          if (!$GLOBALS['PAPAYA_PAGE']->validateAccess($topicId)) {
            continue;
          }
        }
        if (class_exists($topicClass)) {
          $topic = &new $topicClass();
          if (is_object($topic) && method_exists($topic, 'loadOutput')) {
            $topic->loadOutput($topicId, $this->lngId);
            if ($topic->topicId) {
              if ($str = $topic->parseContent(FALSE)) {
                $result .= $str.LF;
                $counter++;
                if ($counter >= $max) {
                  break;
                }
              }
            }
          }
        }
      }
      $result .= '</subtopics>';
    }
    return $result;
  }
}
?>