<?php
/**
* Basic topic tree object
*
* @author Thomas Weinert <weinert@papaya-cms.com>
* @package papaya
* @version $Id: base_topictree.php,v 1.14 2006/05/30 13:13:25 weinert Exp $
*/


/**
* Basic class database access
*/
require_once(PAPAYA_INCLUDE_PATH.'system/sys_base_db.php');
/**
* Basic class check conditions
*/
require_once(PAPAYA_INCLUDE_PATH.'system/sys_checkit.php');
/**
* Basic class XSL
*/
require_once(PAPAYA_INCLUDE_PATH.'system/sys_xsl.php');
/**
* Basic topic tree object
* @author Thomas Weinert <weinert@papaya-cms.com>
* @package papaya
*/
class base_topic_tree extends base_db {
  var $paramName = '';
  var $sessionParamName = '';
  var $params;
  var $baseLink = '';
  var $topics;
  var $subtopicCount;
  var $opened;

  var $maxIndent = 50;

  /**
  * Constructor
  *
  * @param string $paramName optional, default value 'tt'
  * @access public
  */
  function __construct($paramName='tt') {
    $this->paramName = $paramName;
    $this->sessionParamName = 'PAPAYA_SESS_'.$paramName;
  }

  /**
  * Initialize parameters
  *
  * @param mixed $id optional, default value NULL
  * @access public
  */
  function initialize($id=NULL) {
    $this->initializeParams();
    $this->sessionParams = $this->getSessionValue($this->sessionParamName);

    if (isset($this->params['page_id']) && ($this->params['page_id'] > 0)) {
      $this->sessionParams['page_id'] = (int)$this->params['page_id'];
    } elseif (isset($id) && ($id > 0)) {
      $this->params['page_id'] = (int)$id;
      $this->sessionParams['page_id'] = (int)$this->params['page_id'];
    } elseif (isset($this->sessionParams['page_id'])) {
      $this->params['page_id'] = (int)$this->sessionParams['page_id'];
    } else {
      $this->params['page_id'] = 0;
    }
    $this->topicId = $this->params['page_id'];

    $this->setSessionValue($this->sessionParamName, $this->sessionParams);
  }

  /**
  * execute - basic function for handling parameters
  *
  * @access public
  */
  function execute() {
    if (!(($this->topicId) && ($this->hasParent($this->authUser->user['start_node'])))) {
      if ($this->params['page_id'] != $this->authUser->user['start_node']) {
        $protocol = (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] == 'on') ? 'https' : 'http';
        //Zurck auf Los
        if ($this->authUser->user['start_node'] > 0) {
          $toUrl = $protocol."://".$_SERVER['HTTP_HOST'].$this->getBasePath().
            $this->getLink(array('page_id'=>$this->authUser->user['start_node']));
          $this->sessionParams['page_id'] = $this->authUser->user['start_node'];
        } else {
          $toUrl = $protocol."://".$_SERVER['HTTP_HOST'].$this->getBasePath().$this->baseLink;
          $this->sessionParams['page_id'] = 0;
        }
        $this->setSessionValue($this->sessionParamName, $this->sessionParams);
        header("Location: $toUrl");
        exit;
      }
    }

    $this->opened = @$this->sessionParams['opened'];
    if ($this->params['page_id'] == 0) {
      $this->openSubIds((int)$this->params['page_id']);
    }
    switch (@$this->params['cmd']) {
      case 'open' :
        if ($this->params['page_id'] > 0) {
          $this->opened[$this->params['page_id']] = TRUE;
        }
        break;
      case 'close' :
        if (isset($this->opened[$this->params['page_id']])) {
          unset($this->opened[$this->params['page_id']]);
        }
        break;
      case 'openall' :
        $this->openAll((int)$this->params['page_id']);
        break;
      case 'regenerate' :
        if ($this->authUser->hasPerm(33)) {
          $this->regeneratePath();
        }
        break;
      case 'refreshpages' :
        if ($this->authUser->hasPerm(37)) {
          $this->refreshPages();
        }
        break;
    }
    $this->openPrevs($this->topicId);
    $this->sessionParams['opened'] = $this->opened;
    $this->setSessionValue($this->sessionParamName, $this->sessionParams);
  }

  /**
  * Open sub ID's
  *
  * @param integer $baseId
  * @access public
  */
  function openSubIds($baseId) {
    $sql = "SELECT topic_id, topic_weight, topic_created
            FROM %s AS t
            WHERE t.prev = '%s'
            ORDER BY topic_weight ASC, topic_created ASC";
    $params = array($this->tableTopics,$baseId);
    if ($res = $this->databaseQueryFmt($sql,$params)) {
      while ($row = $res->fetchRow()) {
        $this->opened[$row[0]] = TRUE;
      }
    }
  }

  /**
  * Load
  *
  * @param integer $baseId
  * @access public
  */
  function load($baseId, $lngId) {
    unset($this->topics);
    unset($this->topicLinks);
    //tt.topic_trans_created, tt.topic_trans_modified,
    $baseSql = "SELECT t.topic_id, t.prev, t.prev_path, t.is_navigation,
                    t.topic_weight, t.topic_created, t.topic_modified, t.author_id,
                    t.is_navigation, t.is_deleted,
                    tt.lng_id, tt.topic_title,
                    ttp.topic_modified AS topic_published
              FROM %s AS t
              LEFT OUTER JOIN %s AS tt ON (tt.topic_id = t.topic_id AND tt.lng_id = %d)
              LEFT OUTER JOIN %s AS ttp ON (ttp.topic_id = t.topic_id)
              WHERE t.topic_id > 0 \n";
    $params = array($this->tableTopics,$this->tableTopicsTrans, (int)$lngId,$this->tableTopicsPublic);
    $prevs = array();
    if (isset($this->opened)) {
      foreach($this->opened as $id=>$active) {
        $prevs[(int)$id] = $active;
      }
    }
    $prevs[$baseId] = TRUE;
    $prevs = array_keys($prevs);
    $sql = $baseSql;
    if (isset($prevs) && is_array($prevs) && count($prevs) > 1) {
      $sql .= "AND t.prev IN ('".implode("','", $prevs)."') \n";
    } else {
      $sql .= "AND t.prev = '".(int)$baseId."' \n";
    }
    if (!$this->authUser->hasPerm(38)) {
      $sql .= "AND t.is_deleted = 0 \n";
    }
    $sql .= "ORDER BY t.topic_weight ASC, t.topic_created ASC";
    if ($res = $this->databaseQueryFmt($sql,$params)) {
      while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        if ($row['is_deleted']) {
          $row['topic_status'] = 4; //deleted
        } elseif (isset($row['topic_published']) && $row['topic_published'] < $row['topic_modified']) {
          $row['topic_status'] = 3; //published and modified
        } elseif (isset($row['topic_published']) && $row['topic_published'] >= $row['topic_modified']) {
          $row['topic_status'] = 2; //published and up to date
        } else {
          $row['topic_status'] = 1; //created
        }
        $this->topics[$row['topic_id']] = $row;
        $this->topicLinks[$row['prev']]['childs'][] = $row['topic_id'];
      }
    }

    if ($baseId > 0) {
      $sql = $baseSql;
      $sql = $sql." AND t.topic_id = '".(int)$baseId."'";
      if ($res = $this->databaseQueryFmt($sql,$params)) {
        if ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
          if ($row['is_deleted']) {
            $row['topic_status'] = 4; //deleted
          } elseif (isset($row['topic_published']) && $row['topic_published'] < $row['topic_modified']) {
            $row['topic_status'] = 3; //published and modified
          } elseif (isset($row['topic_published']) && $row['topic_published'] >= $row['topic_modified']) {
            $row['topic_status'] = 2; //published and up to date
          } else {
            $row['topic_status'] = 1; //created
          }
          $this->topics[$row['topic_id']] = $row;
          $this->topicLinks[$row['prev']]['childs'][] = $row['topic_id'];
        }
      }
    }
    if (isset($this->topics) && is_array($this->topics)) {
      $sql = "SELECT prev, COUNT(topic_id) as tcount FROM %s GROUP BY prev";
      if ($res = $this->databaseQueryFmt($sql,array($this->tableTopics))) {
        while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
          $this->subtopicCount[$row['prev']] = $row['tcount'];
        }
      }
      $this->optimizeList($baseId);
    }
  }

  /**
  * Is entry parent of $id ?
  *
  * @param integer $id
  * @access public
  * @return boolean
  */
  function hasParent($id) {
    $iId = (integer) $id;
    if ($iId == 0) return TRUE;

    if (isset($this->topics[$this->topicId])) {
      $topic = $this->topics[$this->topicId];
    } else {
      $sql = "SELECT topic_id, prev, prev_path
              FROM %s
              WHERE topic_id = %d";
      $params = array($this->tableTopics, $this->topicId);
      if ($res = $this->databaseQueryFmt($sql, $params)) {
        if ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
          $topic = $row;
        } else {
          return FALSE;
        }
      } else {
        return FALSE;
      }
    }

    if (preg_match_all('/\d+/', $topic['prev_path'], $regs, PREG_PATTERN_ORDER)) {
      $ids = array_flip($regs[0]);
    } else {
      return FALSE;
    }

    if (isset($ids[$iId]) || ($topic['prev'] == $iId) || ($this->topicId == $iId) || ($iId == 0)) {
      $result = TRUE;
    } else {
      $result = FALSE;
    }

    return $result;
  }

  /**
  * Load simply all
  *
  * @access public
  */
  function loadSimplyAll($lngId) {
    unset($this->topics);
    unset($this->topicLinks);
    if ($this->authUser->hasPerm(38)) {
      $filter = '';
    } else {
      $filter = "WHERE t.is_deleted = 0";
    }
    $sql = "SELECT t.topic_id, t.prev, t.prev_path, t.is_navigation,
                    t.topic_weight, t.topic_created, t.topic_modified,
                    t.is_navigation, t.is_deleted,
                    tt.lng_id, tt.topic_title,
                    tp.topic_modified AS topic_published
              FROM %s AS t
              LEFT OUTER JOIN %s AS tt ON (tt.topic_id = t.topic_id AND tt.lng_id = %d)
              LEFT OUTER JOIN %s AS tp ON (tp.topic_id = t.topic_id)
              $filter
            ORDER BY t.topic_weight ASC, t.topic_created ASC, tt.topic_title ASC";
    $params = array($this->tableTopics,$this->tableTopicsTrans, $lngId, $this->tableTopicsPublic);
    if ($res = $this->databaseQueryFmt($sql,$params)) {
      while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $id = (int)$row['topic_id'];
        if ($id > 0) {
          $prev = (int)$row['prev'];
          $this->topics[$id] = $row;
          if (isset($this->topicLinks[$prev]['childs']) && is_array($this->topicLinks[$prev]['childs'])) {
            $this->topicLinks[$prev]['childs'][] = $id;
          } else {
            $this->topicLinks[$prev]['childs'] = array($id);
          }
        }
      }
    }
  }

  /**
  * Regenerate path
  *
  * @access public
  */
  function regeneratePath() {
    $sql = "SELECT t.topic_id, t.prev, t.prev_path,
                    tp.topic_id AS public_topic_id,
                    tp.prev AS public_prev,
                    tp.prev_path AS public_prev_path
              FROM %s AS t
              LEFT OUTER JOIN %s AS tp ON t.topic_id = tp.topic_id
              ORDER BY t.topic_id";
    if ($res = $this->databaseQueryFmt($sql,array($this->tableTopics, $this->tableTopicsPublic))) {
      while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->nodes[$row['topic_id']] = $row;
      }
    }
    if (isset($this->nodes) && is_array($this->nodes)) {
      reset($this->nodes);
      $counter = 0;
      while (list($key) = each($this->nodes)) {
        $this->generatePrevPath($key);
        if (isset($this->nodes[$key]['new_path'])) {
          $newPath = $this->nodes[$key]['new_path'].';';
        } else {
          $newPath = ';';
        }
        $data = array('prev_path' => $newPath, 'prev' => (int)$this->nodes[$key]['prev']);
        if ($newPath != $this->nodes[$key]['prev_path']) {
          if (FALSE !== $this->databaseUpdateRecord($this->tableTopics, array('prev_path' => $newPath), 'topic_id', $key)) {
            if ($this->nodes[$key]['public_topic_id'] > 0) {
              $this->databaseUpdateRecord($this->tableTopicsPublic, $data, 'topic_id', $key);
            }
            $counter++;
          }
        } elseif ($this->nodes[$key]['public_topic_id'] > 0 && (
            $newPath != $this->nodes[$key]['public_prev_path'] ||
            $this->nodes[$key]['public_prev'] != $this->nodes[$key]['prev'])) {
          if (FALSE !== $this->databaseUpdateRecord($this->tableTopicsPublic, $data, 'topic_id', $key)) {
            $counter++;
          }
        }
      }
      if ($counter > 0) {
        $this->addMsg(MSG_INFO, sprintf($this->_gt('%s paths repaired.'), $counter));
      } else {
        $this->addMsg(MSG_INFO, $this->_gt('Path index checked.'));
      }
    }
    unset($this->nodes);
  }

  /**
  * Generate preview path
  *
  * @param integer $id
  * @access public
  * @return string
  */
  function generatePrevPath($id) {
    if (isset($this->nodes[$id]) && (!isset($this->nodes[$id]['new_path']))) {
      if (isset($this->nodes[$this->nodes[$id]['prev']])) {
        $parentNode = $this->nodes[$this->nodes[$id]['prev']];
        $this->nodes[$id]['new_path'] = $this->generatePrevPath($this->nodes[$id]['prev']).";".@(int)$parentNode['prev'];
      }
    }
    if (isset($this->nodes[$id]['new_path'])) {
      return $this->nodes[$id]['new_path'];
    } else {
      return '';
    }
  }

  /**
  * Open previews
  *
  * @param integer $id
  * @access public
  */
  function openPrevs($id) {
    $sql = "SELECT prev, prev_path
                  FROM %s
                  WHERE topic_id = '%d'";
    $params = array($this->tableTopics,$id);
    if ($res = $this->databaseQueryFmt($sql,$params)) {
      if ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        if (preg_match_all('/\d+/',$row['prev_path'],$matches, PREG_PATTERN_ORDER)) {
          $prevs = $matches[0];
        }
        $prevs[] = $row['prev'];
        $prevs = array_unique($prevs);
      }
    }
    if (isset($prevs) && is_array($prevs)) {
      foreach($prevs as $prevId) {
        if (!@$this->opened[$prevId]) {
          $this->opened[$prevId] = TRUE;
        }
      }
    }
  }

  /**
  * Open all
  *
  * @param integer $id optional, default value 0
  * @access public
  */
  function openAll($id=0) {
    if ($id == 0) {
      $sql = "SELECT DISTINCT(prev)
                FROM %s";
      $params = array($this->tableTopics);
      if ($res = $this->databaseQueryFmt($sql,$params)) {
        while ($row = $res->fetchRow()) {
            $this->opened[$row[0]] = TRUE;
          }
      }
    } else {
      $sql = "SELECT prev_path, prev FROM %s WHERE topic_id = '%d'";
      $params = array($this->tableTopics,$id);
      if ($res = $this->databaseQueryFmt($sql,$params)) {
        if ($temp = $res->fetchRow()) {
          if ($prev > 0) {
            $prevPath = str_replace(';;',';',';'.$temp[0].$temp[1].';');
          } else {
            $prevPath = str_replace(';;',';',';0;'.$temp[1].';'.$id.';');
          }
          $sql = "SELECT topic_id
                  FROM %s
                  WHERE (prev = '%s') OR (prev_path LIKE '%s%%')";
          $params = array($this->tableTopics, $id, $prevPath);
          if ($res = $this->databaseQueryFmt($sql,$params)) {
            while ($row = $res->fetchRow()) {
              $this->opened[$row[0]] = TRUE;
            }
          }
          $this->opened[$id] = TRUE;
        }
      }
    }
  }

  /**
  * Connect all childs to the parents build and array
  * of entries ids with no parents.
  * @param interger $baseId ID of Top Entry
  * @return array list of base entry ids
  */
  function optimizeList($baseId=0) {
    $result = FALSE;
    if (isset($this->topics) && is_array($this->topics)) {
      foreach($this->topics as $key=>$val) {
        if (isset($this->topics[$val["prev"]])) {/*
          $this->topics[$val["prev"]]["CHILDREN"][] = $key;
          $this->topics[$val["prev"]]["CHILDCOUNT"]++; */
          if (preg_match_all("/([\d]+)/",$val["prev_path"],$prevReg)) {
            array_unshift($prevReg[1],$val["prev"]);
            $this->topics[$key]["ALLPREV"] = $prevReg[1];
          } else {
            $this->topics[$key]["ALLPREV"] = array($val["prev"]);
          }
        } elseif ((int)$val["prev"] == $baseId) {
          $result[] = $key;
        } else {
          $this->topics[$key]= FALSE;
        }
      }
    }
    $this->rootTopics = $result;
    return $result;
  }

  /**
  * Get XML
  *
  * @access public
  * @return string
  */
  function getXML() {
    $result = '<items>';
    if (isset($this->topicLinks) && is_array($this->topicLinks)) {
      $result .= $this->getXMLSubTree(0,0);
    }
    $result .= '</items>';
    return $result;
  }

  /**
  * Get XML sub tree
  *
  * @param integer $id
  * @param integer $indent
  * @access public
  * @return string
  */
  function getXMLSubTree($id, $indent) {
    $result = '';
    if (isset($this->topicLinks[$id]['childs']) && is_array($this->topicLinks[$id]['childs']) && $indent < $this->maxIndent) {
      foreach($this->topicLinks[$id]['childs'] as $cid) {
        $result .= $this->getXMLElement($cid,$indent);
        $result .= $this->getXMLSubTree($cid,$indent+1);
      }
    }
    return $result;
  }

  /**
  * Get XML element
  *
  * @param integer $id
  * @param integer $indent
  * @access public
  * @return string
  */
  function getXMLElement($id, $indent) {
    $result = '';
    $topic = $this->topics[$id];
    if (isset($topic) && is_array($topic)) {
      if ($topic['is_navigation']) {
        $visible = 'TRUE';
      } else {
        $visible = 'FALSE';
      }
      if ($topic['topic_published']) {
        if ($topic['topic_published'] >= $topic['topic_modified']) {
          $public = 1;
        } else {
          $public = 2;
        }
      } else {
        $public = 0;
      }

      if (isset($topic['topic_title']) && trim($topic['topic_title']) != '') {
        $title = papaya_strings::escapeHTMLChars($topic['topic_title']);
      } else {
        $title = '';
      }
      $result .= sprintf('<item id="%d" prev="%d" indent="%s" visible="%s" public="%s" title="%s" modified="%s" published="%s" weight="%s" />',
        $topic['topic_id'], $topic['prev'], $indent, $visible, $public,
        papaya_strings::escapeHTMLChars($title),
        date('d.m.Y H:i',$topic['topic_modified']), date('d.m.Y H:i',$topic['topic_published']),
        papaya_strings::escapeHTMLChars($topic['topic_weight']));
    }
    return $result;
  }

  /**
  * Get buttons
  *
  * @access public
  * @return mixed boolean or array
  */
  function getButtons() {
    $result = FALSE;
    if ($this->authUser->hasPerm(33)) {
      $result[] = array('Check index', 'Check and correct path index', $this->images[73], 33,
        $this->baseLink.'?'.$this->paramName.'[cmd]=regenerate',
        '_self', FALSE);
    }
    if ($this->authUser->hasPerm(37)) {
      $result[] = array('Empty cache', 'Empty output cache', $this->images[10], 33,
        $this->baseLink.'?'.$this->paramName.'[cmd]=refreshpages',
        '_self', FALSE);
    }
    return $result;
  }

  /**
  * Refresh pages
  *
  * @access public
  */
  function refreshPages() {
    if (file_exists(PAPAYA_PATH_CACHE)  && ($dh = opendir(PAPAYA_PATH_CACHE))) {
      $counter = 0;
      while(is_string($file = readdir($dh))) {
        if (preg_match('#^\.\d+_\d+_[a-z]{0,4}_[a-f\d]{32}\.([a-z]+)(\.gz)?$#i', $file)) {
          unlink(PAPAYA_PATH_CACHE.$file);
          $counter++;
        } elseif (preg_match('#^\.box_\d+_\d+_\d+_[a-f\d]{32}_[a-f\d]{32}\.\d+$#i', $file)) {
          unlink(PAPAYA_PATH_CACHE.$file);
          $counter++;        
        }
      }
      if ($counter > 0) {
        $this->addMsg(MSG_INFO, sprintf($this->_gt('%s files deleted.'), $counter));
      } else {
        $this->addMsg(MSG_INFO, $this->_gt('Cache was empty - no files deleted,'));
      }
      closedir($dh);
    } else {
      $this->addMsg(MSG_WARNING, $this->_gt('Can not find cache directory.'));
    }
  }

  /**
  * Is topic editable
  * @param integer $topicId
  * @access public
  * @return boolean
  */
  function topicEditable($topicId) {
    $result = FALSE;
    if (isset($this->topics[$topicId])) {
      $topic = $this->topics[$topicId];
      if (
        //own entry or other allowed
        (($topic['author_id'] == $this->authUser->userId) || ($this->authUser->hasPerm(2))) &&
        //Indent level ok or all allowed
        ((count(@$topic['ALLPREVS']) <= $this->authUser->subLevel) || ($this->authUser->subLevel == 0)))
      {
        //go!
        $result = TRUE;
      }
    }
    return $result;
  }
}

?>