<?php
/**
* Implementation common database access
*
* @package papaya_base
* @author Thomas Weinert <weinert@papaya-cms.com>
* @version $Id: base.php,v 1.7 2006/06/06 11:35:42 weinert Exp $
*/

/**
* available BASIS-class
*/
require_once(PAPAYA_INCLUDE_PATH.'system/sys_base_object.php');

/**
* Data record as array with fieldname and Indizes
*/
define("DB_FETCHMODE_DEFAULT", 0);

/**
* Data record as numeric array
*/
define("DB_FETCHMODE_ORDERED", 1);

/**
* Data record with fieldname as array key
*/
define("DB_FETCHMODE_ASSOC", 2);

/**
* DB-abstraction layer - connection objekt
*
* @package papaya_base
* @author Thomas Weinert <weinert@papaya-cms.com>
*/
class dbcon_base extends base_object {
  /**
  * @var array $databaseConfiguration Configuration
  * @access private
  */
  var $databaseConfiguration;

  /**
  * @var array $databaseConnection Connection-ID
  * @access public
  */
  var $databaseConnection = NULL;

  /**
  * @var object dbresult_base $lastResult last database result
  * @access public
  */
  var $lastResult;

  /**
  * @var string $lastSQLQuery last Query
  * @access public
  */
  var $lastSQLQuery = '';

  /**
  * Constructor
  *
  * @param array &$conf Konfiguration
  * @return void
  * @access public
  */
  function __construct($conf) {
    $this->databaseConfiguration = $conf;
  }
  
  function extensionFound() {
    return FALSE;
  }

  /**
  * connect to Database
  *
  * @return boolean Erfolg
  * @access public
  */
  function connect() {
    return FALSE;
  }

  /**
  * Supply error
  *
  * @return array error data
  * @access public
  */
  function getError() {
    return NULL;
  }

  /**
  * String ecsaping for database
  *
  * @param mixed $value Value to escape
  * @return string escaped value.
  */
  function escapeStr($value) {
    return '';
  }

  /**
  * Database request
  *
  * @param string $sql SQL-string
  * @param integer $max data record limit
  * @param integer $offset startindex - data record limit
  * @param boolean $debug Debug Query?
  * @access public
  * @return mixed FALSE or number of affected_rows or database result object
  */
  function &query($sql, $max=NULL, $offset=NULL, $debug = FALSE) {
    $this->lastSQLQuery = '';
    $result = FALSE;
    return $result;
  }

  /**
  * Insert new record
  *
  * @param string $table table
  * @param string $idField Index column
  * @param array $values values
  * @access public
  * @return mixed FALSE or Id of new record
  */
  function insertRecord($table, $idField, $values = NULL) {
    $this->lastSQLQuery = '';
    return FALSE;
  }

  /**
  * Insert many records at once
  *
  * @param string $table tablen
  * @param array $values values
  * @access public
  * @return mixed FALSE or Id of new record
  */
  function insertRecords($table, $values) {
    $this->lastSQLQuery = '';
    return FALSE;
  }

  function getSQLCondition($filter) {
    if (is_array($filter)) {
      $str = '';
      if (count($filter) > 0) {
        $op = '';
        foreach($filter as $field=>$value) {
          $fieldValue = @strtoupper(trim($value));
          if (is_int($field) && ($fieldValue == 'OR' || $fieldValue == 'AND')) {
            if ($str != '') $op = $fieldValue;
          } else {
            if (is_bool($value)) {
              $str .= $op.' '.$this->escapeStr($field)." = '".(int)$value."' ";
            } elseif (is_array($value)) {
              $subValues = array();
              foreach($value as $subValue) {
                $subValues[] = $this->escapeStr($subValue);
              }
              if (count($subValues) > 1) {
                $str .= $op.' '.$this->escapeStr($field)." IN ('".implode("', '", $subValues)."') ";
              } else {
                $str .= $op.' '.$this->escapeStr($field)." = '".$subValues[0]."' ";
              }
            } else {
              $str .= $op.' '.$this->escapeStr($field)." = '".$this->escapeStr($value)."' ";
            }
            $op = 'AND';
          }
        }
      }
      if ($str != '') {
        return $str;
      } else {
        return '1=0';
      }
    } elseif (is_string($filter)) {
      return $filter;
    } elseif (!isset($filter)) {
      return '1=1';
    } else {
      return '1=0';
    }
  }

  /**
  * Change database records
  *
  * @param string $table Table
  * @param array $values values
  * @param string $filter condition
  * @access public
  * @return mixed FALSE or number of affected_rows or database result object
  */
  function updateRecord($table, $values, $filter) {
    $this->lastSQLQuery = '';
    return FALSE;
  }

  /**
  * delete database record
  *
  * @param string $table table
  * @param string $filter condition
  * @access public
  * @return mixed FALSE or number of affected_rows or database result object
  */
  function deleteRecord($table, $filter) {
    $this->lastSQLQuery = '';
    return FALSE;
  }

  /**
  * Get all table names
  *
  * @access public
  * @return array
  */
  function queryTableNames() {
    return array();
  }

  /**
  * table structur as array
  *
  * @param string $tableName table name
  * @param string $tablePrefix Prefix
  * @access public
  * @return array
  */
  function queryTableStructure($tableName, $tablePrefix = '') {
    return array();
  }

  /**
  * Create table
  *
  * @param string &$tableData
  * @param string $tablePrefix
  * @access public
  * @return boolean
  */
  function createTable(&$tableData, $tablePrefix) {
    return FALSE;
  }

  /**
  * Add field
  *
  * @param string $table
  * @param array &$fieldData
  * @access public
  * @return boolean
  */
  function addField($table, &$fieldData) {
    return FALSE;
  }

  /**
  * Change field
  *
  * @param string $table
  * @param array &$fieldData
  * @access public
  * @return boolean
  */
  function changeField($table, &$fieldData) {
    return FALSE;
  }

  /**
  * Delete field
  *
  * @param string $table
  * @param string $field
  * @access public
  * @return boolean
  */
  function dropField($table, $field) {
    return FALSE;
  }

  /**
  * Add index
  *
  * @param string $table
  * @param array &$index
  * @access public
  * @return boolean
  */
  function addIndex($table, &$index) {
    return FALSE;
  }

  /**
  * Change index
  *
  * @param string $table
  * @param array &$index
  * @param boolean $dropCurrent
  * @access public
  * @return boolean
  */
  function changeIndex($table, &$index, $dropCurrent = TRUE) {
    return FALSE;
  }

  /**
  * Delete index
  *
  * @param string $table
  * @param string $name
  * @access public
  * @return boolean
  */
  function dropIndex($table, $name) {
    return FALSE;
  }

  /**
  * DBMS spezific SQL source
  *
  * @param $function sql function
  * @param $params params
  * @access public
  * @return mixed sql string or FALSE
  */
  function getSQLSource($function, $params) {
    return FALSE;
  }


  /**
  * Compare the field structure
  *
  * @param &$xmlField
  * @param &$databaseField
  * @access public
  * @return Different TRUE/FALSE
  */
  function compareFieldStructure(&$xmlField, &$databaseField) {
    return FALSE;
  }

  /**
  * Compare the key/index structure
  *
  * @param &$xmlKey
  * @param &$databaseKey
  * @access public
  * @return Different TRUE/FALSE
  */
  function compareKeyStructure(&$xmlKey, &$databaseKey) {
    return FALSE;
  }
}

/**
* DB-abstraction layer - result objekt
*
* @package papaya_base
* @author Thomas Weinert <weinert@papaya-cms.com>
*/
class dbresult_base extends base_object {
  /**
  * @var resource $result result ressources
  * @access private
  */
  var $result = NULL;

  /**
  * @var boolean $hasLimit data record limit
  * @access public
  */
  var $hasLimit = FALSE;

  /**
  * @var integer $limitMax data record limit amount
  */
  var $limitMax = NULL;

  /**
  * @var integer $limitOffset data record limit offset
  */
  var $limitOffset = NULL;

  /**
  * @var integer $recNo current data record
  */
  var $recNo = 0;

  /**
  * @var string $_absCount absolute number
  */
  var $_absCount = 0;

  /**
  * Constructor
  *
  * @param resource &$result result ressource
  */
  function __construct(&$result) {
    $this->result = &$result;
  }

  /**
  * PHP 4 Constructor
  *
  * @param resource &$result result ressource
  */
  function dbresult_base(&$result) {
    $this->__construct($result);
  }

  /**
  * Destruktor
  * @access public
  */
  function free() {
    if (@is_resource($this->result)) {
      unset($this->result);
      unset($this);
    }
  }

  /**
  * data record as array
  *
  * @param integer $mode modus (numeric oder fieldname)
  * @access public
  * @return array data record
  */
  function fetchRow($mode = DB_FETCHMODE_DEFAULT) {
    return FALSE;
  }

  /**
  * fetch data from field
  *
  * @param mixed $fieldIndex Index/Name of field
  * @access public
  * @return string
  */
  function fetchField($fieldIndex = 0) {
    if (is_int($fieldIndex)) {
      $data = $this->fetchRow(DB_FETCHMODE_ORDERED);
      return $data[$fieldIndex];
    } elseif (is_string($fieldIndex)) {
      $data = $this->fetchRow(DB_FETCHMODE_ASSOC);
      return $data[$fieldIndex];
    }
    return FALSE;
  }

  /**
  * Put data records in an array
  *
  * @param array &$row Array for data records
  * @param integer $mode Modus (numeric oder fieldname)
  * @access public
  * @return mixed 0 or FALSE
  */
  function fetchInto(&$row, $mode = DB_FETCHMODE_DEFAULT) {
    $row = $this->fetchRow($mode);
    if (@is_array($row)) {
      return 0;
    } else {
      return NULL;
    }
  }

  /**
  * Aquire number of database records
  *
  * @return boolean FALSE
  * @access public
  */
  function count() {
    return FALSE;
  }

  /**
  * Aquire absolute number of database records
  *
  * @return boolean FALSE
  * @access public
  */
  function absCount() {
    return FALSE;
  }

  /**
  * Move record pointer
  *
  * @param integer $index Neue Position
  * @access public
  * @return boolean success ?
  */
  function seek($index) {
    return FALSE;
  }

  /**
  * Move database record pointer to first record
  *
  * @access public
  * @return boolean success ?
  */
  function seekFirst() {
    return $this->seek(0);
  }

  /**
  * Move record pointer to last record
  *
  * @access public
  * @return boolean success ?
  */
  function seekLast() {
    if (FALSE !== ($count = $this->count())) {
      return $this->seek($count);
    }
    return FALSE;
  }

  /**
  * Set record limiter
  *
  * @param integer $max data record limit
  * @param integer $offset start index data record limit
  * @access public
  */
  function setLimit($max=NULL, $offset=NULL) {
    if ($max > 0) {
      $this->hasLimit = TRUE;
      $this->limitMax = (int)$max;
      $this->limitOffset = (isset($offset)) ? (int)$offset : NULL;
    } else {
      $this->hasLimit = FALSE;
      unset($this->limitMax);
      unset($this->limitOffset);
    }
    return 0;
  }
}
?>