<?php
/**
* manage an output filter
* 
* @package papaya_base
* @author Thomas Weinert <weinert@papaya-cms.com>
* @version $Id: papaya_output.php,v 1.10 2006/04/25 10:18:59 weinert Exp $
*/


/**
* base class database access
*/
require_once(PAPAYA_INCLUDE_PATH.'system/sys_base_db.php');

/**
* manage an output filter
* @package papaya_base
* @author Thomas Weinert <weinert@papaya-cms.com>
* 
*/
class papaya_output extends base_db {
  var $tableViewModes = PAPAYA_DB_TBL_VIEWMODES;
  var $tableViewLinks = PAPAYA_DB_TBL_VIEWLINKS;
  var $tableModules = PAPAYA_DB_TBL_MODULES;
  
  
  var $viewMode = NULL;
  var $viewLink = NULL;
  
  function loadViewModeData($ext, $id = NULL) {
    unset($this->viewMode);
    if (isset($id)) {
      $sql = "SELECT viewmode_id, viewmode_ext, viewmode_charset, viewmode_contenttype FROM %s WHERE viewmode_id = '%d'";
      $params = array($this->tableViewModes, $id);
    } else {
      $sql = "SELECT viewmode_id, viewmode_ext, viewmode_charset, viewmode_contenttype FROM %s WHERE viewmode_ext = '%s'";
      $params = array($this->tableViewModes, $ext);    
    }
    if ($res = $this->databaseQueryFmt($sql, $params)) {
      if ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->viewMode = $row;
        return TRUE;
      }
    }  
    return FALSE;  
  }
  
  function sendHeader() {
    if (isset($this->viewMode)) {
      $contenType = (trim($this->viewMode['viewmode_contenttype']) != '') ? $this->viewMode['viewmode_contenttype'] : 'text/html';
      $charset = (trim($this->viewMode['viewmode_charset']) != '') ? $this->viewMode['viewmode_charset'] : 'utf-8';
      header('Content-type: '.$contenType.'; charset='.$charset);
    }
  }
  
  function loadViewLinkData($viewId, $viewModeId) {
    unset($this->viewLink);
    $sql = "SELECT vl.view_id, vl.viewmode_id, vl.viewlink_data,
                   vm.viewmode_ext,
                   m.module_guid, m.module_path, m.module_file, m.module_class, m.module_title
              FROM %s AS vl
              LEFT OUTER JOIN %s AS vm ON (vm.viewmode_id = vl.viewmode_id)
              LEFT OUTER JOIN %s AS m ON (m.module_guid = vm.module_guid)
             WHERE vl.view_id = %d AND vl.viewmode_id = %d";
    $params = array($this->tableViewLinks, $this->tableViewModes, $this->tableModules, $viewId, $viewModeId);
    if ($res = $this->databaseQueryFmt($sql, $params)) {
      if ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->viewLink = $row;
        return TRUE;
      }
    }
    return FALSE;
  }
  
  function loadViewsList($viewId) {
    $result = array();
    $sql = "SELECT vl.view_id, vl.viewmode_id, vm.viewmode_ext 
              FROM %s AS vl, %s AS vm 
             WHERE vl.view_id = %d AND vm.viewmode_id = vl.viewmode_id";
    if ($res = $this->databaseQueryFmt($sql, array($this->tableViewLinks, $this->tableViewModes, $viewId))) {
      while($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $result[$row['viewmode_id']] = $row;
      }
    }
    return $result;
  }
  
  function getViewsList($viewId, $pageTitle = '') {
    $result = '';
    $data = $this->loadViewsList($viewId);
    if (is_array($data) && count($data) > 0) {
      foreach($data as $mode) {
        $link = $this->getWebLink(NULL, NULL, $mode['viewmode_ext'], NULL, NULL, $pageTitle);
        if (isset($_SERVER['QUERY_STRING']) && trim($_SERVER['QUERY_STRING'] != '')) {
          $link .= '?'.$_SERVER['QUERY_STRING'];
        }
        $result .= sprintf('<viewmode ext="%s" href="%s"/>', 
          papaya_strings::escapeHTMLChars($mode['viewmode_ext']), $link);
      }
    }
    return $result;
  }
  
  function &getFilter($viewId) {
    $result = FALSE;
    if (isset($this->viewMode) && $this->loadViewLinkData($viewId, $this->viewMode['viewmode_id'])) {
      $parent = NULL;
      $result = &$this->createPluginObject($this->viewLink['module_path'].$this->viewLink['module_file'],
        $this->viewLink['module_class'], $this->viewLink['module_guid'], $parent);
      if ($result) {
        $result->setData($this->viewLink['viewlink_data']);
      }
    }      
    return $result;
  }
}

?>