<?php
/**
* Output page
*
* @package papaya
* @author Thomas Weinert <weinert@papaya-cms.com>
* @version $Id: papaya_page.php,v 1.57 2006/04/24 16:20:58 weinert Exp $
*/

/**
* basic class
*/
require_once(PAPAYA_INCLUDE_PATH.'system/sys_base_object.php');

/**
* Output page
*
* @package papaya
* @author Thomas Weinert <weinert@papaya-cms.com>
*/
class papaya_page extends base_object {

  /**
  * ID of Topic
  * @var integer $topicId
  */
  var $topicId = 0;

  /**
  * Topic
  * @var string $topic
  */
  var $topic = NULL;
  /**
  * Ouput filter manager
  * @var string $output
  */
  var $output = NULL;

  /**
  * Layout
  * @var ? $layout
  */
  var $layout = NULL;


  /**
  * Output modus
  * @var string $mode
  */
  var $mode = 'html';

  /**
  * Request data array
  * @var array $requestData
  */
  var $requestData = array();

  /**
  * Accept .gz compressed files
  * @var boolean $acceptGzip
  */
  var $acceptGzip = FALSE;

  /**
  * Session Name Suffix
  * @var string
  */
  var $sessionName = '';

  /**
  * Global Session Params
  * @var array
  */
  var $sessionParams = array();

  /**
  * Constructor
  */
  function __construct() {
    $this->startTimer();
    define('PAPAYA_ADMIN_PAGE', FALSE);
        
    if (!isset($_SERVER['HTTP_HOST'])) {
      $_SERVER['HTTP_HOST'] = $_SERVER['SERVER_NAME'];
    }

    if (!$this->loadOptions()) {
      if (isset($_GET['redirect'])) {
        $this->sendHTTPStatus(503);
        if (defined('PAPAYA_ERRORDOCUMENT_503') && file_exists(PAPAYA_ERRORDOCUMENT_503) && 
            is_file(PAPAYA_ERRORDOCUMENT_503) && is_readable(PAPAYA_ERRORDOCUMENT_503)) {
          header('Content-type: text/html; charset=utf-8;');
          readfile(PAPAYA_ERRORDOCUMENT_503);
        } else {
          $this->getErrorHTML(503, 'Service Unavailable');
        }
        exit;
      } else {
        $this->sendHTTPStatus(302);
        $protocol = (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] == 'on') ? 'https' : 'http';
        $url = strtr($protocol.'://'.$_SERVER['HTTP_HOST'].dirname($_SERVER['PHP_SELF']).'index.html?redirect=503', '\\', '/');
        header('Location: '.$url);
        exit;
      }
    }
        
    $this->requestData = base_object::parseRequestURI();    

    /* exit script handling */
    if (isset($_GET['exit'])) {
      if (preg_match("#^/sid".preg_quote($this->sessionName)."^/([0-9a-z]{32})([^?]*)#i", $_SERVER['REQUEST_URI'], $regs)) {
        $protocol = (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] == 'on') ? 'https' : 'http';
        $url = $protocol.'://'.strtolower(PAPAYA_DEFAULT_HOST).$regs[2].'?exit='.urlencode($_GET['exit']);
      } else {
        $url = $_GET['exit'];
      }
      $this->logRequestExitPage($url);
      $this->sendHTTPStatus(301);
      header('Location: '.$url);
      exit;
    }

    /* redirect script handling */
    if (isset($_REQUEST['redirect'])) {
      $url = base_object::getAbsoluteURL($_REQUEST['redirect'], @$_REQUEST['title']);
      $this->sendHTTPStatus(302);
      header('Location: '.$url);
      exit;
    }

    if (defined('PAPAYA_DEFAULT_HOST') && trim(PAPAYA_DEFAULT_HOST) != '' &&
        defined('PAPAYA_DEFAULT_HOST_ACTION') && PAPAYA_DEFAULT_HOST_ACTION > 0 &&
        strtolower($_SERVER['HTTP_HOST']) != strtolower(PAPAYA_DEFAULT_HOST)) {
      $this->sendHTTPStatus(301);
      $protocol = (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] == 'on') ? 'https' : 'http';
      $url = $_SERVER['REQUEST_URI'];
      header('Location: '.$protocol.'://'.strtolower(PAPAYA_DEFAULT_HOST).$url);
      exit;
    }

    if (defined('PAPAYA_DBG_PHP_ERROR')) {
      error_reporting(PAPAYA_DBG_PHP_ERROR);
    }

    if (defined('PAPAYA_GLOBAL_LOGOBJECT') && (trim(PAPAYA_GLOBAL_LOGOBJECT) != '')) {
      include_once(PAPAYA_INCLUDE_PATH.'system/base_log.php');
      $GLOBALS[PAPAYA_GLOBAL_LOGOBJECT] = &new base_log();
    }

    $this->initPageMode();

    $startSession = (PAPAYA_START_SESSION != 0) || (!$this->public);

    $this->startSession('', $startSession);

    $this->acceptGzip = (preg_match('#gzip#', getenv('HTTP_ACCEPT_ENCODING')) &&
                        function_exists('gzopen') &&
                        (defined('PAPAYA_COMPRESS_OUTPUT') && PAPAYA_COMPRESS_OUTPUT === '1') &&
                        (PAPAYA_CACHE_TIME > 0));

    $this->initializeParams();
  }

  /**
  * PHP 4 constuctor redirect
  *
  * @access public
  */
  function papaya_page() {
    $args = func_get_args();
    call_user_func_array(array(&$this, '__construct'), $args);
  }

  /**
  * Initialize parameters
  *
  * @access public
  */
  function initializeParams() {
    if (isset($_REQUEST['tt']['page_id']) && $_REQUEST['tt']['page_id'] > 0) {
      $this->topicId = (int)$_REQUEST['tt']['page_id'];
    } elseif (isset($_REQUEST['p_id']) && $_REQUEST['p_id'] > 0) {
      $this->topicId = (int)$_REQUEST['p_id'];
    } elseif (isset($this->requestData['page_id']) && $this->requestData['page_id'] > 0) {
      $this->topicId = (int)$this->requestData['page_id'];
      $_REQUEST['p_id'] = (int)$this->requestData['page_id'];
    } else {
      if ($this->checkAlias()) {
        $this->topicId = -1;
      } else {
        $this->topicId = (int)PAPAYA_PAGEID_DEFAULT;
      }
      $_REQUEST['p_id'] = $this->topicId;
    }
    if (isset($this->requestData['language'])) {
      $_REQUEST['language'] = $this->requestData['language'];
    }
    if (isset($_GET['box']) && $_GET['box'] > 0) {
      $this->boxId = (int)$_GET['box'];
      $this->topicId = -1;
    } else {
      $this->boxId = -1;
    }
  }

  /**
  * Check alias
  *
  * @return boolean
  */
  function checkAlias() {
    if (isset($_SERVER['REDIRECT_STATUS']) && $_SERVER['REDIRECT_STATUS'] == 404) {
      /* mac-ie css hack handling - path starts with ( or " */
      if (isset($_SERVER['REQUEST_URI']) && 
          preg_match('~^/?([("]|\\\\")~', $_SERVER['REQUEST_URI'])) {
        $this->sendHTTPStatus(200);
        exit;
      } else {
        include_once(PAPAYA_INCLUDE_PATH.'system/base_urlmounter.php');
        $urlMounter = &new base_urlmounter;
        if ($url = $urlMounter->locate()) {
          $this->sendHTTPStatus(302);
          @header("Location: $url");
          exit;
        }
      }
      return TRUE;
    }
    return FALSE;
  }

  /**
  * Load options
  */
  function loadOptions() {
    include_once(PAPAYA_INCLUDE_PATH.'system/base_options.php');
    $options = &new base_options();
    return $options->loadAndDefine();
  }

  /**
  * Start new seassion
  */
  function startSession() {
    $startSession = (bool)(PAPAYA_START_SESSION || (!$this->public));
    $fallback = defined('PAPAYA_SESSION_ID_FALLBACK') ? PAPAYA_SESSION_ID_FALLBACK : 'rewrite';
    if (!($this->public && ($fallback == 'rewrite' || $fallback == 'get'))) $fallback = 'rewrite';

    include_once(PAPAYA_INCLUDE_PATH.'system/sys_session.php');
    $GLOBALS['PAPAYA_SESSION'] = &new rewrite_session($this->sessionName, $startSession, $fallback);
    $this->sessionParams = $GLOBALS['PAPAYA_SESSION']->getValue('PAPAYA_SESSION_PAGE_PARAMS');

    include_once(PAPAYA_INCLUDE_PATH.'system/base_surfer.php');
    if (!(isset($GLOBALS['PAPAYA_SURFER']) && is_object($GLOBALS['PAPAYA_SURFER']))) {
      $GLOBALS['PAPAYA_SURFER'] = &new base_surfer(TRUE);
      $GLOBALS['PAPAYA_SURFER']->loadTopicIdList();
    }
  }

  function setPageOption($name, $value) {
    if (isset($value)) {
      $this->sessionParams[$name] = $value;
    } else {
      unset($this->sessionParams[$name]);
    }
    $GLOBALS['PAPAYA_SESSION']->setValue('PAPAYA_SESSION_PAGE_PARAMS', $this->sessionParams);
  }

  function getPageOption($name) {
    if (isset($this->sessionParams[$name])) {
      return $this->sessionParams[$name];
    } else {
      return NULL;
    }
  }

  /**
  * Is access valid
  * @return boolean
  */
  function validateAccess($topicId) {
    include_once(PAPAYA_INCLUDE_PATH.'system/base_surfer.php');
    if (!(isset($GLOBALS['PAPAYA_SURFER']) && is_object($GLOBALS['PAPAYA_SURFER']))) {
      $GLOBALS['PAPAYA_SURFER'] = &new base_surfer(TRUE);
    }
    if ($this->public) {
      $GLOBALS['PAPAYA_SURFER']->loadTopicIdList(!$GLOBALS['PAPAYA_SURFER']->isValid);
      return ($GLOBALS['PAPAYA_SURFER']->canView($topicId));
      } else {
      $GLOBALS['PAPAYA_SURFER']->loadTopicIdList(!$GLOBALS['PAPAYA_SURFER']->isValid);
      include_once(PAPAYA_INCLUDE_PATH.'system/base_auth.php');
      $GLOBALS['PAPAYA_USER'] = &new base_auth();
      $GLOBALS['PAPAYA_USER']->initialize();
      $GLOBALS['PAPAYA_USER']->execLogin();
      return ($GLOBALS['PAPAYA_USER']->isValid);
    }
  }

  /**
  * Create topic
  */
  function createTopic() {
    unset($this->topic);
    if ($this->public) {
      include_once(PAPAYA_INCLUDE_PATH.'system/papaya_publictopic.php');
      $this->topic = &new papaya_publictopic;
    } else {
      include_once(PAPAYA_INCLUDE_PATH.'system/papaya_topic.php');
      $this->topic = &new papaya_topic;
    }
  }

  /**
  * Create output manager object
  */
  function createOutput() {
    include_once(PAPAYA_INCLUDE_PATH.'system/papaya_output.php');
    unset($this->output);
    $this->output = &new papaya_output;
  }

  /**
  * Get cache ID
  * @return mixed
  */
  function getCacheId() {
    if (file_exists(PAPAYA_PATH_CACHE) && is_dir(PAPAYA_PATH_CACHE)) {
      $surferId = (int)$GLOBALS['PAPAYA_SURFER']->surferId;
      $pageEnv = $_SERVER['HTTP_HOST'].dirname($_SERVER['SCRIPT_FILENAME']).
         $this->requestData['categ_id'].$_SERVER['QUERY_STRING'].serialize($this->sessionParams);
      $lng = $this->requestData['language'];
      $cacheId = $this->topicId.'_'.$surferId.'_'.$lng.'_'.md5($pageEnv).'.'.$this->mode;
      return $cacheId;
    }
    return FALSE;
  }

  function useCache() {
    $method = @strtoupper($_SERVER['REQUEST_METHOD']);
    $agent = @strtolower($_SERVER['HTTP_USER_AGENT']);
    return ($this->public && $method == 'GET' && PAPAYA_CACHE_TIME > 0 && $agent != 'mnogosearch-dimensional');
  }

  /**
  * Get cache
  *
  * @param integer $cacheId
  * @access public
  * @return boolean
  */
  function getCache($cacheId) {
    if (isset($_SERVER['HTTP_IF_NONE_MATCH'])) {
      if ($cacheId == $_SERVER['HTTP_IF_NONE_MATCH']) {
        if (isset($_SERVER['HTTP_IF_MODIFIED_SINCE']) &&
            time() < (strtotime($_SERVER['HTTP_IF_MODIFIED_SINCE']) + (int)PAPAYA_CACHE_TIME)) {
          $this->sendHTTPStatus(304);
          exit;
        }
      }
    }
    $cacheFileName = PAPAYA_PATH_CACHE.'.'.$cacheId;
    $cacheFileGzipName = $cacheFileName.'.gz';
    $now = time();

    $this->output->sendHeader();

    if ($this->acceptGzip && file_exists($cacheFileGzipName) && is_file($cacheFileGzipName)) {
      //read and send compressed file
      $cacheTime = filectime($cacheFileGzipName);
      if ($now < ($cacheTime + (int)PAPAYA_CACHE_TIME)) {
        header('Last-modified: '.gmdate("D, d M Y H:i:s", $cacheTime)." GMT");
        header('Content-Encoding: gzip');
        readfile($cacheFileGzipName);
        flush();
        return TRUE;
      }
    } elseif (file_exists($cacheFileName) && is_file($cacheFileName)) {
      //read uncompressed cache file
      $cacheTime = filectime($cacheFileName);
      if ($now < ($cacheTime+(int)PAPAYA_CACHE_TIME)) {
        if ($this->acceptGzip) {
          //write and send compressed cache file
          $cacheFileGzipName = $cacheFileName.'.gz';
          if ($fh = fopen($cacheFileName, 'r')) {
            $content = fread($fh, filesize($cacheFileName));
            fclose($fh);
            if ($fh = gzopen($cacheFileGzipName, 'w')) {
              gzwrite($fh, $content);
              gzclose($fh);
              header('Last-modified: '.gmdate("D, d M Y H:i:s")." GMT");
              header('Content-Encoding: gzip');
              readfile($cacheFileGzipName);
              flush();
              return TRUE;
            }
          }
        }
        header('Last-modified: '.gmdate("D, d M Y H:i:s", $cacheTime)." GMT");
        readfile($cacheFileName);
        flush();
        return TRUE;
      }
    } elseif (file_exists($cacheFileGzipName) && is_file($cacheFileGzipName)) {
      //read compressed file - send uncomressed
      header('Last-modified: '.gmdate("D, d M Y H:i:s", $cacheTime)." GMT");
      readgzfile($cacheFileGzipName);
      flush();
      return TRUE;
    }
    return FALSE;
  }

  /**
  * Set cache
  *
  * @param integer $cacheId
  * @param string $page
  * @access public
  * @return boolean
  */
  function setCache($cacheId, $page) {
    $cacheFileName = PAPAYA_PATH_CACHE.'.'.$cacheId;
    $this->invalidateCache($cacheId);
    if ($this->acceptGzip) {
      if ($fh = gzopen($cacheFileName.'.gz', 'w')) {
        gzwrite($fh, $page);
        gzclose($fh);
        return TRUE;
      }
    }
    if ($fh = fopen($cacheFileName, 'w')) {
      fwrite($fh, $page);
      fclose($fh);
      return TRUE;
    }
    return FALSE;
  }

  /**
  * Invalid date cache
  *
  * @param integer $cacheId
  * @access public
  */
  function invalidateCache($cacheId) {
    if (preg_match('#^\d+#', $cacheId, $regs)) {
      $page = $regs[0];
      if ($dh = opendir(PAPAYA_PATH_CACHE)) {
        $cacheFileName = '.'.$cacheId;
        while($file = readdir($dh)) {
          if ($file != $cacheFileName) {
            if (preg_match('#^\.'.(int)$page.'_\d+_[a-f\d]{32}\.html(\.gz)?$#', $file)) {
              $now = time();
              $cacheTime = filectime(PAPAYA_PATH_CACHE.$file)+(int)PAPAYA_CACHE_TIME;
              if ($now > $cacheTime) {
                unlink(PAPAYA_PATH_CACHE.$file);
              }
            }
          }
        }
        closedir($dh);
      }
    }
  }

  /**
  * Initialize page mode
  *
  * @access public
  */
  function initPageMode() {
    $this->versionDateTime = 0;
    if (isset($_GET['p_date'])) {
      $this->versionDateTime = (int)$_GET['p_date'];
    } elseif (isset($this->requestData['datetime']) && $this->requestData['datetime'] > 0) {
      $this->versionDateTime = (int)$this->requestData['datetime'];
    }
    if (isset($this->requestData['preview']) && $this->requestData['preview']) {
      $this->public = FALSE;
    } elseif (isset($_GET['public'])) {
      $this->public = ($_GET['public'] == 'yes');
    } else {
      $this->versionDateTime = 0;
      $this->public = TRUE;
    }
    switch (@$this->requestData['ext']) {
      case 'xml' :
        $_GET['XML'] = 1;
        $this->mode = 'xml';
        break;
      case 'php' :
        $this->mode = defined('PAPAYA_URL_EXTENSION') ? PAPAYA_URL_EXTENSION : 'html';
        break;
      default :
        $this->mode = $this->requestData['ext'];
    }

  }

  /**
  * Get
  *
  * @access public
  */
  function get() {
    header('X-Generator: papaya CMS');
    switch ($this->mode) {
    case 'urls' :
      $this->logRequest();
      $this->getUrls();
      break;
    case 'image' :
      $this->logRequest();
      $this->getDynamicImage();
      break;
    case 'thumb' :
      $this->logRequest();
      $this->getMediaThumbFile();
      break;
    case 'media' :
      $this->logRequest();
      $this->getMediaFile();
      break;
    case 'popup' :
      $this->getMediaPopup();
      break;
    case 'download' :
      $this->logRequest();
      $this->getDownload();
      break;
    case 'xml' :
      $this->createTopic();
      $this->createOutput();
      if (!PAPAYA_DBG_XML_OUTPUT) {
        $this->getError(403, 'Access forbidden');
      } else if ($this->topic->topicExists($this->topicId)) {
        if ($str = $this->getPageXML()) {
          header('Content-type: text/xml; charset: utf-8');
          echo $str;
        } else {
          $this->getError(404, 'Page #'.(int)$this->topicId.' no content found');
        }
      } elseif ($this->topicId == -1) {
        $this->getError(404, 'File/path '.$_SERVER['REDIRECT_URL'].' not found');
      } else {
        $this->getError(404, 'Page #'.(int)$this->topicId.' not found');
      }
      break;
    default :
      $this->createTopic();
      $this->createOutput();
      if ($this->topic->topicExists($this->topicId)) {
        if ($this->output->loadViewModeData($this->mode)) {
          //Caching ist nicht immer angebracht
          if ($this->useCache()) {
            //Cache prfen
            if ($cacheId = $this->getCacheId()) {
              header('ETag: '.$cacheId);
              header('Expires: '.gmdate("D, d M Y H:i:s", time() + @(int)PAPAYA_CACHE_TIME)." GMT");
              //Cache holen
              if (!($this->getCache($cacheId))) {
                //Seite erzeugen
                print ($page = $this->getPage());
                //Cache setzen
                if ($this->topic->cacheable) {
                  $this->setCache($cacheId, $page);
                }
              } else {
                $this->logRequest($this->topic->languageIdentToId($this->requestData['language']), TRUE);
              }
            } else {
              print $this->getPage();
            }
          } else {
            print $this->getPage();
          }
        } else {
          $this->getError(404, 'Output mode "'.$this->mode.'" not found');
        }
      } elseif (($this->boxId > 0) && (!$this->public)) {
        print $this->getBox();
      } elseif ($this->topicId == -1) {
        $this->getError(404, 'File/path '.$_SERVER['REDIRECT_URL'].' not found');
      } else {
        $this->getError(404, 'Page #'.(int)$this->topicId.' not found');
      }

      if (defined('PAPAYA_DBG_RUNTIME_PHP') && PAPAYA_DBG_RUNTIME_PHP === '1')
        $this->showDebug(array('time'=>$this->stopTimer()));
      return '';
    }
    $GLOBALS['PAPAYA_SESSION']->setValue('PAPAYA_SESSION_PAGE_PARAMS', $this->sessionParams);
  }

  /**
  * Get URLs
  *
  * @access public
  */
  function getUrls() {
    include_once(PAPAYA_INCLUDE_PATH.'system/base_sitemap.php');
    $this->createTopic();
    $this->map = &new base_sitemap($this->topic,
      array('root'=>0, 'format'=>'static', 'forstart'=>0, 'forend'=>999));
    @header("Content-type: text/html");
    echo $this->map->getUrls();
  }


  function getPageXML() {
    if ($this->topic->loadOutput($this->topicId, $this->requestData['language'], $this->versionDateTime)) {
      if ($this->topic->checkPublishPeriod($this->topicId)) {
        if ($this->validateAccess($this->topicId)) {
          $this->generatePage();
          header('Content-type: text/xml; charset=utf-8');
          return $this->layout->xml();
        } else {
          $this->getError(403, 'Access forbidden');
        }
      } else {
        $this->getError(404, 'Page not Published');
      }
    } else {
      $this->getError(404, 'Page not found');
    }
    return '';
  }

  /**
  * Get page
  *
  * @access public
  * @return string
  */
  function getPage() {
    if ($this->topic->loadOutput($this->topicId, $this->requestData['language'], $this->versionDateTime)) {
      $this->logRequest($this->topic->getContentLanguageId());
      if ($this->topic->checkPublishPeriod($this->topicId)) {
        if ($this->validateAccess($this->topicId)) {
          $viewId = $this->topic->getViewId();
          if ($viewId > 0) {
            if ($this->filter = $this->output->getFilter($viewId)) {
              $this->generatePage(@$this->filter->data);
              if ($this->filter->checkConfiguration()) {
                $str = $this->filter->parsePage($this->topic, $this->layout);
                header('Last-modified: '.gmdate("D, d M Y H:i:s")." GMT");
                $this->output->sendHeader();
                return $str;
              } else {
                $this->getError(500, 'Output mode "'.papaya_strings::escapeHTMLChars(basename($this->mode)).'"for page #'.$this->topicId.' not configured: '.$this->filter->errorMessage);
              }
            } else {
              $this->getError(500, 'Output mode "'.papaya_strings::escapeHTMLChars(basename($this->mode)).'" for page #'.$this->topicId.' not found');
            }
          } else {
            $this->getError(500, 'View "'.papaya_strings::escapeHTMLChars(basename($this->mode)).'" for page #'.$this->topicId.' not found');
          }
        } else {
          $this->getError(403, 'Access forbidden');
        }
      } else {
        $this->getError(404, 'Page not published');
      }
    } else {
      $this->getError(404, 'Page not found');
    }
    return '';
  }

  /**
  * Generate page
  *
  * @access public
  * @return string
  */
  function generatePage($filterParams = NULL) {
    include_once(PAPAYA_INCLUDE_PATH.'system/papaya_xsl.php');
    $this->layout = &new papaya_xsl();
    $this->layout->add($this->topic->parseContent(TRUE, $filterParams), 'content');
    $this->layout->add($this->topic->getMetaInfos(), 'meta');
    $this->layout->add($this->output->getViewsList($this->topic->getViewId(),
      $this->topic->topic['TRANSLATION']['topic_title']), 'views');
    $this->layout->add($this->topic->getTranslationsData(
      $this->topic->topic['TRANSLATION']['lng_id']), 'translations');

      
    $protocol = (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] == 'on') ? 'https' : 'http';
    $url = strtr($protocol.'://'.$_SERVER['HTTP_HOST'].PAPAYA_PATH_WEB, '\\', '/');
    $this->layout->setParam('PAGE_BASE_URL', $url);
    
    $this->layout->setParam('PAGE_TITLE', $this->topic->topic['TRANSLATION']['topic_title']);
    $this->layout->setParam('PAGE_TODAY', date('Y-m-d H:i:s'));
    $this->layout->setParam('PAGE_THEME_PATH', PAPAYA_PATH_WEB.'papaya-themes/'.PAPAYA_LAYOUT_THEME.'/');
    $this->layout->setParam('PAGE_WEB_PATH', PAPAYA_PATH_WEB);
    $this->layout->setParam('PAGE_LANGUAGE', $this->topic->currentLanguage['lng_short']);
    
    $this->layout->setParam('PAGE_MODE_PUBLIC', ($this->public) ? 1 : 0 );
    
    if (@$_SERVER['HTTP_USER_AGENT'] != 'mnogosearch-dimensional') {
      if ($boxesId = $this->topic->getBoxesTopicID()) {
        if ($this->public) {
          include_once(PAPAYA_INCLUDE_PATH.'system/papaya_publicboxeslinks.php');
          $boxesList = &new papaya_public_boxeslinks($this->topic);
        } else {
          include_once(PAPAYA_INCLUDE_PATH.'system/base_boxeslinks.php');
          $boxesList = &new base_boxeslinks($this->topic);
        }
        if ($data = $boxesList->parsed($boxesId, $this->topic->topic['TRANSLATION']['lng_id'],
            @(int)$this->output->viewMode['viewmode_id'])) {
          $this->layout->add($data, 'boxes');
        }
        unset($boxesList);
      }
    }
  }

  /**
  * Get box
  *
  * @access public
  * @return string
  */
  function getBox() {
    include_once(PAPAYA_INCLUDE_PATH.'system/base_boxes.php');
    $boxes = &new base_boxes;
    $boxes->msgs = &$this->msgs;
    if ($boxes->load($this->boxId, @$_GET['lng_id'])) {
      return $boxes->parsedBox();
    }
    return '';
  }

  function getDynamicImage() {
    include_once(PAPAYA_INCLUDE_PATH.'system/base_imagegenerator.php');
    $imgGenerator = &new base_imagegenerator();
    if ($imgGenerator->loadByIdent(@$this->requestData['ident'])) {
      if (!$imgGenerator->generateImage()) {
        $this->getError(500, $imgGenerator->lastError);
      }
    } else {
      $this->getError(404, 'Image identifier not found');
    }
  }

  /**
  * Get download
  *
  * @access public
  * @return mixed
  */
  function getDownload() {
    if (isset($this->requestData['media_id'])) {
      if ($file = $this->checkMediaPerm($this->requestData['media_id'])) {
        $fileName = PAPAYA_PATH_MEDIAFILES.$file['file_id'];
        if ($fh = fopen($fileName, 'r')) {
          $packetSize = 512;
          $agent = $this->getUserAgent();
          $mimeType = ($agent == 'IE' || $agent == 'OPERA')
            ? 'application/octetstream'
            : 'application/octet-stream';
          if ($agent == 'IE') {
            header('Last-modified: '.gmdate("D, d M Y H:i:s", $file['file_date'])." GMT");
            header('Content-Disposition: inline; filename="'.$file['file_name'].'"');
          } else {
            header('Last-modified: '.gmdate("D, d M Y H:i:s", $file['file_date'])." GMT");
            header('Content-Disposition: attachment; filename="'.$file['file_name'].'"');
          }

          $contentSize = filesize($fileName);

          header('Content-type: ' . $mimeType);
          header("Content-length: ".$contentSize);

          $packets = floor($contentSize/$packetSize);
          for ($i = 1; $i <= $packets; $i++) {
            echo fread($fh, $packetSize);
            flush();
          }
          $lastBytes = ($contentSize % $packetSize);
          if ($lastBytes > 0) {
            echo fread($fh, $lastBytes);
            flush();
          }

          fclose($fh);
          exit;
        }
      }
    }
    $this->getError(404, 'File not found');
  }

  /**
  * Load media file data
  *
  * @param integer $id
  * @access public
  * @return mixed array or boolean
  */
  function loadMediaFileData($id) {
    include_once(PAPAYA_INCLUDE_PATH.'system/base_fileman.php');
    $fileManager = &new base_filemanager();
    if ($fileManager->dataLoadFile($id)) {
      $data = $fileManager->selectedFile;
      $data['mimetype'] = $fileManager->getMimeType($data['file_type'], $data['file_name']);
      if ($data['folder_id'] == 0 || $data['folder_id'] == -1 || $fileManager->dataLoadFolder($data['folder_id'])) {
        if (isset($fileManager->folder)) {
          $data['folder'] = $fileManager->folder;
        }
        return $data;
      } else {
        $this->getError(404, 'Folder not found');
      }
    } else {
      $this->getError(404, 'File record "'.$id.'" not found');
    }
    return FALSE;
  }

  function getMediaPopup() {
    if ($file = $this->checkMediaPerm($this->requestData['media_id'])) {
      if ($file['file_type'] >= 1 && $file['file_type'] <= 3) {
        header('Content-type: text/html');
        printf('<html><head><title>%s</title></head><body style="padding: 0px; margin: 0px;">',
          papaya_strings::escapeHTMLChars(
            (trim(@$file['file_title']) != '') ? $file['file_title'] : $file['file_name']));
        printf('<img src="%s" style="border: none; margin: 0px;" alt="%s" />',
          'image.media.'.($this->public ? '' : 'preview.').$file['file_id'],
          papaya_strings::escapeHTMLChars(
            (trim(@$file['file_title']) != '') ? $file['file_title'] : $file['file_name']));
        print('</body></html>');
      } else {
        $this->outputFile(PAPAYA_PATH_MEDIAFILES.$this->requestData['media_id'], $file);
      }
      exit;
    }
  }

  /**
  * Get media file
  *
  * @access public
  */
  function getMediaFile() {
    if (isset($this->requestData['media_id'])) {
      if ($file = $this->checkMediaPerm($this->requestData['media_id'])) {
        $this->outputFile(PAPAYA_PATH_MEDIAFILES.$this->requestData['media_id'], $file);
      }
    }
  }

  /**
  * Get media tumbnail file
  *
  * @access public
  */
  function getMediaThumbFile() {
    $thumb = $this->requestData['media_id'];
    if (preg_match('#^(.*?)(_([a-z]+))?_(\d+x\d+)(\.\w+)$#', $thumb, $regs)) {
      if ($file = $this->checkMediaPerm($regs[1].$regs[5])) {
        $this->outputFile(PAPAYA_PATH_THUMBFILES.$this->requestData['media_id'], $file);
      }
    } elseif (preg_match('#^(\w{32}\.\w+)$#', $thumb, $regs)) {
      if ($file = $this->checkMediaPerm($regs[1])) {
        $this->outputFile(PAPAYA_PATH_MEDIAFILES.$this->requestData['media_id'], $file);
      }
    } else {
      $this->getError(404, 'Invalid FileId');
    }
  }

  function filenameToMimeType($fileName) {
    if (preg_match('/\.([\w\d]+)$/', $fileName, $regs)) {
      if (!(isset($this->mimeTypes) && is_object($this->mimeTypes))) {
        include_once(PAPAYA_INCLUDE_PATH.'system/base_mimetypes.php');
        $this->mimeTypes = new base_mimetypes();
      }
      if ($mime = $this->mimeTypes->loadMimeTypeByExt($regs[1])) {
        return $mime['mimetype_text'];
      }
    }
    $agent = $this->getUserAgent();
    $mimeType = ($agent == 'IE' || $agent == 'OPERA')
      ? 'application/octetstream'
      : 'application/octet-stream';
    return NULL;
  }

  /**
  * Output file
  *
  * @param string $fileName
  * @param array &$data
  * @access public
  */
  function outputFile($fileName, &$data) {
    if ($fh = @fopen($fileName, 'r')) {
      if (isset($data['mimetype'])) {
        header('Content-type: '.@$data['mimetype']);
      } elseif ($mimeType = $this->filenameToMimeType($fileName)) {
        header('Content-type: '.@$mimeType);
      }
      header('Last-modified: '.gmdate("D, d M Y H:i:s", @$data['file_date'])." GMT");
      header('Expires: '.gmdate("D, d M Y H:i:s", time() + @(int)PAPAYA_CACHE_TIME)." GMT");
      //header("Content-Length: ".filesize($fileName));
      fpassthru($fh);
      fclose($fh);
      exit;
    } else {
      $this->getError(404, 'File not found');
    }
  }

  /**
  * Check media permissions
  *
  * @param integer $mid
  * @access public
  * @return array
  */
  function checkMediaPerm($mid) {
    if ($file = $this->loadMediaFileData($mid)) {
      if ($this->public) {
        if (!isset($file['folder'])) {
          return $file;
        }
        if (!(isset($GLOBALS['PAPAYA_SURFER']) && is_object($GLOBALS['PAPAYA_SURFER']))) {
          include_once(PAPAYA_INCLUDE_PATH.'system/base_surfer.php');
          $GLOBALS['PAPAYA_SURFER'] = &new base_surfer(TRUE);
        }
        $filePerms = @array_merge(array_keys($file['folder']['perms']), array_keys($file['folder']['inh_perms']));
        if ($GLOBALS['PAPAYA_SURFER']->hasOnPermOf($filePerms)) {
          return $file;
        }
        } else {
        include_once(PAPAYA_INCLUDE_PATH.'system/base_auth.php');
        $GLOBALS['PAPAYA_USER'] = &new base_auth();
        $GLOBALS['PAPAYA_USER']->initialize();
        $GLOBALS['PAPAYA_USER']->execLogin();
        if ($GLOBALS['PAPAYA_USER']->isValid) {
          return $file;
        }
      }
    }
    $this->getError(403, 'Access forbidden to media file', TRUE);
    return FALSE;
  }

  /**
  * Get user agent
  *
  * @access public
  * @return string
  */
  function getUserAgent() {
    $agentString = strtolower(@$_SERVER["HTTP_USER_AGENT"]);
    if (strpos($agentString, 'opera') !== FALSE) {
      return 'OPERA';
    } elseif (strpos($agentString, 'msie') !== FALSE) {
      return 'IE';
    } else {
      return 'STD';
    }
  }
  
  function logRequest($lngId = 0, $cachedPage = FALSE) { 
    if (defined('PAPAYA_PAGE_STATISTIC') && PAPAYA_PAGE_STATISTIC) {
      include_once(PAPAYA_INCLUDE_PATH.'system/base_statistic.php');
      $statObj = &new base_statistic();
      $this->statisticRequestId = $statObj->logRequest($this->requestData, $lngId, $cachedPage);
    }
  }
  
  function logRequestExitPage($url) { 
    if (defined('PAPAYA_PAGE_STATISTIC') && PAPAYA_PAGE_STATISTIC) {
      include_once(PAPAYA_INCLUDE_PATH.'system/base_statistic.php');
      $statObj = &new base_statistic();
      $this->statisticRequestId = $statObj->logRequest($this->requestData);
      $statObj->logExitPage($this->statisticRequestId, $url);
    }  
  }
  

  /**
  * Start timer
  *
  * @access public
  */
  function startTimer() {
    $this->startTime = microtime();
  }

  /**
  * Stop timer
  *
  * @param integer $digits optional, default value 4
  * @access public
  */
  function stopTimer($digits = 4) {
    $to = explode(' ',microtime());
    $from = explode(' ',$this->startTime);
    $to = $to[0]+(double)$to[1];
    $from = $from[1]+(double)$from[0];
    return number_format(($to-$from)*1000, $digits);
  }

  /**
  * Show debug
  *
  * @param array $msg
  * @access public
  * @return integer
  */
  function showDebug($msg) {
    echo '</form>', '</table><br>';
    echo '<table style="position: relative; z-index: 10000;" border="1" width="800" cellpadding="2" cellspacing="0" align="center" bgcolor="#C0C0C0">';
    echo sprintf('<tr><th>Debugausgabe (%s)</th><th align="right">'.$msg['time'].' ms</th></tr>', 'Gesamtlaufzeit');
    echo '</table><br/>';
    flush();
    return 0;
  }

  /**
  * Get error
  *
  * @param integer $error
  * @param string $errorString
  * @param boolean $verbose optional, default value FALSE
  * @access public
  */
  function getError($error, $errorString, $verbose = FALSE) {
    header('Content-type: text/html; charset: utf-8');
    if (!$verbose) {
      $this->createTopic();
      switch ($error) {
        case 404 :
          $this->sendHTTPStatus(404);
          $this->topic->loadOutput((int)PAPAYA_PAGEID_ERROR_404, $this->requestData['language']);
          break;
        case 403 :
          $this->sendHTTPStatus(403);
          $this->topic->loadOutput((int)PAPAYA_PAGEID_ERROR_403, $this->requestData['language']);
          break;
        default :
          $this->sendHTTPStatus(500);
          $this->topic->loadOutput((int)PAPAYA_PAGEID_ERROR_500, $this->requestData['language']);
      }
      if ($this->topic->topicId) {
        print $this->generatePage();
      } else {
        print $this->getErrorHTML($error, $errorString);
      }
    } else {
      print $this->getErrorHTML($error, $errorString);
    }
  }

  /**
  * Get error HTML
  *
  * @param integer $error
  * @param string $errorString
  * @access public
  */
  function getErrorHTML($error, $errorString) {
    print ('
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0 Transitional//EN">
<html>
<head>
<title>Fehler '.$error.'</title>
<style type="text/css">
<!--
  th, td {
    font-family: Verdana, Geneva, Arial, Helvetica, sans-serif;
  }
//-->
</style>
</head>
<body>
<div align="center">
<br />
<br />
<br />
<table cellpadding="4" border="0" width="400">
  <tr>
    <th colspan="2">Error in page request!</th>
  </tr>
  <tr valign="top">
    <td align="center"><pre>'.$this->getNumber($error).'</pre></td>
  </tr>
  <tr valign="top">
    <th><h3>'.(int)$error.' - '.papaya_strings::escapeHTMLChars($errorString).'</h3></th>
  </tr>
  <tr valign="top">
    <th><hr style="border: none; border-bottom: 1px solid black;"><h4><a href="http://'.papaya_strings::escapeHTMLChars($_SERVER['HTTP_HOST']).'/">http://'.papaya_strings::escapeHTMLChars($_SERVER['HTTP_HOST']).'/</a></h4></th>
  </tr>
</table>
</div>
</body>
</html>');
    exit();
  }

  /**
  * Get number
  *
  * @param interger $error
  * @access public
  * @return string
  */
  function getNumber($error) {
    include_once(PAPAYA_INCLUDE_PATH.'system/base_ascii_gfx.php');
    $ascii = &new base_ascii_gfx();
    return $ascii->getAsciiString($error);
  }

  /**
  * Send HTTP status
  *
  * @param integer $status
  * @access public
  */
  function sendHTTPStatus($status) {
    switch ($status) {
      case 206 : $str = '206 Partial Content'; break;
      case 300 : $str = '300 Multiple Choices'; break;
      case 301 : $str = '301 Moved Permanently'; break;
      case 302 : $str = '302 Found'; break;
      case 303 : $str = '303 See Other'; break;
      case 304 : $str = '304 Not Modified'; break;
      case 400 : $str = '400 Bad Request'; break;
      case 401 : $str = '401 Unauthorized'; break;
      case 401 : $str = '402 Payment Required'; break;
      case 403 : $str = '403 Forbidden'; break;
      case 404 : $str = '404 Not Found'; break;
      case 405 : $str = '405 Method Not Allowed'; break;
      case 406 : $str = '406 Not Acceptable'; break;
      case 407 : $str = '407 Proxy Authentication Required'; break;
      case 408 : $str = '408 Request Timeout'; break;
      case 409 : $str = '409 Conflict'; break;
      case 410 : $str = '410 Gone'; break;
      case 500 : $str = '500 Unknown'; break;
      case 503 : $str = '503 Service Unavailable'; break;
      default  : $str = '200 OK'; break;
    }
    if (php_sapi_name() == 'cgi' || php_sapi_name() == 'fast-cgi') {
      @header('Status: '.$str);
    } else {
      @header('HTTP/1.1 '.$str);
    }
  }
}


?>
