<?php
/**
* Parser for special tags like papaya:media, papaya:link and papaya:popup
*
* @package papaya
* @author Thomas Weinert <weinert@papaya-cms.com>
* @version $Id: papaya_parser.php,v 1.25 2006/05/26 13:16:32 kelm Exp $
*/

/**
* Base object database access
*/
require_once(PAPAYA_INCLUDE_PATH.'system/sys_base_db.php');

/**
* Parser for special tags like papaya:media, papaya:link and papaya:popup
*
* @package papaya
* @author Thomas Weinert <weinert@papaya-cms.com>
*/
class papaya_parser extends base_db {
  /**
  * Table media files
  * @var string $tableMediaFiles
  */
  var $tableMediaFiles = PAPAYA_DB_TBL_MEDIA_FILES;
  /**
  * Table media files translations
  * @var string $tableMediaFilesTrans
  */
  var $tableMediaFilesTrans = PAPAYA_DB_TBL_MEDIA_FILES_TRANS;

  /**
  * Table topics
  * @var string $tableTopics
  */
  var $tableTopicsTrans = PAPAYA_DB_TBL_TOPICS_TRANS;

  /**
  * Dynamic Image Configurations Table
  * @var string $tableImageConfs
  */
  var $tableImageConfs = PAPAYA_DB_TBL_IMAGES;

  /**
  * Modules/Plugins
  * @var string $tableModules
  */
  var $tableModules = PAPAYA_DB_TBL_MODULES;

  /**
  * Media orgiginal path
  * @var string $mediaOrgPath
  */
  var $mediaOrgPath = PAPAYA_PATH_MEDIAFILES;

  /**
  * Media thumbnail path
  * @var string $mediaThumbPath
  */
  var $mediaThumbPath = PAPAYA_PATH_THUMBFILES;

  /**
  * papaya tag pattern
  * @var string $papayaTagPattern
  */
  var $papayaTagPattern = '~<(papaya|ndim):([a-z]\w+)\s*([^>]*)(/>|(>(</(\1):(\2)>)))~i';

  /**
  * papaya parameters pattern
  * @var string $papayaParamsPattern
  */
  var $papayaParamsPattern = '/(?:^|\s)([a-z]\w+)(?:=)(?:(?:\'([^\']+)\')|(?:"([^"]*)")|([^\s,]+))/i';

  /**
  * Tags
  * @var string $tags
  */
  var $tags;

  /**
  * Data
  * @var string $data
  */
  var $data;

  /**
  * Linked Files
  * @var array $files
  */
  var $files = NULL;

  /**
  * Linked page topics
  * @var array $topics
  */
  var $topics = NULL;

  /**
  * Linked Dynamic Images (Attributes)
  * @var array $dynImages
  */
  var $dynImages = NULL;

  /**
  * Linked Addons
  * @var array $addOns
  */
  var $addOns = NULL;
  /**
  * Addon tag data
  * @var array $addOnTags
  */
  var $addOnTags = NULL;

  /**
  * Parse function
  *
  * @param string $data
  * @param boolean $expand optional, default value FALSE
  * @param boolean $stripAttributes optional, default value FALSE
  * @access public
  * @return string
  */
  function parse($data, $lngId, $expand = FALSE, $stripAttributes = FALSE) {
    $this->data = $data;
    $this->lngId = $lngId;
    if (preg_match_all($this->papayaTagPattern, $data, $matches, PREG_SET_ORDER)) {
      foreach($matches as $match) {
        $tag = $match[0];
        if (!isset($this->tags[$tag])) {
          $this->tags[$tag] = &$this->parseTag($match[2], $match[3]);
        }
      }
      if (isset($this->tags) && is_array($this->tags)) {
        $this->initializeAddOns();
        $this->getTopicInfos();
        $this->getMediaInfos();
        $this->getImageConfInfos();
        foreach($this->tags as $tagString=>$tag) {
          $str = ($expand) ? $this->expandTag($tag, $tagString, $stripAttributes) : $this->createTag($tag);
          if ($str) {
            $this->tags[$tagString]['parsed'] = $str;
          } else {
            $this->tags[$tagString]['parsed'] = '<!--'.$tag['str'].'-->';
          }
        }
      }
    } else {
      unset($this->tags);
    }
    $this->replaceTags();
    return $this->data;
  }

  /**
  * Get additional data from parser addons
  *
  * @access public
  * @return string
  */
  function getParsedData() {
    $result = '';
    if (isset($this->addOns) && is_array($this->addOns) && count($this->addOns) > 0) {
      foreach($this->addOns as $addOn) {
        $result .= $addOn->getParsedData();
      }
    }
    return $result;
  }

  /**
  * Replace tags
  *
  * @access public
  */
  function replaceTags() {
    if (isset($this->tags) && is_array($this->tags)) {
      foreach($this->tags as $tag=>$parsed) {
        $this->data = str_replace($tag, $parsed['parsed'], $this->data);
      }
    }
  }

  /**
  * Parse tag
  *
  * @param string $tagName
  * @param string $tag
  * @access public
  * @return array
  */
  function &parseTag($tagName, $tag) {
    $result = FALSE;
    if (preg_match_all($this->papayaParamsPattern, $tag, $matches, PREG_SET_ORDER)) {
      foreach($matches as $match) {
        $paramName = $match[1];
        $paramValue = @($match[2].$match[3].$match[4]);
        $params[$paramName] = $paramValue;
        switch ($paramName) {
          case 'src' :
            $this->files[$paramValue] = FALSE;
            break;
          case 'topic' :
            $this->topics[(int)$paramValue] = FALSE;
            break;
          case 'image' :
            $this->dynImages[strtolower($paramValue)] = FALSE;
            break;
          case 'addon' :
            $this->addOns[strtolower($paramValue)] = FALSE;
            break;
          case 'href' :
            if (checkit::isNum($paramValue, TRUE)) {
              $this->topics[(int)$paramValue] = FALSE;
            }
            break;
        }
      }
      $result = array('name' => $tagName, 'params' => $params, 'str' => $tag);
      if (isset($params['addon'])) {
        $this->addOnTags[$params['addon']][] = &$result;
      }
    }
    return $result;
  }

  /**
  * Load addon data and initialize addons
  *
  * @access public
  */
  function initializeAddOns() {
    if (isset($this->addOns) && is_array($this->addOns)) {
      if (count($this->addOns) > 1) {
        $filter = "WHERE module_guid IN ('".implode("','",array_keys($this->addOns))."')";
      } else {
        reset($this->addOns);
        $filter = "WHERE module_guid = '".key($this->addOns)."'";
      }
      $sql = "SELECT m.module_guid, m.module_path, m.module_file, m.module_class
                FROM %s AS m
                $filter";
      if ($res = $this->databaseQueryFmt($sql, array($this->tableModules))) {
        $parent = NULL;
        while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
          $moduleObj = &$this->createPluginObject($row['module_path'].$row['module_file'],
          $row['module_class'], $row['module_guid'], $parent);
          if (isset($moduleObj) && is_object($moduleObj)) {
            $this->addOns[$row['module_guid']] = &$moduleObj;
            if (isset($this->addOnTags[$row['module_guid']])) {
              $moduleObj->initialize($this);
              $moduleObj->compileData($this->addOnTags[$row['module_guid']]);
            }
          }
        }
      }
    }
  }

  /**
  * Fetch informaiton to given topic id out of database
  */
  function getTopicInfos() {
    if (isset($this->topics) && is_array($this->topics)) {
      if (count($this->topics) > 1) {
        $filter = "WHERE topic_id IN ('".implode("','",array_keys($this->topics))."')";
      } else {
        reset($this->topics);
        $filter = "WHERE topic_id = '".key($this->topics)."'";
      }
      $sql = "SELECT topic_id, topic_title FROM %s $filter AND lng_id = %d";
      $params = array($this->tableTopicsTrans, $this->lngId);
      if ($res = $this->databaseQueryFmt($sql,$params)) {
        while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
          $this->topics[$row['topic_id']] = $row;
        }
      }
    }
  }

  /**
  * Fetch informaiton to given media object out of database
  */
  function getMediaInfos() {
    if (isset($this->files) && is_array($this->files)) {
      if (count($this->files) > 1) {
        $filter = "IN ('".implode("','",array_keys($this->files))."')";
      } else {
        reset($this->files);
        $filter = "= '".key($this->files)."'";
      }
      $sql = "SELECT f.file_id, f.file_name, ft.file_title, f.file_size
                FROM %s AS f
                LEFT OUTER JOIN %s AS ft ON (ft.file_id = f.file_id and ft.lng_id = %d)
               WHERE f.file_id $filter";
      $params = array($this->tableMediaFiles, $this->tableMediaFilesTrans, $this->lngId);
      if ($res = $this->databaseQueryFmt($sql, $params)) {
        while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
          $this->files[$row['file_id']] = $this->getFileInfos($row['file_id'],
            $row['file_name'], $row['file_title'], $row['file_size']);
        }
      }
    }
  }

  function getImageConfInfos() {
    if (isset($this->dynImages) && is_array($this->dynImages) && count($this->dynImages) > 0) {
      $moduleData = array();
      if (count($this->files) > 1) {
        $filter = "IN ('".implode("','",array_keys($this->dynImages))."')";
      } else {
        reset($this->dynImages);
        $filter = "= '".key($this->dynImages)."'";
      }
      $sql = "SELECT i.image_ident,
                     m.module_guid, m.module_path, m.module_file, m.module_class, m.module_title
                FROM %s AS i, %s AS m
                WHERE i.module_guid = m.module_guid
                  AND image_ident $filter";
      if ($res = $this->databaseQueryFmt($sql, array($this->tableImageConfs, $this->tableModules))) {
        while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
          if (isset($moduleData[$row['module_guid']])) {
            $row['ATTRS'] = $moduleData[$row['module_guid']];
          } else {
            $parent = NULL;
            $moduleObj = &$this->createPluginObject($row['module_path'].$row['module_file'],
              $row['module_class'], $row['module_guid'], $parent);
            if (isset($moduleObj) && is_object($moduleObj)) {
              $row['ATTRS'] = $moduleObj->attributeFields;
              $moduleData[$row['module_guid']] = $moduleObj->attributeFields;
            }
          }
          $this->dynImages[$row['image_ident']] = $row;
        }
      }
    }
  }

  /**
  * Get file Info
  *
  * @return mixed Array with width, height, type, title and size or FALSE
  */
  function getFileInfos($fileId, $fileName, $fileTitle, $fileSize) {
    if (file_exists($this->mediaOrgPath.$fileId)) {
      $data = @getImageSize($this->mediaOrgPath.$fileId);
      $result = array(
        'width'=>$data[0], 'height'=>$data[1],
        'type'=>$data[2], 'size'=>$fileSize,
        'file'=>$fileName, 'title'=>$fileTitle);
      return $result;
    }
    return FALSE;
  }

  /**
  * Function creates links by given tag
  *
  * @param array $tag data of papaya-tag for example <papaya:media.../ <papaya:link... or <papaya:popup...>
  * @return mixed Link or FALSE if unknown tag is given to function
  */
  function createTag(&$tag) {
    switch($tag['name']) {
    case 'media' :
      return $this->createMediaTag($tag['params']);
    case 'image' :
      return $this->createDynImageTag($tag['params']);
    case 'link' :
      return $this->createLink($tag['params']);
    case 'popup' :
      return $this->createPopupLink($tag['params']);
    case 'addon' :
      $guid = @(string)$tag['params']['addon'];
      if (isset($this->addOns[$guid]) && is_object($this->addOns[$guid])) {
        if ($tagStr = $this->addOns[$guid]->createTag($tag)) {
          return $tagStr;
        }
      }
      break;
    }
    return FALSE;
  }

  function getTagAttributeStr(&$attrDefs, &$tag, &$data, $stripDynamicAttributes = FALSE) {
    $result = '';
    if (isset($attrDefs) && is_array($attrDefs) && count($attrDefs) > 0) {
      foreach($attrDefs as $attrName=>$attrDef) {
        if (!($attrDef[2] && $stripDynamicAttributes)) {
          switch ($attrDef[0]) {
          case 'int' :
            if ($attrDef[2] && isset($data[$attrName]) && $data[$attrName] > 0) {
              $result .= ' '.$attrName.'="'.(int)$data[$attrName].'"';
            } elseif (isset($tag['params'][$attrName]) && $tag['params'][$attrName] > 0) {
              $result .= ' '.$attrName.'="'.(int)$tag['params'][$attrName].'"';
            } elseif (isset($attrDef[1]) && $attrDef[1] > 0) {
              $result .= ' '.$attrName.'="'.(int)$attrDef[1].'"';
            }
            break;
          case 'yesno' :
            if ($attrDef[2] && isset($data[$attrName]) && ($data[$attrName] == 'yes' || $data == 'no')) {
              $result .= ' '.$attrName.'="'.$data[$attrName].'"';
            } elseif (isset($tag['params'][$attrName]) && ($tag['params'][$attrName] == 'yes' || $tag['params'][$attrName] == 'no')) {
              $result .= ' '.$attrName.'="'.$tag['params'][$attrName].'"';
            } elseif (isset($attrDef[1]) && ($attrDef[1] == 'yes' || $attrDef[1] == 'no')) {
              $result .= ' '.$attrName.'="'.$attrDef[1].'"';
            }
            break;
          case 'bool' :
            if ($attrDef[2] && isset($data[$attrName])) {
              $result .= ' '.$attrName.'="'.(int)$data[$attrName].'"';
            } elseif (isset($tag['params'][$attrName]) && $tag['params'][$attrName] > 0) {
              $result .= ' '.$attrName.'="'.(int)$tag['params'][$attrName].'"';
            } elseif (isset($attrDef[1]) && $attrDef[1] > 0) {
              $result .= ' '.$attrName.'="'.(int)$attrDef[1].'"';
            }
            break;
          default :
            if ($attrDef[2] && isset($data[$attrName]) && $data[$attrName] != '') {
              $result .= ' '.$attrName.'="'.$data[$attrName].'"';
            } elseif (isset($tag['params'][$attrName]) && $tag['params'][$attrName] != '') {
              $result .= ' '.$attrName.'="'.$tag['params'][$attrName].'"';
            } elseif (isset($attrDef[1]) && $attrDef[1] != '') {
              $result .= ' '.$attrName.'="'.$attrDef[1].'"';
            }
            break;
          }
        }
      }
    }
    return $result;
  }

  /**
  * Expand tag with aditional inforamtions for JS up-to-date-check
  *
  * @param array $tag data of papaya-tag for example <papaya:media.../ <papaya:link... or <papaya:popup...>
  * @return mixed Link or FALSE if unknown tag is given to function
  */
  function expandTag($tag, $original, $stripDynamicAttributes = FALSE) {
    switch($tag['name']) {
    case 'addon' :
      $guid = @(string)$tag['params']['addon'];
      $result = sprintf('<papaya:addon addon="%s"', $guid);
      if (isset($this->addOns[$guid]) && is_object($this->addOns[$guid])) {
        if ($attrStr = $this->addOns[$guid]->expandTag($tag, $original, $stripDynamicAttributes)) {
          $result .= $attrStr;
        }
      }
      return $result .= ' />';
    case 'media' :
      $data = $this->files[$tag['params']['src']];
      $result = '<papaya:media src="'.$tag['params']['src'].'"';
      if (isset($tag['params']['download']) && $tag['params']['download']) {
        $result .= ' download="yes"';
        $attrDefs = array(
          'text' => array('string', NULL, FALSE),
          'target' => array('string', NULL, FALSE),
          'hint' => array('string', NULL, FALSE)
        );
        $result .= $this->getTagAttributeStr($attrDefs, $tag, $data, $stripDynamicAttributes);
      } else {
        $attrDefs = array(
          'subtitle' => array('string', NULL, FALSE),
          'alt' => array('string', NULL, FALSE),
          'target' => array('string', NULL, FALSE),
          'width' => array('int', NULL, FALSE),
          'height' => array('int', NULL, FALSE),
          'align' => array('string', NULL, FALSE),
          'resize' => array('string', 'max', FALSE),
          'align' => array('string', NULL, FALSE),
          'tspace' => array('int', NULL, FALSE),
          'rspace' => array('int', NULL, FALSE),
          'bspace' => array('int', NULL, FALSE),
          'lspace' => array('int', NULL, FALSE),
          'popup' => array('string', NULL, FALSE),
          'dyn_exists' => array('bool', NULL, TRUE),
          'dyn_src' => array('string', NULL, TRUE),
          'dyn_title' => array('string', NULL, TRUE),
          'dyn_size' => array('int', NULL, TRUE),
          'dyn_file' => array('string', NULL, TRUE),
          'dyn_type' => array('int', NULL, TRUE),
          'dyn_width' => array('int', NULL, TRUE),
          'dyn_height' => array('int', NULL, TRUE),
          'dyn_orgwidth' => array('int', NULL, TRUE),
          'dyn_orgheight' => array('int', NULL, TRUE),
        );

        if (isset($data) && is_array($data) && (!$stripDynamicAttributes)) {
          $data['orgwidth'] = $data['width'];
          $data['orgheight'] = $data['height'];
          if (((@(int)$tag['params']['width'] > 0) || (@(int)$tag['params']['height'] > 0)) &&
              ($data['width'] > 0) && ($data['height'] > 0)) {
            $data = $this->createThumbnail($tag['params'], $data);
            $data['filename'] = $this->getWebMediaLink($data['filename'], 'thumb');
          } else {
            $data['filename'] = $this->getWebMediaLink($tag['params']['src'], 'media');
          }

          $data['dyn_exists'] = 1;
          $data['dyn_src'] = $this->getAbsoluteURL($data['filename']);
          $data['dyn_title'] = papaya_strings::escapeHTMLChars($data['title']);
          $data['dyn_size'] = (int)$data['size'];
          $data['dyn_file'] = papaya_strings::escapeHTMLChars($data['file']);
          $data['dyn_type'] = (int)$data['type'];
          $data['dyn_width'] = (int)$data['width'];
          $data['dyn_height'] = (int)$data['height'];
          $data['dyn_orgwidth'] = (int)$data['orgwidth'];
          $data['dyn_orgheight'] = (int)$data['orgheight'];
        } else {
          $data['dyn_exists'] = 0;
        }

        if (isset($tag['params']['topic']) && $tag['params']['topic'] > 0 &&
            isset($this->topics[$tag['params']['topic']]) &&
            is_array($this->topics[$tag['params']['topic']])) {
          $attrDefs['topic'] = array('int', NULL, FALSE);
        } else {
          $attrDefs['href'] = array('string', NULL, FALSE);
        }

        $result .= $this->getTagAttributeStr($attrDefs, $tag, $data, $stripDynamicAttributes);
      }
      return $result .= ' />';
    case 'image' :
      $result = '<papaya:image image="'.strtolower($tag['params']['image']).'"';
      $imgConf = @$this->dynImages[strtolower($tag['params']['image'])];
      $attrDefs = array();
      $data = array();
      $paramName = 'img';
      $paramStr = '';
      if (isset($imgConf) && isset($imgConf['ATTRS']) && is_array($imgConf['ATTRS']) && count($imgConf['ATTRS']) > 0) {
        foreach($imgConf['ATTRS'] as $attrName=>$attrDef) {
          $attrDefs[$attrName] = array('string', papaya_strings::escapeHTMLChars($attrDef[6]), FALSE);
          if (isset($tag['params'][$attrName])) {
            $data[$attrName] = $tag['params'][$attrName];
            $paramStr .= '&'.$paramName.'['.urlencode($attrName).']='.urlencode($tag['params'][$attrName]);
          } else {
            $paramStr .= '&'.$paramName.'['.urlencode($attrName).']='.urlencode($attrDef[6]);
          }
        }
        $attrDefs['dyn_exists'] = array('bool', 1, TRUE);
        $src = '../'.$this->escapeForFilename($tag['params']['image']).'.image.jpg';
        $attrDefs['dyn_src'] = array('string',  $this->getAbsoluteURL($src).'?'.substr($paramStr, 1), TRUE);
      } else {
        $attrDefs['dyn_exists'] = array('bool', 0, TRUE);
      }
      $result .= $this->getTagAttributeStr($attrDefs, $tag, $data, $stripDynamicAttributes);
      return $result .= ' />';
    case 'link' :
      $result = '<papaya:link';
      if (isset($tag['params']['topic']) && $tag['params']['topic'] > 0) {
        $data = $this->topics[$tag['params']['topic']];
        if (isset($data) && is_array($data)) {
          $topicExists = 1;
          $topicTitle = papaya_strings::escapeHTMLChars($data['topic_title']);
        } else {
          $topicExists = 0;
          $topicTitle = $this->_gt('Invalid link');
        }
        if (isset($tag['params']['alttext'])) {
          switch ($tag['params']['alttext']) {
            case 'on': $tag['params']['altmode'] = 'yes'; break;
            case 'off': $tag['params']['altmode'] = 'no'; break;
            default : $tag['params']['altmode'] = 'auto'; break;
          }
        }
        $attrDefs = array(
          'text' => array('string', NULL, FALSE),
          'target' => array('string', NULL, FALSE),
          'topic' => array('int', NULL, FALSE),
          'altmode' => array('string', NULL, FALSE),
          'dyn_exists' => array('bool', NULL, TRUE),
          'dyn_title' => array('string', NULL, TRUE),
        );
        $data['dyn_exists'] = $topicExists;
        $data['dyn_title'] = $topicTitle;

        $result .= $this->getTagAttributeStr($attrDefs, $tag, $data, $stripDynamicAttributes);
      } elseif (isset($tag['params']['href']) && trim($tag['params']['href']) != '') {
        $attrDefs = array(
          'text' => array('string', NULL, FALSE),
          'target' => array('string', NULL, FALSE),
          'href' => array('string', NULL, FALSE)
        );
        $result .= $this->getTagAttributeStr($attrDefs, $tag, $data, $stripDynamicAttributes);
      }
      return $result .= ' />';
    case 'popup' :
      $data = $this->topics[$tag['params']['topic']];
      $result = '<papaya:popup';
      if (isset($tag['params']['topic'])) {
        $attrDefs = array(
          'topic' => array('int', NULL, FALSE),
          'text' => array('string', NULL, FALSE),
          'altmode' => array('string', NULL, FALSE),
          'name' => array('string', NULL, FALSE),
          'width' => array('int', NULL, FALSE),
          'height' => array('int', NULL, FALSE),
          'scrollbars' => array('yesno', 'no', FALSE),
          'resizeable' => array('yesno', 'no', FALSE),
          'toolbar' => array('yesno', 'no', FALSE),
          'dyn_exists' => array('bool', NULL, TRUE),
          'dyn_title' => array('string', NULL, TRUE),
        );

        if (isset($data) && is_array($data)) {
          $data['dyn_exists'] = 1;
          $data['dyn_title'] = papaya_strings::escapeHTMLChars($data['topic_title']);;
        } else {
          $data['dyn_exists'] = 0;
        }
        $result .= $this->getTagAttributeStr($attrDefs, $tag, $data, $stripDynamicAttributes);
      } elseif (isset($tag['params']['href'])) {
        $attrDefs = array(
          'href' => array('string', NULL, FALSE),
          'text' => array('string', NULL, FALSE),
          'name' => array('string', NULL, FALSE),
          'width' => array('int', NULL, FALSE),
          'height' => array('int', NULL, FALSE),
          'scrollbars' => array('yesno', 'no', FALSE),
          'resizeable' => array('yesno', 'no', FALSE),
          'toolbar' => array('yesno', 'no', FALSE),
        );
        $result .= $this->getTagAttributeStr($attrDefs, $tag, $data, $stripDynamicAttributes);
      }
      return $result .= ' />';
    }
    return $original;
  }

  /**
  * Create media tag
  *
  * @param array $params Parameters
  * @return mixed Link or FALSE
  *
  */
  function createMediaTag($params) {
    $data = @$this->files[$params['src']];
    if (isset($params) && is_array($params) && isset($data) && is_array($data)) {
      if (isset($params['download']) && $params['download'] == 'yes') {
        return $this->createDownloadTag($params, $data);
      }
      switch($data['type']) {
        case 1:
        case 2:
        case 3:
          return $this->createImageTag($params, $data);
        case 4:
          return $this->createFlashTag($params, $data);
        default :
          $ext = strtolower(strrchr($params['src'],'.'));
          switch ($ext) {
            case '.pdf' :
              return $this->createPDFTag($params, $data);
          }
          return $this->createDownloadTag($params, $data);
      }
    }
    return FALSE;
  }

  function createDynImageTag($params) {
    $paramName = 'img';
    $data = @$this->dynImages[$params['image']];
    if (isset($params) && is_array($params) && isset($data) && isset($data['ATTRS']) && is_array($data['ATTRS'])) {
      $paramStr = '';
      foreach($data['ATTRS'] as $attrName=>$attrDef) {
        if (isset($params[$attrName])) {
          $paramStr .= '&'.$paramName.'['.urlencode($attrName).']='.urlencode($params[$attrName]);
        } else {
          $paramStr .= '&'.$paramName.'['.urlencode($attrName).']='.urlencode($attrDef[6]);
        }
      }
      $src = $this->escapeForFileName($params['image']).'.image.jpg';
      return sprintf('<img src="%s?%s" class="papayaDynamicImage"/>', $src, substr($paramStr, 1));
    }
    return FALSE;
  }

  /**
  * Create image tag
  *
  * @param array $params Link parameters
  * @param array $data Parser given data like hight width of object
  * @return string
  */
  function createImageTag($params, $data) {
    $orgWidth = @(int)$data['width'];
    $orgHeight = @(int)$data['height'];
    if (((@$params['width'] > 0) || (@$params['height'] > 0)) &&
        (@$data['width'] > 0) && (@$data['height'] > 0)) {
      $data = $this->createThumbnail($params, $data);
      $imageData = array(
        'src' => $this->getWebMediaLink($data['filename'], 'thumb', $data['title']),
        'width' => @(int)$data['width'],
        'height' => @(int)$data['height']
      );
    } else {
      $imageData = array(
        'src' => $this->getWebMediaLink($params['src'], 'media', $data['title']),
        'width' => @(int)$data['width'],
        'height' => @(int)$data['height']
      );
    }

    if (isset($params['topic']) &&
        isset($this->topics[$params['topic']]) && is_array($this->topics[$params['topic']])) {
      $topicData = $this->topics[$params['topic']];
      $hrefData = array(
        'href' => $this->getWebLink((int)$params['topic']),
        'target' => ((isset($params['target'])) ? $params['target'] : '_self'),
        'title' => papaya_strings::escapeHTMLChars($topicData['topic_title'])
      );
    } elseif (isset($params['href']) && checkit::isNum($params['href'], TRUE) &&
        isset($this->topics[$params['href']]) && is_array($this->topics[$params['href']])) {
      $hrefData = array(
        'href' => $this->getWebLink((int)$params['href']),
        'target' => ((isset($params['target'])) ? $params['target'] : '_self'),
        'title' => ((isset($params['title'])) ? papaya_strings::escapeHTMLChars($params['title']) : '')
      );
    } elseif (isset($params['href']) && checkit::isHTTPX($params['href'], TRUE)) {
      $hrefData = array(
        'href' => $this->getWebLink(0, '', 'page', array('exit'=>$params['href'])),
        'target' => ((isset($params['target'])) ? $params['target'] : '_self'),
        'title' =>  ((isset($params['title'])) ? papaya_strings::escapeHTMLChars($params['title']) : '')
      );
    } elseif (isset($params['href']) && trim($params['href']) != '') {
      $hrefData = array(
        'href' => $params['href'],
        'target' => ((isset($params['target'])) ? $params['target'] : '_self'),
        'title' =>  ((isset($params['title'])) ? papaya_strings::escapeHTMLChars($params['title']) : '')
      );
    } else {
      $hrefData = FALSE;
    }

    $altText = ((isset($params['alt'])) ? papaya_strings::escapeHTMLChars($params['alt']) : $hrefData['title']);

    $borderData = $this->calcBorders($params, $data);

    $style = sprintf('border: none; margin: %dpx %dpx %dpx %dpx; width: %dpx;',
      $borderData['top'], $borderData['right'], $borderData['bottom'], $borderData['left'],
      $data['width']);
    switch (@$params['align'])  {
      case 'left' :
        $style .= 'float: left;';
        break;
      case 'right' :
        $style .= 'float: right;';
        break;
      case 'none' :
        $style = sprintf('border: none; width: %dpx;',
          $data['width']);
        break;
      case 'center' :
        $style = sprintf('display: block;  text-align: center; border: none; margin-top: %dpx; margin-bottom: %dpx; margin-left: auto; margin-right: auto; width: %dpx;',
          $borderData['top'], $borderData['bottom'], $data['width']);
        break;
    }

    if ($borderData['subtitle']) {
      $result = sprintf('<img src="%s" class="papayaImage" style="border: none; width: %dpx; height: %dpx;" alt="%s" />',
        $imageData['src'], $imageData['width'], $imageData['height'], $altText);
    } else {
      $result = sprintf('<img src="%s" class="papayaImage" style="%s height: %dpx;" alt="%s" />',
        $imageData['src'], $style, $imageData['height'], $altText);
    }

    if ($hrefData) {
      $result = sprintf('<a href="%s" target="%s" title="%s">%s</a>',
        $hrefData['href'], $hrefData['target'], $hrefData['title'], $result);
    } elseif (isset($params['popup']) && trim($params['popup']) != '' && $orgWidth > 0 && $orgHeight > 0) {
      $result = sprintf('<a href="%s" onclick="return openPopup(\'%s\', \'%s\', %d, %d, false, false, false);">%s</a>',
        $this->getWebMediaLink($params['src'], 'popup', $data['title']),
        $this->getWebMediaLink($params['src'], 'popup', $data['title']),
        $params['popup'], $orgWidth, $orgHeight, $result);
    }

    if ($borderData['subtitle']) {
      $result = sprintf('<div class="papayaImage" style="%s">%s
        <div class="papayaImageSubtitle">%s</div></div>',
        $style, $result, $borderData['subtitle']);
    }

    return $result;
  }

  /**
  * Calculate borders
  *
  * @param array $params Link parameters
  * @param array $data Parser given data like hight width of object
  * @return array ('left'=>$left, 'right'=>$right, 'top'=>$top, 'bottom'=>$bottom, 'subtitle'=>$subTitle)
  */
  function calcBorders($params, $data) {
    //Defaults sind von der Ausrichtung abhngig
    if (isset($params['align']) && $params['align'] == 'left') {
      $left = 0;
      $right = 4;
    } elseif (isset($params['align']) && $params['align'] == 'right') {
      $left = 4;
      $right = 0;
    } else {
      $left = 0;
      $right = 0;
    }

    //Userwerte
    $left   = (isset($params['lspace'])) ? $params['lspace'] : $left;
    $right  = (isset($params['rspace'])) ? $params['rspace'] : $right;
    $top    = (isset($params['tspace'])) ? $params['tspace'] : 0;
    $bottom = (isset($params['bspace'])) ? $params['bspace'] : 0;

    //Mit Untertitel werden 2 zustzliche Zeilen bentigt
    if (isset($params['subtitle']) && trim($params['subtitle']) != '') {
      $subTitle = $params['subtitle'];
    } else {
      $subTitle = FALSE;
    }

    //Rckgabe
    return array('left'=>$left, 'right'=>$right, 'top'=>$top, 'bottom'=>$bottom, 'subtitle'=>$subTitle);
  }

  /**
  * Create a thumbnail in given size
  *
  * @param array $params parameters
  * @param array $data Parser given data like hight width of object
  * @return string Tag in XHTML which can be used in all HTML-files
  */
  function createThumbnail($params, $data) {
    $result = $data;
    $result['filename'] = $params['src'];

    $tWidth = (isset($params['width']) && $params['width']>0) ? $params['width'] : $data['width'];
    $tHeight = (isset($params['height']) && $params['height']>0) ? $params['height'] : $data['height'];

    if (($tWidth != $data['width']) || ($tHeight != $data['height'])) {

      include_once(PAPAYA_INCLUDE_PATH.'system/base_thumbnail.php');
      $thumbnail = &new base_thumbnail;
      $thumbnail->baseDirectory = $this->mediaOrgPath;
      $thumbnail->thumbnailDirectory = $this->mediaThumbPath;
      $thumbnail->width = $tWidth;
      $thumbnail->height = $tHeight;
      $thumbnail->resizeFlag = (isset($params['resize'])) ? $params['resize'] : 'max';

      if ($thumbFileName = $thumbnail->getThumb($params['src'])) {
        $result['filename'] = $thumbFileName;
      }

      list($tWidth, $tHeight) = $thumbnail->calcSize($data['width'],$data['height']);
      $result['width'] = $tWidth;
      $result['height'] = $tHeight;

    }
    return $result;
  }

  /**
  * Create tag for flash object
  *
  * @param array $params Used parameters
  * @param array $data Parser given data like hight width of object
  * @return string Tag in XHTML which can be used in all HTML-files
  */
  function createFlashTag($params, $data) {
    $width = (isset($params['width']) ? $params['width'] : $data['width']);
    $height = (isset($params['height']) ? $params['height'] : $data['height']);
    $play = (isset($params['play']) ? $params['play'] : 'TRUE');
    $loop = (isset($params['loop']) ? $params['loop'] : 'TRUE');
    $align = (isset($params['align']) ? $params['align'] : 'left');
    $movie = $this->getWebMediaLink($params['src'], 'media', $data['title']);
    $bgcolor = (isset($params['bgcolor']) ? $params['bgcolor'] : '');
    $tag = sprintf('<object classid="clsid:D27CDB6E-AE6D-11cf-96B8-444553540000"   codebase="http://download.macromedia.com/pub/shockwave/cabs/flash/swflash.cab#version=5,0,0,0" width="%s" height="%s" align="%s">
      <param name="movie" value="%s" />
      <param name="play" value="%s" />
      <param name="loop" value="%s" />
      <param name="quality" value="high" />
      <embed src="%s" play="%s" loop="%s" quality="high" bgcolor="%s" width="%s" height="%s" align="%s" type="application/x-shockwave-flash" pluginspage="http://www.macromedia.com/shockwave/download/index.cgi?P1_Prod_Version=ShockwaveFlash"></embed>
      </object>',
      $width, $height, $align, $movie, $play, $loop,
      $movie, $play, $loop, $bgcolor, $width, $height, $align);
    return $tag;
  }

  /**
  * Create tag for pdf-object
  *
  * @param array $params Used parameters
  * @param array $data Parser given data like hight width of object
  * @return string Tag in XHTML which can be used in all HTML-files
  */
  function createPDFTag($params, $data) {
    $width = (isset($params['width']) ? $params['width'] : '100%');
    $height = (isset($params['height']) ? $params['height'] : '400');
    $play = (isset($params['play']) ? $params['play'] : 'TRUE');
    $loop = (isset($params['loop']) ? $params['loop'] : 'TRUE');
    $align = (isset($params['align']) ? $params['align'] : 'left');
    $file = $this->getWebMediaLink($params['src'], 'media', $data['title']);
    $bgcolor = (isset($params['bgcolor']) ? $params['bgcolor'] : '');
    $tag = sprintf('
      <object classid="clsid:CA8A9780-280D-11CF-A24D-444553540000" width="%s" height="%s" align="%s">
        <param name="src" value="%s" />
        <!-- <iframe src="%s" width="%s" height="%s" align="%s">
          <noframes> //-->
          <embed src="%s" width="%s" height="%s" align="%s" href="%s">
            <noembed>
              %s<br />
              <a href="http://www.adobe.de/products/acrobat/readstep2.html" target="_blank">www.adobe.de/products/acrobat/readstep2.html</a>
            </noembed>
          </embed>
          <!-- </noframes>
        </iframe> //-->
      </object>',
      $width, $height, $align, $file,
      $file, $width, $height, $align,
      $file, $width, $height, $align, $file,
      $params['text']);
    return $tag;
  }

  /**
  * Create download link tag for example parameter "download=yes" is given.
  *
  * @param array $params Used parameters
  * @param array $data Parser given data like hight width of object
  * @return string Tag in XHTML which can be used in all HTML-files
  */
  function createDownloadTag($params, $data) {
    if ($data['size'] > 1048576) {
      $size = number_format(($data['size']/1048576),2,',','.').' MB';
    } elseif ($data['size'] > 1024) {
      $size = number_format(($data['size']/1024),0,',','.').' kB';
    } else {
      $size = number_format($data['size'],0,',','.').' Bytes';
    }
    $result = sprintf('<a href="%s" title="%s" target="%s">%s</a>',
      $this->getWebMediaLink($params['src'], 'download', $data['title']),
      ((isset($params['hint']))   ? $params['hint']   : ''),
      ((isset($params['target'])) ? $params['target'] : '_blank'),
      ((isset($params['text']))   ? $params['text']   : ($data['title'].' ('.$size.')'))
    );
    return $result;
  }

  /**
  * Create Link
  *
  * @param array $params Used parameters
  * @param array $data Parser given data like hight width of object
  * @return string Tag in XHTML which can be used in all HTML-files
  */
  function createLink($params) {
    if (isset($params['topic']) && $params['topic'] > 0) {
      $data = $this->topics[$params['topic']];
      if (isset($params) && is_array($params)) {
        if (isset($params['alttext'])) {
          if (isset($data) && is_array($data)) {
            switch ($params['alttext']) {
              case 'on'  :
                $caption =  (isset($params['text'])) ? $params['text'] : $data['topic_title'];
                break;
              default :
                $caption =  $data['topic_title'];
            }
            $target = isset($params['target']) ? ' target="'.$params['target'].'"' : '';
            $result = sprintf('<a href="%s" class="%s"%s>%s</a>',
              $this->getWebLink((int)$params['topic']),
              (isset($params['class'])) ? $params['class'] : 'papayaLink',
              $target, papaya_strings::escapeHTMLChars($caption));
          } else {
            switch ($params['alttext']) {
              case 'off' :
                $caption =  '';
                  break;
              default :
                $caption =  (isset($params['text'])) ? $params['text'] : '';
            }
            $result = $caption;
          }
        } else {
          if (isset($data) && is_array($data)) {
            switch (@$params['altmode']) {
              case 'yes' :
                $caption =  (isset($params['text'])) ? $params['text'] : $data['topic_title'];
                break;
              case 'no' :
                $caption =  $data['topic_title'];
                break;
              default :
                $caption =  $data['topic_title'];
            }
            $target = isset($params['target']) ? ' target="'.$params['target'].'"' : '';
            $result = sprintf('<a href="%s" class="%s"%s>%s</a>',
              $this->getWebLink((int)$params['topic']),
              (isset($params['class'])) ? $params['class'] : 'papayaLink',
              $target, papaya_strings::escapeHTMLChars($caption));
          } else {
            switch ($params['altmode']) {
              case 'yes' :
                $caption =  (isset($params['text'])) ? $params['text'] : '';
                break;
              case 'no' :
                $caption =  '';
                break;
              default :
                $caption =  (isset($params['text'])) ? $params['text'] : '';
            }
            $result = $caption;
          }
        }
        return $result;
      }
    } elseif (isset($params['href']) && trim($params['href']) != '') {
      $target = isset($params['target']) ? ' target="'.$params['target'].'"' : '';
      if (preg_match('~^mailto:~i', $params['href'])) {
        return sprintf('<a href="%s" class="%s"%s>%s</a>',
          $params['href'],
          (isset($params['class'])) ? $params['class'] : 'papayaLinkMailTo', $target,
          (isset($params['text']) ? $params['text'] : $params['href']));
      } else {
        if (isset($params['class']) && $params['class'] == 'papayaReferenceLink') {
          return sprintf('<a href="%s" class="%s"%s>%s</a>',
            $params['href'],
            (isset($params['class'])) ? $params['class'] : 'papayaLink', $target,
            (isset($params['text']) ? $params['text'] : $params['href']));
        }
        if (preg_match('~^\d+$~', $params['href'])) {
          $link = $this->getWebLink($params['href']);
        } elseif (preg_match('~^\w+://$~', $params['href'])) {
          $link = $this->getWebLink(NULL, NULL, NULL, array('exit' => $params['href']));
        } else {        
          $link = $params['href'];
        }
        return sprintf('<a href="%s" class="%s"%s>%s</a>', $link,
          (isset($params['class'])) ? $params['class'] : 'papayaLink', $target,
          (isset($params['text']) ? $params['text'] : $params['href']));
      }
    }
  }

  /**
  * Create Link for popup window
  *
  * @param array $params Used parameters
  * @return string Tag in XHTML which can be used in all HTML-files
  */
  function createPopUpLink($params) {
    $href = NULL;
    if (isset($this->topics[$params['topic']]) && is_array($this->topics[$params['topic']])) {
      $data = $this->topics[$params['topic']];
      $href = $this->getWebLink((int)$data['topic_id'], '', 'page', NULL, NULL, $data['topic_title']);
    } else {
      $href = $this->getAbsoluteURL($params['href']);
    }
    if (isset($href)) {
      $height = (@$params['height'] > 50) ? (int)$params['height'] : 400;
      $width = (@$params['width'] > 50) ? (int)$params['width'] : 300;
      $name = (trim(@$params['name']) != '') ? (int)$params['name'] : 'papaya_popup';
      $scrollbars = (@$params['scrollbars'] == 'yes') ? 'yes' : 'no';
      $resizeable = (@$params['resizeable'] == 'yes') ? 'yes' : 'no';
      $toolbar = (@$params['toolbar'] == 'yes') ? 'yes' : 'no';
      if (isset($data) && is_array($data)) {
        switch ($params['altmode']) {
          case 'yes' :
                    $caption =  (isset($params['text'])) ? $params['text'] : $data['topic_title'];
            break;
          case 'no' :
                  $caption =  $data['topic_title'];
            break;
          default :
                  $caption =  $data['topic_title'];
        }
      } else {
        $caption =  (isset($params['text'])) ? $params['text'] : $href;
      }
      return sprintf('<a href="%s" onclick="return openPopup(\'%s\',\'%s\',\'%d\',\'%d\',\'%s\',\'%s\',\'%s\');">%s</a>',
          papaya_strings::escapeHTMLChars($href),
          papaya_strings::escapeHTMLChars($href),
          papaya_strings::escapeHTMLChars($name),
          $width, $height, $scrollbars, $resizeable, $toolbar,
          papaya_strings::escapeHTMLChars($caption)
        );
    } else {
      switch ($params['altmode']) {
        case 'yes' :
              $caption =  (isset($params['text'])) ? $params['text'] : '';
          break;
        case 'no' :
              $caption =  '';
          break;
        default :
          $caption =  (isset($params['text'])) ? $params['text'] : '';
      }
          return $caption;
    }
    return '';
  }

  function mediaFileExists($fileId) {
    return (isset($this->files[$fileId]) && is_array($this->files[$fileId]));
  }
}
?>