<?php
/**
* Object to display user todo list
*
* @package papaya
* @author  Raphael Kalla <kalla@papaya-cms.com>
*/


/**
* Basic class database access
*/
require_once(PAPAYA_INCLUDE_PATH.'system/sys_base_db.php');
/**
* Object to display user todo list
*
* @package papaya
* @author Raphael Kalla <kalla@papaya-cms.com>
*/
class papaya_todo extends base_db {
  var $tableAuthUser = PAPAYA_DB_TBL_AUTHUSER;
  var $tableAuthGroups = PAPAYA_DB_TBL_AUTHGROUPS;
  var $tableTodos = PAPAYA_DB_TBL_TODOS;

  var $todo;
  var $todoList;

  var $paramName = 'todo';

  function initialize() {
    $this->initializeParams();
  }

  /**
  * Execute - base function for handlig parameters
  *
  * @access public
  */
  function execute() {
      switch (@$this->params['cmd']) {
        case 'todo_delete' :
          if (isset($this->params['todo_id']) && $this->params['todo_id'] > 0 &&
              isset($this->params['confirm_delete']) && $this->params['confirm_delete']) {
            if ($this->deleteTodo((int)$this->params['todo_id'])) {
              unset($this->params['cmd']);
              $this->addMsg(MSG_INFO, $this->_gt('Entry deleted.'));
            } else {
              $this->addMsg(MSG_WARNING, $this->_gt('Database error'));
            }
          }
          break;
        case 'todo_create' :
            $this->initializeTodoDialog();
            if ($this->dialogTodo->checkDialogInput() && $this->checkTodoInput()) {
              if ($this->createTodo()) {
                unset($this->dialogTodo);
                unset($this->todo);
                unset($this->params['todo_id']);
                $this->addMsg(MSG_INFO,$this->_gt('New entry saved.'));
              } else {
                $this->addMsg(MSG_WARNING,$this->_gt('Database error! New Entry not saved.'));
              }
            }
          break;
       case 'todo_edit' :
          if (isset($this->params['todo_id']) && $this->params['todo_id'] > 0 &&
              $this->loadTodo($this->params['todo_id'])) {
            $this->initializeTodoDialog();
            if ($this->dialogTodo->checkDialogInput() && $this->checkTodoInput()) {
              if ($this->saveTodo()) {
                unset($this->dialogTodo);
                unset($this->todo);
                $this->addMsg(MSG_INFO,$this->_gt('Changes saved.'));
              } else {
                $this->addMsg(MSG_WARNING,$this->_gt('Database error! Changes not saved.'));
              }
            }
          }
          break;
        case 'todo_forward' :
          if (isset($this->params['todo_id']) && $this->params['todo_id'] > 0 &&
              isset($this->params['confirm_forward']) && $this->params['confirm_forward'] ) {
            $this->initializeForwardDialog();
              if ($this->forwardTodo()) {
                unset($this->params['cmd']);
                unset($this->dialogForwardTodo);
                unset($this->params['todo_id']);
                $this->addMsg(MSG_INFO,$this->_gt('Todo forwarded.'));
              } else {
                $this->addMsg(MSG_WARNING,$this->_gt('Database error! Changes not saved.'));
              }
          }
          break;
      }
      if (isset($this->params['todo_id']) && $this->params['todo_id'] > 0) {
        $this->loadTodo($this->params['todo_id']);
      }
  }

  function getXML() {
    $this->loadTodoList();
    $this->layout->addLeft($this->getTodoList());
    switch(@$this->params['cmd']) {
      case 'todo_forward' :
        $this->layout->add($this->getForwardDialog());
        break;
      case 'todo_delete' :
        $this->layout->add($this->getDeleteForm());
        break;
      default :
        $this->layout->add($this->getTodoDialog());
        break;
    }
  }

  function getUserList($perms = NULL) {
    $sql = "SELECT user_id, givenname, surname
                FROM %s
               ORDER BY givenname, surname ASC";

    $params = array( $this->tableAuthUser );
    if ($res = $this->databaseQueryFmt($sql,$params)) {
      while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $users[$row['user_id']] = $row['surname'].", ".$row['givenname'];
      }
    }
    return $users;
  }


  /**
  * Load users Todo's
  *
  * @param integer $baseId
  * @access public
  */
  function loadTodoList() {
    unset($this->todoList);
    $sql = "SELECT td.todo_id, td.title, td.priority, td.date_from, td.date_to, td.status, td.comment, td.user_id_from, td.topic_id, au.givenname, au.surname
              FROM %s AS td
              LEFT OUTER JOIN %s AS au ON ( au.user_id = td.user_id_from )
             WHERE td.user_id_to = '".$this->authUser->user['user_id']."'
             ORDER BY td.date_to ASC\r\n";
    $params = array( $this->tableTodos,$this->tableAuthUser );
    if ($res = $this->databaseQueryFmt($sql,$params)) {
      while ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->todoList[$row['todo_id']] = $row;
      }
    }
  }

  /**
  * Try to load a todo entry
  *
  * @param integer $todoId
  * @access public
  * @return boolean
  */
  function loadTodo($todoId) {
    unset($this->todo);
    $sql = "SELECT todo_id, title, priority, date_from, date_to, status, topic_id, comment, user_id_from, user_id_to
              FROM %s
             WHERE todo_id = %d";
    if ($res = $this->databaseQueryFmt($sql, array($this->tableTodos, $todoId))) {
      if ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
        $this->todo = $row;
        return TRUE;
      }
    }
    return FALSE;
  }

  /**
  *
  * @access public
  * @return string
  */
  function getTodoList() {
    $result = '';
    if (isset($this->todoList) && is_array($this->todoList) && count($this->todoList) > 0) {
      $result .= sprintf('<listview width="550" title="%s">'.LF, $this->_gt('Todo list'));
      $result .= '<cols>'.LF;
      $result .= sprintf('<col>%s</col>'.LF, $this->_gt('Title'));
      $result .= sprintf('<col>%s</col>'.LF, $this->_gt('Priority'));
      $result .= sprintf('<col>%s</col>'.LF, $this->_gt('Date To'));
      $result .= sprintf('<col>%s</col>'.LF, $this->_gt('Status'));
      $result .= sprintf('<col>%s</col>'.LF, $this->_gt('Comment'));
      $result .= sprintf('<col>%s</col>'.LF, $this->_gt('Published by'));
      $result .= '</cols>'.LF;
      $result .= '<items>'.LF;
      $counter = '0';
      $result .= $this->getTodoElements();
      $result .=  '</items></listview>'.LF;
    } else {
      $this->addMsg(MSG_INFO, $this->_gt('No todo\'s defined.'));
    }
    return $result;
  }

  function getTodoElements() {
    $result = '';
      if (isset($this->todoList) && is_array($this->todoList)) {
        foreach ($this->todoList as $id=>$values) {
          if (isset($values) && is_array($values)) {

             if($values['date_to'] > 0 ) {
               $dateToStr = date("Y-m-d H:i",($values['date_to']));
             } else {
               $dateToStr = $this->_gt("undefined");
             }
            $result .= sprintf('<listitem title="%s" href="%s" %s>',
              papaya_strings::escapeHTMLChars($values['title']),
              @$this->Link.'?'.$this->paramName.'[todo_id]='.$values['todo_id'],
              (($values['todo_id'] == @$this->params['todo_id']) ? ' selected="selected"' : ''));
            $result .= sprintf('<subitem align="center"><glyph src="%s"/></subitem>'.LF,
              $this->images[$this->getPriorityGlyphIndex($values['priority'])]);
            $result .= sprintf('<subitem>%s</subitem>'.LF, $dateToStr);
            $result .= sprintf('<subitem align="center"><glyph src="%s"/></subitem>'.LF,
              $this->images[$this->getStatusGlyphIndex($values['status'])]);
            $result .= sprintf('<subitem>%s</subitem>'.LF,
              papaya_strings::escapeHTMLChars($values['comment']));
            $result .= sprintf('<subitem>%s</subitem>'.LF,
              papaya_strings::escapeHTMLChars($values['givenname']." ".$values['surname']));
            $result .= sprintf('<subitem align="center"><a href="%s"><glyph src="%s"/></a></subitem>'.LF,
              $this->getLink(array('page_id'=>$values['topic_id']), 'tt', 'topic.php'),
              $this->images[4]);
            $result .= '</listitem>'."\r\n";
          }
        }
      }
    return $result;
  }

  /**
  * Check input
  *
  * @access public
  * @return boolean
  */
  function checkTodoInput() {
    $result = TRUE;
    if (isset($this->params['topic_id']) && $this->params['topic_id'] > 0) {
      $topic = &new papaya_topic;
      if (!$topic->topicExists(@(int)$this->params['topic_id'])) {
        $this->addMsg(MSG_ERROR, $this->_gt('Specified page doesnt exists.'));
        $result = FALSE;
      }
    } else {
      $this->addMsg(MSG_ERROR, $this->_gt('No page specified.'));
      $result = FALSE;
    }
    return $result;
  }

  /**
  * Create a new Todo entry
  *
  * @access public
  * @return mixed Id of new Todo (or NULL)
  */
  function createTodo() {
    $timeTo = split(":", @$this->params['time_to']);
    $time = (@$timeTo[0] * 60 * 60) + @$timeTo[1] * 60;

    $dateTo = split("-", @$this->params['date_to']);
    $date = mktime(0, 0, 0, @$dateTo[1], @$dateTo[2], @$dateTo[0]);

    if (isset($this->authUser->user['user_id']) &&
        $this->authUser->user['user_id'] != "") {
      $idFrom = $this->authUser->user['user_id'];
    } else {
      $idFrom = $this->params['user_id_from'];
    }

    $values = array(
      'topic_id'     => $this->params['topic_id'],
      'title'        => $this->params['title'],
      'priority'     => $this->params['priority'],
      'date_from'    => time(),
      'date_to'      => ($date + $time),
      'status'       => $this->params['status'],
      'comment'      => $this->params['comment'],
      'user_id_from' => $idFrom,
      'user_id_to'   => $this->params['user_id_to']
    );
    if (FALSE !== $this->databaseInsertRecord($this->tableTodos,
        'todo_id', $values)) {
      $this->notifyOfTodo($values);
      return TRUE;
    }
    return FALSE;
  }

  /**
  * Save todo
  *
  * @access public
  * @return boolean
  */
  function saveTodo() {
    $timeTo = split(":", $this->params['time_to']);
    $time = (@$timeTo[0] * 60 * 60)+ @$timeTo[1] * 60;
    $dateTo = split("-", $this->params['date_to']);
    $date = mktime(0, 0, 0, @$dateTo[1], @$dateTo[2], @$dateTo[0]);

    $values = array(
      'topic_id'   => $this->params['topic_id'],
      'title'      => $this->params['title'],
      'priority'   => $this->params['priority'],
      'date_to'    => ($date + $time),
      'status'     => $this->params['status'],
      'comment'    => $this->params['comment'],
      'user_id_to' => $this->params['user_id_to']
    );

    if (FALSE !== $this->databaseUpdateRecord($this->tableTodos, $values,
        'todo_id', (int)$this->params['todo_id'])) {
      $this->notifyOfTodo($values);
      return TRUE;
    }
    return FALSE;
  }

  /**
  * Delete Todo entry
  *
  * @param integer $id
  * @access public
  * @return mixed
  */
  function deleteTodo($todoId) {
    return (FALSE !== $this->databaseDeleteRecord($this->tableTodos,
      'todo_id', $todoId));
  }

  /**
  * send email of todo as reminder
  *
  * @param array $values values of inserted/updated todo
  * @return boolean
  */
  function notifyOfTodo($values) {
    if (defined('PAPAYA_OVERVIEW_TASK_NOTIFY') && PAPAYA_OVERVIEW_TASK_NOTIFY &&
        isset($values) && is_array($values) && count($values) > 0) {
      $additional =  (isset($values['user_id_from'])) ? sprintf(" OR user_id = '%s' ", $values['user_id_from']) : '';
      $sql = "SELECT email
              FROM %s WHERE user_id = '%s' $additional";
      $params = array($this->tableAuthUser, $values['user_id_to']);
      if ($res = $this->databaseQueryFmt($sql, $params)) {
        if ($row = $res->fetchRow(DB_FETCHMODE_ASSOC)) {
          include_once(PAPAYA_INCLUDE_PATH.'system/sys_email.php');
          $emailObj = &new email;

          require_once(PAPAYA_INCLUDE_PATH.'system/base_statictables.php');
          $states = base_statictables::getTodoStates();
          $priorities = base_statictables::getTodoPriorities();

          $bodyTemplate =
            $this->_gt('Task'). ': {%TASK%}'.LF.
            $this->_gt('Page Id'). ': {%TOPIC_ID%}'.LF.
            $this->_gt('Date'). ': {%DATE%}'.LF.
            $this->_gt('Status'). ': {%STATUS%}'.LF.
            $this->_gt('Priority'). ': {%PRIORITY%}'.LF.
            $this->_gt('Comment'). ': {%COMMENT%}'.LF;
          $data = array(
            'TASK' => $values['title'],
            'TOPIC_ID' => $values['topic_id'],
            'DATE' => date('Y-m-d H:i', $values['date_to']),
            'PRIORITY' => $priorities[$values['priority']],
            'STATUS' => $states[$values['status']],
            'COMMENT' => $values['comment'],
          );
          $emailObj->setTemplate('body', $bodyTemplate, $data);
          $subject  = sprintf($this->_gt('Todo reminder: %s'), $values['title']);
          while ($row['email'] != '') {
            if ($emailObj->send($row['email'], $subject)) {
              $this->addMsg(MSG_INFO, sprintf('Notification was sent to %s', $row['email']));
              return TRUE;
            }
          }
        }
      }
    }
    return FALSE;
  }

  /**
  * Forward todo
  *
  * @access public
  * @return boolean
  */
  function forwardTodo() {

    $values = array(
      'user_id_to' => $this->params['user_id_to']
    );
    return (FALSE !== $this->databaseUpdateRecord($this->tableTodos, $values, 'todo_id', (int)$this->params['todo_id']));
  }

  /**
  * Get buttons
  *
  * @access public
  * @return mixed
  */
  function getButtons() {
    $result = NULL;
    $result[] = array(
      'New Todo', 'Create a new Todo Entry', $this->images[70], 16,
      $this->baseLink.'?'.$this->paramName.'[todo_id]=0',
      '_self'
    );
    $result[] = '-';
    if (isset($this->todo)) {
      $result[] = array(
        'Forward', 'Forward Todo to another User', $this->images[126], 16,
        $this->baseLink.'?'.$this->paramName.'[cmd]=todo_forward&amp;'.$this->paramName.'[todo_id]='.(int)$this->params['todo_id'],
        '_self'
      );
      $result[] = array(
        'Delete', 'Delete selected Todo', $this->images[10], 16,
        $this->baseLink.'?'.$this->paramName.'[cmd]=todo_delete&amp;'.$this->paramName.'[todo_id]='.(int)$this->params['todo_id'],
        '_self'
      );
    }
    return $result;
  }

  /**
  * Initialize the create / modify Dialog
  *
  * @access public
  */
  function initializeTodoDialog() {
    if (!(isset($this->dialogTodo) && is_object($this->dialogTodo))) {
      include_once(PAPAYA_INCLUDE_PATH.'system/base_dialog.php');
      if (isset($this->todo)) {
        $dialogTitle = $this->_gt('Edit Todo');
        $data = $this->todo;
        $selectedUser = @$this->params['user_id_to'];

        $data['date_from'] =  date("Y-m-d H:i", $this->todo['date_from']);

        if($data['date_to'] > 0) {
          $data['date_to'] =  date("Y-m-d", $this->todo['date_to']);
          $data['time_to'] =  date("H:i", $this->todo['date_to']);
        } else {
          $data['date_to'] = '';
          $data['time_to'] = '';
        }
        $hidden = array(
          'cmd' => 'todo_edit',
          'todo_id' => (int)$this->todo['todo_id']
        );
        $btnCaption = 'Edit';
      } else {
        $dialogTitle = $this->_gt('New Todo');
        $selectedUser = $this->authUser->user['user_id'];
        $data = array();
        $hidden = array(
          'cmd' => 'todo_create'
        );
        $btnCaption = 'Add';
      }

      include_once(PAPAYA_INCLUDE_PATH.'system/base_statictables.php');
      $fields = array(
        'user_id_to' => array('Create for', 'isSometext', TRUE, 'combo', $this->getUserList(), '', $selectedUser),
        'title' => array('Title', 'isSometext', TRUE, 'input', 255, '', ''),
        'topic_id' => array('Page id', 'isNum', TRUE, 'pageid', 5, '', ''),
        'priority'=>array('Priority', 'isNum', TRUE, 'combo', base_statictables::getTodoPriorities()),
        'date_to' => array('Date to', 'isISODate', FALSE, 'input', 20, 'ISO Date YYYY-MM-DD', ''),
        'time_to' => array('Time to', 'isSomeText', FALSE, 'input', 20, 'ISO Time HH:MM', ''),
        'status'=>array('Status', 'isNum', TRUE, 'combo', base_statictables::getTodoStates()),
        'comment' => array('Comment', 'isSometext', FALSE, 'input', 255, '', ''),
      );
      $this->dialogTodo = &new base_dialog($this, $this->paramName, $fields, $data, $hidden);
      $this->dialogTodo->msgs = &$this->msgs;
      $this->dialogTodo->loadParams();
      $this->dialogTodo->dialogTitle = $dialogTitle;
      $this->dialogTodo->buttonTitle = $btnCaption;
      $this->dialogTodo->dialogDoubleButtons = FALSE;
    }
  }

  /**
  * Build xml for the create / modify Dialog
  *
  * @access public
  * @return string
  */
  function getTodoDialog() {
    $this->initializeTodoDialog();
    return $this->dialogTodo->getDialogXML();
  }

  /**
  * Get delete formular
  *
  * @access public
  * @return string
  */
  function getDeleteForm() {
    include_once(PAPAYA_INCLUDE_PATH.'system/base_msgdialog.php');
    $hidden = array(
      'cmd' => 'todo_delete',
      'todo_id' => $this->todo['todo_id'],
      'confirm_delete'=>1,
    );
    $msg = sprintf($this->_gt('Really delete TodoEntry "%s"?'), $this->todo['todo_id']);
    $dialog = &new base_msgdialog($this, $this->paramName, $hidden, $msg, 'question');
    $dialog->msgs = &$this->msgs;
    $dialog->buttonTitle = 'Delete';
    return $dialog->getMsgDialog();
  }

  /**
  * Get forward formular
  *
  * @access public
  * @return string
  */
  function initializeForwardDialog() {
    include_once(PAPAYA_INCLUDE_PATH.'system/base_dialog.php');
    $hidden = array(
      'cmd' => 'todo_forward',
      'todo_id' => $this->todo['todo_id'],
      'confirm_forward'=>1,
    );

    $fields = array(
        'user_id_to' => array('User', 'isSometext', TRUE, 'combo', $this->getUserList(), '', '')
    );

    $this->dialogForwardTodo = &new base_dialog($this, $this->paramName, $fields, $data, $hidden);
    $this->dialogForwardTodo->msgs = &$this->msgs;
    $this->dialogForwardTodo->loadParams();
    $this->dialogForwardTodo->dialogTitle = $this->_gt('Forward to');
    $this->dialogForwardTodo->buttonTitle = 'Forward';
    $this->dialogForwardTodo->dialogDoubleButtons = FALSE;
  }

  /**
  * Build xml for the forward Dialog
  *
  * @access public
  * @return string
  */
   function getForwardDialog() {
    $this->initializeForwardDialog();
    return $this->dialogForwardTodo->getDialogXML();
  }

  function getPriorityGlyphIndex($priority) {
    switch($priority) {
      case 2 : return 85;
      case 1 : return 84;
      default : return 0;
    }
  }
  function getStatusGlyphIndex($status) {
    switch($status) {
      case 2 : return 123;
      case 1 : return 124;
      default : return 125;
    }
  }
}
?>
