<?php
/**
* XSL Layout-object with XML data buffer
*
* @package papaya
* @author Thomas Weinert <weinert@papaya-cms.com>
* @version $Id: papaya_xsl.php,v 1.7 2006/04/25 10:48:18 weinert Exp $
*/

/**
* XSL-transformation
*/
require_once(PAPAYA_INCLUDE_PATH.'system/sys_xsl.php');
/**
* XSL Layout-object with XML data buffer
*
* @package papaya
*/
class papaya_xsl extends sys_xsl {

  /**
  * base target for navigation links
  * @var string $baseTarget
  */
  var $baseTarget = "_top";

  /**
  * Buffer for XHTML-Data
  * @var array $data
  */
  var $data;

  /**
  * Preferred XSLT-Extension configurable by constants
  * @var string $preferred
  */
  var $preferred = PAPAYA_XSLT_EXTENSION;

  /**
  * Constuctor
  * @param string $xslFileName XSL-Stylesheet
  */
  function __construct($xslFileName="") {
    $this->setXSL($xslFileName);
  }

  /**
  * Set XSL-file
  *
  * @param string $xslFileName
  */
  function setXSL($xslFileName="") {
    $this->xslFileName = $xslFileName;
    if (file_exists($xslFileName)) {
      $this->xslData = $this->loadFile($xslFileName);
    }
  }

  /**
  * Add XML data
  *
  * @param string $col column
  * @param string $xmlData data
  * @param boolean $encode encode special characters ?
  */
  function addData($col,$xmlData, $encode=TRUE) {
    if ($encode) {
      $str = papaya_strings::entityToXML($xmlData);
    } else {
      $str = $xmlData;
    }
    if (isset($this->data[$col])) {
      $this->data[$col] .= $str;
    } else {
      $this->data[$col] = $str;
    }
  }

  /**
  * Add Content for middle column (XML / XHTML)
  *
  * @param string $xmlData data
  * @param string  $tagName
  * @param boolean $encode encode spezial characters ?
  */
  function add($xmlData, $tagName=NULL, $encode=TRUE) {
    if (isset($tagName)) {
      $this->addData($tagName, $xmlData, $encode);
    } else {
      $this->addData("centercol", $xmlData, $encode);
    }
  }

  /**
  * Add content for left column (XML)
  * @param string $xmlData data
  * @param boolean $encode encode special character ?
  */
  function addLeft($xmlData, $encode=TRUE) {
    $this->addData("leftcol",$xmlData, $encode);
  }

  /**
  * Add content for middle column (XML)
  * @param string $xmlData data
  * @param boolean $encode encode special character ?
  */
  function addCenter($xmlData, $encode=TRUE) {
    $this->addData("centercol",$xmlData, $encode);
  }

  /**
  * Add content for right column (XML)
  * @param string $xmlData data
  * @param boolean $encode encode special character ?
  */
  function addRight($xmlData, $encode=TRUE) {
    $this->addData("rightcol",$xmlData, $encode);
  }

  /**
  * Add content for menu (XML)
  * @param string $xmlData data
  * @param boolean $encode encode special character ?
  */
  function addMenu($xmlData, $encode=TRUE) {
    $this->addData("menus",$xmlData, $encode);
  }

  /**
  * Output XML
  *
  * @return string
  */
  function xml() {
    $result = '<?xml version="1.0" encoding="utf-8" ?>';
    $result .= "<page>\n";
    if (isset($this->data) && is_array($this->data)) {
      foreach($this->data as $tag=>$c) {
        if (strlen($c)>0) {
          $result .= "<{$tag}>\n";
          $result .= $c.LF;
          $result .= "</{$tag}>\n";
        }
      }
    }
    $result .= "</page>";
    if (defined('PAPAYA_LATIN1_COMPATIBILITY') && PAPAYA_LATIN1_COMPATIBILITY) {
      include_once(PAPAYA_INCLUDE_PATH.'system/papaya_strings.php');
      return papaya_strings::ensureUTF8($result);
    } else {
      return $result;
    }
  }

  /**
  * Parse
  *
  * @access public
  * @return mixed boolean or string
  */
  function parse() {
    if (!(FALSE === ($str = parent::parse()))) {
      $replace = array(
        '#<\?xml[^>]+\?>#',
        '#\s*xmlns(:[a-zA-Z]+)?="\s*"#',
        '#\s*xmlns="[^"]*"#',
        '#<([\w:-]+)\s\s*>#s'
      );
      $with = array('', '', '', '<\\1>');
      $str = preg_replace($replace, $with, $str);
      $this->result = $str;
      return $this->result;
    }
    return FALSE;
  }

  /**
  * Transform XML with XSL to HTML
  *
  * @access public
  * @return string
  */
  function xhtml() {
    $this->xmlData = $this->xml();
    if ((defined('PAPAYA_DBG_XML_OUTPUT') && PAPAYA_DBG_XML_OUTPUT === '1') && 
        (isset($_GET['XML']) && $_GET['XML'] || isset($_GET['PAGEXML']) && $_GET['PAGEXML'])) {
      if (defined('PAPAYA_ADMIN_PAGE') && PAPAYA_ADMIN_PAGE) {
        header('Content-type: text/xml; charset=utf-8');
      }
      echo $this->xmlData;
    } else {
      if ($res = $this->parse()) {
        return $this->reWrap($this->result);
      } else {
        return $this->getLastErrorHTML();
      }
    }
  }

  /**
  * Rewrap
  *
  * @param string $str
  * @access public
  * @return string
  */
  function reWrap($str) {
    return preg_replace('#(<[\w])\s#', "\\1\n", $str);
  }
}
?>