<?php
/**
* DB- abstraction layer
*
* Class for handle database requests
* @package papaya_base
* @author Thomas Weinert <weinert@papaya-cms.com>
* @version $Id: sys_db_simple.php,v 1.7 2006/06/06 11:36:57 weinert Exp $
*/



/**
* no error
*/
define('DB_ERROR_NONE', 0);

/**
* no connection
*/
define('DB_ERROR_CONNECTION', 1);

/**
* file error messages will be written in
*/
define('DB_ERROR_FILE', 2);

/**
* Error Protokoll
*/
define('DB_ERROR_PROTOCOL', 3);

/**
* Error in processing query
*/
define('DB_ERROR_QUERY', 4);

/**
* Error handling request in database
*/
define('DB_ERROR_DBACTION', 5);

/**
* Error handling request extension not found
*/
define('DB_ERROR_EXTENSION', 6);

/**
* basic class
*/
require_once(PAPAYA_INCLUDE_PATH.'system/sys_base_object.php');

/**
* DB - abstraction layer
*
* @package papaya_base
* @author Thomas Weinert <weinert@papaya-cms.com>
*/
class db_simple extends base_object {
  /**
  * database URI
  * @var string $databaseURI
  */
  var $databaseURI = "";

  /**
  * database configuratuion
  * @var string $databaseConfiguration
  */
  var $databaseConfiguration;

  /**
  * database connection
  * @var object $databaseObject
  */
  var $databaseObject = NULL;


  /**
  * database
  * @var string $dbsyntaxArr
  */
  var $dbsyntaxArr = array('mysql', 'mysqli', 'pgsql', 'sqlite');


  /**
  * Debug the next n queries
  * @var integer $debugCounter
  */
  var $debugCounter = 0;




  /**
  * connect to database
  * @return object dbcon_base
  */
  function connect(&$object) {
    if (!(isset($this->databaseObject) && is_object($this->databaseObject) && $this->databaseObject->connect())) {
      $this->createConnection($object);
    }
    if (!(isset($this->databaseObject) && is_object($this->databaseObject))) {
      unset($this->databaseObject);
      return FALSE;
    } else {
      return $this->databaseObject->connect();
    }
  }

  /**
  * establish connection
  *
  * @return boolean Success
  */
  function createConnection(&$object) {
    $this->databaseConfiguration = $this->parseURI($this->databaseURI);
    if (isset($this->databaseObject) && is_object($this->databaseObject)) {
      if (!$this->databaseObject->connect($object)) {
        if ($this->lastError[0] != DB_ERROR_CONNECTION) {
          $this->setError($object, DB_ERROR_CONNECTION, $this->databaseObject->getError());
        }
      } else {
        return TRUE;
      }
    } else {
      if (in_array($this->databaseConfiguration['dbsyntax'],$this->dbsyntaxArr)) {
        $file = PAPAYA_INCLUDE_PATH.'system/db/'.strtolower($this->databaseConfiguration['dbsyntax']).'.php';
        if (defined('PAPAYA_DBG_DEVMODE') && PAPAYA_DBG_DEVMODE) {
          $found = include_once($file);
        } else {
          $found = @include_once($file);
        }
        if ($found) {
          $class = 'dbcon_'.$this->databaseConfiguration['dbsyntax'];
          $this->databaseObject = &new $class($this->databaseConfiguration);
          if ($this->databaseObject->extensionFound()) {
            if (!$this->databaseObject->connect($object)) {
              $this->setError($object, DB_ERROR_CONNECTION, $this->databaseObject->getError());
            } else {
              return TRUE;
            }
          } else {
            $this->setError($object, DB_ERROR_EXTENSION);
          }
        } else {
          $this->setError($object, DB_ERROR_FILE);
        }
      } else {
        $this->setError($object, DB_ERROR_PROTOCOL);
      }
    }
    return FALSE;
  }

  /**
  * Parse URI contains connection data
  *
  * @param mixed $dsn Verbindungsdaten
  * @return array
  */
  function parseURI($dsn) {
    if (isset($dsn) && is_array($dsn)) {
      return $dsn;
    }
    $parsed = array(
      'phptype'  => FALSE,
      'dbsyntax' => FALSE,
      'username' => FALSE,
      'password' => FALSE,
      'protocol' => FALSE,
      'hostspec' => FALSE,
      'port'     => FALSE,
      'socket'   => FALSE,
      'database' => FALSE
    );
    // Find phptype and dbsyntax
    if (($pos = strpos($dsn, '://')) !== FALSE) {
      $str = substr($dsn, 0, $pos);
      $dsn = substr($dsn, $pos + 3);
    } else {
      $str = $dsn;
      $dsn = NULL;
    }

    // Get phptype and dbsyntax
    // $str => phptype(dbsyntax)
    if (preg_match('|^(.+?)\((.*?)\)$|', $str, $arr)) {
      $parsed['phptype']  = $arr[1];
      $parsed['dbsyntax'] = (empty($arr[2])) ? $arr[1] : $arr[2];
    } else {
      $parsed['phptype']  = $str;
      $parsed['dbsyntax'] = $str;
    }

    if (empty($dsn)) {
      return $parsed;
    }

    // Get (if found): username and password
    // $dsn => username:password@protocol+hostspec/database
    if (($at = strrpos($dsn,'@')) !== FALSE) {
      $str = substr($dsn, 0, $at);
      $dsn = substr($dsn, $at + 1);
      if (($pos = strpos($str, ':')) !== FALSE) {
        $parsed['username'] = urldecode(substr($str, 0, $pos));
        $parsed['password'] = urldecode(substr($str, $pos + 1));
      } else {
        $parsed['username'] = urldecode($str);
      }
    }

        // Find protocol and hostspec

        // $dsn => proto(proto_opts)/database
    if (preg_match('|^([^(]+)\((.*?)\)/?(.*?)$|', $dsn, $match)) {
      $proto       = $match[1];
      $protoOpts  = (!empty($match[2])) ? $match[2] : FALSE;
      $dsn         = $match[3];

    // $dsn => protocol+hostspec/database (old format)
    } else {
      if (strpos($dsn, '+') !== FALSE) {
        list($proto, $dsn) = explode('+', $dsn, 2);
      }
      if (strpos($dsn, '/') !== FALSE) {
        list($protoOpts, $dsn) = explode('/', $dsn, 2);
      } else {
        $protoOpts = $dsn;
        $dsn = NULL;
      }
    }

    // process the different protocol options
    $parsed['protocol'] = (!empty($proto)) ? $proto : 'tcp';
    $protoOpts = urldecode($protoOpts);
    if ($parsed['protocol'] == 'tcp') {
      if (strpos($protoOpts, ':') !== FALSE) {
        list($parsed['hostspec'], $parsed['port']) = explode(':', $protoOpts);
      } else {
        $parsed['hostspec'] = $protoOpts;
      }
    } elseif ($parsed['protocol'] == 'unix') {
      $parsed['socket'] = $protoOpts;
    }

    // Get dabase if any
    // $dsn => database
    if (!empty($dsn)) {
    // /database
      if (($pos = strpos($dsn, '?')) === FALSE) {
      $parsed['database'] = $dsn;
      // /database?param1=value1&param2=value2
      } else {
        $parsed['database'] = substr($dsn, 0, $pos);
        $dsn = substr($dsn, $pos + 1);
        if (strpos($dsn, '&') !== FALSE) {
          $opts = explode('&', $dsn);
        } else { // database?param1=value1
          $opts = array($dsn);
        }
        foreach ($opts as $opt) {
          list($key, $value) = explode('=', $opt);
          if (!isset($parsed[$key])) { // don't allow params overwrite
            $parsed[$key] = urldecode($value);
          }
        }
      }
    }
    return $parsed;
  }

  function debugNextQuery($count = 1) {
    $this->debugCounter = $count;
  }

  /**
  *
  *
  * @param base_db &$object calling object
  * @param mixed $value value to escape
  * @access public
  * @return string
  */
  function escapeStr(&$object, $value) {
    if (is_bool($value)) {
      return ($value) ? '1' : '0';
    }
    if ($this->connect($object)) {
      return $this->databaseObject->escapeStr($value);
    }
    return '';
  }

  /**
  * Process a query
  *
  * @param string $sql
  * @param object base_db &$object calling object
  * @param integer $max limit data sets
  * @param integer $offset limit data sets - Start
  * @return integer Result Index
  */
  function query($sql, &$object, $max = NULL, $offset = NULL) {
    if ($this->connect($object)) {
      if ($this->debugCounter > 0 || (defined('PAPAYA_DBG_DATABASE_EXPLAIN') && PAPAYA_DBG_DATABASE_EXPLAIN === '1')) {
        $timeStart = microtime();
        $res = $this->databaseObject->query($sql, $max, $offset, TRUE);
      } else {
        $res = $this->databaseObject->query($sql, $max, $offset, FALSE);
      }
      if ((!isset($res)) || ($res === FALSE)) {
        $error = $this->databaseObject->getError();
        $error['query'] = $sql;
        $this->setError($object, DB_ERROR_QUERY, $error);
      } else {
        if ((defined('PAPAYA_DBG_DATABASE_EXPLAIN') && PAPAYA_DBG_DATABASE_EXPLAIN === '1') || $this->debugCounter > 0)  {
          $timeEnd = microtime();
          $from = explode(' ',$timeStart);
          $to = explode(' ',$timeEnd);
          $to = $to[0]+(double)$to[1];
          $from = $from[1]+(double)$from[0];
          $time = $to-$from;
          @$this->timeSum += $time;
          $error['time'] = number_format(($time)*1000, 4).'ms ('.number_format(($this->timeSum)*1000, 4).' ms)';
          $error['query'] = $sql;
          if (method_exists($this->databaseObject, 'getexplain')) {
            $error['explain'] = $this->databaseObject->getExplain();
            if (is_object($res)) {
              $error['records'] = $res->absCount();
              if (isset($max)) $error['perpage'] = $res->count();
              if (isset($offset)) $error['offset'] = $offset;
            } else {
              $error['records'] = NULL;
            }
          }
          $this->setError($object, DB_ERROR_NONE, $error);
          $this->printError();
        }
        if ($this->debugCounter > 0) {
          $this->debugCounter--;
        }
        if (is_int($res) && $res == 0) {
          return TRUE;
        } else {
          return $res;
        }
      }
    }
    return FALSE;
  }

  /**
  * create new record, return id, set default values
  *
  * @param object BASE_DB &$object calling object
  * @param string $table table
  * @param string $idField fields with ID
  * @param array $values default values
  * @return integer data sets number
  */
  function insertRecord(&$object, $table, $idField, $values = NULL) {
    if ($this->connect($object)) {
      $res = $this->databaseObject->insertRecord($table, $idField, $values);
      if (!isset($res) || ($res === FALSE)) {
        $error = $this->databaseObject->getError();
        $error['action'] = 'INSERT RECORD';
        $error['query'] = $this->databaseObject->lastSQLQuery;
        $this->setError($object, DB_ERROR_DBACTION, $error);
      } else {
        return $res;
      }
    }
    return FALSE;
  }

  /**
  * insert records
  *
  * @param object BASE_DB &$object calling object
  * @param string $table table
  * @param array $values default values
  * @return boolean
  */
  function insertRecords(&$object, $table, $values) {
    if ($this->connect($object)) {
      $res = $this->databaseObject->insertRecords($table, $values);
      if (!isset($res) || ($res === FALSE)) {
        $error = $this->databaseObject->getError();
        $error['action'] = 'INSERT RECORDS';
        $error['query'] = $this->databaseObject->lastSQLQuery;
        $this->setError($object, DB_ERROR_DBACTION, $error);
      } else {
        return TRUE;
      }
    }
    return FALSE;
  }

  /**
  * change data sets
  *
  * @param object BASE_DB &$object calling object
  * @param string $table table
  * @param array $values default values
  * @param array $filter condition
  * @return integer number changed records
  */
  function updateRecord(&$object, $table, &$values, $filter) {
    if ($this->connect($object)) {
      if (!(isset($values) && is_array($values) && count($values) > 0)) {
        $error['action'] = 'UPDATE RECORDS';
        $error['query'] = '';
        $error['no'] = '0';
        $error['str'] = 'NO DATA';
        $this->setError($object, DB_ERROR_DBACTION, $error);
      } else {
        $res = $this->databaseObject->updateRecord($table, $values, $filter);
        if (!isset($res) || ($res === FALSE)) {
          $error = $this->databaseObject->getError();
          $error['action'] = 'UPDATE RECORDS';
          $error['query'] = $this->databaseObject->lastSQLQuery;
          $this->setError($object, DB_ERROR_DBACTION, $error);
        } else {
          return $res;
        }
      }
    }
    return FALSE;
  }

  /**
  * delete data sets
  *
  * @param object BASE_DB &$object calling objekt
  * @param string $table table
  * @param string $filter condition
  * @return integer number deleted records
  */
  function deleteRecord(&$object, $table, $filter) {
    if ($this->connect($object)) {
      $res = $this->databaseObject->deleteRecord($table, $filter);
      if (!isset($res) || ($res === FALSE)) {
        $error = $this->databaseObject->getError();
        $error['action'] = 'DELETE RECORDS';
        $error['query'] = $this->databaseObject->lastSQLQuery;
        $this->setError($object, DB_ERROR_DBACTION, $error);
      } else {
        return $res;
      }
    }
    return FALSE;
  }

  /**
  * Get all table names
  *
  * @param object base_db &$object
  * @access public
  * @return array
  */
  function queryTableNames(&$object) {
    if ($this->connect($object)) {
      return $this->databaseObject->queryTableNames();
    }
    return array();
  }

  /**
  * Return table structur as arrays
  *
  * @param object base_db &$object
  * @param string $tableName table name
  * @param string $tablePrefix Prefix
  * @access public
  * @return array
  */
  function queryTableStructure(&$object, $tableName, $tablePrefix = '') {
    if ($this->connect($object)) {
      return $this->databaseObject->queryTableStructure($tableName, $tablePrefix);
    }
    return array();
  }


  /**
  * Create given table
  *
  * @param string &$tableData
  * @param string $tablePrefix
  * @access public
  * @return boolean
  */
  function createTable(&$object, &$tableData, $tablePrefix) {
    if ($this->connect($object)) {
      if (!$this->databaseObject->createTable($tableData, $tablePrefix)) {
        $error = $this->databaseObject->getError();
        $error['action'] = 'CREATE TABLE';
        $error['query'] = $this->databaseObject->lastSQLQuery;
        $this->setError($object, DB_ERROR_DBACTION, $error);
      } else {
        return TRUE;
      }
    }
    return False;
  }

  /**
  * Add field
  *
  * @param object base_db &$object
  * @param string $table
  * @param array &$fieldData
  * @access public
  * @return boolean
  */
  function addField(&$object, $table, &$fieldData) {
    if ($this->connect($object)) {
      if (!$this->databaseObject->addField($table, $fieldData)) {
        $error = $this->databaseObject->getError();
        $error['action'] = 'ADD FIELD';
        $error['query'] = $this->databaseObject->lastSQLQuery;
        $this->setError($object, DB_ERROR_DBACTION, $error);
      } else {
        return TRUE;
      }
    }
    return FALSE;
  }

  /**
  * Change field
  *
  * @param object base_db &$object
  * @param string $table
  * @param array &$fieldData
  * @access public
  * @return boolean
  */
  function changeField(&$object, $table, &$fieldData) {
    if ($this->connect($object)) {
      if (!$this->databaseObject->changeField($table, $fieldData)) {
        $error = $this->databaseObject->getError();
        $error['action'] = 'MODIFY FIELD';
        $error['query'] = $this->databaseObject->lastSQLQuery;
        $this->setError($object, DB_ERROR_DBACTION, $error);
      } else {
        return TRUE;
      }
    }
    return FALSE;
  }

  /**
  * Delete field
  *
  * @param object base_db &$object
  * @param string $table
  * @param string $field
  * @access public
  * @return boolean
  */
  function dropField(&$object, $table, &$field) {
    if ($this->connect($object)) {
      if (!$this->databaseObject->dropField($table, $field)) {
        $error = $this->databaseObject->getError();
        $error['action'] = 'DROP FIELD';
        $error['query'] = $this->databaseObject->lastSQLQuery;
        $this->setError($object, DB_ERROR_DBACTION, $error);
      } else {
        return TRUE;
      }
    }
    return FALSE;
  }

  /**
  * Add index
  *
  * @param object base_db &$object
  * @param string $table
  * @param array &$index
  * @access public
  * @return boolean
  */
  function addIndex(&$object, $table, &$index) {
    if ($this->connect($object)) {
      if ($this->databaseObject->addIndex($table, $index)) {
        return TRUE;
      }
    }
    return FALSE;
  }

  /**
  * Change index
  *
  * @param object base_db &$object
  * @param string $table
  * @param array &$index
  * @access public
  * @return boolean
  */
  function changeIndex(&$object, $table, &$index) {
    if ($this->connect($object)) {
      if ($this->databaseObject->changeIndex($table, $index)) {
        return TRUE;
      }
    }
    return FALSE;
  }

  /**
  * DBMS spezific SQL source
  *
  * @param string $function sql function
  * @param array $params params
  * @access public
  * @return mixed sql string or FALSE
  */
  function getSQLSource($function, $params) {
    if ($this->connect($object)) {
      if ($str = $this->databaseObject->getSQLSource($function, $params)) {
        return $str;
      }
    }
    return FALSE;
  }

  /**
  * DBMS spezific SQL condition string
  *
  * @param array $filter sql condition array
  * @access public
  * @return mixed sql string or FALSE
  */
  function getSQLCondition($filter) {
    if ($this->connect($object)) {
      if ($str = $this->databaseObject->getSQLCondition($filter)) {
        return $str;
      }
    }
    return FALSE;
  }

  /**
  * Delete index
  *
  * @param object base_db &$object
  * @param string $table
  * @param string $name
  * @access public
  * @return boolean
  */
  function dropIndex(&$object, $table, $name) {
    if ($this->connect($object)) {
      if ($this->databaseObject->dropIndex($table, $name)) {
        return TRUE;
      }
    }
    return FALSE;
  }


  function compareFieldStructure(&$object, &$xmlField, &$databaseField) {
    if ($this->connect($object)) {
      return $this->databaseObject->compareFieldStructure($xmlField, $databaseField);
    }
    return FALSE;
  }

  function compareKeyStructure(&$object, &$xmlKey, &$databaseKey) {
    if ($this->connect($object)) {
      return $this->databaseObject->compareKeyStructure($xmlField, $databaseField);
    }
    return FALSE;
  }

  /**
  * Place error ,dump error, log error
  *
  * @param integer $errorLevel error priority
  * @param array $error error detail
  * @param boolean $log send error message tolog object
  */
  function setError(&$object, $errorLevel, $error = NULL, $log = TRUE) {
    $this->lastError = array($errorLevel, $error, $this->getBacktrace());
    $this->lastError[1]['class'] = get_class($object);
    if ($errorLevel > 0) {
      if (defined('PAPAYA_DBG_DATABASE_ERROR') && PAPAYA_DBG_DATABASE_ERROR === '1') {
        $this->printError();
      } elseif ((!defined('PAPAYA_DBG_DATABASE_ERROR')) && defined('PAPAYA_DBG_DEVMODE') && PAPAYA_DBG_DEVMODE) {
        $this->printError();
      }
    }
    if (function_exists('is_a') && is_subclass_of($object, 'base_log')) {
      return 0;
    } elseif (function_exists('is_a') && is_a($object, 'base_log')) {
      return 0;
    }
    if ($errorLevel != DB_ERROR_NONE && $log) {
      if (($this->lastError[0] == @DB_ERROR_QUERY)) {
        $this->logMsg(MSG_ERROR, PAPAYA_LOGTYPE_DATABASE, 'Error in query',
          sprintf('<p>Error "%d" on execution.</p><p>The database server says:<br/> "%s".</p><p>The query was:<br/><tt>%s</tt></p>',
            $this->lastError[1]['no'], papaya_strings::escapeHTMLChars($this->lastError[1]['str']),
          papaya_strings::escapeHTMLChars($this->lastError[1]['query'])));
      } elseif (isset($this->lastError[1]['no'])) {
        $this->logMsg(MSG_ERROR, PAPAYA_LOGTYPE_DATABASE, 'Database error',
          sprintf('<p>Error "%d" on execution.</p><p>The database server says:<br/> "%s".</p>',
            $this->lastError[1]['no'], papaya_strings::escapeHTMLChars($this->lastError[1]['str'])));
      }
    }
  }

  /**
  * Get back trace
  *
  * @access public
  * @return mixed FALSE or string
  */
  function getBacktrace() {
    if (function_exists('debug_backtrace')) {
      $bt = debug_backtrace();
      if (isset($bt) && is_array($bt) && count($bt) > 3) {
        $result = '<table border="1" cellpadding="2" cellspacing="0" width="100%">';
        for($i = 3; $i < count($bt); $i++) {
          $result .= '<tr>';
          $result .= '<td>'.@$bt[$i]['function'].'</td>';
          $result .= '<td>'.@$bt[$i]['class'].'</td>';
          $result .= '<td>'.@$bt[$i]['file'].'</td>';
          $result .= '<td align="right">'.@$bt[$i]['line'].'</td>';
          $result .= '</tr>';
        }
        $result .= '</table>';
        return $result;
      }
    }
    return FALSE;
  }

  /**
  * error message in browser
  *
  * @access public
  */
  function printError() {
    echo '</form>', '</table>';
    echo '<div class="errorDatabase" style="position: relative; z-index: 10000;">';
    echo '<table border="1" width="800" cellpadding="2" cellspacing="0" align="center" bgcolor="#C0C0C0">';
    switch ($this->lastError[0]) {
      case DB_ERROR_NONE :
        echo sprintf('<tr><th>Debug output (%s)</th><th align="right">'.$this->lastError[1]['time'].'</th></tr>', $this->lastError[1]['class']);
        echo '<tr><td colspan="2"><tt>'.nl2br(papaya_strings::escapeHTMLChars($this->lastError[1]['query'])).'</tt></td></tr>';
        if (isset($this->lastError[1]['records'])) {
          echo '<tr><td colspan="2">';
          if (isset($this->lastError[1]['perpage'])) {
            echo $this->lastError[1]['perpage'].' of ';
          }
          echo $this->lastError[1]['records'].' record(s)';
          if (isset($this->lastError[1]['offset'])) {
            echo ' from '.$this->lastError[1]['offset'];
          }
          echo '</td></tr>';
        }
        if (isset($this->lastError[1]['explain'])) {
          echo '<tr><td colspan="2">'.$this->lastError[1]['explain'].'</td></tr>';
        }
        break;
      case DB_ERROR_CONNECTION :
        echo '<tr><th>DB connection failed</th></tr>';
        break;
      case DB_ERROR_DBACTION :
        if (isset($this->lastError[1]['class'])) {
          echo sprintf('<tr><th>Action %s in class %s failed.</th></tr>',
            $this->lastError[1]['action'], papaya_strings::escapeHTMLChars($this->lastError[1]['class']));
        } else {
          echo sprintf('<tr><th>Action %s failed.</th></tr>',
            $this->lastError[1]['action']);
        }
        echo '<tr><td><b>'.(isset($this->lastError[1]['no']) ? $this->lastError[1]['no'].':' : '').'</b> '.
          papaya_strings::escapeHTMLChars($this->lastError[1]['str']).'</tt></td></tr>';
        echo '<tr><td><tt>'.papaya_strings::escapeHTMLChars($this->lastError[1]['query']).'</tt></td></tr>';
        break;
      case DB_ERROR_FILE :
        echo '<tr><th>DB class "'.$this->databaseConfiguration['dbsyntax'].'" not found.</th></tr>';
        break;
      case DB_ERROR_PROTOCOL :
        echo '<tr><th>Invalid database protocol "'.$this->databaseConfiguration['dbsyntax'].'".</th></tr>';
        break;
      case DB_ERROR_EXTENSION :
        echo '<tr><th><br>Database extension for protocol "'.$this->databaseConfiguration['dbsyntax'].'" not found.</th></tr>';
        break;
      case DB_ERROR_QUERY :
        echo sprintf('<tr><th>Error in query (%s)</th></tr>', papaya_strings::escapeHTMLChars($this->lastError[1]['class']));
        echo '<tr><td><b>'.(isset($this->lastError[1]['no']) ? $this->lastError[1]['no'].':' : '').'</b> '.
          papaya_strings::escapeHTMLChars($this->lastError[1]['str']).'</tt></td></tr>';
        echo '<tr><td><tt>'.papaya_strings::escapeHTMLChars($this->lastError[1]['query']).'</tt></td></tr>';
        break;
    }
    if (in_array($this->lastError[0], array(DB_ERROR_QUERY, DB_ERROR_DBACTION, DB_ERROR_NONE))) {
      if (isset($this->lastError[2]) && $this->lastError[2] !== FALSE) {
        echo '<tr><td colspan="2"><tt>'.$this->lastError[2].'</tt></td></tr>';
      }
    }
    echo '</table><br/>';
    echo '</div>';
    flush();
  }
}
?>