<?php
/**
* Implementation expat xml access
*
* @package papaya_base
* @author Thomas Weinert <weinert@papaya-cms.com>
* @version $Id: expat.php,v 1.3 2006/04/20 18:24:00 weinert Exp $
*/

/**
* base class xmltree and xmlnodes
*/
require_once(dirname(__FILE__).'/common.php');

/**
* Expat xml handling class
* @package papaya_base
*/
class xmltree_expat extends xmltree_common {

  function loadFromFile($fileName) {
    unset($this->root);
    unset($this->_currentNode);

    if (!($fp = fopen($fileName, "r"))) {
      $this->setError(XML_ERROR_FILE, array('file'=>$fileName));
      return XML_ERROR_FILE;
    }

    $parser = $this->initParser();

    while ($data = fread($fp, 4096)) {
      if (!xml_parse($parser, $data, feof($fp))) {
        $errorNumber = xml_get_error_code($parser);
        $this->setError(XML_ERROR_PARSE,
          array('errno'=> $errorNumber, 'error'=>xml_error_string($errorNumber),
            'file'=>$fileName,
            'line'=>xml_get_current_line_number($parser)
          )
        );
        return XML_ERROR_PARSE;
      }
    }
    xml_parser_free($parser);
    return 0;
  }

  function loadFromString($str) {
    unset($this->root);
    unset($this->_currentNode);

    if (trim($str) == '') {
      $this->setError(XML_ERROR_EMPTY);
      return XML_ERROR_EMPTY;
    }

    $parser = $this->initParser();
    if (!xml_parse($parser, $str, TRUE)) {
      $errorNumber = xml_get_error_code($parser);
      $this->setError(XML_ERROR_PARSE,
        array('errno'=> $errorNumber, 'error'=>xml_error_string($errorNumber),
          'line'=>xml_get_current_line_number($parser)));
      return XML_ERROR_PARSE;
    }
    xml_parser_free($parser);
    return 0;
  }

  /**
  * Init ext/xml Parser and define method handlers
  *
  * @access public
  * @return object xml_parser
  */
  function initParser() {
    $parser = xml_parser_create();

    xml_parser_set_option($parser, XML_OPTION_CASE_FOLDING, FALSE);
    xml_set_element_handler($parser, array(&$this, 'parserStartElement'), array(&$this, 'parserEndElement'));
    xml_set_character_data_handler($parser, array(&$this, 'parserCharacterData'));

    return $parser;
  }

  /**
  * Handler - new element in xml started
  *
  * @param object $parser
  * @param string $name
  * @param array $attrs
  * @access public
  */
  function parserStartElement(&$parser, $name, $attrs) {
    if (isset($this->_currentNode) && is_object($this->_currentNode)) {
      $parent = &$this->_currentNode;
      $this->_currentNode = &new simple_xmlnode($parent, $name, $attrs);
    } else {
      $parent = NULL;
      $this->root = &new simple_xmlnode($parent, $name, $attrs);
      $this->_currentNode = &$this->root;
    }
  }

  /**
  * Handler - current element in xml closed
  *
  * @param object $parser
  * @param string $name
  * @access public
  */
  function parserEndElement(&$parser, $name) {
    $this->_currentNode = &$this->_currentNode->parent;
  }

  /**
  * Handler - text data element
  *
  * @param object $parser
  * @param string $data
  * @access public
  */
  function parserCharacterData(&$parser, $data) {
    if (trim($data) != '') {
      $parent = &$this->_currentNode;
      new simple_xmltextnode($parent, $data);
    }
  }
  
  function unserializeArrayFromXML($tagName, &$dataArr, $str) {
    $this->loadFromString($str);
    if (isset($this->root)) {
      $this->unserializeArrayFromXMLNode($tagName, $dataArr, $this->root);
      $this->root->free();
      unset($this->root);
    }
  }
  
  
  function unserializeArrayFromXMLNode($tagName, &$dataArr, &$node) {
    if (isset($node) && isset($node->children) && is_array($node->children)) {
      foreach($node->children as $subNode) {
        if (is_object($subNode) && isset($subNode->attributes['name']) && isset($subNode->name)) {
          $name = trim($subNode->attributes['name']);
          if ($subNode->name == $tagName.'-list') {
            $dataArr[$name] = array();
            $this->unserializeArrayFromXMLNode($tagName, $dataArr[$name], $subNode);
          } elseif ($subNode->name == $tagName.'-element')  {
            $text = '';
            $subNode->printText($text);
            $dataArr[$name] = simple_xmltree::XMLToStr($text);
          }
        }
      }
    }
  }
}

?>